#ifndef CODEGEN_H
#define CODEGEN_H
//
// codegen.h - This file contains the header file for codegen.c
//             for Mr. Ham's ECMA-55 Minimal BASIC compiler.
//
// Copyright (C) 2013,2014  John Gatewood Ham
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// version 2 of the Free Software Foundation License.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program; if not, write to the Free Software Foundation, Inc.,
// 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//
#include <stdio.h>
#include <stdbool.h>
#include <inttypes.h>

enum relop { LT, LE, EQ, GE, GT, NE }; // type definition for comparison operators 
extern const char * const relop_names[]; // map used to convert relop values to strings for display
bool write_prologue(FILE *f,const char *inname);
bool write_epilogue(FILE *f);
bool do_INPUT(FILE *f,const char *nfmt);
bool push_float_literal(FILE *f,const char *flitname,const char *flitval);
bool push_string_literal(FILE *f,const char *slitname,const char *txt);
bool push_nvar(FILE *f, const char *varname);
bool push_nparm(FILE *f, const char *varname, const char *fname);
bool push_svar(FILE *f, const char *varname);
bool push_1D_navar(FILE *f, const char *varname, const uint32_t maxindex);
bool push_2D_navar(FILE *f, const char *varname, const uint32_t rows, const uint32_t columns);
bool donegate(FILE *f);
bool doadd(FILE *f);
bool dosubtract(FILE *f);
bool domultiply(FILE *f);
bool dodivide(FILE *f);
bool dopow(FILE *f);
bool doabs(FILE *f);
bool dosgn(FILE *f);
bool dosqr(FILE *f);
bool doint(FILE *f);
bool dornd(FILE *f);
bool dosin(FILE *f);
bool docos(FILE *f);
bool dotan(FILE *f);
bool doatan(FILE *f);
bool dolog(FILE *f);
bool doexp(FILE *f);
bool dorandomize(FILE *f);
bool dostore_nvar(FILE *f, const char *varname);
bool dostore_svar(FILE *f, const char *varname);
bool dostore_1D_navar(FILE *f, const char *varname, const uint32_t maxrows);
bool dostore_2D_navar(FILE *f, const char *varname, const uint32_t maxrows, const uint32_t maxcolumns);
bool doloc(FILE *f, const uint32_t lineno);
bool doprint_number(FILE *f);
bool doprint_string(FILE *f);
bool doprint_newline(FILE *f);
bool doprint_nextfield(FILE *f);
bool dostop(FILE *f);
bool dogoto(FILE *f, const uint32_t target_lineno);
bool donumeric_comparison(FILE *f, const enum relop op, const uint32_t target_lineno);
bool dostring_comparison(FILE *f, const enum relop op, const uint32_t target_lineno);
bool dogosub(FILE *f, const uint32_t target_lineno);
bool doreturn(FILE *f);
bool doongotostart(FILE *f, const uint32_t maxchoice);
bool doongotochoice(FILE *f, const uint32_t target_lineno);
bool doongotoend(FILE *f);
bool doforpush1(FILE *f);
bool doforloopstart(FILE *f, const char *ndex_var, char *tol_label, char *bol_label, char *inc_var, char *limit_var);
bool doforloopend(FILE *f,const char *tol_label, const char *bol_label, const char *ndex_var, const char *inc_var);
bool dotab(FILE *f);
bool doinput_nvar(FILE *f, const char *varname,const uint32_t slotno,const unsigned int lno);
bool doinput_svar(FILE *f, const char *varname,const uint32_t slotno);
bool doinput_navar(FILE *f, const char *varname, const uint32_t slotno,const uint32_t lno);
bool doinput_nvar_phase2(FILE *f, const char *varname,const uint32_t slotno,const uint32_t lno);
bool doinput_svar_phase2(FILE *f, const char *varname,const uint32_t slotno);
bool doinput_navar_phase2(FILE *f, const char *varname, const uint32_t rows, const uint32_t columns,const uint32_t slotno,const uint32_t lno);
bool doread_nvar(FILE *f, const char *varname);
bool doread_svar(FILE *f, const char *varname);
bool doread_navar(FILE *f, const char *varname, const uint32_t rows, const uint32_t columns);
bool dorestore(FILE *f);
bool doudf(FILE *f,const char *fname,bool hasarg);
bool doudf_header(FILE *f,const char *fname,const char *argname);
bool doudf_footer(FILE *f,const char *fname);
bool emit_comment(FILE *f,const char *fmt, ...)  __attribute__ ((format (printf, 2, 3)));
void gen_stack(FILE *f);
#endif
