//
// jenkins.c - This file contains the code generation routines
//             for Bob Jenkin's  ISAAC-64 random number generator
//             used to support the Minimal BASIC RND() function
//             and the Minimal BASIC RANDOMIZE statemen
//             for Mr. Ham's ECMA-55 Minimal BASIC compiler.
//             This code is in the public domain.
//             Upstream URL:
//             http://burtleburtle.net/bob/rand/isaacafa.html
//
#include <stdlib.h>
#include "jenkins.h"

//
//  This procedure will emit code for Bob Jenkin's ISAAC-64 random
//  number generator.  The f parameter is the file handle for the
//  destination assembly language file, and the file must already
//  be open.  This is used to support the Minimal BASIC RND function.
//  This code was created by running a C compiler on Jenkin's 
//  isaac64.[ch] files and having it emit assembly.  This requires
//  SSE2 instruction set support.
//
void write_RND(FILE *f) {
  if (NULL==f) {
    fprintf(stderr,"write_RND(): file parameter is NULL\n");
    abort();
  }
  // begin public domain derived from ISAAC-64/Bob Jenkins' C code
  fprintf(f,"# http://burtleburtle.net/bob/rand/isaacafa.html\n");
  fprintf(f,"        .section .rodata.cst16,\"aM\",@progbits,16\n");
  fprintf(f,"        .align  16, 0\n");
  fprintf(f,".LRAND_0:\n");
  fprintf(f,"        .long   1127219200              # 0x43300000\n");
  fprintf(f,"        .long   1160773632              # 0x45300000\n");
  fprintf(f,"        .long   0                       # 0x0\n");
  fprintf(f,"        .long   0                       # 0x0\n");
  fprintf(f,".LRAND_1:\n");
  fprintf(f,"        .quad   4841369599423283200     # double 4.503600e+15\n");
  fprintf(f,"        .quad   4985484787499139072     # double 1.934281e+25\n\n");
  fprintf(f,"        .section .rodata.cst8,\"aM\",@progbits,8\n");
  fprintf(f,"        .align  8, 0\n");
  fprintf(f,".LRAND_2:\n");
  fprintf(f,"        .quad   4323455642275676160     # double 1.084202172485504E-19\n\n");

  fprintf(f,"        .section .data,\"aw\",@progbits\n\n");

  fprintf(f,"        .align  8\n");
  fprintf(f,"        .type   .LRAND_aa,@object\n");
  fprintf(f,".LRAND_aa:\n");
  fprintf(f,"        .quad   0\n");
  fprintf(f,"        .size   .LRAND_aa, 8\n");
  fprintf(f,"        .type   .LRAND_bb,@object\n");
  fprintf(f,".LRAND_bb:\n");
  fprintf(f,"        .quad   0\n");
  fprintf(f,"        .size   .LRAND_bb, 8\n");
  fprintf(f,"        .type   .LRAND_cc,@object\n");
  fprintf(f,".LRAND_cc:\n");
  fprintf(f,"        .quad   0\n");
  fprintf(f,"        .size   .LRAND_cc, 8\n");
  fprintf(f,"        .type   .LRAND_randcnt,@object\n");
  fprintf(f,".LRAND_randcnt:\n");
  fprintf(f,"        .quad   0\n");
  fprintf(f,"        .size   .LRAND_randcnt, 8\n");
  fprintf(f,"        .type   .LRAND_mm,@object\n");
  fprintf(f,"        .align  16, 0\n");
  fprintf(f,".LRAND_mm:\n");
  fprintf(f,"        .skip   2048, 0\n");
  fprintf(f,"        .size   .LRAND_mm, 2048\n");
  fprintf(f,"        .type   .LRAND_randrsl,@object\n");
  fprintf(f,"        .align  16\n");
  fprintf(f,".LRAND_randrsl:\n");
  fprintf(f,"        .skip   2048, 0\n");
  fprintf(f,"        .size   .LRAND_randrsl, 2048\n\n");

  fprintf(f,"        .section .text,\"ax\",@progbits\n\n");

  fprintf(f,"        .type   isaac64,@function\n");
  fprintf(f,"isaac64:\n");
  fprintf(f,"        pushq   %%rbp\n");
  fprintf(f,"        movq    %%rsp, %%rbp\n");
  fprintf(f,"        movabsq $.LRAND_mm, %%rax\n");
  fprintf(f,"        movq    %%rax, -40(%%rbp)\n");
  fprintf(f,"        movabsq $.LRAND_randrsl, %%rcx\n");
  fprintf(f,"        movq    %%rcx, -56(%%rbp)\n");
  fprintf(f,"        movabsq $.LRAND_aa, %%rcx\n");
  fprintf(f,"        movq    (%%rcx), %%rcx\n");
  fprintf(f,"        movq    %%rcx, -8(%%rbp)\n");
  fprintf(f,"        movabsq $.LRAND_bb, %%rcx\n");
  fprintf(f,"        movq    (%%rcx), %%rcx\n");
  fprintf(f,"        movabsq $.LRAND_cc, %%rdx\n");
  fprintf(f,"        movq    (%%rdx), %%rsi\n");
  fprintf(f,"        movq    %%rsi, %%rdi\n");
  fprintf(f,"        incq    %%rdi\n");
  fprintf(f,"        movq    %%rdi, (%%rdx)\n");
  fprintf(f,"        leaq    1(%%rcx,%%rsi), %%rcx\n");
  fprintf(f,"        movq    %%rcx, -16(%%rbp)\n");
  fprintf(f,"        movq    %%rax, -40(%%rbp)\n");
  fprintf(f,"        movq    -40(%%rbp), %%rax\n");
  fprintf(f,"        addq    $1024, %%rax             # imm = 0x400\n");
  fprintf(f,"        movq    %%rax, -48(%%rbp)\n");
  fprintf(f,"        movq    %%rax, -64(%%rbp)\n");
  fprintf(f,"0:\n");
  fprintf(f,"        movq    -40(%%rbp), %%rax\n");
  fprintf(f,"        cmpq    -64(%%rbp), %%rax\n");
  fprintf(f,"        jb      1f\n");
  fprintf(f,"        jmp     2f\n");
  fprintf(f,"1:\n");
  fprintf(f,"        movq    -40(%%rbp), %%rax\n");
  fprintf(f,"        movq    (%%rax), %%rax\n");
  fprintf(f,"        movq    %%rax, -24(%%rbp)\n");
  fprintf(f,"        movq    -8(%%rbp), %%rax\n");
  fprintf(f,"        movq    %%rax, %%rcx\n");
  fprintf(f,"        shlq    $21, %%rcx\n");
  fprintf(f,"        xorq    %%rcx, %%rax\n");
  fprintf(f,"        notq    %%rax\n");
  fprintf(f,"        movq    -48(%%rbp), %%rcx\n");
  fprintf(f,"        movq    %%rcx, %%rdx\n");
  fprintf(f,"        addq    $8, %%rdx\n");
  fprintf(f,"        movq    %%rdx, -48(%%rbp)\n");
  fprintf(f,"        movq    (%%rcx), %%rcx\n");
  fprintf(f,"        addq    %%rcx, %%rax\n");
  fprintf(f,"        movq    %%rax, -8(%%rbp)\n");
  fprintf(f,"        movq    -24(%%rbp), %%rcx\n");
  fprintf(f,"        andq    $2040, %%rcx             # imm = 0x7F8\n");
  fprintf(f,"        movabsq $.LRAND_mm, %%rdx\n");
  fprintf(f,"        movq    (%%rdx,%%rcx), %%rcx\n");
  fprintf(f,"        addq    %%rax, %%rcx\n");
  fprintf(f,"        movq    -16(%%rbp), %%rax\n");
  fprintf(f,"        addq    %%rax, %%rcx\n");
  fprintf(f,"        movq    %%rcx, -32(%%rbp)\n");
  fprintf(f,"        movq    -40(%%rbp), %%rax\n");
  fprintf(f,"        movq    %%rax, %%rsi\n");
  fprintf(f,"        addq    $8, %%rsi\n");
  fprintf(f,"        movq    %%rsi, -40(%%rbp)\n");
  fprintf(f,"        movq    %%rcx, (%%rax)\n");
  fprintf(f,"        movq    -32(%%rbp), %%rax\n");
  fprintf(f,"        shrq    $8, %%rax\n");
  fprintf(f,"        andq    $2040, %%rax             # imm = 0x7F8\n");
  fprintf(f,"        movq    (%%rdx,%%rax), %%rax\n");
  fprintf(f,"        movq    -24(%%rbp), %%rcx\n");
  fprintf(f,"        addq    %%rcx, %%rax\n");
  fprintf(f,"        movq    %%rax, -16(%%rbp)\n");
  fprintf(f,"        movq    -56(%%rbp), %%rcx\n");
  fprintf(f,"        movq    %%rcx, %%rsi\n");
  fprintf(f,"        addq    $8, %%rsi\n");
  fprintf(f,"        movq    %%rsi, -56(%%rbp)\n");
  fprintf(f,"        movq    %%rax, (%%rcx)\n");
  fprintf(f,"        movq    -40(%%rbp), %%rax\n");
  fprintf(f,"        movq    (%%rax), %%rax\n");
  fprintf(f,"        movq    %%rax, -24(%%rbp)\n");
  fprintf(f,"        movq    -8(%%rbp), %%rax\n");
  fprintf(f,"        movq    %%rax, %%rcx\n");
  fprintf(f,"        shrq    $5, %%rcx\n");
  fprintf(f,"        xorq    %%rcx, %%rax\n");
  fprintf(f,"        movq    -48(%%rbp), %%rcx\n");
  fprintf(f,"        movq    %%rcx, %%rsi\n");
  fprintf(f,"        addq    $8, %%rsi\n");
  fprintf(f,"        movq    %%rsi, -48(%%rbp)\n");
  fprintf(f,"        movq    (%%rcx), %%rcx\n");
  fprintf(f,"        addq    %%rcx, %%rax\n");
  fprintf(f,"        movq    %%rax, -8(%%rbp)\n");
  fprintf(f,"        movq    -24(%%rbp), %%rcx\n");
  fprintf(f,"        andq    $2040, %%rcx             # imm = 0x7F8\n");
  fprintf(f,"        movq    (%%rdx,%%rcx), %%rcx\n");
  fprintf(f,"        addq    %%rax, %%rcx\n");
  fprintf(f,"        movq    -16(%%rbp), %%rax\n");
  fprintf(f,"        addq    %%rax, %%rcx\n");
  fprintf(f,"        movq    %%rcx, -32(%%rbp)\n");
  fprintf(f,"        movq    -40(%%rbp), %%rax\n");
  fprintf(f,"        movq    %%rax, %%rsi\n");
  fprintf(f,"        addq    $8, %%rsi\n");
  fprintf(f,"        movq    %%rsi, -40(%%rbp)\n");
  fprintf(f,"        movq    %%rcx, (%%rax)\n");
  fprintf(f,"        movq    -32(%%rbp), %%rax\n");
  fprintf(f,"        shrq    $8, %%rax\n");
  fprintf(f,"        andq    $2040, %%rax             # imm = 0x7F8\n");
  fprintf(f,"        movq    (%%rdx,%%rax), %%rax\n");
  fprintf(f,"        movq    -24(%%rbp), %%rcx\n");
  fprintf(f,"        addq    %%rcx, %%rax\n");
  fprintf(f,"        movq    %%rax, -16(%%rbp)\n");
  fprintf(f,"        movq    -56(%%rbp), %%rcx\n");
  fprintf(f,"        movq    %%rcx, %%rsi\n");
  fprintf(f,"        addq    $8, %%rsi\n");
  fprintf(f,"        movq    %%rsi, -56(%%rbp)\n");
  fprintf(f,"        movq    %%rax, (%%rcx)\n");
  fprintf(f,"        movq    -40(%%rbp), %%rax\n");
  fprintf(f,"        movq    (%%rax), %%rax\n");
  fprintf(f,"        movq    %%rax, -24(%%rbp)\n");
  fprintf(f,"        movq    -8(%%rbp), %%rax\n");
  fprintf(f,"        movq    %%rax, %%rcx\n");
  fprintf(f,"        shlq    $12, %%rcx\n");
  fprintf(f,"        xorq    %%rcx, %%rax\n");
  fprintf(f,"        movq    -48(%%rbp), %%rcx\n");
  fprintf(f,"        movq    %%rcx, %%rsi\n");
  fprintf(f,"        addq    $8, %%rsi\n");
  fprintf(f,"        movq    %%rsi, -48(%%rbp)\n");
  fprintf(f,"        movq    (%%rcx), %%rcx\n");
  fprintf(f,"        addq    %%rcx, %%rax\n");
  fprintf(f,"        movq    %%rax, -8(%%rbp)\n");
  fprintf(f,"        movq    -24(%%rbp), %%rcx\n");
  fprintf(f,"        andq    $2040, %%rcx             # imm = 0x7F8\n");
  fprintf(f,"        movq    (%%rdx,%%rcx), %%rcx\n");
  fprintf(f,"        addq    %%rax, %%rcx\n");
  fprintf(f,"        movq    -16(%%rbp), %%rax\n");
  fprintf(f,"        addq    %%rax, %%rcx\n");
  fprintf(f,"        movq    %%rcx, -32(%%rbp)\n");
  fprintf(f,"        movq    -40(%%rbp), %%rax\n");
  fprintf(f,"        movq    %%rax, %%rsi\n");
  fprintf(f,"        addq    $8, %%rsi\n");
  fprintf(f,"        movq    %%rsi, -40(%%rbp)\n");
  fprintf(f,"        movq    %%rcx, (%%rax)\n");
  fprintf(f,"        movq    -32(%%rbp), %%rax\n");
  fprintf(f,"        shrq    $8, %%rax\n");
  fprintf(f,"        andq    $2040, %%rax             # imm = 0x7F8\n");
  fprintf(f,"        movq    (%%rdx,%%rax), %%rax\n");
  fprintf(f,"        movq    -24(%%rbp), %%rcx\n");
  fprintf(f,"        addq    %%rcx, %%rax\n");
  fprintf(f,"        movq    %%rax, -16(%%rbp)\n");
  fprintf(f,"        movq    -56(%%rbp), %%rcx\n");
  fprintf(f,"        movq    %%rcx, %%rsi\n");
  fprintf(f,"        addq    $8, %%rsi\n");
  fprintf(f,"        movq    %%rsi, -56(%%rbp)\n");
  fprintf(f,"        movq    %%rax, (%%rcx)\n");
  fprintf(f,"        movq    -40(%%rbp), %%rax\n");
  fprintf(f,"        movq    (%%rax), %%rax\n");
  fprintf(f,"        movq    %%rax, -24(%%rbp)\n");
  fprintf(f,"        movq    -8(%%rbp), %%rax\n");
  fprintf(f,"        movq    %%rax, %%rcx\n");
  fprintf(f,"        shrq    $33, %%rcx\n");
  fprintf(f,"        xorq    %%rcx, %%rax\n");
  fprintf(f,"        movq    -48(%%rbp), %%rcx\n");
  fprintf(f,"        movq    %%rcx, %%rsi\n");
  fprintf(f,"        addq    $8, %%rsi\n");
  fprintf(f,"        movq    %%rsi, -48(%%rbp)\n");
  fprintf(f,"        movq    (%%rcx), %%rcx\n");
  fprintf(f,"        addq    %%rcx, %%rax\n");
  fprintf(f,"        movq    %%rax, -8(%%rbp)\n");
  fprintf(f,"        movq    -24(%%rbp), %%rcx\n");
  fprintf(f,"        andq    $2040, %%rcx             # imm = 0x7F8\n");
  fprintf(f,"        movq    (%%rdx,%%rcx), %%rcx\n");
  fprintf(f,"        addq    %%rax, %%rcx\n");
  fprintf(f,"        movq    -16(%%rbp), %%rax\n");
  fprintf(f,"        addq    %%rax, %%rcx\n");
  fprintf(f,"        movq    %%rcx, -32(%%rbp)\n");
  fprintf(f,"        movq    -40(%%rbp), %%rax\n");
  fprintf(f,"        movq    %%rax, %%rsi\n");
  fprintf(f,"        addq    $8, %%rsi\n");
  fprintf(f,"        movq    %%rsi, -40(%%rbp)\n");
  fprintf(f,"        movq    %%rcx, (%%rax)\n");
  fprintf(f,"        movq    -32(%%rbp), %%rax\n");
  fprintf(f,"        shrq    $8, %%rax\n");
  fprintf(f,"        andq    $2040, %%rax             # imm = 0x7F8\n");
  fprintf(f,"        addq    %%rax, %%rdx\n");
  fprintf(f,"        movq    (%%rdx), %%rax\n");
  fprintf(f,"        addq    -24(%%rbp), %%rax\n");
  fprintf(f,"        movq    %%rax, -16(%%rbp)\n");
  fprintf(f,"        movq    -56(%%rbp), %%rcx\n");
  fprintf(f,"        movq    %%rcx, %%rdx\n");
  fprintf(f,"        addq    $8, %%rdx\n");
  fprintf(f,"        movq    %%rdx, -56(%%rbp)\n");
  fprintf(f,"        movq    %%rax, (%%rcx)\n");
  fprintf(f,"        jmp     0b\n");
  fprintf(f,"2:\n");
  fprintf(f,"        movabsq $.LRAND_mm, %%rax\n");
  fprintf(f,"        movq    %%rax, -48(%%rbp)\n");
  fprintf(f,"3:\n");
  fprintf(f,"        movq    -48(%%rbp), %%rax\n");
  fprintf(f,"        cmpq    -64(%%rbp), %%rax\n");
  fprintf(f,"        jb      4f\n");
  fprintf(f,"        jmp     5f\n");
  fprintf(f,"4:\n");
  fprintf(f,"        movq    -40(%%rbp), %%rax\n");
  fprintf(f,"        movq    (%%rax), %%rax\n");
  fprintf(f,"        movq    %%rax, -24(%%rbp)\n");
  fprintf(f,"        movq    -8(%%rbp), %%rax\n");
  fprintf(f,"        movq    %%rax, %%rcx\n");
  fprintf(f,"        shlq    $21, %%rcx\n");
  fprintf(f,"        xorq    %%rcx, %%rax\n");
  fprintf(f,"        notq    %%rax\n");
  fprintf(f,"        movq    -48(%%rbp), %%rcx\n");
  fprintf(f,"        movq    %%rcx, %%rdx\n");
  fprintf(f,"        addq    $8, %%rdx\n");
  fprintf(f,"        movq    %%rdx, -48(%%rbp)\n");
  fprintf(f,"        movq    (%%rcx), %%rcx\n");
  fprintf(f,"        addq    %%rcx, %%rax\n");
  fprintf(f,"        movq    %%rax, -8(%%rbp)\n");
  fprintf(f,"        movq    -24(%%rbp), %%rcx\n");
  fprintf(f,"        andq    $2040, %%rcx             # imm = 0x7F8\n");
  fprintf(f,"        movabsq $.LRAND_mm, %%rdx\n");
  fprintf(f,"        movq    (%%rdx,%%rcx), %%rcx\n");
  fprintf(f,"        addq    %%rax, %%rcx\n");
  fprintf(f,"        movq    -16(%%rbp), %%rax\n");
  fprintf(f,"        addq    %%rax, %%rcx\n");
  fprintf(f,"        movq    %%rcx, -32(%%rbp)\n");
  fprintf(f,"        movq    -40(%%rbp), %%rax\n");
  fprintf(f,"        movq    %%rax, %%rsi\n");
  fprintf(f,"        addq    $8, %%rsi\n");
  fprintf(f,"        movq    %%rsi, -40(%%rbp)\n");
  fprintf(f,"        movq    %%rcx, (%%rax)\n");
  fprintf(f,"        movq    -32(%%rbp), %%rax\n");
  fprintf(f,"        shrq    $8, %%rax\n");
  fprintf(f,"        andq    $2040, %%rax             # imm = 0x7F8\n");
  fprintf(f,"        movq    (%%rdx,%%rax), %%rax\n");
  fprintf(f,"        movq    -24(%%rbp), %%rcx\n");
  fprintf(f,"        addq    %%rcx, %%rax\n");
  fprintf(f,"        movq    %%rax, -16(%%rbp)\n");
  fprintf(f,"        movq    -56(%%rbp), %%rcx\n");
  fprintf(f,"        movq    %%rcx, %%rsi\n");
  fprintf(f,"        addq    $8, %%rsi\n");
  fprintf(f,"        movq    %%rsi, -56(%%rbp)\n");
  fprintf(f,"        movq    %%rax, (%%rcx)\n");
  fprintf(f,"        movq    -40(%%rbp), %%rax\n");
  fprintf(f,"        movq    (%%rax), %%rax\n");
  fprintf(f,"        movq    %%rax, -24(%%rbp)\n");
  fprintf(f,"        movq    -8(%%rbp), %%rax\n");
  fprintf(f,"        movq    %%rax, %%rcx\n");
  fprintf(f,"        shrq    $5, %%rcx\n");
  fprintf(f,"        xorq    %%rcx, %%rax\n");
  fprintf(f,"        movq    -48(%%rbp), %%rcx\n");
  fprintf(f,"        movq    %%rcx, %%rsi\n");
  fprintf(f,"        addq    $8, %%rsi\n");
  fprintf(f,"        movq    %%rsi, -48(%%rbp)\n");
  fprintf(f,"        movq    (%%rcx), %%rcx\n");
  fprintf(f,"        addq    %%rcx, %%rax\n");
  fprintf(f,"        movq    %%rax, -8(%%rbp)\n");
  fprintf(f,"        movq    -24(%%rbp), %%rcx\n");
  fprintf(f,"        andq    $2040, %%rcx             # imm = 0x7F8\n");
  fprintf(f,"        movq    (%%rdx,%%rcx), %%rcx\n");
  fprintf(f,"        addq    %%rax, %%rcx\n");
  fprintf(f,"        movq    -16(%%rbp), %%rax\n");
  fprintf(f,"        addq    %%rax, %%rcx\n");
  fprintf(f,"        movq    %%rcx, -32(%%rbp)\n");
  fprintf(f,"        movq    -40(%%rbp), %%rax\n");
  fprintf(f,"        movq    %%rax, %%rsi\n");
  fprintf(f,"        addq    $8, %%rsi\n");
  fprintf(f,"        movq    %%rsi, -40(%%rbp)\n");
  fprintf(f,"        movq    %%rcx, (%%rax)\n");
  fprintf(f,"        movq    -32(%%rbp), %%rax\n");
  fprintf(f,"        shrq    $8, %%rax\n");
  fprintf(f,"        andq    $2040, %%rax             # imm = 0x7F8\n");
  fprintf(f,"        movq    (%%rdx,%%rax), %%rax\n");
  fprintf(f,"        movq    -24(%%rbp), %%rcx\n");
  fprintf(f,"        addq    %%rcx, %%rax\n");
  fprintf(f,"        movq    %%rax, -16(%%rbp)\n");
  fprintf(f,"        movq    -56(%%rbp), %%rcx\n");
  fprintf(f,"        movq    %%rcx, %%rsi\n");
  fprintf(f,"        addq    $8, %%rsi\n");
  fprintf(f,"        movq    %%rsi, -56(%%rbp)\n");
  fprintf(f,"        movq    %%rax, (%%rcx)\n");
  fprintf(f,"        movq    -40(%%rbp), %%rax\n");
  fprintf(f,"        movq    (%%rax), %%rax\n");
  fprintf(f,"        movq    %%rax, -24(%%rbp)\n");
  fprintf(f,"        movq    -8(%%rbp), %%rax\n");
  fprintf(f,"        movq    %%rax, %%rcx\n");
  fprintf(f,"        shlq    $12, %%rcx\n");
  fprintf(f,"        xorq    %%rcx, %%rax\n");
  fprintf(f,"        movq    -48(%%rbp), %%rcx\n");
  fprintf(f,"        movq    %%rcx, %%rsi\n");
  fprintf(f,"        addq    $8, %%rsi\n");
  fprintf(f,"        movq    %%rsi, -48(%%rbp)\n");
  fprintf(f,"        movq    (%%rcx), %%rcx\n");
  fprintf(f,"        addq    %%rcx, %%rax\n");
  fprintf(f,"        movq    %%rax, -8(%%rbp)\n");
  fprintf(f,"        movq    -24(%%rbp), %%rcx\n");
  fprintf(f,"        andq    $2040, %%rcx             # imm = 0x7F8\n");
  fprintf(f,"        movq    (%%rdx,%%rcx), %%rcx\n");
  fprintf(f,"        addq    %%rax, %%rcx\n");
  fprintf(f,"        movq    -16(%%rbp), %%rax\n");
  fprintf(f,"        addq    %%rax, %%rcx\n");
  fprintf(f,"        movq    %%rcx, -32(%%rbp)\n");
  fprintf(f,"        movq    -40(%%rbp), %%rax\n");
  fprintf(f,"        movq    %%rax, %%rsi\n");
  fprintf(f,"        addq    $8, %%rsi\n");
  fprintf(f,"        movq    %%rsi, -40(%%rbp)\n");
  fprintf(f,"        movq    %%rcx, (%%rax)\n");
  fprintf(f,"        movq    -32(%%rbp), %%rax\n");
  fprintf(f,"        shrq    $8, %%rax\n");
  fprintf(f,"        andq    $2040, %%rax             # imm = 0x7F8\n");
  fprintf(f,"        movq    (%%rdx,%%rax), %%rax\n");
  fprintf(f,"        movq    -24(%%rbp), %%rcx\n");
  fprintf(f,"        addq    %%rcx, %%rax\n");
  fprintf(f,"        movq    %%rax, -16(%%rbp)\n");
  fprintf(f,"        movq    -56(%%rbp), %%rcx\n");
  fprintf(f,"        movq    %%rcx, %%rsi\n");
  fprintf(f,"        addq    $8, %%rsi\n");
  fprintf(f,"        movq    %%rsi, -56(%%rbp)\n");
  fprintf(f,"        movq    %%rax, (%%rcx)\n");
  fprintf(f,"        movq    -40(%%rbp), %%rax\n");
  fprintf(f,"        movq    (%%rax), %%rax\n");
  fprintf(f,"        movq    %%rax, -24(%%rbp)\n");
  fprintf(f,"        movq    -8(%%rbp), %%rax\n");
  fprintf(f,"        movq    %%rax, %%rcx\n");
  fprintf(f,"        shrq    $33, %%rcx\n");
  fprintf(f,"        xorq    %%rcx, %%rax\n");
  fprintf(f,"        movq    -48(%%rbp), %%rcx\n");
  fprintf(f,"        movq    %%rcx, %%rsi\n");
  fprintf(f,"        addq    $8, %%rsi\n");
  fprintf(f,"        movq    %%rsi, -48(%%rbp)\n");
  fprintf(f,"        movq    (%%rcx), %%rcx\n");
  fprintf(f,"        addq    %%rcx, %%rax\n");
  fprintf(f,"        movq    %%rax, -8(%%rbp)\n");
  fprintf(f,"        movq    -24(%%rbp), %%rcx\n");
  fprintf(f,"        andq    $2040, %%rcx             # imm = 0x7F8\n");
  fprintf(f,"        movq    (%%rdx,%%rcx), %%rcx\n");
  fprintf(f,"        addq    %%rax, %%rcx\n");
  fprintf(f,"        movq    -16(%%rbp), %%rax\n");
  fprintf(f,"        addq    %%rax, %%rcx\n");
  fprintf(f,"        movq    %%rcx, -32(%%rbp)\n");
  fprintf(f,"        movq    -40(%%rbp), %%rax\n");
  fprintf(f,"        movq    %%rax, %%rsi\n");
  fprintf(f,"        addq    $8, %%rsi\n");
  fprintf(f,"        movq    %%rsi, -40(%%rbp)\n");
  fprintf(f,"        movq    %%rcx, (%%rax)\n");
  fprintf(f,"        movq    -32(%%rbp), %%rax\n");
  fprintf(f,"        shrq    $8, %%rax\n");
  fprintf(f,"        andq    $2040, %%rax             # imm = 0x7F8\n");
  fprintf(f,"        addq    %%rax, %%rdx\n");
  fprintf(f,"        movq    (%%rdx), %%rax\n");
  fprintf(f,"        addq    -24(%%rbp), %%rax\n");
  fprintf(f,"        movq    %%rax, -16(%%rbp)\n");
  fprintf(f,"        movq    -56(%%rbp), %%rcx\n");
  fprintf(f,"        movq    %%rcx, %%rdx\n");
  fprintf(f,"        addq    $8, %%rdx\n");
  fprintf(f,"        movq    %%rdx, -56(%%rbp)\n");
  fprintf(f,"        movq    %%rax, (%%rcx)\n");
  fprintf(f,"        jmp     3b\n");
  fprintf(f,"5:\n");
  fprintf(f,"        movq    -16(%%rbp), %%rax\n");
  fprintf(f,"        movabsq $.LRAND_bb, %%rcx\n");
  fprintf(f,"        movq    %%rax, (%%rcx)\n");
  fprintf(f,"        movq    -8(%%rbp), %%rax\n");
  fprintf(f,"        movabsq $.LRAND_aa, %%rcx\n");
  fprintf(f,"        movq    %%rax, (%%rcx)\n");
  fprintf(f,"        popq    %%rbp\n");
  fprintf(f,"        retq\n");
  fprintf(f,"        .size   isaac64, .-isaac64\n\n");
  fprintf(f,"        .type   randinit,@function\n");
  fprintf(f,"randinit:\n");
  fprintf(f,"        pushq   %%rbp\n");
  fprintf(f,"        movq    %%rsp, %%rbp\n");
  fprintf(f,"        subq    $80, %%rsp\n");
  fprintf(f,"        movabsq $0x9E3779B97F4A7C13, %%rax\n");
  fprintf(f,"        movl    %%edi, -4(%%rbp)\n");
  fprintf(f,"        movabsq $.LRAND_cc, %%rcx\n");
  fprintf(f,"        movq    $0, (%%rcx)\n");
  fprintf(f,"        movabsq $.LRAND_bb, %%rcx\n");
  fprintf(f,"        movq    $0, (%%rcx)\n");
  fprintf(f,"        movabsq $.LRAND_aa, %%rcx\n");
  fprintf(f,"        movq    $0, (%%rcx)\n");
  fprintf(f,"        movq    %%rax, -72(%%rbp)\n");
  fprintf(f,"        movq    %%rax, -64(%%rbp)\n");
  fprintf(f,"        movq    %%rax, -56(%%rbp)\n");
  fprintf(f,"        movq    %%rax, -48(%%rbp)\n");
  fprintf(f,"        movq    %%rax, -40(%%rbp)\n");
  fprintf(f,"        movq    %%rax, -32(%%rbp)\n");
  fprintf(f,"        movq    %%rax, -24(%%rbp)\n");
  fprintf(f,"        movq    %%rax, -16(%%rbp)\n");
  fprintf(f,"        movl    $0, -8(%%rbp)\n");
  fprintf(f,"0:\n");
  fprintf(f,"        cmpl    $4, -8(%%rbp)\n");
  fprintf(f,"        jl      1f\n");
  fprintf(f,"        jmp     2f\n");
  fprintf(f,"1:\n");
  fprintf(f,"        movq    -48(%%rbp), %%rax\n");
  fprintf(f,"        movq    -16(%%rbp), %%rcx\n");
  fprintf(f,"        subq    %%rax, %%rcx\n");
  fprintf(f,"        movq    %%rcx, -16(%%rbp)\n");
  fprintf(f,"        movq    -72(%%rbp), %%rax\n");
  fprintf(f,"        shrq    $9, %%rax\n");
  fprintf(f,"        movq    -56(%%rbp), %%rcx\n");
  fprintf(f,"        xorq    %%rax, %%rcx\n");
  fprintf(f,"        movq    %%rcx, -56(%%rbp)\n");
  fprintf(f,"        movq    -16(%%rbp), %%rax\n");
  fprintf(f,"        movq    -72(%%rbp), %%rcx\n");
  fprintf(f,"        addq    %%rax, %%rcx\n");
  fprintf(f,"        movq    %%rcx, -72(%%rbp)\n");
  fprintf(f,"        movq    -56(%%rbp), %%rax\n");
  fprintf(f,"        movq    -24(%%rbp), %%rcx\n");
  fprintf(f,"        subq    %%rax, %%rcx\n");
  fprintf(f,"        movq    %%rcx, -24(%%rbp)\n");
  fprintf(f,"        movq    -16(%%rbp), %%rax\n");
  fprintf(f,"        shlq    $9, %%rax\n");
  fprintf(f,"        movq    -64(%%rbp), %%rcx\n");
  fprintf(f,"        xorq    %%rax, %%rcx\n");
  fprintf(f,"        movq    %%rcx, -64(%%rbp)\n");
  fprintf(f,"        movq    -24(%%rbp), %%rax\n");
  fprintf(f,"        movq    -16(%%rbp), %%rcx\n");
  fprintf(f,"        addq    %%rax, %%rcx\n");
  fprintf(f,"        movq    %%rcx, -16(%%rbp)\n");
  fprintf(f,"        movq    -64(%%rbp), %%rax\n");
  fprintf(f,"        movq    -32(%%rbp), %%rcx\n");
  fprintf(f,"        subq    %%rax, %%rcx\n");
  fprintf(f,"        movq    %%rcx, -32(%%rbp)\n");
  fprintf(f,"        movq    -24(%%rbp), %%rax\n");
  fprintf(f,"        shrq    $23, %%rax\n");
  fprintf(f,"        movq    -72(%%rbp), %%rcx\n");
  fprintf(f,"        xorq    %%rax, %%rcx\n");
  fprintf(f,"        movq    %%rcx, -72(%%rbp)\n");
  fprintf(f,"        movq    -32(%%rbp), %%rax\n");
  fprintf(f,"        movq    -24(%%rbp), %%rcx\n");
  fprintf(f,"        addq    %%rax, %%rcx\n");
  fprintf(f,"        movq    %%rcx, -24(%%rbp)\n");
  fprintf(f,"        movq    -72(%%rbp), %%rax\n");
  fprintf(f,"        movq    -40(%%rbp), %%rcx\n");
  fprintf(f,"        subq    %%rax, %%rcx\n");
  fprintf(f,"        movq    %%rcx, -40(%%rbp)\n");
  fprintf(f,"        movq    -32(%%rbp), %%rax\n");
  fprintf(f,"        shlq    $15, %%rax\n");
  fprintf(f,"        movq    -16(%%rbp), %%rcx\n");
  fprintf(f,"        xorq    %%rax, %%rcx\n");
  fprintf(f,"        movq    %%rcx, -16(%%rbp)\n");
  fprintf(f,"        movq    -40(%%rbp), %%rax\n");
  fprintf(f,"        movq    -32(%%rbp), %%rcx\n");
  fprintf(f,"        addq    %%rax, %%rcx\n");
  fprintf(f,"        movq    %%rcx, -32(%%rbp)\n");
  fprintf(f,"        movq    -16(%%rbp), %%rax\n");
  fprintf(f,"        movq    -48(%%rbp), %%rcx\n");
  fprintf(f,"        subq    %%rax, %%rcx\n");
  fprintf(f,"        movq    %%rcx, -48(%%rbp)\n");
  fprintf(f,"        movq    -40(%%rbp), %%rax\n");
  fprintf(f,"        shrq    $14, %%rax\n");
  fprintf(f,"        movq    -24(%%rbp), %%rcx\n");
  fprintf(f,"        xorq    %%rax, %%rcx\n");
  fprintf(f,"        movq    %%rcx, -24(%%rbp)\n");
  fprintf(f,"        movq    -48(%%rbp), %%rax\n");
  fprintf(f,"        movq    -40(%%rbp), %%rcx\n");
  fprintf(f,"        addq    %%rax, %%rcx\n");
  fprintf(f,"        movq    %%rcx, -40(%%rbp)\n");
  fprintf(f,"        movq    -24(%%rbp), %%rax\n");
  fprintf(f,"        movq    -56(%%rbp), %%rcx\n");
  fprintf(f,"        subq    %%rax, %%rcx\n");
  fprintf(f,"        movq    %%rcx, -56(%%rbp)\n");
  fprintf(f,"        movq    -48(%%rbp), %%rax\n");
  fprintf(f,"        shlq    $20, %%rax\n");
  fprintf(f,"        movq    -32(%%rbp), %%rcx\n");
  fprintf(f,"        xorq    %%rax, %%rcx\n");
  fprintf(f,"        movq    %%rcx, -32(%%rbp)\n");
  fprintf(f,"        movq    -56(%%rbp), %%rax\n");
  fprintf(f,"        movq    -48(%%rbp), %%rcx\n");
  fprintf(f,"        addq    %%rax, %%rcx\n");
  fprintf(f,"        movq    %%rcx, -48(%%rbp)\n");
  fprintf(f,"        movq    -32(%%rbp), %%rax\n");
  fprintf(f,"        movq    -64(%%rbp), %%rcx\n");
  fprintf(f,"        subq    %%rax, %%rcx\n");
  fprintf(f,"        movq    %%rcx, -64(%%rbp)\n");
  fprintf(f,"        movq    -56(%%rbp), %%rax\n");
  fprintf(f,"        shrq    $17, %%rax\n");
  fprintf(f,"        movq    -40(%%rbp), %%rcx\n");
  fprintf(f,"        xorq    %%rax, %%rcx\n");
  fprintf(f,"        movq    %%rcx, -40(%%rbp)\n");
  fprintf(f,"        movq    -64(%%rbp), %%rax\n");
  fprintf(f,"        movq    -56(%%rbp), %%rcx\n");
  fprintf(f,"        addq    %%rax, %%rcx\n");
  fprintf(f,"        movq    %%rcx, -56(%%rbp)\n");
  fprintf(f,"        movq    -40(%%rbp), %%rax\n");
  fprintf(f,"        movq    -72(%%rbp), %%rcx\n");
  fprintf(f,"        subq    %%rax, %%rcx\n");
  fprintf(f,"        movq    %%rcx, -72(%%rbp)\n");
  fprintf(f,"        movq    -64(%%rbp), %%rax\n");
  fprintf(f,"        shlq    $14, %%rax\n");
  fprintf(f,"        movq    -48(%%rbp), %%rcx\n");
  fprintf(f,"        xorq    %%rax, %%rcx\n");
  fprintf(f,"        movq    %%rcx, -48(%%rbp)\n");
  fprintf(f,"        movq    -72(%%rbp), %%rax\n");
  fprintf(f,"        movq    -64(%%rbp), %%rcx\n");
  fprintf(f,"        addq    %%rax, %%rcx\n");
  fprintf(f,"        movq    %%rcx, -64(%%rbp)\n");
  fprintf(f,"        movl    -8(%%rbp), %%eax\n");
  fprintf(f,"        addl    $1, %%eax\n");
  fprintf(f,"        movl    %%eax, -8(%%rbp)\n");
  fprintf(f,"        jmp     0b\n");
  fprintf(f,"2:\n");
  fprintf(f,"        movl    $0, -8(%%rbp)\n");
  fprintf(f,"3:\n");
  fprintf(f,"        cmpl    $256, -8(%%rbp)          # imm = 0x100\n");
  fprintf(f,"        jl      4f\n");
  fprintf(f,"        jmp     7f\n");
  fprintf(f,"4:\n");
  fprintf(f,"        cmpl    $0, -4(%%rbp)\n");
  fprintf(f,"        jne     5f\n");
  fprintf(f,"        jmp     6f\n");
  fprintf(f,"5:\n");
  fprintf(f,"        movslq  -8(%%rbp), %%rax\n");
  fprintf(f,"        movabsq $.LRAND_randrsl, %%rcx\n");
  fprintf(f,"        movq    (%%rcx,%%rax,8), %%rax\n");
  fprintf(f,"        movq    -16(%%rbp), %%rdx\n");
  fprintf(f,"        addq    %%rax, %%rdx\n");
  fprintf(f,"        movq    %%rdx, -16(%%rbp)\n");
  fprintf(f,"        movl    -8(%%rbp), %%esi\n");
  fprintf(f,"        incl    %%esi\n");
  fprintf(f,"        movslq  %%esi, %%rax\n");
  fprintf(f,"        movq    (%%rcx,%%rax,8), %%rax\n");
  fprintf(f,"        movq    -24(%%rbp), %%rdx\n");
  fprintf(f,"        addq    %%rax, %%rdx\n");
  fprintf(f,"        movq    %%rdx, -24(%%rbp)\n");
  fprintf(f,"        movl    -8(%%rbp), %%esi\n");
  fprintf(f,"        addl    $2, %%esi\n");
  fprintf(f,"        movslq  %%esi, %%rax\n");
  fprintf(f,"        movq    (%%rcx,%%rax,8), %%rax\n");
  fprintf(f,"        movq    -32(%%rbp), %%rdx\n");
  fprintf(f,"        addq    %%rax, %%rdx\n");
  fprintf(f,"        movq    %%rdx, -32(%%rbp)\n");
  fprintf(f,"        movl    -8(%%rbp), %%esi\n");
  fprintf(f,"        addl    $3, %%esi\n");
  fprintf(f,"        movslq  %%esi, %%rax\n");
  fprintf(f,"        movq    (%%rcx,%%rax,8), %%rax\n");
  fprintf(f,"        movq    -40(%%rbp), %%rdx\n");
  fprintf(f,"        addq    %%rax, %%rdx\n");
  fprintf(f,"        movq    %%rdx, -40(%%rbp)\n");
  fprintf(f,"        movl    -8(%%rbp), %%esi\n");
  fprintf(f,"        addl    $4, %%esi\n");
  fprintf(f,"        movslq  %%esi, %%rax\n");
  fprintf(f,"        movq    (%%rcx,%%rax,8), %%rax\n");
  fprintf(f,"        movq    -48(%%rbp), %%rdx\n");
  fprintf(f,"        addq    %%rax, %%rdx\n");
  fprintf(f,"        movq    %%rdx, -48(%%rbp)\n");
  fprintf(f,"        movl    -8(%%rbp), %%esi\n");
  fprintf(f,"        addl    $5, %%esi\n");
  fprintf(f,"        movslq  %%esi, %%rax\n");
  fprintf(f,"        movq    (%%rcx,%%rax,8), %%rax\n");
  fprintf(f,"        movq    -56(%%rbp), %%rdx\n");
  fprintf(f,"        addq    %%rax, %%rdx\n");
  fprintf(f,"        movq    %%rdx, -56(%%rbp)\n");
  fprintf(f,"        movl    -8(%%rbp), %%esi\n");
  fprintf(f,"        addl    $6, %%esi\n");
  fprintf(f,"        movslq  %%esi, %%rax\n");
  fprintf(f,"        movq    (%%rcx,%%rax,8), %%rax\n");
  fprintf(f,"        movq    -64(%%rbp), %%rdx\n");
  fprintf(f,"        addq    %%rax, %%rdx\n");
  fprintf(f,"        movq    %%rdx, -64(%%rbp)\n");
  fprintf(f,"        movl    -8(%%rbp), %%esi\n");
  fprintf(f,"        addl    $7, %%esi\n");
  fprintf(f,"        movslq  %%esi, %%rax\n");
  fprintf(f,"        leaq    (%%rcx,%%rax,8), %%rax\n");
  fprintf(f,"        movq    (%%rax), %%rax\n");
  fprintf(f,"        movq    -72(%%rbp), %%rcx\n");
  fprintf(f,"        addq    %%rax, %%rcx\n");
  fprintf(f,"        movq    %%rcx, -72(%%rbp)\n");
  fprintf(f,"6:\n");
  fprintf(f,"        movq    -48(%%rbp), %%rax\n");
  fprintf(f,"        movq    -16(%%rbp), %%rcx\n");
  fprintf(f,"        subq    %%rax, %%rcx\n");
  fprintf(f,"        movq    %%rcx, -16(%%rbp)\n");
  fprintf(f,"        movq    -72(%%rbp), %%rax\n");
  fprintf(f,"        shrq    $9, %%rax\n");
  fprintf(f,"        movq    -56(%%rbp), %%rcx\n");
  fprintf(f,"        xorq    %%rax, %%rcx\n");
  fprintf(f,"        movq    %%rcx, -56(%%rbp)\n");
  fprintf(f,"        movq    -16(%%rbp), %%rax\n");
  fprintf(f,"        movq    -72(%%rbp), %%rcx\n");
  fprintf(f,"        addq    %%rax, %%rcx\n");
  fprintf(f,"        movq    %%rcx, -72(%%rbp)\n");
  fprintf(f,"        movq    -56(%%rbp), %%rax\n");
  fprintf(f,"        movq    -24(%%rbp), %%rcx\n");
  fprintf(f,"        subq    %%rax, %%rcx\n");
  fprintf(f,"        movq    %%rcx, -24(%%rbp)\n");
  fprintf(f,"        movq    -16(%%rbp), %%rax\n");
  fprintf(f,"        shlq    $9, %%rax\n");
  fprintf(f,"        movq    -64(%%rbp), %%rcx\n");
  fprintf(f,"        xorq    %%rax, %%rcx\n");
  fprintf(f,"        movq    %%rcx, -64(%%rbp)\n");
  fprintf(f,"        movq    -24(%%rbp), %%rax\n");
  fprintf(f,"        movq    -16(%%rbp), %%rcx\n");
  fprintf(f,"        addq    %%rax, %%rcx\n");
  fprintf(f,"        movq    %%rcx, -16(%%rbp)\n");
  fprintf(f,"        movq    -64(%%rbp), %%rax\n");
  fprintf(f,"        movq    -32(%%rbp), %%rcx\n");
  fprintf(f,"        subq    %%rax, %%rcx\n");
  fprintf(f,"        movq    %%rcx, -32(%%rbp)\n");
  fprintf(f,"        movq    -24(%%rbp), %%rax\n");
  fprintf(f,"        shrq    $23, %%rax\n");
  fprintf(f,"        movq    -72(%%rbp), %%rcx\n");
  fprintf(f,"        xorq    %%rax, %%rcx\n");
  fprintf(f,"        movq    %%rcx, -72(%%rbp)\n");
  fprintf(f,"        movq    -32(%%rbp), %%rax\n");
  fprintf(f,"        movq    -24(%%rbp), %%rcx\n");
  fprintf(f,"        addq    %%rax, %%rcx\n");
  fprintf(f,"        movq    %%rcx, -24(%%rbp)\n");
  fprintf(f,"        movq    -72(%%rbp), %%rax\n");
  fprintf(f,"        movq    -40(%%rbp), %%rcx\n");
  fprintf(f,"        subq    %%rax, %%rcx\n");
  fprintf(f,"        movq    %%rcx, -40(%%rbp)\n");
  fprintf(f,"        movq    -32(%%rbp), %%rax\n");
  fprintf(f,"        shlq    $15, %%rax\n");
  fprintf(f,"        movq    -16(%%rbp), %%rcx\n");
  fprintf(f,"        xorq    %%rax, %%rcx\n");
  fprintf(f,"        movq    %%rcx, -16(%%rbp)\n");
  fprintf(f,"        movq    -40(%%rbp), %%rax\n");
  fprintf(f,"        movq    -32(%%rbp), %%rcx\n");
  fprintf(f,"        addq    %%rax, %%rcx\n");
  fprintf(f,"        movq    %%rcx, -32(%%rbp)\n");
  fprintf(f,"        movq    -16(%%rbp), %%rax\n");
  fprintf(f,"        movq    -48(%%rbp), %%rcx\n");
  fprintf(f,"        subq    %%rax, %%rcx\n");
  fprintf(f,"        movq    %%rcx, -48(%%rbp)\n");
  fprintf(f,"        movq    -40(%%rbp), %%rax\n");
  fprintf(f,"        shrq    $14, %%rax\n");
  fprintf(f,"        movq    -24(%%rbp), %%rcx\n");
  fprintf(f,"        xorq    %%rax, %%rcx\n");
  fprintf(f,"        movq    %%rcx, -24(%%rbp)\n");
  fprintf(f,"        movq    -48(%%rbp), %%rax\n");
  fprintf(f,"        movq    -40(%%rbp), %%rcx\n");
  fprintf(f,"        addq    %%rax, %%rcx\n");
  fprintf(f,"        movq    %%rcx, -40(%%rbp)\n");
  fprintf(f,"        movq    -24(%%rbp), %%rax\n");
  fprintf(f,"        movq    -56(%%rbp), %%rcx\n");
  fprintf(f,"        subq    %%rax, %%rcx\n");
  fprintf(f,"        movq    %%rcx, -56(%%rbp)\n");
  fprintf(f,"        movq    -48(%%rbp), %%rax\n");
  fprintf(f,"        shlq    $20, %%rax\n");
  fprintf(f,"        movq    -32(%%rbp), %%rcx\n");
  fprintf(f,"        xorq    %%rax, %%rcx\n");
  fprintf(f,"        movq    %%rcx, -32(%%rbp)\n");
  fprintf(f,"        movq    -56(%%rbp), %%rax\n");
  fprintf(f,"        movq    -48(%%rbp), %%rcx\n");
  fprintf(f,"        addq    %%rax, %%rcx\n");
  fprintf(f,"        movq    %%rcx, -48(%%rbp)\n");
  fprintf(f,"        movq    -32(%%rbp), %%rax\n");
  fprintf(f,"        movq    -64(%%rbp), %%rcx\n");
  fprintf(f,"        subq    %%rax, %%rcx\n");
  fprintf(f,"        movq    %%rcx, -64(%%rbp)\n");
  fprintf(f,"        movq    -56(%%rbp), %%rax\n");
  fprintf(f,"        shrq    $17, %%rax\n");
  fprintf(f,"        movq    -40(%%rbp), %%rcx\n");
  fprintf(f,"        xorq    %%rax, %%rcx\n");
  fprintf(f,"        movq    %%rcx, -40(%%rbp)\n");
  fprintf(f,"        movq    -64(%%rbp), %%rax\n");
  fprintf(f,"        movq    -56(%%rbp), %%rcx\n");
  fprintf(f,"        addq    %%rax, %%rcx\n");
  fprintf(f,"        movq    %%rcx, -56(%%rbp)\n");
  fprintf(f,"        movq    -40(%%rbp), %%rax\n");
  fprintf(f,"        movq    -72(%%rbp), %%rcx\n");
  fprintf(f,"        subq    %%rax, %%rcx\n");
  fprintf(f,"        movq    %%rcx, -72(%%rbp)\n");
  fprintf(f,"        movq    -64(%%rbp), %%rax\n");
  fprintf(f,"        shlq    $14, %%rax\n");
  fprintf(f,"        movq    -48(%%rbp), %%rcx\n");
  fprintf(f,"        xorq    %%rax, %%rcx\n");
  fprintf(f,"        movq    %%rcx, -48(%%rbp)\n");
  fprintf(f,"        movq    -72(%%rbp), %%rax\n");
  fprintf(f,"        movq    -64(%%rbp), %%rcx\n");
  fprintf(f,"        addq    %%rax, %%rcx\n");
  fprintf(f,"        movq    %%rcx, -64(%%rbp)\n");
  fprintf(f,"        movq    -16(%%rbp), %%rax\n");
  fprintf(f,"        movslq  -8(%%rbp), %%rcx\n");
  fprintf(f,"        movabsq $.LRAND_mm, %%rdx\n");
  fprintf(f,"        movq    %%rax, (%%rdx,%%rcx,8)\n");
  fprintf(f,"        movq    -24(%%rbp), %%rax\n");
  fprintf(f,"        movl    -8(%%rbp), %%esi\n");
  fprintf(f,"        incl    %%esi\n");
  fprintf(f,"        movslq  %%esi, %%rcx\n");
  fprintf(f,"        movq    %%rax, (%%rdx,%%rcx,8)\n");
  fprintf(f,"        movq    -32(%%rbp), %%rax\n");
  fprintf(f,"        movl    -8(%%rbp), %%esi\n");
  fprintf(f,"        addl    $2, %%esi\n");
  fprintf(f,"        movslq  %%esi, %%rcx\n");
  fprintf(f,"        movq    %%rax, (%%rdx,%%rcx,8)\n");
  fprintf(f,"        movq    -40(%%rbp), %%rax\n");
  fprintf(f,"        movl    -8(%%rbp), %%esi\n");
  fprintf(f,"        addl    $3, %%esi\n");
  fprintf(f,"        movslq  %%esi, %%rcx\n");
  fprintf(f,"        movq    %%rax, (%%rdx,%%rcx,8)\n");
  fprintf(f,"        movq    -48(%%rbp), %%rax\n");
  fprintf(f,"        movl    -8(%%rbp), %%esi\n");
  fprintf(f,"        addl    $4, %%esi\n");
  fprintf(f,"        movslq  %%esi, %%rcx\n");
  fprintf(f,"        movq    %%rax, (%%rdx,%%rcx,8)\n");
  fprintf(f,"        movq    -56(%%rbp), %%rax\n");
  fprintf(f,"        movl    -8(%%rbp), %%esi\n");
  fprintf(f,"        addl    $5, %%esi\n");
  fprintf(f,"        movslq  %%esi, %%rcx\n");
  fprintf(f,"        movq    %%rax, (%%rdx,%%rcx,8)\n");
  fprintf(f,"        movq    -64(%%rbp), %%rax\n");
  fprintf(f,"        movl    -8(%%rbp), %%esi\n");
  fprintf(f,"        addl    $6, %%esi\n");
  fprintf(f,"        movslq  %%esi, %%rcx\n");
  fprintf(f,"        movq    %%rax, (%%rdx,%%rcx,8)\n");
  fprintf(f,"        movq    -72(%%rbp), %%rax\n");
  fprintf(f,"        movl    -8(%%rbp), %%esi\n");
  fprintf(f,"        addl    $7, %%esi\n");
  fprintf(f,"        movslq  %%esi, %%rcx\n");
  fprintf(f,"        leaq    (%%rdx,%%rcx,8), %%rcx\n");
  fprintf(f,"        movq    %%rax, (%%rcx)\n");
  fprintf(f,"        movl    -8(%%rbp), %%eax\n");
  fprintf(f,"        addl    $8, %%eax\n");
  fprintf(f,"        movl    %%eax, -8(%%rbp)\n");
  fprintf(f,"        jmp     3b\n");
  fprintf(f,"7:\n");
  fprintf(f,"        cmpl    $0, -4(%%rbp)\n");
  fprintf(f,"        jne     8f\n");
  fprintf(f,"        jmp     11f\n");
  fprintf(f,"8:\n");
  fprintf(f,"        movl    $0, -8(%%rbp)\n");
  fprintf(f,"9:\n");
  fprintf(f,"        cmpl    $256, -8(%%rbp)          # imm = 0x100\n");
  fprintf(f,"        jl      10f\n");
  fprintf(f,"        jmp     11f\n");
  fprintf(f,"10:\n");
  fprintf(f,"        movslq  -8(%%rbp), %%rax\n");
  fprintf(f,"        movabsq $.LRAND_mm, %%rcx\n");
  fprintf(f,"        movq    (%%rcx,%%rax,8), %%rax\n");
  fprintf(f,"        movq    -16(%%rbp), %%rdx\n");
  fprintf(f,"        addq    %%rax, %%rdx\n");
  fprintf(f,"        movq    %%rdx, -16(%%rbp)\n");
  fprintf(f,"        movl    -8(%%rbp), %%esi\n");
  fprintf(f,"        incl    %%esi\n");
  fprintf(f,"        movslq  %%esi, %%rax\n");
  fprintf(f,"        movq    (%%rcx,%%rax,8), %%rax\n");
  fprintf(f,"        movq    -24(%%rbp), %%rdx\n");
  fprintf(f,"        addq    %%rax, %%rdx\n");
  fprintf(f,"        movq    %%rdx, -24(%%rbp)\n");
  fprintf(f,"        movl    -8(%%rbp), %%esi\n");
  fprintf(f,"        addl    $2, %%esi\n");
  fprintf(f,"        movslq  %%esi, %%rax\n");
  fprintf(f,"        movq    (%%rcx,%%rax,8), %%rax\n");
  fprintf(f,"        movq    -32(%%rbp), %%rdx\n");
  fprintf(f,"        addq    %%rax, %%rdx\n");
  fprintf(f,"        movq    %%rdx, -32(%%rbp)\n");
  fprintf(f,"        movl    -8(%%rbp), %%esi\n");
  fprintf(f,"        addl    $3, %%esi\n");
  fprintf(f,"        movslq  %%esi, %%rax\n");
  fprintf(f,"        movq    (%%rcx,%%rax,8), %%rax\n");
  fprintf(f,"        movq    -40(%%rbp), %%rdx\n");
  fprintf(f,"        addq    %%rax, %%rdx\n");
  fprintf(f,"        movq    %%rdx, -40(%%rbp)\n");
  fprintf(f,"        movl    -8(%%rbp), %%esi\n");
  fprintf(f,"        addl    $4, %%esi\n");
  fprintf(f,"        movslq  %%esi, %%rax\n");
  fprintf(f,"        movq    (%%rcx,%%rax,8), %%rax\n");
  fprintf(f,"        movq    -48(%%rbp), %%rdx\n");
  fprintf(f,"        addq    %%rax, %%rdx\n");
  fprintf(f,"        movq    %%rdx, -48(%%rbp)\n");
  fprintf(f,"        movl    -8(%%rbp), %%esi\n");
  fprintf(f,"        addl    $5, %%esi\n");
  fprintf(f,"        movslq  %%esi, %%rax\n");
  fprintf(f,"        movq    (%%rcx,%%rax,8), %%rax\n");
  fprintf(f,"        movq    -56(%%rbp), %%rdx\n");
  fprintf(f,"        addq    %%rax, %%rdx\n");
  fprintf(f,"        movq    %%rdx, -56(%%rbp)\n");
  fprintf(f,"        movl    -8(%%rbp), %%esi\n");
  fprintf(f,"        addl    $6, %%esi\n");
  fprintf(f,"        movslq  %%esi, %%rax\n");
  fprintf(f,"        movq    (%%rcx,%%rax,8), %%rax\n");
  fprintf(f,"        movq    -64(%%rbp), %%rdx\n");
  fprintf(f,"        addq    %%rax, %%rdx\n");
  fprintf(f,"        movq    %%rdx, -64(%%rbp)\n");
  fprintf(f,"        movl    -8(%%rbp), %%esi\n");
  fprintf(f,"        addl    $7, %%esi\n");
  fprintf(f,"        movslq  %%esi, %%rax\n");
  fprintf(f,"        movq    (%%rcx,%%rax,8), %%rax\n");
  fprintf(f,"        movq    -72(%%rbp), %%rdx\n");
  fprintf(f,"        addq    %%rax, %%rdx\n");
  fprintf(f,"        movq    %%rdx, -72(%%rbp)\n");
  fprintf(f,"        movq    -48(%%rbp), %%rax\n");
  fprintf(f,"        movq    -16(%%rbp), %%rdx\n");
  fprintf(f,"        subq    %%rax, %%rdx\n");
  fprintf(f,"        movq    %%rdx, -16(%%rbp)\n");
  fprintf(f,"        movq    -72(%%rbp), %%rax\n");
  fprintf(f,"        shrq    $9, %%rax\n");
  fprintf(f,"        movq    -56(%%rbp), %%rdx\n");
  fprintf(f,"        xorq    %%rax, %%rdx\n");
  fprintf(f,"        movq    %%rdx, -56(%%rbp)\n");
  fprintf(f,"        movq    -16(%%rbp), %%rax\n");
  fprintf(f,"        movq    -72(%%rbp), %%rdx\n");
  fprintf(f,"        addq    %%rax, %%rdx\n");
  fprintf(f,"        movq    %%rdx, -72(%%rbp)\n");
  fprintf(f,"        movq    -56(%%rbp), %%rax\n");
  fprintf(f,"        movq    -24(%%rbp), %%rdx\n");
  fprintf(f,"        subq    %%rax, %%rdx\n");
  fprintf(f,"        movq    %%rdx, -24(%%rbp)\n");
  fprintf(f,"        movq    -16(%%rbp), %%rax\n");
  fprintf(f,"        shlq    $9, %%rax\n");
  fprintf(f,"        movq    -64(%%rbp), %%rdx\n");
  fprintf(f,"        xorq    %%rax, %%rdx\n");
  fprintf(f,"        movq    %%rdx, -64(%%rbp)\n");
  fprintf(f,"        movq    -24(%%rbp), %%rax\n");
  fprintf(f,"        movq    -16(%%rbp), %%rdx\n");
  fprintf(f,"        addq    %%rax, %%rdx\n");
  fprintf(f,"        movq    %%rdx, -16(%%rbp)\n");
  fprintf(f,"        movq    -64(%%rbp), %%rax\n");
  fprintf(f,"        movq    -32(%%rbp), %%rdx\n");
  fprintf(f,"        subq    %%rax, %%rdx\n");
  fprintf(f,"        movq    %%rdx, -32(%%rbp)\n");
  fprintf(f,"        movq    -24(%%rbp), %%rax\n");
  fprintf(f,"        shrq    $23, %%rax\n");
  fprintf(f,"        movq    -72(%%rbp), %%rdx\n");
  fprintf(f,"        xorq    %%rax, %%rdx\n");
  fprintf(f,"        movq    %%rdx, -72(%%rbp)\n");
  fprintf(f,"        movq    -32(%%rbp), %%rax\n");
  fprintf(f,"        movq    -24(%%rbp), %%rdx\n");
  fprintf(f,"        addq    %%rax, %%rdx\n");
  fprintf(f,"        movq    %%rdx, -24(%%rbp)\n");
  fprintf(f,"        movq    -72(%%rbp), %%rax\n");
  fprintf(f,"        movq    -40(%%rbp), %%rdx\n");
  fprintf(f,"        subq    %%rax, %%rdx\n");
  fprintf(f,"        movq    %%rdx, -40(%%rbp)\n");
  fprintf(f,"        movq    -32(%%rbp), %%rax\n");
  fprintf(f,"        shlq    $15, %%rax\n");
  fprintf(f,"        movq    -16(%%rbp), %%rdx\n");
  fprintf(f,"        xorq    %%rax, %%rdx\n");
  fprintf(f,"        movq    %%rdx, -16(%%rbp)\n");
  fprintf(f,"        movq    -40(%%rbp), %%rax\n");
  fprintf(f,"        movq    -32(%%rbp), %%rdx\n");
  fprintf(f,"        addq    %%rax, %%rdx\n");
  fprintf(f,"        movq    %%rdx, -32(%%rbp)\n");
  fprintf(f,"        movq    -16(%%rbp), %%rax\n");
  fprintf(f,"        movq    -48(%%rbp), %%rdx\n");
  fprintf(f,"        subq    %%rax, %%rdx\n");
  fprintf(f,"        movq    %%rdx, -48(%%rbp)\n");
  fprintf(f,"        movq    -40(%%rbp), %%rax\n");
  fprintf(f,"        shrq    $14, %%rax\n");
  fprintf(f,"        movq    -24(%%rbp), %%rdx\n");
  fprintf(f,"        xorq    %%rax, %%rdx\n");
  fprintf(f,"        movq    %%rdx, -24(%%rbp)\n");
  fprintf(f,"        movq    -48(%%rbp), %%rax\n");
  fprintf(f,"        movq    -40(%%rbp), %%rdx\n");
  fprintf(f,"        addq    %%rax, %%rdx\n");
  fprintf(f,"        movq    %%rdx, -40(%%rbp)\n");
  fprintf(f,"        movq    -24(%%rbp), %%rax\n");
  fprintf(f,"        movq    -56(%%rbp), %%rdx\n");
  fprintf(f,"        subq    %%rax, %%rdx\n");
  fprintf(f,"        movq    %%rdx, -56(%%rbp)\n");
  fprintf(f,"        movq    -48(%%rbp), %%rax\n");
  fprintf(f,"        shlq    $20, %%rax\n");
  fprintf(f,"        movq    -32(%%rbp), %%rdx\n");
  fprintf(f,"        xorq    %%rax, %%rdx\n");
  fprintf(f,"        movq    %%rdx, -32(%%rbp)\n");
  fprintf(f,"        movq    -56(%%rbp), %%rax\n");
  fprintf(f,"        movq    -48(%%rbp), %%rdx\n");
  fprintf(f,"        addq    %%rax, %%rdx\n");
  fprintf(f,"        movq    %%rdx, -48(%%rbp)\n");
  fprintf(f,"        movq    -32(%%rbp), %%rax\n");
  fprintf(f,"        movq    -64(%%rbp), %%rdx\n");
  fprintf(f,"        subq    %%rax, %%rdx\n");
  fprintf(f,"        movq    %%rdx, -64(%%rbp)\n");
  fprintf(f,"        movq    -56(%%rbp), %%rax\n");
  fprintf(f,"        shrq    $17, %%rax\n");
  fprintf(f,"        movq    -40(%%rbp), %%rdx\n");
  fprintf(f,"        xorq    %%rax, %%rdx\n");
  fprintf(f,"        movq    %%rdx, -40(%%rbp)\n");
  fprintf(f,"        movq    -64(%%rbp), %%rax\n");
  fprintf(f,"        movq    -56(%%rbp), %%rdx\n");
  fprintf(f,"        addq    %%rax, %%rdx\n");
  fprintf(f,"        movq    %%rdx, -56(%%rbp)\n");
  fprintf(f,"        movq    -40(%%rbp), %%rax\n");
  fprintf(f,"        movq    -72(%%rbp), %%rdx\n");
  fprintf(f,"        subq    %%rax, %%rdx\n");
  fprintf(f,"        movq    %%rdx, -72(%%rbp)\n");
  fprintf(f,"        movq    -64(%%rbp), %%rax\n");
  fprintf(f,"        shlq    $14, %%rax\n");
  fprintf(f,"        movq    -48(%%rbp), %%rdx\n");
  fprintf(f,"        xorq    %%rax, %%rdx\n");
  fprintf(f,"        movq    %%rdx, -48(%%rbp)\n");
  fprintf(f,"        movq    -72(%%rbp), %%rax\n");
  fprintf(f,"        movq    -64(%%rbp), %%rdx\n");
  fprintf(f,"        addq    %%rax, %%rdx\n");
  fprintf(f,"        movq    %%rdx, -64(%%rbp)\n");
  fprintf(f,"        movq    -16(%%rbp), %%rax\n");
  fprintf(f,"        movslq  -8(%%rbp), %%rdx\n");
  fprintf(f,"        movq    %%rax, (%%rcx,%%rdx,8)\n");
  fprintf(f,"        movq    -24(%%rbp), %%rax\n");
  fprintf(f,"        movl    -8(%%rbp), %%esi\n");
  fprintf(f,"        incl    %%esi\n");
  fprintf(f,"        movslq  %%esi, %%rdx\n");
  fprintf(f,"        movq    %%rax, (%%rcx,%%rdx,8)\n");
  fprintf(f,"        movq    -32(%%rbp), %%rax\n");
  fprintf(f,"        movl    -8(%%rbp), %%esi\n");
  fprintf(f,"        addl    $2, %%esi\n");
  fprintf(f,"        movslq  %%esi, %%rdx\n");
  fprintf(f,"        movq    %%rax, (%%rcx,%%rdx,8)\n");
  fprintf(f,"        movq    -40(%%rbp), %%rax\n");
  fprintf(f,"        movl    -8(%%rbp), %%esi\n");
  fprintf(f,"        addl    $3, %%esi\n");
  fprintf(f,"        movslq  %%esi, %%rdx\n");
  fprintf(f,"        movq    %%rax, (%%rcx,%%rdx,8)\n");
  fprintf(f,"        movq    -48(%%rbp), %%rax\n");
  fprintf(f,"        movl    -8(%%rbp), %%esi\n");
  fprintf(f,"        addl    $4, %%esi\n");
  fprintf(f,"        movslq  %%esi, %%rdx\n");
  fprintf(f,"        movq    %%rax, (%%rcx,%%rdx,8)\n");
  fprintf(f,"        movq    -56(%%rbp), %%rax\n");
  fprintf(f,"        movl    -8(%%rbp), %%esi\n");
  fprintf(f,"        addl    $5, %%esi\n");
  fprintf(f,"        movslq  %%esi, %%rdx\n");
  fprintf(f,"        movq    %%rax, (%%rcx,%%rdx,8)\n");
  fprintf(f,"        movq    -64(%%rbp), %%rax\n");
  fprintf(f,"        movl    -8(%%rbp), %%esi\n");
  fprintf(f,"        addl    $6, %%esi\n");
  fprintf(f,"        movslq  %%esi, %%rdx\n");
  fprintf(f,"        movq    %%rax, (%%rcx,%%rdx,8)\n");
  fprintf(f,"        movq    -72(%%rbp), %%rax\n");
  fprintf(f,"        movl    -8(%%rbp), %%esi\n");
  fprintf(f,"        addl    $7, %%esi\n");
  fprintf(f,"        movslq  %%esi, %%rdx\n");
  fprintf(f,"        leaq    (%%rcx,%%rdx,8), %%rcx\n");
  fprintf(f,"        movq    %%rax, (%%rcx)\n");
  fprintf(f,"        movl    -8(%%rbp), %%eax\n");
  fprintf(f,"        addl    $8, %%eax\n");
  fprintf(f,"        movl    %%eax, -8(%%rbp)\n");
  fprintf(f,"        jmp     9b\n");
  fprintf(f,"11:\n");
  fprintf(f,"        movabsq $isaac64, %%rax\n");
  fprintf(f,"        callq   *%%rax\n");
  fprintf(f,"        movabsq $.LRAND_randcnt, %%rax\n");
  fprintf(f,"        movq    $256, (%%rax)            # imm = 0x100\n");
  fprintf(f,"        addq    $80, %%rsp\n");
  fprintf(f,"        popq    %%rbp\n");
  fprintf(f,"        retq\n");
  fprintf(f,"        .size   randinit, .-randinit\n\n");
  fprintf(f,"        .type   drand,@function\n");
  fprintf(f,"drand:\n");
  fprintf(f,"        pushq   %%rbp\n");
  fprintf(f,"        movq    %%rsp, %%rbp\n");
  fprintf(f,"        subq    $80, %%rsp\n");
  fprintf(f,"0:\n");
  fprintf(f,"        movabsq $.LRAND_randcnt, %%rax\n");
  fprintf(f,"        movq    (%%rax), %%rcx\n");
  fprintf(f,"        movq    %%rcx, %%rdx\n");
  fprintf(f,"        decq    %%rdx\n");
  fprintf(f,"        movq    %%rdx, (%%rax)\n");
  fprintf(f,"        cmpq    $0, %%rcx\n");
  fprintf(f,"        jne     1f\n");
  fprintf(f,"        movabsq $isaac64, %%rax\n");
  fprintf(f,"        callq   *%%rax\n");
  fprintf(f,"        movabsq $.LRAND_randcnt, %%rax\n");
  fprintf(f,"        movq    $255, (%%rax)\n");
  fprintf(f,"        movl    $255, %%eax\n");
  fprintf(f,"        movabsq $.LRAND_randrsl, %%rcx\n");
  fprintf(f,"        addq    $2040, %%rcx             # imm = 0x7F8\n");
  fprintf(f,"        movq    (%%rcx), %%rcx\n");
  fprintf(f,"        movq    %%rax, -16(%%rbp)\n");
  fprintf(f,"        movq    %%rcx, -24(%%rbp)\n");
  fprintf(f,"        jmp     2f\n");
  fprintf(f,"1:\n");
  fprintf(f,"        movabsq $.LRAND_randcnt, %%rax\n");
  fprintf(f,"        movq    (%%rax), %%rax\n");
  fprintf(f,"        movabsq $.LRAND_randrsl, %%rcx\n");
  fprintf(f,"        leaq    (%%rcx,%%rax,8), %%rax\n");
  fprintf(f,"        movq    (%%rax), %%rax\n");
  fprintf(f,"        movq    %%rax, -24(%%rbp)\n");
  fprintf(f,"2:\n");
  fprintf(f,"        movq    -24(%%rbp), %%rax\n");
  fprintf(f,"        movabsq $.LRAND_0, %%rcx\n");
  fprintf(f,"        movaps  (%%rcx), %%xmm0\n");
  fprintf(f,"        movd    %%rax, %%xmm1\n");
  fprintf(f,"        punpckldq       %%xmm0, %%xmm1    # xmm1 = xmm1[0],xmm0[0],xmm1[1],xmm0[1]\n");
  fprintf(f,"        movabsq $.LRAND_1, %%rax\n");
  fprintf(f,"        movapd  (%%rax), %%xmm0\n");
  fprintf(f,"        subpd   %%xmm0, %%xmm1\n");
  fprintf(f,"        haddpd  %%xmm1, %%xmm1\n");
  fprintf(f,"        movabsq $.LRAND_2, %%rax\n");
  fprintf(f,"        movsd   (%%rax), %%xmm0\n");
  fprintf(f,"        mulsd   %%xmm0, %%xmm1\n");
  fprintf(f,"        movabsq $.LRAND_randcnt, %%rax\n");
  fprintf(f,"        movq    (%%rax), %%rcx\n");
  fprintf(f,"        movq    %%rcx, %%rdx\n");
  fprintf(f,"        decq    %%rdx\n");
  fprintf(f,"        movq    %%rdx, (%%rax)\n");
  fprintf(f,"        cmpq    $0, %%rcx\n");
  fprintf(f,"        movsd   %%xmm1, -32(%%rbp)\n");
  fprintf(f,"        jne     3f\n");
  fprintf(f,"        movabsq $isaac64, %%rax\n");
  fprintf(f,"        callq   *%%rax\n");
  fprintf(f,"        movabsq $.LRAND_randcnt, %%rax\n");
  fprintf(f,"        movq    $255, (%%rax)\n");
  fprintf(f,"        movl    $255, %%eax\n");
  fprintf(f,"        movabsq $.LRAND_randrsl, %%rcx\n");
  fprintf(f,"        addq    $2040, %%rcx             # imm = 0x7F8\n");
  fprintf(f,"        movq    (%%rcx), %%rcx\n");
  fprintf(f,"        movq    %%rax, -40(%%rbp)\n");
  fprintf(f,"        movq    %%rcx, -48(%%rbp)\n");
  fprintf(f,"        jmp     4f\n");
  fprintf(f,"3:\n");
  fprintf(f,"        movabsq $.LRAND_randcnt, %%rax\n");
  fprintf(f,"        movq    (%%rax), %%rax\n");
  fprintf(f,"        movabsq $.LRAND_randrsl, %%rcx\n");
  fprintf(f,"        leaq    (%%rcx,%%rax,8), %%rax\n");
  fprintf(f,"        movq    (%%rax), %%rax\n");
  fprintf(f,"        movq    %%rax, -48(%%rbp)\n");
  fprintf(f,"4:\n");
  fprintf(f,"        movq    -48(%%rbp), %%rax\n");
  fprintf(f,"        movabsq $.LRAND_0, %%rcx\n");
  fprintf(f,"        movaps  (%%rcx), %%xmm0\n");
  fprintf(f,"        movd    %%rax, %%xmm1\n");
  fprintf(f,"        punpckldq       %%xmm0, %%xmm1    # xmm1 = xmm1[0],xmm0[0],xmm1[1],xmm0[1]\n");
  fprintf(f,"        movabsq $.LRAND_1, %%rax\n");
  fprintf(f,"        movapd  (%%rax), %%xmm0\n");
  fprintf(f,"        subpd   %%xmm0, %%xmm1\n");
  fprintf(f,"        haddpd  %%xmm1, %%xmm1\n");
  fprintf(f,"        movsd   -32(%%rbp), %%xmm0\n");
  fprintf(f,"        addsd   %%xmm1, %%xmm0\n");
  fprintf(f,"        movabsq $.LRAND_2, %%rax\n");
  fprintf(f,"        movsd   (%%rax), %%xmm1\n");
  fprintf(f,"        mulsd   %%xmm1, %%xmm0\n");
  fprintf(f,"        movabsq $.LRAND_randcnt, %%rax\n");
  fprintf(f,"        movq    (%%rax), %%rcx\n");
  fprintf(f,"        movq    %%rcx, %%rdx\n");
  fprintf(f,"        decq    %%rdx\n");
  fprintf(f,"        movq    %%rdx, (%%rax)\n");
  fprintf(f,"        cmpq    $0, %%rcx\n");
  fprintf(f,"        movsd   %%xmm0, -56(%%rbp)\n");
  fprintf(f,"        jne     5f\n");
  fprintf(f,"        movabsq $isaac64, %%rax\n");
  fprintf(f,"        callq   *%%rax\n");
  fprintf(f,"        movabsq $.LRAND_randcnt, %%rax\n");
  fprintf(f,"        movq    $255, (%%rax)\n");
  fprintf(f,"        movl    $255, %%eax\n");
  fprintf(f,"        movabsq $.LRAND_randrsl, %%rcx\n");
  fprintf(f,"        addq    $2040, %%rcx             # imm = 0x7F8\n");
  fprintf(f,"        movq    (%%rcx), %%rcx\n");
  fprintf(f,"        movq    %%rax, -64(%%rbp)\n");
  fprintf(f,"        movq    %%rcx, -72(%%rbp)\n");
  fprintf(f,"        jmp     6f\n");
  fprintf(f,"5:\n");
  fprintf(f,"        movabsq $.LRAND_randcnt, %%rax\n");
  fprintf(f,"        movq    (%%rax), %%rax\n");
  fprintf(f,"        movabsq $.LRAND_randrsl, %%rcx\n");
  fprintf(f,"        leaq    (%%rcx,%%rax,8), %%rax\n");
  fprintf(f,"        movq    (%%rax), %%rax\n");
  fprintf(f,"        movq    %%rax, -72(%%rbp)\n");
  fprintf(f,"6:\n");
  fprintf(f,"        movq    -72(%%rbp), %%rax\n");
  fprintf(f,"        movabsq $.LRAND_0, %%rcx\n");
  fprintf(f,"        movaps  (%%rcx), %%xmm0\n");
  fprintf(f,"        movd    %%rax, %%xmm1\n");
  fprintf(f,"        punpckldq       %%xmm0, %%xmm1    # xmm1 = xmm1[0],xmm0[0],xmm1[1],xmm0[1]\n");
  fprintf(f,"        movabsq $.LRAND_1, %%rax\n");
  fprintf(f,"        movapd  (%%rax), %%xmm0\n");
  fprintf(f,"        subpd   %%xmm0, %%xmm1\n");
  fprintf(f,"        haddpd  %%xmm1, %%xmm1\n");
  fprintf(f,"        movsd   -56(%%rbp), %%xmm0\n");
  fprintf(f,"        addsd   %%xmm1, %%xmm0\n");
  fprintf(f,"        movabsq $.LRAND_2, %%rax\n");
  fprintf(f,"        movsd   (%%rax), %%xmm1\n");
  fprintf(f,"        mulsd   %%xmm1, %%xmm0\n");
  fprintf(f,"        movsd   %%xmm0, -8(%%rbp)\n");
  fprintf(f,"        movabsq $1, %%rax\n");
  fprintf(f,"        cvtsi2sdq       %%rax, %%xmm0\n");
  fprintf(f,"        movsd   -8(%%rbp), %%xmm1\n");
  fprintf(f,"        ucomisd %%xmm0, %%xmm1\n");
  fprintf(f,"        jb      7f\n");
  fprintf(f,"        jmp     0b\n");
  fprintf(f,"7:\n");
  fprintf(f,"        movsd   -8(%%rbp), %%xmm0\n");
  fprintf(f,"        addq    $80, %%rsp\n");
  fprintf(f,"        popq    %%rbp\n");
  fprintf(f,"        retq\n");
  fprintf(f,"        .size   drand, .-drand\n\n");

  fprintf(f,"        .type   realinit, @function\n");
  fprintf(f,"realinit:\n");
  fprintf(f,"        pushq   %%rbp\n");
  fprintf(f,"        movq    %%rsp, %%rbp\n");
  fprintf(f,"        pushq   %%rbx\n");
  fprintf(f,"        subq    $24, %%rsp\n");
  fprintf(f,"        movabsq $.LRAND_mm, %%rdx\n");
  fprintf(f,"        xorl    %%eax, %%eax\n");
  fprintf(f,"        movl    %%edi, %%ebx\n");
  fprintf(f,"0:\n");
  fprintf(f,"        movq    $0, (%%rdx,%%rax,8)\n");
  fprintf(f,"        incq    %%rax\n");
  fprintf(f,"        cmpq    $256, %%rax\n");
  fprintf(f,"        jne     0b\n");
  fprintf(f,"        leaq    -32(%%rbp), %%rdi # &tv\n");
  fprintf(f,"        xorq    %%rsi, %%rsi      # NULL\n");
  fprintf(f,"        movabsq $__NR_gettimeofday, %%rax\n");
  fprintf(f,"        syscall\n");
  fprintf(f,"        xorq    %%rdi, %%rdi\n");
  fprintf(f,"        testl   %%ebx, %%ebx\n");
  fprintf(f,"        movabsq $.LRAND_cc, %%rax\n");
  fprintf(f,"        movabsq $randinit, %%rcx\n");
  fprintf(f,"        movq    $0, (%%rax)\n");
  fprintf(f,"        movabsq $.LRAND_bb, %%rax\n");
  fprintf(f,"        movq    $0, (%%rax)\n");
  fprintf(f,"        movabsq $.LRAND_aa, %%rax\n");
  fprintf(f,"        movq    $0, (%%rax)\n");
  fprintf(f,"        je      1f\n");
  fprintf(f,"        movq    -32(%%rbp), %%rbx\n");
  fprintf(f,"        movl    $1, %%edi\n");
  fprintf(f,"        movq    -24(%%rbp), %%rdx\n");
  fprintf(f,"        movq    %%rbx, %%rax\n");
  fprintf(f,"        leaq    0(,%%rdx,8), %%rsi\n");
  fprintf(f,"        movabsq %%rax, .LRAND_randrsl+16\n");
  fprintf(f,"        imulq   %%rbx, %%rdx\n");
  fprintf(f,"        movq    %%rsi, %%rax\n");
  fprintf(f,"        movabsq %%rax, .LRAND_randrsl+32\n");
  fprintf(f,"        salq    $9, %%rdx\n");
  fprintf(f,"        movq    %%rdx, %%rax\n");
  fprintf(f,"        movq    %%rbx, %%rdx\n");
  fprintf(f,"        movabsq %%rax, .LRAND_randrsl+48\n");
  fprintf(f,"        salq    $13, %%rdx\n");
  fprintf(f,"        movq    %%rdx, %%rax\n");
  fprintf(f,"        movabsq %%rax, .LRAND_randrsl+64\n");
  fprintf(f,"        movq    %%rbx, %%rax\n");
  fprintf(f,"        salq    $29, %%rax\n");
  fprintf(f,"        movabsq %%rax, .LRAND_randrsl+80\n");
  fprintf(f,"1:\n");
  fprintf(f,"        callq   *%%rcx\n");
  fprintf(f,"        addq    $24, %%rsp\n");
  fprintf(f,"        popq    %%rbx\n");
  fprintf(f,"        popq    %%rbp\n");
  fprintf(f,"        retq\n");
  fprintf(f,"        .size   realinit, .-realinit\n\n");
  // end public domain derived from ISAAC-64/Bob Jenkins' C code
  fflush(f);
  return;
}
