#ifndef SCANNER2_H
#define SCANNER2_H
//
// scanner2.h - This file contains the header file for scanner2.c
//              for Mr. Ham's ECMA-55 Minimal BASIC compiler.
//
// Copyright (C) 2013,2014  John Gatewood Ham
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// version 2 of the Free Software Foundation License.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program; if not, write to the Free Software Foundation, Inc.,
// 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//
#include <inttypes.h>
#include <stdbool.h>

// token numeric identifier constants
enum token_ids {
T_ABS=0,        // ABS() function
T_ADD,          // binary add operator
T_ATN,          // ATN() function
T_BASE,         // part of OPTION BASE statement
T_COMMA,        // comma used in INPUT, PRINT, DATA
T_COS,          // COS() function
T_DATA,         // DATA statement
T_DEF,          // DEF statement
T_DIM,          // DIM statement
T_DIV,          // binary divide operator
T_END,          // END statement
T_EOF,          // synthetic end-of-file token
T_EOL,          // end-of-line token
T_EQ,           // equals relational operator
T_EXP,          // EXP() function
T_FNID,         // numeric user-defined function
T_FOR,          // part of FOR statement
T_GE,           // greater than or equals relational operator
T_GO,           // part of GO TO or GO SUB
T_GOSUB,        // GOSUB statement
T_GOTO,         // GOTO statement
T_GT,           // greater than relational operator
T_IF,           // part of IF statement
T_INPUT,        // INPUT statement
T_INT,          // INT() function
T_INTEGER,      // integer literal
T_LE,           // less than or equal to relational operator
T_LET,          // LET statement
T_LOG,          // LOG() function
T_LPAREN,       // left parenthesis
T_LT,           // less than relational operator
T_MUL,          // binary multiply operator
T_NAVAR,        // numeric array variable
T_NE,           // not equals relational operator
T_NEXT,         // part of FOR statement
T_NVAR,         // numeric scalar variable
T_ON,           // part of ON .. GOTO statement
T_OPTION,       // part of OPTION BASE statement
T_POW,          // power operator
T_PRINT,        // PRINT statement
T_QSTRING,      // quoted string used in PRINT, DATA
T_RANDOMIZE,    // RANDOMIZE statement
T_READ,         // READ statement
T_REAL,         // floating point literal value
T_REM,          // REM statement
T_RESTORE,      // RESTORE statement
T_RETURN,       // RETURN statement
T_RND,          // RND function
T_RPAREN,       // right parenthesis
T_SEMI,         // semicolon
T_SGN,          // SGN() function
T_SIN,          // SIN() function
T_SQR,          // SQR() function
T_STEP,         // part of FOR statement
T_STOP,         // STOP statement
T_SUB,          // part of GOSUB statement
T_SUBTRACT,     // binary subtraction operator
T_SVAR,         // string scalar variable
T_TAB,          // TAB() function
T_TAN,          // TAN() function
T_THEN,         // part of IF statement
T_TO,           // part of GOTO and ON .. GOTO
T_UQSTRING,     // unquoted string used in DATA and INPUT
};

// token definition
struct token {
  enum token_ids tid; // token numeric identifier
  char *toketext; // pointer to string of token text
  uint32_t lno; // line number in input program
  uint32_t cno; // column number in input program
};

// element of vector implementing token_names map
struct tndata {
  const char * const namestring; // text displaying name of token
  bool iskeyword;                // if true, the token is a Minimal BASIC keyword, otherwise false
};
// map of token numbers to token names for display in error messages
extern const struct tndata token_names[];

struct token peek_next_token(void);
struct token get_next_token(void);
struct token *clone_next_token(void);
bool scan_buffer(const char *buffer);
void dump_token_stream(void);
void clear_token_stream(void);

#endif
