#ifndef SYMBOL_TABLE_H
#define SYMBOL_TABLE_H
//
// symbol_table.h - This is the header file for symbol_table.c
//                  for Mr. Ham's ECMA-55 Minimal BASIC compiler.
//
// Copyright (C) 2013,2014  John Gatewood Ham
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// version 2 of the Free Software Foundation License.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program; if not, write to the Free Software Foundation, Inc.,
// 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//
#include <stdio.h>
#include <stdbool.h>
#include <inttypes.h>

struct arraydata {
  const char * const namestring; // ASCIIZ name of array
  unsigned int dims; // number of dimensions (1 or 2, but if 0 this means no array by this name
                     // has been used in the program)
  unsigned int dim1; // maximum permitted index for first dimension
  unsigned int dim2; // maximum permitted index for second dimension (not used if dims is 1)
};

// Type numbers
//    QSTR is a quoted string
//    NUM is a floating point value
//    UQSTR is an unquoted string (used in READ and INPUT)
enum data_item_type { QSTR=0, NUM, UQSTR };

// lookup information in symbol table module
bool nvar_exists(const char *s);
bool navar_exists(const char *s);
bool svar_exists(const char *s);
bool udf_exists(const char *s,char *argname);
struct arraydata *get_array_data(const char *s);
void dump_symbol_table(bool onlyused);
// add/update information in symbol table module
bool assign_nvar(const char *s,const uint32_t lno);
bool assign_navar(const char *s,const uint32_t lno);
bool assign_svar(const char *s);
bool assign_magicvar(const char *s);
bool add_udf(const char *fname,const char *argname,uint32_t lno);
bool add_string_literal(const char *s, char **symbol);
bool add_input_format(const char *fmt, char **symbol, uint32_t lno);
bool add_float_literal(const char *s, char **symbol);
bool put_array_data(const char *s,unsigned int dims,unsigned int dim1, unsigned int dim2);
bool add_data_literal(const char *s, enum data_item_type t);
// other functions
bool generate_symbol_table(FILE *f);
void clear_symbol_table(void);
#endif
