//
// gay.c - This file contains the code generation routines
//         for David M. Gay's  ASCIIZ<-->double conversion.
//         used to support Minimal BASIC's input and output
//         of numeric values in the INPUT, READ, and PRINT
//         statements for Mr. Ham's ECMA-55 Minimal BASIC 
//         compiler.
//
// The author of this software is David M. Gay.
//
// Copyright (c) 1991, 2000, 2001 by Lucent Technologies.
//
// Permission to use, copy, modify, and distribute this software for any
// purpose without fee is hereby granted, provided that this entire notice
// is included in all copies of any software which is or includes a copy
// or modification of this software and in all copies of the supporting
// documentation for such software.
//
// THIS SOFTWARE IS BEING PROVIDED "AS IS", WITHOUT ANY EXPRESS OR IMPLIED
// WARRANTY.  IN PARTICULAR, NEITHER THE AUTHOR NOR LUCENT MAKES ANY
// REPRESENTATION OR WARRANTY OF ANY KIND CONCERNING THE MERCHANTABILITY
// OF THIS SOFTWARE OR ITS FITNESS FOR ANY PARTICULAR PURPOSE.
//
// Upstream URL:
// http://netlib.sandia.gov/fp/index.html
//
#include <stdlib.h>
#include "gay.h"

//
// This procedure will emit code for David M. Gay's ASCIIZ<-->double
// conversion routines and accompanying support routines.
// The g_fmt routine is used to convert a floating point number to an ASCIIZ string,
// and the strtod is used to convert ASCIIZ string to a floating point number.
// The floating point numbers are 64bit (double), and any conversion to 32 bit
// floating point must be done by the caller.
// This code was created by running a C compiler on Gay's dtoa.c and g_fmt.c
// files and having it emit assembly.  This requires SSE2 and CMOV instruction 
// set support.
//
void dgay_code(FILE *f) {
  if (NULL==f) {
    fprintf(stderr,"dgay_code(): file parameter is NULL\n");
    abort();
  }
  fprintf(f,"################################################################\n");
  fprintf(f,"#\n");
  fprintf(f,"# The author of this software is David M. Gay.\n");
  fprintf(f,"#\n");
  fprintf(f,"# Copyright (c) 1991, 2000, 2001 by Lucent Technologies.\n");
  fprintf(f,"#\n");
  fprintf(f,"# Permission to use, copy, modify, and distribute this software for any\n");
  fprintf(f,"# purpose without fee is hereby granted, provided that this entire notice\n");
  fprintf(f,"# is included in all copies of any software which is or includes a copy\n");
  fprintf(f,"# or modification of this software and in all copies of the supporting\n");
  fprintf(f,"# documentation for such software.\n");
  fprintf(f,"#\n");
  fprintf(f,"# THIS SOFTWARE IS BEING PROVIDED \"AS IS\", WITHOUT ANY EXPRESS OR IMPLIED\n");
  fprintf(f,"# WARRANTY.  IN PARTICULAR, NEITHER THE AUTHOR NOR LUCENT MAKES ANY\n");
  fprintf(f,"# REPRESENTATION OR WARRANTY OF ANY KIND CONCERNING THE MERCHANTABILITY\n");
  fprintf(f,"# OF THIS SOFTWARE OR ITS FITNESS FOR ANY PARTICULAR PURPOSE.\n");
  fprintf(f,"#\n");
  fprintf(f,"################################################################\n");
  fprintf(f,"        .text\n");
  fprintf(f,"        .type   hi0bits, @function\n");
  fprintf(f,"hi0bits:\n");
  fprintf(f,"        pushq   %%rbp\n");
  fprintf(f,"        xorl    %%edx, %%edx\n");
  fprintf(f,"        testl   $-65536, %%edi\n");
  fprintf(f,"        movq    %%rsp, %%rbp\n");
  fprintf(f,"        jne     0f\n");
  fprintf(f,"        sall    $16, %%edi\n");
  fprintf(f,"        movb    $16, %%dl\n");
  fprintf(f,"0:\n");
  fprintf(f,"        testl   $-16777216, %%edi\n");
  fprintf(f,"        jne     1f\n");
  fprintf(f,"        addl    $8, %%edx\n");
  fprintf(f,"        sall    $8, %%edi\n");
  fprintf(f,"1:\n");
  fprintf(f,"        testl   $-268435456, %%edi\n");
  fprintf(f,"        jne     2f\n");
  fprintf(f,"        addl    $4, %%edx\n");
  fprintf(f,"        sall    $4, %%edi\n");
  fprintf(f,"2:\n");
  fprintf(f,"        testl   $-1073741824, %%edi\n");
  fprintf(f,"        jne     3f\n");
  fprintf(f,"        addl    $2, %%edx\n");
  fprintf(f,"        sall    $2, %%edi\n");
  fprintf(f,"3:\n");
  fprintf(f,"        testl   %%edi, %%edi\n");
  fprintf(f,"        js      4f\n");
  fprintf(f,"        andl    $1073741824, %%edi\n");
  fprintf(f,"        movl    $32, %%eax\n");
  fprintf(f,"        je      5f\n");
  fprintf(f,"        leal    1(%%rdx), %%eax\n");
  fprintf(f,"        jmp     5f\n");
  fprintf(f,"4:\n");
  fprintf(f,"        movl    %%edx, %%eax\n");
  fprintf(f,"5:\n");
  fprintf(f,"        popq    %%rbp\n");
  fprintf(f,"        retq\n");
  fprintf(f,"        .size   hi0bits, .-hi0bits\n\n");

  fprintf(f,"        .type   lo0bits, @function\n");
  fprintf(f,"lo0bits:\n");
  fprintf(f,"        movl    (%%rdi), %%edx\n");
  fprintf(f,"        pushq   %%rbp\n");
  fprintf(f,"        movq    %%rsp, %%rbp\n");
  fprintf(f,"        testb   $7, %%dl\n");
  fprintf(f,"        je      1f\n");
  fprintf(f,"        xorl    %%eax, %%eax\n");
  fprintf(f,"        testb   $1, %%dl\n");
  fprintf(f,"        jne     7f\n");
  fprintf(f,"        testb   $2, %%dl\n");
  fprintf(f,"        je      0f\n");
  fprintf(f,"        shrl    %%edx\n");
  fprintf(f,"        movb    $1, %%al\n");
  fprintf(f,"        movl    %%edx, (%%rdi)\n");
  fprintf(f,"        jmp     7f\n");
  fprintf(f,"0:\n");
  fprintf(f,"        shrl    $2, %%edx\n");
  fprintf(f,"        movl    $2, %%eax\n");
  fprintf(f,"        movl    %%edx, (%%rdi)\n");
  fprintf(f,"        jmp     7f\n");
  fprintf(f,"1:\n");
  fprintf(f,"        xorl    %%ecx, %%ecx\n");
  fprintf(f,"        testw   %%dx, %%dx\n");
  fprintf(f,"        jne     2f\n");
  fprintf(f,"        shrl    $16, %%edx\n");
  fprintf(f,"        movb    $16, %%cl\n");
  fprintf(f,"2:\n");
  fprintf(f,"        testb   %%dl, %%dl\n");
  fprintf(f,"        jne     3f\n");
  fprintf(f,"        addl    $8, %%ecx\n");
  fprintf(f,"        shrl    $8, %%edx\n");
  fprintf(f,"3:\n");
  fprintf(f,"        testb   $15, %%dl\n");
  fprintf(f,"        jne     4f\n");
  fprintf(f,"        addl    $4, %%ecx\n");
  fprintf(f,"        shrl    $4, %%edx\n");
  fprintf(f,"4:\n");
  fprintf(f,"        testb   $3, %%dl\n");
  fprintf(f,"        jne     5f\n");
  fprintf(f,"        addl    $2, %%ecx\n");
  fprintf(f,"        shrl    $2, %%edx\n");
  fprintf(f,"5:\n");
  fprintf(f,"        testb   $1, %%dl\n");
  fprintf(f,"        jne     6f\n");
  fprintf(f,"        shrl    %%edx\n");
  fprintf(f,"        movl    $32, %%eax\n");
  fprintf(f,"        je      7f\n");
  fprintf(f,"        incl    %%ecx\n");
  fprintf(f,"6:\n");
  fprintf(f,"        movl    %%edx, (%%rdi)\n");
  fprintf(f,"        movl    %%ecx, %%eax\n");
  fprintf(f,"7:\n");
  fprintf(f,"        popq    %%rbp\n");
  fprintf(f,"        retq\n");
  fprintf(f,"        .size   lo0bits, .-lo0bits\n\n");

  fprintf(f,"        .type   cmp, @function\n");
  fprintf(f,"cmp:\n");
  fprintf(f,"        pushq   %%rbp\n");
  fprintf(f,"        movslq  20(%%rsi), %%rdx\n");
  fprintf(f,"        movl    20(%%rdi), %%eax\n");
  fprintf(f,"        movq    %%rsp, %%rbp\n");
  fprintf(f,"        subl    %%edx, %%eax\n");
  fprintf(f,"        testl   %%eax, %%eax\n");
  fprintf(f,"        jne     2f\n");
  fprintf(f,"        salq    $2, %%rdx\n");
  fprintf(f,"        addq    $24, %%rdi\n");
  fprintf(f,"        leaq    (%%rdi,%%rdx), %%rax\n");
  fprintf(f,"        leaq    24(%%rsi,%%rdx), %%rdx\n");
  fprintf(f,"0:\n");
  fprintf(f,"        subq    $4, %%rdx\n");
  fprintf(f,"        movl    (%%rdx), %%ecx\n");
  fprintf(f,"        subq    $4, %%rax\n");
  fprintf(f,"        cmpl    %%ecx, (%%rax)\n");
  fprintf(f,"        je      1f\n");
  fprintf(f,"        sbbl    %%eax, %%eax\n");
  fprintf(f,"        orl     $1, %%eax\n");
  fprintf(f,"        jmp     2f\n");
  fprintf(f,"1:\n");
  fprintf(f,"        cmpq    %%rdi, %%rax\n");
  fprintf(f,"        ja      0b\n");
  fprintf(f,"        xorl    %%eax, %%eax\n");
  fprintf(f,"2:\n");
  fprintf(f,"        popq    %%rbp\n");
  fprintf(f,"        retq\n");
  fprintf(f,"        .size   cmp, .-cmp\n\n");

  fprintf(f,"        .type   b2d, @function\n");
  fprintf(f,"b2d:\n");
  fprintf(f,"        pushq   %%rbp\n");
  fprintf(f,"        movq    %%rsp, %%rbp\n");
  fprintf(f,"        pushq   %%r15\n");
  fprintf(f,"        movq    %%rsi, %%r15\n");
  fprintf(f,"        pushq   %%r14\n");
  fprintf(f,"        leaq    24(%%rdi), %%r14\n");
  fprintf(f,"        pushq   %%r13\n");
  fprintf(f,"        pushq   %%r12\n");
  fprintf(f,"        pushq   %%rbx\n");
  fprintf(f,"        movslq  20(%%rdi), %%rax\n");
  fprintf(f,"        leaq    (%%r14,%%rax,4), %%r13\n");
  fprintf(f,"        movabsq $hi0bits, %%rax\n");
  fprintf(f,"        movl    -4(%%r13), %%ebx\n");
  fprintf(f,"        leaq    -4(%%r13), %%r12\n");
  fprintf(f,"        movl    %%ebx, %%edi\n");
  fprintf(f,"        callq   *%%rax\n");
  fprintf(f,"        movl    $32, %%edx\n");
  fprintf(f,"        subl    %%eax, %%edx\n");
  fprintf(f,"        cmpl    $10, %%eax\n");
  fprintf(f,"        movl    %%edx, (%%r15)\n");
  fprintf(f,"        jg      1f\n");
  fprintf(f,"        movl    $11, %%ecx\n");
  fprintf(f,"        movl    %%ebx, %%edx\n");
  fprintf(f,"        xorl    %%edi, %%edi\n");
  fprintf(f,"        subl    %%eax, %%ecx\n");
  fprintf(f,"        shrl    %%cl, %%edx\n");
  fprintf(f,"        orl     $1072693248, %%edx\n");
  fprintf(f,"        salq    $32, %%rdx\n");
  fprintf(f,"        cmpq    %%r14, %%r12\n");
  fprintf(f,"        jbe     0f\n");
  fprintf(f,"        movl    -8(%%r13), %%edi\n");
  fprintf(f,"0:\n");
  fprintf(f,"        shrl    %%cl, %%edi\n");
  fprintf(f,"        leal    21(%%rax), %%ecx\n");
  fprintf(f,"        sall    %%cl, %%ebx\n");
  fprintf(f,"        orl     %%ebx, %%edi\n");
  fprintf(f,"        jmp     5f\n");
  fprintf(f,"1:\n");
  fprintf(f,"        xorl    %%edx, %%edx\n");
  fprintf(f,"        cmpq    %%r14, %%r12\n");
  fprintf(f,"        jbe     2f\n");
  fprintf(f,"        movl    -8(%%r13), %%edx\n");
  fprintf(f,"        leaq    -8(%%r13), %%r12\n");
  fprintf(f,"2:\n");
  fprintf(f,"        movl    %%eax, %%r8d\n");
  fprintf(f,"        subl    $11, %%r8d\n");
  fprintf(f,"        je      4f\n");
  fprintf(f,"        movl    $43, %%esi\n");
  fprintf(f,"        movb    %%r8b, %%cl\n");
  fprintf(f,"        movl    %%edx, %%edi\n");
  fprintf(f,"        subl    %%eax, %%esi\n");
  fprintf(f,"        sall    %%cl, %%ebx\n");
  fprintf(f,"        movb    %%sil, %%cl\n");
  fprintf(f,"        orl     $1072693248, %%ebx\n");
  fprintf(f,"        shrl    %%cl, %%edi\n");
  fprintf(f,"        orl     %%edi, %%ebx\n");
  fprintf(f,"        xorl    %%edi, %%edi\n");
  fprintf(f,"        salq    $32, %%rbx\n");
  fprintf(f,"        cmpq    %%r14, %%r12\n");
  fprintf(f,"        jbe     3f\n");
  fprintf(f,"        movl    -4(%%r12), %%edi\n");
  fprintf(f,"3:\n");
  fprintf(f,"        movb    %%sil, %%cl\n");
  fprintf(f,"        shrl    %%cl, %%edi\n");
  fprintf(f,"        movb    %%r8b, %%cl\n");
  fprintf(f,"        sall    %%cl, %%edx\n");
  fprintf(f,"        orl     %%edx, %%edi\n");
  fprintf(f,"        orq     %%rbx, %%rdi\n");
  fprintf(f,"        jmp     6f\n");
  fprintf(f,"4:\n");
  fprintf(f,"        movl    %%ebx, %%edi\n");
  fprintf(f,"        orl     $1072693248, %%edi\n");
  fprintf(f,"        salq    $32, %%rdi\n");
  fprintf(f,"5:\n");
  fprintf(f,"        orq     %%rdx, %%rdi\n");
  fprintf(f,"6:\n");
  fprintf(f,"        popq    %%rbx\n");
  fprintf(f,"        movd    %%rdi, %%xmm0\n");
  fprintf(f,"        popq    %%r12\n");
  fprintf(f,"        popq    %%r13\n");
  fprintf(f,"        popq    %%r14\n");
  fprintf(f,"        popq    %%r15\n");
  fprintf(f,"        popq    %%rbp\n");
  fprintf(f,"        retq\n");
  fprintf(f,"        .size   b2d, .-b2d\n\n");

  fprintf(f,"        .type   ratio, @function\n");
  fprintf(f,"ratio:\n");
  fprintf(f,"        pushq   %%rbp\n");
  fprintf(f,"        movq    %%rsp, %%rbp\n");
  fprintf(f,"        pushq   %%r14\n");
  fprintf(f,"        movabsq $b2d, %%r14\n");
  fprintf(f,"        pushq   %%r13\n");
  fprintf(f,"        movq    %%rdi, %%r13\n");
  fprintf(f,"        pushq   %%r12\n");
  fprintf(f,"        movq    %%rsi, %%r12\n");
  fprintf(f,"        pushq   %%rbx\n");
  fprintf(f,"        leaq    -40(%%rbp), %%rsi\n");
  fprintf(f,"        subq    $16, %%rsp\n");
  fprintf(f,"        callq   *%%r14\n");
  fprintf(f,"        leaq    -36(%%rbp), %%rsi\n");
  fprintf(f,"        movq    %%r12, %%rdi\n");
  fprintf(f,"        movd    %%xmm0, %%rbx\n");
  fprintf(f,"        callq   *%%r14\n");
  fprintf(f,"        movl    20(%%r13), %%edx\n");
  fprintf(f,"        movl    -40(%%rbp), %%ecx\n");
  fprintf(f,"        movd    %%xmm0, %%rax\n");
  fprintf(f,"        subl    20(%%r12), %%edx\n");
  fprintf(f,"        subl    -36(%%rbp), %%ecx\n");
  fprintf(f,"        sall    $5, %%edx\n");
  fprintf(f,"        addl    %%ecx, %%edx\n");
  fprintf(f,"        testl   %%edx, %%edx\n");
  fprintf(f,"        jle     0f\n");
  fprintf(f,"        movq    %%rbx, %%rcx\n");
  fprintf(f,"        sall    $20, %%edx\n");
  fprintf(f,"        movl    %%ebx, %%ebx\n");
  fprintf(f,"        shrq    $32, %%rcx\n");
  fprintf(f,"        addl    %%ecx, %%edx\n");
  fprintf(f,"        salq    $32, %%rdx\n");
  fprintf(f,"        orq     %%rdx, %%rbx\n");
  fprintf(f,"        jmp     1f\n");
  fprintf(f,"0:\n");
  fprintf(f,"        movl    %%edx, %%ecx\n");
  fprintf(f,"        movd    %%xmm0, %%rdx\n");
  fprintf(f,"        movl    %%eax, %%eax\n");
  fprintf(f,"        sall    $20, %%ecx\n");
  fprintf(f,"        shrq    $32, %%rdx\n");
  fprintf(f,"        subl    %%ecx, %%edx\n");
  fprintf(f,"        salq    $32, %%rdx\n");
  fprintf(f,"        orq     %%rdx, %%rax\n");
  fprintf(f,"1:\n");
  fprintf(f,"        movd    %%rax, %%xmm1\n");
  fprintf(f,"        popq    %%rax\n");
  fprintf(f,"        movd    %%rbx, %%xmm0\n");
  fprintf(f,"        divsd   %%xmm1, %%xmm0\n");
  fprintf(f,"        popq    %%rdx\n");
  fprintf(f,"        popq    %%rbx\n");
  fprintf(f,"        popq    %%r12\n");
  fprintf(f,"        popq    %%r13\n");
  fprintf(f,"        popq    %%r14\n");
  fprintf(f,"        popq    %%rbp\n");
  fprintf(f,"        retq\n");
  fprintf(f,"        .size   ratio, .-ratio\n\n");

  fprintf(f,"        .type   match, @function\n");
  fprintf(f,"match:\n");
  fprintf(f,"        movq    (%%rdi), %%rdx\n");
  fprintf(f,"        pushq   %%rbp\n");
  fprintf(f,"        movq    %%rsp, %%rbp\n");
  fprintf(f,"0:\n");
  fprintf(f,"        movsbl  (%%rsi), %%ecx\n");
  fprintf(f,"        leaq    1(%%rdx), %%rdx\n");
  fprintf(f,"        incq    %%rsi\n");
  fprintf(f,"        testl   %%ecx, %%ecx\n");
  fprintf(f,"        je      1f\n");
  fprintf(f,"        movb    (%%rdx), %%r8b\n");
  fprintf(f,"        movsbl  %%r8b, %%eax\n");
  fprintf(f,"        subl    $65, %%r8d\n");
  fprintf(f,"        leal    32(%%rax), %%r9d\n");
  fprintf(f,"        cmpb    $25, %%r8b\n");
  fprintf(f,"        cmovbe  %%r9d, %%eax\n");
  fprintf(f,"        cmpl    %%ecx, %%eax\n");
  fprintf(f,"        je      0b\n");
  fprintf(f,"        xorl    %%eax, %%eax\n");
  fprintf(f,"        jmp     2f\n");
  fprintf(f,"1:\n");
  fprintf(f,"        movq    %%rdx, (%%rdi)\n");
  fprintf(f,"        movl    $1, %%eax\n");
  fprintf(f,"2:\n");
  fprintf(f,"        popq    %%rbp\n");
  fprintf(f,"        retq\n");
  fprintf(f,"        .size   match, .-match\n\n");

  fprintf(f,"        .type   hexnan, @function\n");
  fprintf(f,"hexnan:\n");
  fprintf(f,"        movq    (%%rsi), %%rax\n");
  fprintf(f,"0:\n");
  fprintf(f,"        movb    1(%%rax), %%dl\n");
  fprintf(f,"        leal    -1(%%rdx), %%ecx\n");
  fprintf(f,"        cmpb    $31, %%cl\n");
  fprintf(f,"        ja      1f\n");
  fprintf(f,"        incq    %%rax\n");
  fprintf(f,"        jmp     0b\n");
  fprintf(f,"1:\n");
  fprintf(f,"        pushq   %%rbp\n");
  fprintf(f,"        cmpb    $48, %%dl\n");
  fprintf(f,"        movq    %%rsp, %%rbp\n");
  fprintf(f,"        pushq   %%r14\n");
  fprintf(f,"        pushq   %%r13\n");
  fprintf(f,"        pushq   %%r12\n");
  fprintf(f,"        pushq   %%rbx\n");
  fprintf(f,"        jne     2f\n");
  fprintf(f,"        movb    2(%%rax), %%dl\n");
  fprintf(f,"        leaq    2(%%rax), %%rcx\n");
  fprintf(f,"        andl    $-33, %%edx\n");
  fprintf(f,"        cmpb    $88, %%dl\n");
  fprintf(f,"        cmove   %%rcx, %%rax\n");
  fprintf(f,"2:\n");
  fprintf(f,"        xorl    %%edx, %%edx\n");
  fprintf(f,"        xorl    %%r8d, %%r8d\n");
  fprintf(f,"        xorl    %%r11d, %%r11d\n");
  fprintf(f,"        movl    $1, %%r10d\n");
  fprintf(f,"        xorl    %%r9d, %%r9d\n");
  fprintf(f,"        movl    $1, %%ebx\n");
  fprintf(f,"        movabsq $hexdig, %%r12\n");
  fprintf(f,"        xorl    %%r13d, %%r13d\n");
  fprintf(f,"3:\n");
  fprintf(f,"        incq    %%rax\n");
  fprintf(f,"        movzbl  (%%rax), %%ecx\n");
  fprintf(f,"        testl   %%ecx, %%ecx\n");
  fprintf(f,"        je      9f\n");
  fprintf(f,"        movl    %%ecx, %%r14d\n");
  fprintf(f,"        movb    (%%r12,%%r14), %%r14b\n");
  fprintf(f,"        testb   %%r14b, %%r14b\n");
  fprintf(f,"        je      4f\n");
  fprintf(f,"        andl    $15, %%r14d\n");
  fprintf(f,"        xorl    %%ecx, %%ecx\n");
  fprintf(f,"        testl   %%r11d, %%r11d\n");
  fprintf(f,"        jne     7f\n");
  fprintf(f,"        movl    %%edx, %%ecx\n");
  fprintf(f,"        movl    %%r8d, %%edx\n");
  fprintf(f,"        jmp     7f\n");
  fprintf(f,"4:\n");
  fprintf(f,"        cmpl    $32, %%ecx\n");
  fprintf(f,"        ja      6f\n");
  fprintf(f,"        testb   $1, %%r9b\n");
  fprintf(f,"        je      3b\n");
  fprintf(f,"        testl   %%r10d, %%r10d\n");
  fprintf(f,"        cmovne  %%ebx, %%r11d\n");
  fprintf(f,"        setne   %%cl\n");
  fprintf(f,"        testb   %%cl, %%cl\n");
  fprintf(f,"        cmovne  %%r13d, %%r10d\n");
  fprintf(f,"        cmovne  %%ebx, %%r9d\n");
  fprintf(f,"        jmp     3b\n");
  fprintf(f,"5:\n");
  fprintf(f,"        movsbl  (%%rax), %%ecx\n");
  fprintf(f,"        testl   %%ecx, %%ecx\n");
  fprintf(f,"        je      9f\n");
  fprintf(f,"6:\n");
  fprintf(f,"        leaq    1(%%rax), %%rax\n");
  fprintf(f,"        cmpl    $41, %%ecx\n");
  fprintf(f,"        jne     5b\n");
  fprintf(f,"        movq    %%rax, (%%rsi)\n");
  fprintf(f,"        jmp     9f\n");
  fprintf(f,"7:\n");
  fprintf(f,"        testl   %%r10d, %%r10d\n");
  fprintf(f,"        je      8f\n");
  fprintf(f,"        sall    $4, %%edx\n");
  fprintf(f,"        movl    %%edx, %%r8d\n");
  fprintf(f,"        movl    %%ecx, %%edx\n");
  fprintf(f,"        shrl    $28, %%edx\n");
  fprintf(f,"        orl     %%r8d, %%edx\n");
  fprintf(f,"8:\n");
  fprintf(f,"        sall    $4, %%ecx\n");
  fprintf(f,"        movl    %%edx, %%r8d\n");
  fprintf(f,"        xorl    %%r11d, %%r11d\n");
  fprintf(f,"        orl     %%r14d, %%ecx\n");
  fprintf(f,"        movl    $1, %%r9d\n");
  fprintf(f,"        movl    %%ecx, %%edx\n");
  fprintf(f,"        jmp     3b\n");
  fprintf(f,"9:\n");
  fprintf(f,"        andl    $1048575, %%r8d\n");
  fprintf(f,"        jne     10f\n");
  fprintf(f,"        testl   %%edx, %%edx\n");
  fprintf(f,"        je      11f\n");
  fprintf(f,"10:\n");
  fprintf(f,"        orl     $2146435072, %%r8d\n");
  fprintf(f,"        movl    %%edx, (%%rdi)\n");
  fprintf(f,"        movl    %%r8d, 4(%%rdi)\n");
  fprintf(f,"11:\n");
  fprintf(f,"        popq    %%rbx\n");
  fprintf(f,"        popq    %%r12\n");
  fprintf(f,"        popq    %%r13\n");
  fprintf(f,"        popq    %%r14\n");
  fprintf(f,"        popq    %%rbp\n");
  fprintf(f,"        retq\n");
  fprintf(f,"        .size   hexnan, .-hexnan\n\n");

  fprintf(f,"        .type   rshift, @function\n");
  fprintf(f,"rshift:\n");
  fprintf(f,"        movslq  20(%%rdi), %%rcx\n");
  fprintf(f,"        movl    %%esi, %%edx\n");
  fprintf(f,"        pushq   %%rbp\n");
  fprintf(f,"        leaq    24(%%rdi), %%r8\n");
  fprintf(f,"        sarl    $5, %%edx\n");
  fprintf(f,"        movq    %%rsp, %%rbp\n");
  fprintf(f,"        movq    %%r8, %%rax\n");
  fprintf(f,"        pushq   %%rbx\n");
  fprintf(f,"        cmpl    %%ecx, %%edx\n");
  fprintf(f,"        jge     3f\n");
  fprintf(f,"        leaq    (%%r8,%%rcx,4), %%rbx\n");
  fprintf(f,"        movslq  %%edx, %%rdx\n");
  fprintf(f,"        andl    $31, %%esi\n");
  fprintf(f,"        leaq    (%%r8,%%rdx,4), %%rdx\n");
  fprintf(f,"        je      2f\n");
  fprintf(f,"        leaq    4(%%rdx), %%r9\n");
  fprintf(f,"        movl    (%%rdx), %%edx\n");
  fprintf(f,"        movl    $32, %%r11d\n");
  fprintf(f,"        movb    %%sil, %%cl\n");
  fprintf(f,"        subl    %%esi, %%r11d\n");
  fprintf(f,"        movq    %%r8, %%rax\n");
  fprintf(f,"        shrl    %%cl, %%edx\n");
  fprintf(f,"0:\n");
  fprintf(f,"        cmpq    %%rbx, %%r9\n");
  fprintf(f,"        jae     1f\n");
  fprintf(f,"        movl    (%%r9), %%r10d\n");
  fprintf(f,"        movb    %%r11b, %%cl\n");
  fprintf(f,"        addq    $4, %%rax\n");
  fprintf(f,"        addq    $4, %%r9\n");
  fprintf(f,"        sall    %%cl, %%r10d\n");
  fprintf(f,"        movb    %%sil, %%cl\n");
  fprintf(f,"        orl     %%edx, %%r10d\n");
  fprintf(f,"        movl    %%r10d, -4(%%rax)\n");
  fprintf(f,"        movl    -4(%%r9), %%edx\n");
  fprintf(f,"        shrl    %%cl, %%edx\n");
  fprintf(f,"        jmp     0b\n");
  fprintf(f,"1:\n");
  fprintf(f,"        leaq    4(%%rax), %%rcx\n");
  fprintf(f,"        testl   %%edx, %%edx\n");
  fprintf(f,"        movl    %%edx, (%%rax)\n");
  fprintf(f,"        cmovne  %%rcx, %%rax\n");
  fprintf(f,"        jmp     3f\n");
  fprintf(f,"2:\n");
  fprintf(f,"        cmpq    %%rbx, %%rdx\n");
  fprintf(f,"        jae     3f\n");
  fprintf(f,"        addq    $4, %%rdx\n");
  fprintf(f,"        movl    -4(%%rdx), %%ecx\n");
  fprintf(f,"        addq    $4, %%rax\n");
  fprintf(f,"        movl    %%ecx, -4(%%rax)\n");
  fprintf(f,"        jmp     2b\n");
  fprintf(f,"3:\n");
  fprintf(f,"        subq    %%r8, %%rax\n");
  fprintf(f,"        sarq    $2, %%rax\n");
  fprintf(f,"        testl   %%eax, %%eax\n");
  fprintf(f,"        movl    %%eax, 20(%%rdi)\n");
  fprintf(f,"        jne     4f\n");
  fprintf(f,"        movl    $0, 24(%%rdi)\n");
  fprintf(f,"4:\n");
  fprintf(f,"        popq    %%rbx\n");
  fprintf(f,"        popq    %%rbp\n");
  fprintf(f,"        retq\n");
  fprintf(f,"        .size   rshift, .-rshift\n\n");

  fprintf(f,"        .type   any_on, @function\n");
  fprintf(f,"any_on:\n");
  fprintf(f,"        pushq   %%rbp\n");
  fprintf(f,"        leaq    24(%%rdi), %%r8\n");
  fprintf(f,"        movl    %%esi, %%edx\n");
  fprintf(f,"        movl    20(%%rdi), %%eax\n");
  fprintf(f,"        sarl    $5, %%edx\n");
  fprintf(f,"        movq    %%rsp, %%rbp\n");
  fprintf(f,"        cmpl    %%eax, %%edx\n");
  fprintf(f,"        jg      0f\n");
  fprintf(f,"        jge     1f\n");
  fprintf(f,"        movl    %%esi, %%ecx\n");
  fprintf(f,"        andl    $31, %%ecx\n");
  fprintf(f,"        je      1f\n");
  fprintf(f,"        movslq  %%edx, %%rax\n");
  fprintf(f,"        movl    (%%r8,%%rax,4), %%r9d\n");
  fprintf(f,"        movl    $1, %%eax\n");
  fprintf(f,"        movl    %%r9d, %%edi\n");
  fprintf(f,"        shrl    %%cl, %%edi\n");
  fprintf(f,"        sall    %%cl, %%edi\n");
  fprintf(f,"        cmpl    %%r9d, %%edi\n");
  fprintf(f,"        jne     4f\n");
  fprintf(f,"        jmp     1f\n");
  fprintf(f,"0:\n");
  fprintf(f,"        movl    %%eax, %%edx\n");
  fprintf(f,"1:\n");
  fprintf(f,"        movslq  %%edx, %%rdx\n");
  fprintf(f,"        leaq    (%%r8,%%rdx,4), %%rax\n");
  fprintf(f,"2:\n");
  fprintf(f,"        cmpq    %%r8, %%rax\n");
  fprintf(f,"        jbe     3f\n");
  fprintf(f,"        subq    $4, %%rax\n");
  fprintf(f,"        cmpl    $0, (%%rax)\n");
  fprintf(f,"        je      2b\n");
  fprintf(f,"        movl    $1, %%eax\n");
  fprintf(f,"        jmp     4f\n");
  fprintf(f,"3:\n");
  fprintf(f,"        xorl    %%eax, %%eax\n");
  fprintf(f,"4:\n");
  fprintf(f,"        popq    %%rbp\n");
  fprintf(f,"        retq\n");
  fprintf(f,"        .size   any_on, .-any_on\n\n");

  fprintf(f,"        .type   dshift, @function\n");
  fprintf(f,"dshift:\n");
  fprintf(f,"        pushq   %%rbp\n");
  fprintf(f,"        movq    %%rsp, %%rbp\n");
  fprintf(f,"        pushq   %%rbx\n");
  fprintf(f,"        movl    %%esi, %%ebx\n");
  fprintf(f,"        movl    20(%%rdi), %%eax\n");
  fprintf(f,"        decl    %%eax\n");
  fprintf(f,"        cltq\n");
  fprintf(f,"        movl    24(%%rdi,%%rax,4), %%edi\n");
  fprintf(f,"        movabsq $hi0bits, %%rax\n");
  fprintf(f,"        callq   *%%rax\n");
  fprintf(f,"        subl    $4, %%eax\n");
  fprintf(f,"        movl    %%eax, %%edx\n");
  fprintf(f,"        subl    %%ebx, %%edx\n");
  fprintf(f,"        testl   %%ebx, %%ebx\n");
  fprintf(f,"        popq    %%rbx\n");
  fprintf(f,"        cmovg   %%edx, %%eax\n");
  fprintf(f,"        andl    $31, %%eax\n");
  fprintf(f,"        popq    %%rbp\n");
  fprintf(f,"        retq\n");
  fprintf(f,"        .size   dshift, .-dshift\n\n");

  fprintf(f,"        .type   quorem, @function\n");
  fprintf(f,"quorem:\n");
  fprintf(f,"        pushq   %%rbp\n");
  fprintf(f,"        xorl    %%eax, %%eax\n");
  fprintf(f,"        movq    %%rsp, %%rbp\n");
  fprintf(f,"        pushq   %%r15\n");
  fprintf(f,"        pushq   %%r14\n");
  fprintf(f,"        movq    %%rdi, %%r14\n");
  fprintf(f,"        pushq   %%r13\n");
  fprintf(f,"        pushq   %%r12\n");
  fprintf(f,"        pushq   %%rbx\n");
  fprintf(f,"        subq    $16, %%rsp\n");
  fprintf(f,"        movl    20(%%rsi), %%ebx\n");
  fprintf(f,"        cmpl    %%ebx, 20(%%rdi)\n");
  fprintf(f,"        jl      10f\n");
  fprintf(f,"        leaq    24(%%rsi), %%r13\n");
  fprintf(f,"        decl    %%ebx\n");
  fprintf(f,"        xorl    %%edx, %%edx\n");
  fprintf(f,"        leaq    24(%%rdi), %%r12\n");
  fprintf(f,"        movslq  %%ebx, %%rcx\n");
  fprintf(f,"        salq    $2, %%rcx\n");
  fprintf(f,"        leaq    0(%%r13,%%rcx), %%r9\n");
  fprintf(f,"        addq    %%r12, %%rcx\n");
  fprintf(f,"        movl    (%%r9), %%eax\n");
  fprintf(f,"        leal    1(%%rax), %%edi\n");
  fprintf(f,"        movl    (%%rcx), %%eax\n");
  fprintf(f,"        divl    %%edi\n");
  fprintf(f,"        testl   %%eax, %%eax\n");
  fprintf(f,"        movl    %%eax, %%r15d\n");
  fprintf(f,"        movl    %%eax, -44(%%rbp)\n");
  fprintf(f,"        je      4f\n");
  fprintf(f,"        movl    %%r15d, %%r8d\n");
  fprintf(f,"        movq    %%r13, %%rax\n");
  fprintf(f,"        movq    %%r12, %%rdx\n");
  fprintf(f,"        xorl    %%r11d, %%r11d\n");
  fprintf(f,"        xorl    %%edi, %%edi\n");
  fprintf(f,"        movq    %%r8, -56(%%rbp)\n");
  fprintf(f,"0:\n");
  fprintf(f,"        addq    $4, %%rax\n");
  fprintf(f,"        movl    -4(%%rax), %%r10d\n");
  fprintf(f,"        addq    $4, %%rdx\n");
  fprintf(f,"        movl    -4(%%rdx), %%r8d\n");
  fprintf(f,"        imulq   -56(%%rbp), %%r10\n");
  fprintf(f,"        addq    %%r11, %%r10\n");
  fprintf(f,"        movq    %%r10, %%r11\n");
  fprintf(f,"        movl    %%r10d, %%r10d\n");
  fprintf(f,"        subq    %%r10, %%r8\n");
  fprintf(f,"        shrq    $32, %%r11\n");
  fprintf(f,"        subq    %%rdi, %%r8\n");
  fprintf(f,"        movq    %%r8, %%rdi\n");
  fprintf(f,"        movl    %%r8d, -4(%%rdx)\n");
  fprintf(f,"        shrq    $32, %%rdi\n");
  fprintf(f,"        andl    $1, %%edi\n");
  fprintf(f,"        cmpq    %%r9, %%rax\n");
  fprintf(f,"        jbe     0b\n");
  fprintf(f,"        cmpl    $0, (%%rcx)\n");
  fprintf(f,"        jne     4f\n");
  fprintf(f,"1:\n");
  fprintf(f,"        subq    $4, %%rcx\n");
  fprintf(f,"        cmpq    %%r12, %%rcx\n");
  fprintf(f,"        ja      3f\n");
  fprintf(f,"2:\n");
  fprintf(f,"        movl    %%ebx, 20(%%r14)\n");
  fprintf(f,"        jmp     4f\n");
  fprintf(f,"3:\n");
  fprintf(f,"        cmpl    $0, (%%rcx)\n");
  fprintf(f,"        jne     2b\n");
  fprintf(f,"        decl    %%ebx\n");
  fprintf(f,"        jmp     1b\n");
  fprintf(f,"4:\n");
  fprintf(f,"        movq    %%r14, %%rdi\n");
  fprintf(f,"        movq    %%r9, -56(%%rbp)\n");
  fprintf(f,"        movabsq $cmp, %%rax\n");
  fprintf(f,"        callq   *%%rax\n");
  fprintf(f,"        testl   %%eax, %%eax\n");
  fprintf(f,"        js      9f\n");
  fprintf(f,"        movq    -56(%%rbp), %%r9\n");
  fprintf(f,"        leal    1(%%r15), %%eax\n");
  fprintf(f,"        movq    %%r12, %%rdx\n");
  fprintf(f,"        movl    %%eax, -44(%%rbp)\n");
  fprintf(f,"        xorl    %%ecx, %%ecx\n");
  fprintf(f,"5:\n");
  fprintf(f,"        addq    $4, %%r13\n");
  fprintf(f,"        movl    (%%rdx), %%eax\n");
  fprintf(f,"        addq    $4, %%rdx\n");
  fprintf(f,"        movl    -4(%%r13), %%esi\n");
  fprintf(f,"        subq    %%rsi, %%rax\n");
  fprintf(f,"        subq    %%rcx, %%rax\n");
  fprintf(f,"        movq    %%rax, %%rcx\n");
  fprintf(f,"        movl    %%eax, -4(%%rdx)\n");
  fprintf(f,"        shrq    $32, %%rcx\n");
  fprintf(f,"        andl    $1, %%ecx\n");
  fprintf(f,"        cmpq    %%r9, %%r13\n");
  fprintf(f,"        jbe     5b\n");
  fprintf(f,"        movslq  %%ebx, %%rax\n");
  fprintf(f,"        leaq    (%%r12,%%rax,4), %%rax\n");
  fprintf(f,"        cmpl    $0, (%%rax)\n");
  fprintf(f,"        jne     9f\n");
  fprintf(f,"6:\n");
  fprintf(f,"        subq    $4, %%rax\n");
  fprintf(f,"        cmpq    %%r12, %%rax\n");
  fprintf(f,"        ja      8f\n");
  fprintf(f,"7:\n");
  fprintf(f,"        movl    %%ebx, 20(%%r14)\n");
  fprintf(f,"        jmp     9f\n");
  fprintf(f,"8:\n");
  fprintf(f,"        cmpl    $0, (%%rax)\n");
  fprintf(f,"        jne     7b\n");
  fprintf(f,"        decl    %%ebx\n");
  fprintf(f,"        jmp     6b\n");
  fprintf(f,"9:\n");
  fprintf(f,"        movl    -44(%%rbp), %%eax\n");
  fprintf(f,"10:\n");
  fprintf(f,"        popq    %%rsi\n");
  fprintf(f,"        popq    %%rdi\n");
  fprintf(f,"        popq    %%rbx\n");
  fprintf(f,"        popq    %%r12\n");
  fprintf(f,"        popq    %%r13\n");
  fprintf(f,"        popq    %%r14\n");
  fprintf(f,"        popq    %%r15\n");
  fprintf(f,"        popq    %%rbp\n");
  fprintf(f,"        retq\n");
  fprintf(f,"        .size   quorem, .-quorem\n\n");

  fprintf(f,"        .type   Balloc, @function\n");
  fprintf(f,"Balloc:\n");
  fprintf(f,"        pushq   %%rbp\n");
  fprintf(f,"        cmpl    $7, %%edi\n");
  fprintf(f,"        movq    %%rsp, %%rbp\n");
  fprintf(f,"        jg      0f\n");
  fprintf(f,"        movabsq $freelist, %%rdx\n");
  fprintf(f,"        movslq  %%edi, %%rcx\n");
  fprintf(f,"        movq    (%%rdx,%%rcx,8), %%r8\n");
  fprintf(f,"        testq   %%r8, %%r8\n");
  fprintf(f,"        je      0f\n");
  fprintf(f,"        movq    (%%r8), %%rsi\n");
  fprintf(f,"        movq    %%rsi, (%%rdx,%%rcx,8)\n");
  fprintf(f,"        jmp     2f\n");
  fprintf(f,"0:\n");
  fprintf(f,"        movb    %%dil, %%cl\n");
  fprintf(f,"        movl    $1, %%edx\n");
  fprintf(f,"        sall    %%cl, %%edx\n");
  fprintf(f,"        leal    -1(%%rdx), %%eax\n");
  fprintf(f,"        cltq\n");
  fprintf(f,"        leaq    39(,%%rax,4), %%rcx\n");
  fprintf(f,"        shrq    $3, %%rcx\n");
  fprintf(f,"        cmpl    $7, %%edi\n");
  fprintf(f,"        jg      1f\n");
  fprintf(f,"        movabsq $pmem_next, %%rax\n");
  fprintf(f,"        movl    %%ecx, %%esi\n");
  fprintf(f,"        movabsq $private_mem, %%rcx\n");
  fprintf(f,"        movq    (%%rax), %%r8\n");
  fprintf(f,"        movq    %%r8, %%rax\n");
  fprintf(f,"        subq    %%rcx, %%rax\n");
  fprintf(f,"        movq    %%rax, %%rcx\n");
  fprintf(f,"        sarq    $3, %%rcx\n");
  fprintf(f,"        addq    %%rsi, %%rcx\n");
  fprintf(f,"        cmpq    $2048, %%rcx\n");
  fprintf(f,"        ja      1f\n");
  fprintf(f,"        leaq    (%%r8,%%rsi,8), %%rcx\n");
  fprintf(f,"        movl    %%edi, 8(%%r8)\n");
  fprintf(f,"        movq    %%rcx, %%rax\n");
  fprintf(f,"        movl    %%edx, 12(%%r8)\n");
  fprintf(f,"        movabsq %%rax, pmem_next\n");
  fprintf(f,"        jmp     2f\n");
  fprintf(f,"1:\n");
  fprintf(f,"        movabsq $badxit, %%rax\n");
  fprintf(f,"        callq   *%%rax\n");
  fprintf(f,"2:\n");
  fprintf(f,"        movl    $0, 20(%%r8)\n");
  fprintf(f,"        movq    %%r8, %%rax\n");
  fprintf(f,"        movl    $0, 16(%%r8)\n");
  fprintf(f,"        popq    %%rbp\n");
  fprintf(f,"        retq\n");
  fprintf(f,"        .size   Balloc, .-Balloc\n\n");

  fprintf(f,"        .type   d2b, @function\n");
  fprintf(f,"d2b:\n");
  fprintf(f,"        movabsq $Balloc, %%rax\n");
  fprintf(f,"        pushq   %%rbp\n");
  fprintf(f,"        movq    %%rsp, %%rbp\n");
  fprintf(f,"        pushq   %%r15\n");
  fprintf(f,"        movq    %%rdi, %%r15\n");
  fprintf(f,"        pushq   %%r14\n");
  fprintf(f,"        movl    $1, %%edi\n");
  fprintf(f,"        pushq   %%r13\n");
  fprintf(f,"        movq    %%rsi, %%r13\n");
  fprintf(f,"        pushq   %%r12\n");
  fprintf(f,"        movq    %%rdx, %%r12\n");
  fprintf(f,"        pushq   %%rbx\n");
  fprintf(f,"        subq    $24, %%rsp\n");
  fprintf(f,"        callq   *%%rax\n");
  fprintf(f,"        movl    4(%%r15), %%r14d\n");
  fprintf(f,"        movq    %%rax, %%rbx\n");
  fprintf(f,"        movl    %%r14d, %%eax\n");
  fprintf(f,"        andl    $2147483647, %%r14d\n");
  fprintf(f,"        andl    $1048575, %%eax\n");
  fprintf(f,"        movl    %%r14d, 4(%%r15)\n");
  fprintf(f,"        shrl    $20, %%r14d\n");
  fprintf(f,"        movl    %%eax, -52(%%rbp)\n");
  fprintf(f,"        je      0f\n");
  fprintf(f,"        orl     $1048576, %%eax\n");
  fprintf(f,"        movl    %%eax, -52(%%rbp)\n");
  fprintf(f,"0:\n");
  fprintf(f,"        movl    (%%r15), %%eax\n");
  fprintf(f,"        testl   %%eax, %%eax\n");
  fprintf(f,"        movl    %%eax, -56(%%rbp)\n");
  fprintf(f,"        movabsq $lo0bits, %%rax\n");
  fprintf(f,"        je      3f\n");
  fprintf(f,"        leaq    -56(%%rbp), %%rdi\n");
  fprintf(f,"        callq   *%%rax\n");
  fprintf(f,"        testl   %%eax, %%eax\n");
  fprintf(f,"        je      1f\n");
  fprintf(f,"        movl    -52(%%rbp), %%edx\n");
  fprintf(f,"        movl    $32, %%ecx\n");
  fprintf(f,"        subl    %%eax, %%ecx\n");
  fprintf(f,"        movl    %%edx, %%esi\n");
  fprintf(f,"        sall    %%cl, %%esi\n");
  fprintf(f,"        movl    -56(%%rbp), %%ecx\n");
  fprintf(f,"        orl     %%esi, %%ecx\n");
  fprintf(f,"        movl    %%ecx, 24(%%rbx)\n");
  fprintf(f,"        movb    %%al, %%cl\n");
  fprintf(f,"        shrl    %%cl, %%edx\n");
  fprintf(f,"        movl    %%edx, -52(%%rbp)\n");
  fprintf(f,"        jmp     2f\n");
  fprintf(f,"1:\n");
  fprintf(f,"        movl    -56(%%rbp), %%edx\n");
  fprintf(f,"        movl    %%edx, 24(%%rbx)\n");
  fprintf(f,"2:\n");
  fprintf(f,"        movl    -52(%%rbp), %%edx\n");
  fprintf(f,"        cmpl    $1, %%edx\n");
  fprintf(f,"        movl    %%edx, 28(%%rbx)\n");
  fprintf(f,"        sbbl    %%r15d, %%r15d\n");
  fprintf(f,"        addl    $2, %%r15d\n");
  fprintf(f,"        movl    %%r15d, 20(%%rbx)\n");
  fprintf(f,"        jmp     4f\n");
  fprintf(f,"3:\n");
  fprintf(f,"        leaq    -52(%%rbp), %%rdi\n");
  fprintf(f,"        movl    $1, %%r15d\n");
  fprintf(f,"        callq   *%%rax\n");
  fprintf(f,"        movl    -52(%%rbp), %%edx\n");
  fprintf(f,"        movl    $1, 20(%%rbx)\n");
  fprintf(f,"        addl    $32, %%eax\n");
  fprintf(f,"        movl    %%edx, 24(%%rbx)\n");
  fprintf(f,"4:\n");
  fprintf(f,"        testl   %%r14d, %%r14d\n");
  fprintf(f,"        je      5f\n");
  fprintf(f,"        leal    -1075(%%rax,%%r14), %%edx\n");
  fprintf(f,"        movl    %%edx, 0(%%r13)\n");
  fprintf(f,"        movl    $53, %%edx\n");
  fprintf(f,"        subl    %%eax, %%edx\n");
  fprintf(f,"        movl    %%edx, (%%r12)\n");
  fprintf(f,"        jmp     6f\n");
  fprintf(f,"5:\n");
  fprintf(f,"        subl    $1074, %%eax\n");
  fprintf(f,"        movl    %%eax, 0(%%r13)\n");
  fprintf(f,"        movslq  %%r15d, %%rax\n");
  fprintf(f,"        sall    $5, %%r15d\n");
  fprintf(f,"        movl    20(%%rbx,%%rax,4), %%edi\n");
  fprintf(f,"        movabsq $hi0bits, %%rax\n");
  fprintf(f,"        callq   *%%rax\n");
  fprintf(f,"        subl    %%eax, %%r15d\n");
  fprintf(f,"        movl    %%r15d, (%%r12)\n");
  fprintf(f,"6:\n");
  fprintf(f,"        addq    $24, %%rsp\n");
  fprintf(f,"        movq    %%rbx, %%rax\n");
  fprintf(f,"        popq    %%rbx\n");
  fprintf(f,"        popq    %%r12\n");
  fprintf(f,"        popq    %%r13\n");
  fprintf(f,"        popq    %%r14\n");
  fprintf(f,"        popq    %%r15\n");
  fprintf(f,"        popq    %%rbp\n");
  fprintf(f,"        retq\n");
  fprintf(f,"        .size   d2b, .-d2b\n\n");

  fprintf(f,"        .type   i2b, @function\n");
  fprintf(f,"i2b:\n");
  fprintf(f,"        movabsq $Balloc, %%rax\n");
  fprintf(f,"        pushq   %%rbp\n");
  fprintf(f,"        movq    %%rsp, %%rbp\n");
  fprintf(f,"        pushq   %%rbx\n");
  fprintf(f,"        movl    %%edi, %%ebx\n");
  fprintf(f,"        pushq   %%r9\n");
  fprintf(f,"        movl    $1, %%edi\n");
  fprintf(f,"        callq   *%%rax\n");
  fprintf(f,"        movl    %%ebx, 24(%%rax)\n");
  fprintf(f,"        movl    $1, 20(%%rax)\n");
  fprintf(f,"        popq    %%r10\n");
  fprintf(f,"        popq    %%rbx\n");
  fprintf(f,"        popq    %%rbp\n");
  fprintf(f,"        retq\n");
  fprintf(f,"        .size   i2b, .-i2b\n\n");

  fprintf(f,"        .type   mult, @function\n");
  fprintf(f,"mult:\n");
  fprintf(f,"        pushq   %%rbp\n");
  fprintf(f,"        movq    %%rsp, %%rbp\n");
  fprintf(f,"        pushq   %%r15\n");
  fprintf(f,"        pushq   %%r14\n");
  fprintf(f,"        pushq   %%r13\n");
  fprintf(f,"        movq    %%rsi, %%r13\n");
  fprintf(f,"        pushq   %%r12\n");
  fprintf(f,"        pushq   %%rbx\n");
  fprintf(f,"        pushq   %%rbx\n");
  fprintf(f,"        movq    %%rdi, %%rbx\n");
  fprintf(f,"        movl    20(%%rsi), %%eax\n");
  fprintf(f,"        cmpl    %%eax, 20(%%rdi)\n");
  fprintf(f,"        jge     0f\n");
  fprintf(f,"        movq    %%rsi, %%rbx\n");
  fprintf(f,"        movq    %%rdi, %%r13\n");
  fprintf(f,"0:\n");
  fprintf(f,"        movslq  20(%%rbx), %%r15\n");
  fprintf(f,"        xorl    %%edi, %%edi\n");
  fprintf(f,"        movabsq $Balloc, %%rax\n");
  fprintf(f,"        movslq  20(%%r13), %%r14\n");
  fprintf(f,"        leal    (%%r15,%%r14), %%r12d\n");
  fprintf(f,"        cmpl    12(%%rbx), %%r12d\n");
  fprintf(f,"        setg    %%dil\n");
  fprintf(f,"        addl    8(%%rbx), %%edi\n");
  fprintf(f,"        callq   *%%rax\n");
  fprintf(f,"        movslq  %%r12d, %%rcx\n");
  fprintf(f,"        leaq    24(%%rax), %%rdx\n");
  fprintf(f,"        leaq    (%%rdx,%%rcx,4), %%rcx\n");
  fprintf(f,"        movq    %%rdx, %%rsi\n");
  fprintf(f,"1:\n");
  fprintf(f,"        cmpq    %%rcx, %%rsi\n");
  fprintf(f,"        jae     2f\n");
  fprintf(f,"        movl    $0, (%%rsi)\n");
  fprintf(f,"        addq    $4, %%rsi\n");
  fprintf(f,"        jmp     1b\n");
  fprintf(f,"2:\n");
  fprintf(f,"        addq    $24, %%rbx\n");
  fprintf(f,"        addq    $24, %%r13\n");
  fprintf(f,"        leaq    (%%rbx,%%r15,4), %%r11\n");
  fprintf(f,"        leaq    0(%%r13,%%r14,4), %%r14\n");
  fprintf(f,"3:\n");
  fprintf(f,"        cmpq    %%r14, %%r13\n");
  fprintf(f,"        jae     8f\n");
  fprintf(f,"        addq    $4, %%r13\n");
  fprintf(f,"        movl    -4(%%r13), %%r9d\n");
  fprintf(f,"        testl   %%r9d, %%r9d\n");
  fprintf(f,"        je      6f\n");
  fprintf(f,"        movq    %%rdx, %%rsi\n");
  fprintf(f,"        movq    %%rbx, %%rdi\n");
  fprintf(f,"        xorl    %%r8d, %%r8d\n");
  fprintf(f,"        movl    %%r9d, %%r15d\n");
  fprintf(f,"4:\n");
  fprintf(f,"        addq    $4, %%rdi\n");
  fprintf(f,"        movl    -4(%%rdi), %%r10d\n");
  fprintf(f,"        movl    (%%rsi), %%r9d\n");
  fprintf(f,"        imulq   %%r15, %%r10\n");
  fprintf(f,"        addq    %%r9, %%r10\n");
  fprintf(f,"        leaq    4(%%rsi), %%r9\n");
  fprintf(f,"        addq    %%r8, %%r10\n");
  fprintf(f,"        movq    %%r10, %%r8\n");
  fprintf(f,"        movl    %%r10d, (%%rsi)\n");
  fprintf(f,"        shrq    $32, %%r8\n");
  fprintf(f,"        cmpq    %%r11, %%rdi\n");
  fprintf(f,"        jae     5f\n");
  fprintf(f,"        movq    %%r9, %%rsi\n");
  fprintf(f,"        jmp     4b\n");
  fprintf(f,"5:\n");
  fprintf(f,"        movl    %%r8d, 4(%%rsi)\n");
  fprintf(f,"6:\n");
  fprintf(f,"        addq    $4, %%rdx\n");
  fprintf(f,"        jmp     3b\n");
  fprintf(f,"7:\n");
  fprintf(f,"        subq    $4, %%rcx\n");
  fprintf(f,"        cmpl    $0, (%%rcx)\n");
  fprintf(f,"        jne     9f\n");
  fprintf(f,"        decl    %%r12d\n");
  fprintf(f,"8:\n");
  fprintf(f,"        testl   %%r12d, %%r12d\n");
  fprintf(f,"        jg      7b\n");
  fprintf(f,"9:\n");
  fprintf(f,"        movl    %%r12d, 20(%%rax)\n");
  fprintf(f,"        popq    %%r11\n");
  fprintf(f,"        popq    %%rbx\n");
  fprintf(f,"        popq    %%r12\n");
  fprintf(f,"        popq    %%r13\n");
  fprintf(f,"        popq    %%r14\n");
  fprintf(f,"        popq    %%r15\n");
  fprintf(f,"        popq    %%rbp\n");
  fprintf(f,"        retq\n");
  fprintf(f,"        .size   mult, .-mult\n\n");

  fprintf(f,"        .type   diff, @function\n");
  fprintf(f,"diff:\n");
  fprintf(f,"        movabsq $cmp, %%rax\n");
  fprintf(f,"        pushq   %%rbp\n");
  fprintf(f,"        movq    %%rsp, %%rbp\n");
  fprintf(f,"        pushq   %%r13\n");
  fprintf(f,"        pushq   %%r12\n");
  fprintf(f,"        movq    %%rsi, %%r12\n");
  fprintf(f,"        pushq   %%rbx\n");
  fprintf(f,"        movq    %%rdi, %%rbx\n");
  fprintf(f,"        pushq   %%r13\n");
  fprintf(f,"        callq   *%%rax\n");
  fprintf(f,"        testl   %%eax, %%eax\n");
  fprintf(f,"        movabsq $Balloc, %%rax\n");
  fprintf(f,"        jne     0f\n");
  fprintf(f,"        xorl    %%edi, %%edi\n");
  fprintf(f,"        callq   *%%rax\n");
  fprintf(f,"        movl    $1, 20(%%rax)\n");
  fprintf(f,"        movl    $0, 24(%%rax)\n");
  fprintf(f,"        jmp     7f\n");
  fprintf(f,"0:\n");
  fprintf(f,"        js      1f\n");
  fprintf(f,"        xorl    %%r13d, %%r13d\n");
  fprintf(f,"        jmp     2f\n");
  fprintf(f,"1:\n");
  fprintf(f,"        movq    %%rbx, %%rdx\n");
  fprintf(f,"        movl    $1, %%r13d\n");
  fprintf(f,"        movq    %%r12, %%rbx\n");
  fprintf(f,"        movq    %%rdx, %%r12\n");
  fprintf(f,"2:\n");
  fprintf(f,"        movl    8(%%rbx), %%edi\n");
  fprintf(f,"        callq   *%%rax\n");
  fprintf(f,"        movl    20(%%rbx), %%r9d\n");
  fprintf(f,"        leaq    24(%%rbx), %%rdi\n");
  fprintf(f,"        leaq    24(%%r12), %%r8\n");
  fprintf(f,"        movl    %%r13d, 16(%%rax)\n");
  fprintf(f,"        xorl    %%ecx, %%ecx\n");
  fprintf(f,"        movslq  %%r9d, %%rdx\n");
  fprintf(f,"        leaq    (%%rdi,%%rdx,4), %%r10\n");
  fprintf(f,"        movslq  20(%%r12), %%rdx\n");
  fprintf(f,"        leaq    (%%r8,%%rdx,4), %%r11\n");
  fprintf(f,"        leaq    24(%%rax), %%rdx\n");
  fprintf(f,"3:\n");
  fprintf(f,"        addq    $4, %%rdi\n");
  fprintf(f,"        addq    $4, %%r8\n");
  fprintf(f,"        movl    -4(%%rdi), %%esi\n");
  fprintf(f,"        movl    -4(%%r8), %%ebx\n");
  fprintf(f,"        addq    $4, %%rdx\n");
  fprintf(f,"        subq    %%rbx, %%rsi\n");
  fprintf(f,"        subq    %%rcx, %%rsi\n");
  fprintf(f,"        movq    %%rsi, %%rcx\n");
  fprintf(f,"        movl    %%esi, -4(%%rdx)\n");
  fprintf(f,"        shrq    $32, %%rcx\n");
  fprintf(f,"        andl    $1, %%ecx\n");
  fprintf(f,"        cmpq    %%r11, %%r8\n");
  fprintf(f,"        jb      3b\n");
  fprintf(f,"4:\n");
  fprintf(f,"        cmpq    %%r10, %%rdi\n");
  fprintf(f,"        jae     5f\n");
  fprintf(f,"        addq    $4, %%rdi\n");
  fprintf(f,"        movl    -4(%%rdi), %%esi\n");
  fprintf(f,"        addq    $4, %%rdx\n");
  fprintf(f,"        subq    %%rcx, %%rsi\n");
  fprintf(f,"        movq    %%rsi, %%rcx\n");
  fprintf(f,"        movl    %%esi, -4(%%rdx)\n");
  fprintf(f,"        shrq    $32, %%rcx\n");
  fprintf(f,"        andl    $1, %%ecx\n");
  fprintf(f,"        jmp     4b\n");
  fprintf(f,"5:\n");
  fprintf(f,"        subq    $4, %%rdx\n");
  fprintf(f,"        cmpl    $0, (%%rdx)\n");
  fprintf(f,"        jne     6f\n");
  fprintf(f,"        decl    %%r9d\n");
  fprintf(f,"        jmp     5b\n");
  fprintf(f,"6:\n");
  fprintf(f,"        movl    %%r9d, 20(%%rax)\n");
  fprintf(f,"7:\n");
  fprintf(f,"        popq    %%r12\n");
  fprintf(f,"        popq    %%rbx\n");
  fprintf(f,"        popq    %%r12\n");
  fprintf(f,"        popq    %%r13\n");
  fprintf(f,"        popq    %%rbp\n");
  fprintf(f,"        retq\n");
  fprintf(f,"        .size   diff, .-diff\n\n");

  fprintf(f,"        .type   rv_alloc, @function\n");
  fprintf(f,"rv_alloc:\n");
  fprintf(f,"        pushq   %%rbp\n");
  fprintf(f,"        movl    $4, %%eax\n");
  fprintf(f,"        movslq  %%edi, %%rdi\n");
  fprintf(f,"        movq    %%rsp, %%rbp\n");
  fprintf(f,"        pushq   %%rbx\n");
  fprintf(f,"        xorl    %%ebx, %%ebx\n");
  fprintf(f,"        pushq   %%rcx\n");
  fprintf(f,"0:\n");
  fprintf(f,"        movslq  %%eax, %%rdx\n");
  fprintf(f,"        addq    $24, %%rdx\n");
  fprintf(f,"        cmpq    %%rdi, %%rdx\n");
  fprintf(f,"        ja      1f\n");
  fprintf(f,"        incl    %%ebx\n");
  fprintf(f,"        addl    %%eax, %%eax\n");
  fprintf(f,"        jmp     0b\n");
  fprintf(f,"1:\n");
  fprintf(f,"        movabsq $Balloc, %%rax\n");
  fprintf(f,"        movl    %%ebx, %%edi\n");
  fprintf(f,"        callq   *%%rax\n");
  fprintf(f,"        movl    %%ebx, (%%rax)\n");
  fprintf(f,"        addq    $4, %%rax\n");
  fprintf(f,"        movabsq %%rax, dtoa_result\n");
  fprintf(f,"        popq    %%rdx\n");
  fprintf(f,"        popq    %%rbx\n");
  fprintf(f,"        popq    %%rbp\n");
  fprintf(f,"        retq\n");
  fprintf(f,"        .size   rv_alloc, .-rv_alloc\n\n");

  fprintf(f,"        .type   nrv_alloc, @function\n");
  fprintf(f,"nrv_alloc:\n");
  fprintf(f,"        movabsq $rv_alloc, %%rax\n");
  fprintf(f,"        pushq   %%rbp\n");
  fprintf(f,"        movq    %%rsp, %%rbp\n");
  fprintf(f,"        pushq   %%r12\n");
  fprintf(f,"        movq    %%rsi, %%r12\n");
  fprintf(f,"        pushq   %%rbx\n");
  fprintf(f,"        movq    %%rdi, %%rbx\n");
  fprintf(f,"        movl    %%edx, %%edi\n");
  fprintf(f,"        callq   *%%rax\n");
  fprintf(f,"        movq    %%rax, %%rdx\n");
  fprintf(f,"0:\n");
  fprintf(f,"        incq    %%rbx\n");
  fprintf(f,"        movb    -1(%%rbx), %%cl\n");
  fprintf(f,"        testb   %%cl, %%cl\n");
  fprintf(f,"        movb    %%cl, (%%rdx)\n");
  fprintf(f,"        je      1f\n");
  fprintf(f,"        incq    %%rdx\n");
  fprintf(f,"        jmp     0b\n");
  fprintf(f,"1:\n");
  fprintf(f,"        testq   %%r12, %%r12\n");
  fprintf(f,"        je      2f\n");
  fprintf(f,"        movq    %%rdx, (%%r12)\n");
  fprintf(f,"2:\n");
  fprintf(f,"        popq    %%rbx\n");
  fprintf(f,"        popq    %%r12\n");
  fprintf(f,"        popq    %%rbp\n");
  fprintf(f,"        retq\n");
  fprintf(f,"        .size   nrv_alloc, .-nrv_alloc\n\n");

  fprintf(f,"        .type   Bfree, @function\n");
  fprintf(f,"Bfree:\n");
  fprintf(f,"        testq   %%rdi, %%rdi\n");
  fprintf(f,"        je      1f\n");
  fprintf(f,"        movslq  8(%%rdi), %%rdx\n");
  fprintf(f,"        cmpl    $7, %%edx\n");
  fprintf(f,"        jle     0f\n");
  fprintf(f,"        movabsq $badxit, %%rax\n");
  fprintf(f,"        pushq   %%rbp\n");
  fprintf(f,"        movq    %%rsp, %%rbp\n");
  fprintf(f,"        callq   *%%rax\n");
  fprintf(f,"0:\n");
  fprintf(f,"        movabsq $freelist, %%rax\n");
  fprintf(f,"        movq    (%%rax,%%rdx,8), %%rcx\n");
  fprintf(f,"        movq    %%rdi, (%%rax,%%rdx,8)\n");
  fprintf(f,"        movq    %%rcx, (%%rdi)\n");
  fprintf(f,"1:\n");
  fprintf(f,"        retq\n");
  fprintf(f,"        .size   Bfree, .-Bfree\n\n");

  fprintf(f,"        .type   lshift, @function\n");
  fprintf(f,"lshift:\n");
  fprintf(f,"        pushq   %%rbp\n");
  fprintf(f,"        movq    %%rsp, %%rbp\n");
  fprintf(f,"        pushq   %%r15\n");
  fprintf(f,"        pushq   %%r14\n");
  fprintf(f,"        movl    %%esi, %%r14d\n");
  fprintf(f,"        pushq   %%r13\n");
  fprintf(f,"        movl    %%esi, %%r13d\n");
  fprintf(f,"        pushq   %%r12\n");
  fprintf(f,"        sarl    $5, %%r13d\n");
  fprintf(f,"        pushq   %%rbx\n");
  fprintf(f,"        movq    %%rdi, %%rbx\n");
  fprintf(f,"        subq    $24, %%rsp\n");
  fprintf(f,"        movl    20(%%rbx), %%r15d\n");
  fprintf(f,"        movl    8(%%rdi), %%edi\n");
  fprintf(f,"        movl    12(%%rbx), %%eax\n");
  fprintf(f,"        addl    %%r13d, %%r15d\n");
  fprintf(f,"        leal    1(%%r15), %%r12d\n");
  fprintf(f,"0:\n");
  fprintf(f,"        cmpl    %%eax, %%r12d\n");
  fprintf(f,"        jle     1f\n");
  fprintf(f,"        incl    %%edi\n");
  fprintf(f,"        addl    %%eax, %%eax\n");
  fprintf(f,"        jmp     0b\n");
  fprintf(f,"1:\n");
  fprintf(f,"        movabsq $Balloc, %%rax\n");
  fprintf(f,"        callq   *%%rax\n");
  fprintf(f,"        xorl    %%edx, %%edx\n");
  fprintf(f,"        leaq    24(%%rax), %%rcx\n");
  fprintf(f,"2:\n");
  fprintf(f,"        cmpl    %%edx, %%r13d\n");
  fprintf(f,"        jle     3f\n");
  fprintf(f,"        movl    $0, (%%rcx,%%rdx,4)\n");
  fprintf(f,"        incq    %%rdx\n");
  fprintf(f,"        jmp     2b\n");
  fprintf(f,"3:\n");
  fprintf(f,"        xorl    %%edx, %%edx\n");
  fprintf(f,"        testl   %%r13d, %%r13d\n");
  fprintf(f,"        cmovns  %%r13d, %%edx\n");
  fprintf(f,"        andl    $31, %%r14d\n");
  fprintf(f,"        movslq  %%edx, %%rdx\n");
  fprintf(f,"        leaq    (%%rcx,%%rdx,4), %%rdi\n");
  fprintf(f,"        movslq  20(%%rbx), %%rcx\n");
  fprintf(f,"        leaq    24(%%rbx), %%rdx\n");
  fprintf(f,"        leaq    (%%rdx,%%rcx,4), %%r10\n");
  fprintf(f,"        je      6f\n");
  fprintf(f,"        movl    $32, %%esi\n");
  fprintf(f,"        xorl    %%r8d, %%r8d\n");
  fprintf(f,"        subl    %%r14d, %%esi\n");
  fprintf(f,"4:\n");
  fprintf(f,"        movl    (%%rdx), %%r9d\n");
  fprintf(f,"        movb    %%r14b, %%cl\n");
  fprintf(f,"        addq    $4, %%rdx\n");
  fprintf(f,"        leaq    4(%%rdi), %%r11\n");
  fprintf(f,"        sall    %%cl, %%r9d\n");
  fprintf(f,"        movb    %%sil, %%cl\n");
  fprintf(f,"        orl     %%r8d, %%r9d\n");
  fprintf(f,"        movl    %%r9d, (%%rdi)\n");
  fprintf(f,"        movl    -4(%%rdx), %%r8d\n");
  fprintf(f,"        shrl    %%cl, %%r8d\n");
  fprintf(f,"        cmpq    %%r10, %%rdx\n");
  fprintf(f,"        jae     5f\n");
  fprintf(f,"        movq    %%r11, %%rdi\n");
  fprintf(f,"        jmp     4b\n");
  fprintf(f,"5:\n");
  fprintf(f,"        addl    $2, %%r15d\n");
  fprintf(f,"        testl   %%r8d, %%r8d\n");
  fprintf(f,"        movl    %%r8d, 4(%%rdi)\n");
  fprintf(f,"        cmovne  %%r15d, %%r12d\n");
  fprintf(f,"        jmp     7f\n");
  fprintf(f,"6:\n");
  fprintf(f,"        addl    $2, %%r15d\n");
  fprintf(f,"        addq    $4, %%rdx\n");
  fprintf(f,"        movl    -4(%%rdx), %%ecx\n");
  fprintf(f,"        addq    $4, %%rdi\n");
  fprintf(f,"        cmpq    %%r10, %%rdx\n");
  fprintf(f,"        movl    %%ecx, -4(%%rdi)\n");
  fprintf(f,"        jb      6b\n");
  fprintf(f,"7:\n");
  fprintf(f,"        addl    $2, %%r15d\n");
  fprintf(f,"        decl    %%r12d\n");
  fprintf(f,"        movq    %%rbx, %%rdi\n");
  fprintf(f,"        movq    %%rax, -56(%%rbp)\n");
  fprintf(f,"        movabsq $Bfree, %%rdx\n");
  fprintf(f,"        movl    %%r12d, 20(%%rax)\n");
  fprintf(f,"        callq   *%%rdx\n");
  fprintf(f,"        movq    -56(%%rbp), %%rax\n");
  fprintf(f,"        addq    $24, %%rsp\n");
  fprintf(f,"        popq    %%rbx\n");
  fprintf(f,"        popq    %%r12\n");
  fprintf(f,"        popq    %%r13\n");
  fprintf(f,"        popq    %%r14\n");
  fprintf(f,"        popq    %%r15\n");
  fprintf(f,"        popq    %%rbp\n");
  fprintf(f,"        retq\n");
  fprintf(f,"        .size   lshift, .-lshift\n\n");

  fprintf(f,"        .type   increment, @function\n");
  fprintf(f,"increment:\n");
  fprintf(f,"        pushq   %%rbp\n");
  fprintf(f,"        leaq    24(%%rdi), %%rax\n");
  fprintf(f,"        movq    %%rsp, %%rbp\n");
  fprintf(f,"        pushq   %%r12\n");
  fprintf(f,"        pushq   %%rbx\n");
  fprintf(f,"        movq    %%rdi, %%rbx\n");
  fprintf(f,"        movslq  20(%%rdi), %%rdx\n");
  fprintf(f,"        leaq    (%%rax,%%rdx,4), %%rcx\n");
  fprintf(f,"0:\n");
  fprintf(f,"        movl    (%%rax), %%edx\n");
  fprintf(f,"        cmpl    $-1, %%edx\n");
  fprintf(f,"        je      1f\n");
  fprintf(f,"        incl    %%edx\n");
  fprintf(f,"        movl    %%edx, (%%rax)\n");
  fprintf(f,"        jmp     3f\n");
  fprintf(f,"1:\n");
  fprintf(f,"        addq    $4, %%rax\n");
  fprintf(f,"        movl    $0, -4(%%rax)\n");
  fprintf(f,"        cmpq    %%rcx, %%rax\n");
  fprintf(f,"        jb      0b\n");
  fprintf(f,"        movl    12(%%rbx), %%eax\n");
  fprintf(f,"        cmpl    %%eax, 20(%%rbx)\n");
  fprintf(f,"        jl      2f\n");
  fprintf(f,"        movl    8(%%rbx), %%eax\n");
  fprintf(f,"        leal    1(%%rax), %%edi\n");
  fprintf(f,"        movabsq $Balloc, %%rax\n");
  fprintf(f,"        callq   *%%rax\n");
  fprintf(f,"        leaq    16(%%rbx), %%rsi\n");
  fprintf(f,"        movq    %%rax, %%r12\n");
  fprintf(f,"        movslq  20(%%rbx), %%rax\n");
  fprintf(f,"        leaq    16(%%r12), %%rdi\n");
  fprintf(f,"        leaq    8(,%%rax,4), %%rdx\n");
  fprintf(f,"        movabsq $memcpy, %%rax\n");
  fprintf(f,"        callq   *%%rax\n");
  fprintf(f,"        movq    %%rbx, %%rdi\n");
  fprintf(f,"        movq    %%r12, %%rbx\n");
  fprintf(f,"        movabsq $Bfree, %%rax\n");
  fprintf(f,"        callq   *%%rax\n");
  fprintf(f,"2:\n");
  fprintf(f,"        movslq  20(%%rbx), %%rax\n");
  fprintf(f,"        leal    1(%%rax), %%edx\n");
  fprintf(f,"        movl    %%edx, 20(%%rbx)\n");
  fprintf(f,"        movl    $1, 24(%%rbx,%%rax,4)\n");
  fprintf(f,"3:\n");
  fprintf(f,"        movq    %%rbx, %%rax\n");
  fprintf(f,"        popq    %%rbx\n");
  fprintf(f,"        popq    %%r12\n");
  fprintf(f,"        popq    %%rbp\n");
  fprintf(f,"        retq\n");
  fprintf(f,"        .size   increment, .-increment\n\n");

  fprintf(f,"        .type   multadd, @function\n");
  fprintf(f,"multadd:\n");
  fprintf(f,"        pushq   %%rbp\n");
  fprintf(f,"        leaq    24(%%rdi), %%rcx\n");
  fprintf(f,"        xorl    %%eax, %%eax\n");
  fprintf(f,"        movq    %%rsp, %%rbp\n");
  fprintf(f,"        pushq   %%r14\n");
  fprintf(f,"        movslq  %%esi, %%rsi\n");
  fprintf(f,"        pushq   %%r13\n");
  fprintf(f,"        movslq  %%edx, %%r13\n");
  fprintf(f,"        pushq   %%r12\n");
  fprintf(f,"        pushq   %%rbx\n");
  fprintf(f,"        movq    %%rdi, %%rbx\n");
  fprintf(f,"        movl    20(%%rdi), %%r12d\n");
  fprintf(f,"0:\n");
  fprintf(f,"        movl    (%%rcx,%%rax,4), %%edx\n");
  fprintf(f,"        imulq   %%rsi, %%rdx\n");
  fprintf(f,"        addq    %%r13, %%rdx\n");
  fprintf(f,"        movq    %%rdx, %%r13\n");
  fprintf(f,"        movl    %%edx, (%%rcx,%%rax,4)\n");
  fprintf(f,"        incq    %%rax\n");
  fprintf(f,"        shrq    $32, %%r13\n");
  fprintf(f,"        cmpl    %%eax, %%r12d\n");
  fprintf(f,"        jg      0b\n");
  fprintf(f,"        testq   %%r13, %%r13\n");
  fprintf(f,"        je      2f\n");
  fprintf(f,"        cmpl    12(%%rbx), %%r12d\n");
  fprintf(f,"        jl      1f\n");
  fprintf(f,"        movl    8(%%rbx), %%eax\n");
  fprintf(f,"        leal    1(%%rax), %%edi\n");
  fprintf(f,"        movabsq $Balloc, %%rax\n");
  fprintf(f,"        callq   *%%rax\n");
  fprintf(f,"        leaq    16(%%rbx), %%rsi\n");
  fprintf(f,"        movq    %%rax, %%r14\n");
  fprintf(f,"        movslq  20(%%rbx), %%rax\n");
  fprintf(f,"        leaq    16(%%r14), %%rdi\n");
  fprintf(f,"        leaq    8(,%%rax,4), %%rdx\n");
  fprintf(f,"        movabsq $memcpy, %%rax\n");
  fprintf(f,"        callq   *%%rax\n");
  fprintf(f,"        movq    %%rbx, %%rdi\n");
  fprintf(f,"        movq    %%r14, %%rbx\n");
  fprintf(f,"        movabsq $Bfree, %%rax\n");
  fprintf(f,"        callq   *%%rax\n");
  fprintf(f,"1:\n");
  fprintf(f,"        movslq  %%r12d, %%rax\n");
  fprintf(f,"        incl    %%r12d\n");
  fprintf(f,"        movl    %%r13d, 24(%%rbx,%%rax,4)\n");
  fprintf(f,"        movl    %%r12d, 20(%%rbx)\n");
  fprintf(f,"2:\n");
  fprintf(f,"        movq    %%rbx, %%rax\n");
  fprintf(f,"        popq    %%rbx\n");
  fprintf(f,"        popq    %%r12\n");
  fprintf(f,"        popq    %%r13\n");
  fprintf(f,"        popq    %%r14\n");
  fprintf(f,"        popq    %%rbp\n");
  fprintf(f,"        retq\n");
  fprintf(f,"        .size   multadd, .-multadd\n\n");

  fprintf(f,"        .type   s2b, @function\n");
  fprintf(f,"s2b:\n");
  fprintf(f,"        pushq   %%rbp\n");
  fprintf(f,"        leal    8(%%rdx), %%eax\n");
  fprintf(f,"        movq    %%rsp, %%rbp\n");
  fprintf(f,"        pushq   %%r15\n");
  fprintf(f,"        movq    %%rdi, %%r15\n");
  fprintf(f,"        pushq   %%r14\n");
  fprintf(f,"        movl    %%edx, %%r14d\n");
  fprintf(f,"        cltd\n");
  fprintf(f,"        pushq   %%r13\n");
  fprintf(f,"        xorl    %%edi, %%edi\n");
  fprintf(f,"        movl    %%ecx, %%r13d\n");
  fprintf(f,"        pushq   %%r12\n");
  fprintf(f,"        movslq  %%r8d, %%r12\n");
  fprintf(f,"        pushq   %%rbx\n");
  fprintf(f,"        movl    %%esi, %%ebx\n");
  fprintf(f,"        movl    $9, %%esi\n");
  fprintf(f,"        idivl   %%esi\n");
  fprintf(f,"        subq    $24, %%rsp\n");
  fprintf(f,"        movb    $1, %%sil\n");
  fprintf(f,"0:\n");
  fprintf(f,"        cmpl    %%esi, %%eax\n");
  fprintf(f,"        jle     1f\n");
  fprintf(f,"        addl    %%esi, %%esi\n");
  fprintf(f,"        incl    %%edi\n");
  fprintf(f,"        jmp     0b\n");
  fprintf(f,"1:\n");
  fprintf(f,"        movabsq $Balloc, %%rax\n");
  fprintf(f,"        callq   *%%rax\n");
  fprintf(f,"        cmpl    $9, %%ebx\n");
  fprintf(f,"        movl    %%r13d, 24(%%rax)\n");
  fprintf(f,"        movl    $1, 20(%%rax)\n");
  fprintf(f,"        jle     3f\n");
  fprintf(f,"        leaq    9(%%r15), %%rcx\n");
  fprintf(f,"        xorl    %%r13d, %%r13d\n");
  fprintf(f,"        movabsq $multadd, %%r8\n");
  fprintf(f,"2:\n");
  fprintf(f,"        movsbl  9(%%r15,%%r13), %%edx\n");
  fprintf(f,"        movl    $10, %%esi\n");
  fprintf(f,"        incq    %%r13\n");
  fprintf(f,"        movq    %%rax, %%rdi\n");
  fprintf(f,"        movq    %%rcx, -64(%%rbp)\n");
  fprintf(f,"        movq    %%r8, -56(%%rbp)\n");
  fprintf(f,"        subl    $48, %%edx\n");
  fprintf(f,"        callq   *%%r8\n");
  fprintf(f,"        leal    9(%%r13), %%edx\n");
  fprintf(f,"        movq    -56(%%rbp), %%r8\n");
  fprintf(f,"        cmpl    %%ebx, %%edx\n");
  fprintf(f,"        movq    -64(%%rbp), %%rcx\n");
  fprintf(f,"        jl      2b\n");
  fprintf(f,"        leal    -10(%%rbx), %%edx\n");
  fprintf(f,"        leaq    1(%%rcx,%%rdx), %%rdx\n");
  fprintf(f,"        addq    %%rdx, %%r12\n");
  fprintf(f,"        jmp     4f\n");
  fprintf(f,"3:\n");
  fprintf(f,"        leaq    9(%%r15,%%r12), %%r12\n");
  fprintf(f,"        movl    $9, %%ebx\n");
  fprintf(f,"4:\n");
  fprintf(f,"        subl    %%r12d, %%ebx\n");
  fprintf(f,"5:\n");
  fprintf(f,"        leal    (%%rbx,%%r12), %%edx\n");
  fprintf(f,"        cmpl    %%r14d, %%edx\n");
  fprintf(f,"        jge     6f\n");
  fprintf(f,"        incq    %%r12\n");
  fprintf(f,"        movsbl  -1(%%r12), %%edx\n");
  fprintf(f,"        movq    %%rax, %%rdi\n");
  fprintf(f,"        movabsq $multadd, %%rax\n");
  fprintf(f,"        movl    $10, %%esi\n");
  fprintf(f,"        subl    $48, %%edx\n");
  fprintf(f,"        callq   *%%rax\n");
  fprintf(f,"        jmp     5b\n");
  fprintf(f,"6:\n");
  fprintf(f,"        addq    $24, %%rsp\n");
  fprintf(f,"        popq    %%rbx\n");
  fprintf(f,"        popq    %%r12\n");
  fprintf(f,"        popq    %%r13\n");
  fprintf(f,"        popq    %%r14\n");
  fprintf(f,"        popq    %%r15\n");
  fprintf(f,"        popq    %%rbp\n");
  fprintf(f,"        retq\n");
  fprintf(f,"        .size   s2b, .-s2b\n\n");

  fprintf(f,"        .type   pow5mult, @function\n");
  fprintf(f,"pow5mult:\n");
  fprintf(f,"        pushq   %%rbp\n");
  fprintf(f,"        movl    %%esi, %%edx\n");
  fprintf(f,"        movq    %%rsp, %%rbp\n");
  fprintf(f,"        pushq   %%r15\n");
  fprintf(f,"        andl    $3, %%edx\n");
  fprintf(f,"        pushq   %%r14\n");
  fprintf(f,"        pushq   %%r13\n");
  fprintf(f,"        movl    %%esi, %%r13d\n");
  fprintf(f,"        pushq   %%r12\n");
  fprintf(f,"        movq    %%rdi, %%r12\n");
  fprintf(f,"        pushq   %%rbx\n");
  fprintf(f,"        pushq   %%r8\n");
  fprintf(f,"        je      0f\n");
  fprintf(f,"        movabsq $p05.3208, %%rax\n");
  fprintf(f,"        decl    %%edx\n");
  fprintf(f,"        movslq  %%edx, %%rdx\n");
  fprintf(f,"        movl    (%%rax,%%rdx,4), %%esi\n");
  fprintf(f,"        xorl    %%edx, %%edx\n");
  fprintf(f,"        movabsq $multadd, %%rax\n");
  fprintf(f,"        callq   *%%rax\n");
  fprintf(f,"        movq    %%rax, %%r12\n");
  fprintf(f,"0:\n");
  fprintf(f,"        sarl    $2, %%r13d\n");
  fprintf(f,"        je      6f\n");
  fprintf(f,"        movabsq $p5s, %%rax\n");
  fprintf(f,"        movq    (%%rax), %%rbx\n");
  fprintf(f,"        testq   %%rbx, %%rbx\n");
  fprintf(f,"        jne     1f\n");
  fprintf(f,"        movabsq $i2b, %%rax\n");
  fprintf(f,"        movl    $625, %%edi\n");
  fprintf(f,"        callq   *%%rax\n");
  fprintf(f,"        movabsq %%rax, p5s\n");
  fprintf(f,"        movq    %%rax, %%rbx\n");
  fprintf(f,"        movq    $0, (%%rbx)\n");
  fprintf(f,"1:\n");
  fprintf(f,"        movabsq $mult, %%r14\n");
  fprintf(f,"2:\n");
  fprintf(f,"        testb   $1, %%r13b\n");
  fprintf(f,"        je      3f\n");
  fprintf(f,"        movq    %%r12, %%rdi\n");
  fprintf(f,"        movq    %%rbx, %%rsi\n");
  fprintf(f,"        callq   *%%r14\n");
  fprintf(f,"        movq    %%r12, %%rdi\n");
  fprintf(f,"        movq    %%rax, %%r15\n");
  fprintf(f,"        movabsq $Bfree, %%rax\n");
  fprintf(f,"        callq   *%%rax\n");
  fprintf(f,"        movq    %%r15, %%r12\n");
  fprintf(f,"3:\n");
  fprintf(f,"        sarl    %%r13d\n");
  fprintf(f,"        je      6f\n");
  fprintf(f,"        movq    (%%rbx), %%rax\n");
  fprintf(f,"        testq   %%rax, %%rax\n");
  fprintf(f,"        je      5f\n");
  fprintf(f,"4:\n");
  fprintf(f,"        movq    %%rax, %%rbx\n");
  fprintf(f,"        jmp     2b\n");
  fprintf(f,"5:\n");
  fprintf(f,"        movq    %%rbx, %%rsi\n");
  fprintf(f,"        movq    %%rbx, %%rdi\n");
  fprintf(f,"        callq   *%%r14\n");
  fprintf(f,"        movq    %%rax, (%%rbx)\n");
  fprintf(f,"        movq    $0, (%%rax)\n");
  fprintf(f,"        jmp     4b\n");
  fprintf(f,"6:\n");
  fprintf(f,"        popq    %%rsi\n");
  fprintf(f,"        movq    %%r12, %%rax\n");
  fprintf(f,"        popq    %%rbx\n");
  fprintf(f,"        popq    %%r12\n");
  fprintf(f,"        popq    %%r13\n");
  fprintf(f,"        popq    %%r14\n");
  fprintf(f,"        popq    %%r15\n");
  fprintf(f,"        popq    %%rbp\n");
  fprintf(f,"        retq\n");
  fprintf(f,"        .size   pow5mult, .-pow5mult\n\n");

  fprintf(f,"        .type   ulp.isra.0, @function\n");
  fprintf(f,"ulp.isra.0:\n");
  fprintf(f,"        andl    $2146435072, %%edi\n");
  fprintf(f,"        pushq   %%rbp\n");
  fprintf(f,"        subl    $54525952, %%edi\n");
  fprintf(f,"        movq    %%rsp, %%rbp\n");
  fprintf(f,"        testl   %%edi, %%edi\n");
  fprintf(f,"        jg      0f\n");
  fprintf(f,"        negl    %%edi\n");
  fprintf(f,"        sarl    $20, %%edi\n");
  fprintf(f,"        cmpl    $19, %%edi\n");
  fprintf(f,"        jg      1f\n");
  fprintf(f,"        movl    $524288, %%eax\n");
  fprintf(f,"        movb    %%dil, %%cl\n");
  fprintf(f,"        movl    %%eax, %%edx\n");
  fprintf(f,"        sarl    %%cl, %%edx\n");
  fprintf(f,"        movq    %%rdx, %%rdi\n");
  fprintf(f,"0:\n");
  fprintf(f,"        salq    $32, %%rdi\n");
  fprintf(f,"        jmp     3f\n");
  fprintf(f,"1:\n");
  fprintf(f,"        cmpl    $50, %%edi\n");
  fprintf(f,"        movl    $1, %%eax\n");
  fprintf(f,"        jg      2f\n");
  fprintf(f,"        movl    $51, %%ecx\n");
  fprintf(f,"        subl    %%edi, %%ecx\n");
  fprintf(f,"        sall    %%cl, %%eax\n");
  fprintf(f,"2:\n");
  fprintf(f,"        movl    %%eax, %%edi\n");
  fprintf(f,"3:\n");
  fprintf(f,"        movd    %%rdi, %%xmm0\n");
  fprintf(f,"        popq    %%rbp\n");
  fprintf(f,"        retq\n");
  fprintf(f,"        .size   ulp.isra.0, .-ulp.isra.0\n\n");

  fprintf(f,"        .type   gethex, @function\n");
  fprintf(f,"gethex:\n");
  fprintf(f,"        pushq   %%rbp\n");
  fprintf(f,"        movq    %%rsp, %%rbp\n");
  fprintf(f,"        pushq   %%r15\n");
  fprintf(f,"        pushq   %%r14\n");
  fprintf(f,"        pushq   %%r13\n");
  fprintf(f,"        pushq   %%r12\n");
  fprintf(f,"        pushq   %%rbx\n");
  fprintf(f,"        subq    $40, %%rsp\n");
  fprintf(f,"        movl    %%edx, -56(%%rbp)\n");
  fprintf(f,"        movq    (%%rdi), %%rax\n");
  fprintf(f,"        movq    %%rsi, -64(%%rbp)\n");
  fprintf(f,"        movl    %%ecx, -52(%%rbp)\n");
  fprintf(f,"        leaq    2(%%rax), %%rdx\n");
  fprintf(f,"        movq    %%rdx, %%rbx\n");
  fprintf(f,"0:\n");
  fprintf(f,"        movq    %%rbx, %%r14\n");
  fprintf(f,"        movl    %%ebx, %%r9d\n");
  fprintf(f,"        leaq    1(%%r14), %%rbx\n");
  fprintf(f,"        movb    -1(%%rbx), %%al\n");
  fprintf(f,"        subl    %%edx, %%r9d\n");
  fprintf(f,"        cmpb    $48, %%al\n");
  fprintf(f,"        je      0b\n");
  fprintf(f,"        movabsq $hexdig, %%rdx\n");
  fprintf(f,"        movzbl  %%al, %%ecx\n");
  fprintf(f,"        cmpb    $0, (%%rdx,%%rcx)\n");
  fprintf(f,"        je      2f\n");
  fprintf(f,"        incl    %%r9d\n");
  fprintf(f,"        xorl    %%r8d, %%r8d\n");
  fprintf(f,"        xorl    %%ebx, %%ebx\n");
  fprintf(f,"1:\n");
  fprintf(f,"        movq    %%r14, %%r13\n");
  fprintf(f,"        jmp     5f\n");
  fprintf(f,"2:\n");
  fprintf(f,"        cmpb    $46, %%al\n");
  fprintf(f,"        movq    %%r14, %%r13\n");
  fprintf(f,"        jne     10f\n");
  fprintf(f,"        movzbl  1(%%r14), %%eax\n");
  fprintf(f,"        movq    %%rbx, %%r13\n");
  fprintf(f,"        cmpb    $0, (%%rdx,%%rax)\n");
  fprintf(f,"        je      10f\n");
  fprintf(f,"        movq    %%rbx, %%r14\n");
  fprintf(f,"3:\n");
  fprintf(f,"        movzbl  (%%r14), %%eax\n");
  fprintf(f,"        cmpb    $48, %%al\n");
  fprintf(f,"        jne     4f\n");
  fprintf(f,"        incq    %%r14\n");
  fprintf(f,"        jmp     3b\n");
  fprintf(f,"4:\n");
  fprintf(f,"        xorl    %%r8d, %%r8d\n");
  fprintf(f,"        cmpb    $0, (%%rdx,%%rax)\n");
  fprintf(f,"        movl    $1, %%r9d\n");
  fprintf(f,"        sete    %%r8b\n");
  fprintf(f,"        jmp     1b\n");
  fprintf(f,"5:\n");
  fprintf(f,"        movb    0(%%r13), %%al\n");
  fprintf(f,"        movzbl  %%al, %%ecx\n");
  fprintf(f,"        cmpb    $0, (%%rdx,%%rcx)\n");
  fprintf(f,"        je      6f\n");
  fprintf(f,"        incq    %%r13\n");
  fprintf(f,"        jmp     5b\n");
  fprintf(f,"6:\n");
  fprintf(f,"        cmpb    $46, %%al\n");
  fprintf(f,"        jne     8f\n");
  fprintf(f,"        testq   %%rbx, %%rbx\n");
  fprintf(f,"        jne     9f\n");
  fprintf(f,"        leaq    1(%%r13), %%rbx\n");
  fprintf(f,"        movq    %%rbx, %%r13\n");
  fprintf(f,"7:\n");
  fprintf(f,"        movzbl  0(%%r13), %%eax\n");
  fprintf(f,"        cmpb    $0, (%%rdx,%%rax)\n");
  fprintf(f,"        je      8f\n");
  fprintf(f,"        incq    %%r13\n");
  fprintf(f,"        jmp     7b\n");
  fprintf(f,"8:\n");
  fprintf(f,"        testq   %%rbx, %%rbx\n");
  fprintf(f,"        je      11f\n");
  fprintf(f,"9:\n");
  fprintf(f,"        subl    %%r13d, %%ebx\n");
  fprintf(f,"        sall    $2, %%ebx\n");
  fprintf(f,"        jmp     12f\n");
  fprintf(f,"10:\n");
  fprintf(f,"        movl    $1, %%r8d\n");
  fprintf(f,"11:\n");
  fprintf(f,"        xorl    %%ebx, %%ebx\n");
  fprintf(f,"12:\n");
  fprintf(f,"        movb    0(%%r13), %%al\n");
  fprintf(f,"        cmpb    $80, %%al\n");
  fprintf(f,"        je      13f\n");
  fprintf(f,"        cmpb    $112, %%al\n");
  fprintf(f,"        jne     20f\n");
  fprintf(f,"13:\n");
  fprintf(f,"        movb    1(%%r13), %%al\n");
  fprintf(f,"        cmpb    $43, %%al\n");
  fprintf(f,"        je      14f\n");
  fprintf(f,"        cmpb    $45, %%al\n");
  fprintf(f,"        je      15f\n");
  fprintf(f,"        leaq    1(%%r13), %%rax\n");
  fprintf(f,"        xorl    %%ecx, %%ecx\n");
  fprintf(f,"        jmp     17f\n");
  fprintf(f,"14:\n");
  fprintf(f,"        xorl    %%ecx, %%ecx\n");
  fprintf(f,"        jmp     16f\n");
  fprintf(f,"15:\n");
  fprintf(f,"        movl    $1, %%ecx\n");
  fprintf(f,"16:\n");
  fprintf(f,"        leaq    2(%%r13), %%rax\n");
  fprintf(f,"17:\n");
  fprintf(f,"        movzbl  (%%rax), %%esi\n");
  fprintf(f,"        movzbl  (%%rdx,%%rsi), %%edx\n");
  fprintf(f,"        leal    -1(%%rdx), %%esi\n");
  fprintf(f,"        cmpb    $24, %%sil\n");
  fprintf(f,"        ja      21f\n");
  fprintf(f,"        subl    $16, %%edx\n");
  fprintf(f,"        xorl    %%esi, %%esi\n");
  fprintf(f,"        movl    $1, %%r15d\n");
  fprintf(f,"        movabsq $hexdig, %%r12\n");
  fprintf(f,"18:\n");
  fprintf(f,"        incq    %%rax\n");
  fprintf(f,"        movzbl  (%%rax), %%r10d\n");
  fprintf(f,"        movb    (%%r12,%%r10), %%r10b\n");
  fprintf(f,"        movzbl  %%r10b, %%r11d\n");
  fprintf(f,"        decl    %%r10d\n");
  fprintf(f,"        cmpb    $24, %%r10b\n");
  fprintf(f,"        ja      19f\n");
  fprintf(f,"        testl   $-134217728, %%edx\n");
  fprintf(f,"        cmovne  %%r15d, %%esi\n");
  fprintf(f,"        imull   $10, %%edx, %%edx\n");
  fprintf(f,"        leal    -16(%%r11,%%rdx), %%edx\n");
  fprintf(f,"        jmp     18b\n");
  fprintf(f,"19:\n");
  fprintf(f,"        movl    %%edx, %%r10d\n");
  fprintf(f,"        negl    %%r10d\n");
  fprintf(f,"        testl   %%ecx, %%ecx\n");
  fprintf(f,"        cmovne  %%r10d, %%edx\n");
  fprintf(f,"        addl    %%edx, %%ebx\n");
  fprintf(f,"        jmp     23f\n");
  fprintf(f,"20:\n");
  fprintf(f,"        movq    %%r13, %%rax\n");
  fprintf(f,"        xorl    %%ecx, %%ecx\n");
  fprintf(f,"        jmp     22f\n");
  fprintf(f,"21:\n");
  fprintf(f,"        movq    %%r13, %%rax\n");
  fprintf(f,"22:\n");
  fprintf(f,"        xorl    %%esi, %%esi\n");
  fprintf(f,"23:\n");
  fprintf(f,"        testl   %%r9d, %%r9d\n");
  fprintf(f,"        jne     24f\n");
  fprintf(f,"        leaq    -1(%%r14), %%rax\n");
  fprintf(f,"24:\n");
  fprintf(f,"        testl   %%r8d, %%r8d\n");
  fprintf(f,"        movq    %%rax, (%%rdi)\n");
  fprintf(f,"        jne     59f\n");
  fprintf(f,"        testl   %%esi, %%esi\n");
  fprintf(f,"        je      39f\n");
  fprintf(f,"        testl   %%ecx, %%ecx\n");
  fprintf(f,"        movl    -56(%%rbp), %%eax\n");
  fprintf(f,"        je      34f\n");
  fprintf(f,"        cmpl    $2, %%eax\n");
  fprintf(f,"        je      25f\n");
  fprintf(f,"        cmpl    $3, %%eax\n");
  fprintf(f,"        je      26f\n");
  fprintf(f,"        jmp     58f\n");
  fprintf(f,"25:\n");
  fprintf(f,"        cmpl    $0, -52(%%rbp)\n");
  fprintf(f,"        jne     58f\n");
  fprintf(f,"        jmp     33f\n");
  fprintf(f,"26:\n");
  fprintf(f,"        cmpl    $0, -52(%%rbp)\n");
  fprintf(f,"        je      58f\n");
  fprintf(f,"        jmp     33f\n");
  fprintf(f,"27:\n");
  fprintf(f,"        movabsq $Balloc, %%rax\n");
  fprintf(f,"        callq   *%%rax\n");
  fprintf(f,"        xorl    %%ecx, %%ecx\n");
  fprintf(f,"        xorl    %%edi, %%edi\n");
  fprintf(f,"        leaq    24(%%rax), %%r12\n");
  fprintf(f,"        movq    %%rax, %%r15\n");
  fprintf(f,"        movabsq $hexdig, %%r8\n");
  fprintf(f,"        movq    %%r12, %%rax\n");
  fprintf(f,"28:\n");
  fprintf(f,"        cmpq    %%r14, %%r13\n");
  fprintf(f,"        ja      41f\n");
  fprintf(f,"        leaq    4(%%rax), %%r14\n");
  fprintf(f,"        movl    %%edi, (%%rax)\n");
  fprintf(f,"        movabsq $hi0bits, %%r13\n");
  fprintf(f,"        subq    %%r12, %%r14\n");
  fprintf(f,"        sarq    $2, %%r14\n");
  fprintf(f,"        movl    %%r14d, 20(%%r15)\n");
  fprintf(f,"        sall    $5, %%r14d\n");
  fprintf(f,"        callq   *%%r13\n");
  fprintf(f,"        movq    %%r13, -72(%%rbp)\n");
  fprintf(f,"        subl    %%eax, %%r14d\n");
  fprintf(f,"        cmpl    $53, %%r14d\n");
  fprintf(f,"        jle     46f\n");
  fprintf(f,"        leal    -53(%%r14), %%r13d\n");
  fprintf(f,"        movq    %%r15, %%rdi\n");
  fprintf(f,"        movabsq $any_on, %%r9\n");
  fprintf(f,"        movl    %%r13d, %%esi\n");
  fprintf(f,"        movq    %%r9, -80(%%rbp)\n");
  fprintf(f,"        callq   *%%r9\n");
  fprintf(f,"        xorl    %%r8d, %%r8d\n");
  fprintf(f,"        movq    -80(%%rbp), %%r9\n");
  fprintf(f,"        testl   %%eax, %%eax\n");
  fprintf(f,"        jne     43f\n");
  fprintf(f,"29:\n");
  fprintf(f,"        movl    %%r13d, %%esi\n");
  fprintf(f,"        movq    %%r15, %%rdi\n");
  fprintf(f,"        movl    %%r8d, -80(%%rbp)\n");
  fprintf(f,"        movabsq $rshift, %%rax\n");
  fprintf(f,"        addl    %%r13d, %%ebx\n");
  fprintf(f,"        callq   *%%rax\n");
  fprintf(f,"30:\n");
  fprintf(f,"        movl    -80(%%rbp), %%r8d\n");
  fprintf(f,"31:\n");
  fprintf(f,"        movabsq $Bfree, %%r13\n");
  fprintf(f,"        cmpl    $1023, %%ebx\n");
  fprintf(f,"        jg      52f\n");
  fprintf(f,"        cmpl    $-1074, %%ebx\n");
  fprintf(f,"        jge     61f\n");
  fprintf(f,"        movl    $-1074, %%r9d\n");
  fprintf(f,"        subl    %%ebx, %%r9d\n");
  fprintf(f,"        cmpl    $52, %%r9d\n");
  fprintf(f,"        jle     47f\n");
  fprintf(f,"        movl    -56(%%rbp), %%eax\n");
  fprintf(f,"        cmpl    $2, %%eax\n");
  fprintf(f,"        jne     54f\n");
  fprintf(f,"        cmpl    $0, -52(%%rbp)\n");
  fprintf(f,"        jne     57f\n");
  fprintf(f,"32:\n");
  fprintf(f,"        movq    %%r15, %%rdi\n");
  fprintf(f,"        callq   *%%r13\n");
  fprintf(f,"33:\n");
  fprintf(f,"        movabsq $errno, %%rax\n");
  fprintf(f,"        movl    $34, (%%rax)\n");
  fprintf(f,"        movq    -64(%%rbp), %%rax\n");
  fprintf(f,"        movl    $0, 4(%%rax)\n");
  fprintf(f,"        movl    $1, (%%rax)\n");
  fprintf(f,"        jmp     71f\n");
  fprintf(f,"34:\n");
  fprintf(f,"        cmpl    $2, %%eax\n");
  fprintf(f,"        je      35f\n");
  fprintf(f,"        cmpl    $3, %%eax\n");
  fprintf(f,"        je      37f\n");
  fprintf(f,"        decl    %%eax\n");
  fprintf(f,"        jmp     36f\n");
  fprintf(f,"35:\n");
  fprintf(f,"        cmpl    $0, -52(%%rbp)\n");
  fprintf(f,"36:\n");
  fprintf(f,"        jne     38f\n");
  fprintf(f,"        jmp     53f\n");
  fprintf(f,"37:\n");
  fprintf(f,"        cmpl    $0, -52(%%rbp)\n");
  fprintf(f,"        jne     53f\n");
  fprintf(f,"38:\n");
  fprintf(f,"        movq    -64(%%rbp), %%rax\n");
  fprintf(f,"        movl    $2146435071, 4(%%rax)\n");
  fprintf(f,"        movl    $-1, (%%rax)\n");
  fprintf(f,"        jmp     71f\n");
  fprintf(f,"39:\n");
  fprintf(f,"        movq    %%r13, %%rax\n");
  fprintf(f,"        xorl    %%edi, %%edi\n");
  fprintf(f,"        subq    %%r14, %%rax\n");
  fprintf(f,"        decl    %%eax\n");
  fprintf(f,"40:\n");
  fprintf(f,"        cmpl    $7, %%eax\n");
  fprintf(f,"        jle     27b\n");
  fprintf(f,"        incl    %%edi\n");
  fprintf(f,"        sarl    %%eax\n");
  fprintf(f,"        jmp     40b\n");
  fprintf(f,"41:\n");
  fprintf(f,"        decq    %%r13\n");
  fprintf(f,"        cmpb    $46, 0(%%r13)\n");
  fprintf(f,"        je      28b\n");
  fprintf(f,"        cmpl    $32, %%ecx\n");
  fprintf(f,"        jne     42f\n");
  fprintf(f,"        movl    %%edi, (%%rax)\n");
  fprintf(f,"        xorb    %%cl, %%cl\n");
  fprintf(f,"        addq    $4, %%rax\n");
  fprintf(f,"        xorl    %%edi, %%edi\n");
  fprintf(f,"42:\n");
  fprintf(f,"        movzbl  0(%%r13), %%esi\n");
  fprintf(f,"        movb    (%%r8,%%rsi), %%sil\n");
  fprintf(f,"        andl    $15, %%esi\n");
  fprintf(f,"        sall    %%cl, %%esi\n");
  fprintf(f,"        addl    $4, %%ecx\n");
  fprintf(f,"        orl     %%esi, %%edi\n");
  fprintf(f,"        jmp     28b\n");
  fprintf(f,"43:\n");
  fprintf(f,"        leal    -54(%%r14), %%esi\n");
  fprintf(f,"        movl    $1, %%eax\n");
  fprintf(f,"        movb    $1, %%r8b\n");
  fprintf(f,"        movl    %%esi, %%edi\n");
  fprintf(f,"        movl    %%esi, %%ecx\n");
  fprintf(f,"        sarl    $5, %%edi\n");
  fprintf(f,"        sall    %%cl, %%eax\n");
  fprintf(f,"        movslq  %%edi, %%rdi\n");
  fprintf(f,"        testl   %%eax, (%%r12,%%rdi,4)\n");
  fprintf(f,"        je      29b\n");
  fprintf(f,"        testl   %%esi, %%esi\n");
  fprintf(f,"        jne     45f\n");
  fprintf(f,"44:\n");
  fprintf(f,"        movl    $2, %%r8d\n");
  fprintf(f,"        jmp     29b\n");
  fprintf(f,"45:\n");
  fprintf(f,"        movq    %%r15, %%rdi\n");
  fprintf(f,"        callq   *%%r9\n");
  fprintf(f,"        testl   %%eax, %%eax\n");
  fprintf(f,"        je      44b\n");
  fprintf(f,"        movl    $3, %%r8d\n");
  fprintf(f,"        jmp     29b\n");
  fprintf(f,"46:\n");
  fprintf(f,"        movl    $0, %%r8d\n");
  fprintf(f,"        je      31b\n");
  fprintf(f,"        movl    $53, %%r12d\n");
  fprintf(f,"        movq    %%r15, %%rdi\n");
  fprintf(f,"        movl    %%r8d, -80(%%rbp)\n");
  fprintf(f,"        movabsq $lshift, %%rax\n");
  fprintf(f,"        subl    %%r14d, %%r12d\n");
  fprintf(f,"        movl    %%r12d, %%esi\n");
  fprintf(f,"        subl    %%r12d, %%ebx\n");
  fprintf(f,"        callq   *%%rax\n");
  fprintf(f,"        leaq    24(%%rax), %%r12\n");
  fprintf(f,"        movq    %%rax, %%r15\n");
  fprintf(f,"        jmp     30b\n");
  fprintf(f,"47:\n");
  fprintf(f,"        movl    $-1075, %%r14d\n");
  fprintf(f,"        subl    %%ebx, %%r14d\n");
  fprintf(f,"        testl   %%r8d, %%r8d\n");
  fprintf(f,"        jne     60f\n");
  fprintf(f,"        testl   %%r14d, %%r14d\n");
  fprintf(f,"        je      48f\n");
  fprintf(f,"        movl    %%r14d, %%esi\n");
  fprintf(f,"        movq    %%r15, %%rdi\n");
  fprintf(f,"        movl    %%r9d, -80(%%rbp)\n");
  fprintf(f,"        movabsq $any_on, %%rax\n");
  fprintf(f,"        callq   *%%rax\n");
  fprintf(f,"        movl    -80(%%rbp), %%r9d\n");
  fprintf(f,"        movl    %%eax, %%r8d\n");
  fprintf(f,"48:\n");
  fprintf(f,"        movl    %%r14d, %%esi\n");
  fprintf(f,"        movl    %%r14d, %%ecx\n");
  fprintf(f,"        movl    $1, %%eax\n");
  fprintf(f,"        sarl    $5, %%esi\n");
  fprintf(f,"        sall    %%cl, %%eax\n");
  fprintf(f,"        movl    %%r8d, %%ecx\n");
  fprintf(f,"        movslq  %%esi, %%rsi\n");
  fprintf(f,"        orl     $2, %%ecx\n");
  fprintf(f,"        movq    %%r15, %%rdi\n");
  fprintf(f,"        testl   %%eax, (%%r12,%%rsi,4)\n");
  fprintf(f,"        leal    1127(%%rbx), %%r14d\n");
  fprintf(f,"        movl    %%r9d, %%esi\n");
  fprintf(f,"        movabsq $rshift, %%rax\n");
  fprintf(f,"        movl    $-1074, %%ebx\n");
  fprintf(f,"        cmovne  %%ecx, %%r8d\n");
  fprintf(f,"        movl    %%r8d, -80(%%rbp)\n");
  fprintf(f,"        callq   *%%rax\n");
  fprintf(f,"        movl    -80(%%rbp), %%r8d\n");
  fprintf(f,"        movl    $1, %%ecx\n");
  fprintf(f,"49:\n");
  fprintf(f,"        testl   %%r8d, %%r8d\n");
  fprintf(f,"        je      65f\n");
  fprintf(f,"        movl    -56(%%rbp), %%eax\n");
  fprintf(f,"        cmpl    $2, %%eax\n");
  fprintf(f,"        je      62f\n");
  fprintf(f,"        cmpl    $3, %%eax\n");
  fprintf(f,"        je      63f\n");
  fprintf(f,"        decl    %%eax\n");
  fprintf(f,"        jne     65f\n");
  fprintf(f,"        testb   $2, %%r8b\n");
  fprintf(f,"        je      65f\n");
  fprintf(f,"        orl     (%%r12), %%r8d\n");
  fprintf(f,"        andb    $1, %%r8b\n");
  fprintf(f,"50:\n");
  fprintf(f,"        je      65f\n");
  fprintf(f,"        movq    %%r15, %%rdi\n");
  fprintf(f,"        movl    %%ecx, -52(%%rbp)\n");
  fprintf(f,"        movslq  20(%%r15), %%r12\n");
  fprintf(f,"        movabsq $increment, %%rax\n");
  fprintf(f,"        callq   *%%rax\n");
  fprintf(f,"        movl    -52(%%rbp), %%ecx\n");
  fprintf(f,"        movq    %%rax, %%r15\n");
  fprintf(f,"        testl   %%ecx, %%ecx\n");
  fprintf(f,"        jne     66f\n");
  fprintf(f,"        cmpl    %%r12d, 20(%%rax)\n");
  fprintf(f,"        jle     64f\n");
  fprintf(f,"51:\n");
  fprintf(f,"        movl    $1, %%esi\n");
  fprintf(f,"        movq    %%r15, %%rdi\n");
  fprintf(f,"        incl    %%ebx\n");
  fprintf(f,"        movabsq $rshift, %%rax\n");
  fprintf(f,"        callq   *%%rax\n");
  fprintf(f,"        cmpl    $1024, %%ebx\n");
  fprintf(f,"        jne     69f\n");
  fprintf(f,"52:\n");
  fprintf(f,"        movq    %%r15, %%rdi\n");
  fprintf(f,"        callq   *%%r13\n");
  fprintf(f,"53:\n");
  fprintf(f,"        movabsq $errno, %%rax\n");
  fprintf(f,"        movl    $34, (%%rax)\n");
  fprintf(f,"        movq    -64(%%rbp), %%rax\n");
  fprintf(f,"        movl    $2146435072, 4(%%rax)\n");
  fprintf(f,"        movl    $0, (%%rax)\n");
  fprintf(f,"        jmp     71f\n");
  fprintf(f,"54:\n");
  fprintf(f,"        cmpl    $3, %%eax\n");
  fprintf(f,"        je      55f\n");
  fprintf(f,"        decl    %%eax\n");
  fprintf(f,"        jne     57f\n");
  fprintf(f,"        cmpl    $53, %%r9d\n");
  fprintf(f,"        jne     57f\n");
  fprintf(f,"        movl    $-1075, %%esi\n");
  fprintf(f,"        movq    %%r15, %%rdi\n");
  fprintf(f,"        movabsq $any_on, %%rax\n");
  fprintf(f,"        subl    %%ebx, %%esi\n");
  fprintf(f,"        callq   *%%rax\n");
  fprintf(f,"        testl   %%eax, %%eax\n");
  fprintf(f,"        jmp     56f\n");
  fprintf(f,"55:\n");
  fprintf(f,"        cmpl    $0, -52(%%rbp)\n");
  fprintf(f,"56:\n");
  fprintf(f,"        jne     32b\n");
  fprintf(f,"57:\n");
  fprintf(f,"        movq    %%r15, %%rdi\n");
  fprintf(f,"        callq   *%%r13\n");
  fprintf(f,"58:\n");
  fprintf(f,"        movabsq $errno, %%rax\n");
  fprintf(f,"        movl    $34, (%%rax)\n");
  fprintf(f,"59:\n");
  fprintf(f,"        movq    -64(%%rbp), %%rax\n");
  fprintf(f,"        movq    $0, (%%rax)\n");
  fprintf(f,"        jmp     71f\n");
  fprintf(f,"60:\n");
  fprintf(f,"        movl    $1, %%r8d\n");
  fprintf(f,"        jmp     48b\n");
  fprintf(f,"61:\n");
  fprintf(f,"        movl    $53, %%r14d\n");
  fprintf(f,"        xorl    %%ecx, %%ecx\n");
  fprintf(f,"        jmp     49b\n");
  fprintf(f,"62:\n");
  fprintf(f,"        movl    $1, %%eax\n");
  fprintf(f,"        subl    -52(%%rbp), %%eax\n");
  fprintf(f,"        movl    %%eax, -52(%%rbp)\n");
  fprintf(f,"63:\n");
  fprintf(f,"        cmpl    $0, -52(%%rbp)\n");
  fprintf(f,"        jmp     50b\n");
  fprintf(f,"64:\n");
  fprintf(f,"        andl    $31, %%r14d\n");
  fprintf(f,"        je      69f\n");
  fprintf(f,"        movl    20(%%rax,%%r12,4), %%edi\n");
  fprintf(f,"        movq    -72(%%rbp), %%rax\n");
  fprintf(f,"        callq   *%%rax\n");
  fprintf(f,"        movl    $32, %%ecx\n");
  fprintf(f,"        subl    %%r14d, %%ecx\n");
  fprintf(f,"        cmpl    %%ecx, %%eax\n");
  fprintf(f,"        jge     69f\n");
  fprintf(f,"        jmp     51b\n");
  fprintf(f,"65:\n");
  fprintf(f,"        testl   %%ecx, %%ecx\n");
  fprintf(f,"        je      69f\n");
  fprintf(f,"66:\n");
  fprintf(f,"        cmpl    $1, 20(%%r15)\n");
  fprintf(f,"        jle     67f\n");
  fprintf(f,"        movl    28(%%r15), %%eax\n");
  fprintf(f,"        andl    $-1048577, %%eax\n");
  fprintf(f,"        jmp     68f\n");
  fprintf(f,"67:\n");
  fprintf(f,"        xorl    %%eax, %%eax\n");
  fprintf(f,"68:\n");
  fprintf(f,"        movq    -64(%%rbp), %%rdx\n");
  fprintf(f,"        movl    %%eax, 4(%%rdx)\n");
  fprintf(f,"        jmp     70f\n");
  fprintf(f,"69:\n");
  fprintf(f,"        movl    28(%%r15), %%eax\n");
  fprintf(f,"        addl    $1075, %%ebx\n");
  fprintf(f,"        sall    $20, %%ebx\n");
  fprintf(f,"        andl    $-1048577, %%eax\n");
  fprintf(f,"        orl     %%eax, %%ebx\n");
  fprintf(f,"        movq    -64(%%rbp), %%rax\n");
  fprintf(f,"        movl    %%ebx, 4(%%rax)\n");
  fprintf(f,"70:\n");
  fprintf(f,"        movl    24(%%r15), %%eax\n");
  fprintf(f,"        movq    %%r15, %%rdi\n");
  fprintf(f,"        movq    -64(%%rbp), %%rdx\n");
  fprintf(f,"        movl    %%eax, (%%rdx)\n");
  fprintf(f,"        addq    $40, %%rsp\n");
  fprintf(f,"        movq    %%r13, %%rax\n");
  fprintf(f,"        popq    %%rbx\n");
  fprintf(f,"        popq    %%r12\n");
  fprintf(f,"        popq    %%r13\n");
  fprintf(f,"        popq    %%r14\n");
  fprintf(f,"        popq    %%r15\n");
  fprintf(f,"        popq    %%rbp\n");
  fprintf(f,"        jmp     *%%rax\n");
  fprintf(f,"71:\n");
  fprintf(f,"        addq    $40, %%rsp\n");
  fprintf(f,"        popq    %%rbx\n");
  fprintf(f,"        popq    %%r12\n");
  fprintf(f,"        popq    %%r13\n");
  fprintf(f,"        popq    %%r14\n");
  fprintf(f,"        popq    %%r15\n");
  fprintf(f,"        popq    %%rbp\n");
  fprintf(f,"        retq\n");
  fprintf(f,"        .size   gethex, .-gethex\n\n");

  fprintf(f,"        .section .rodata.str1.1,\"aMS\",@progbits,1\n\n");

  fprintf(f,".LdtoaC4:\n");
  fprintf(f,"        .string \"nf\"\n");
  fprintf(f,".LdtoaC5:\n");
  fprintf(f,"        .string \"inity\"\n");
  fprintf(f,".LdtoaC6:\n");
  fprintf(f,"        .string \"an\"\n\n");

  fprintf(f,"        .section .text,\"ax\",@progbits\n\n");

  fprintf(f,"        .type   strtod, @function\n");
  fprintf(f,"strtod:\n");
  fprintf(f,"        pushq   %%rbp\n");
  fprintf(f,"        movq    %%rsp, %%rbp\n");
  fprintf(f,"        pushq   %%r15\n");
  fprintf(f,"        pushq   %%r14\n");
  fprintf(f,"        pushq   %%r13\n");
  fprintf(f,"        pushq   %%r12\n");
  fprintf(f,"        movq    %%rdi, %%r12\n");
  fprintf(f,"        pushq   %%rbx\n");
  fprintf(f,"        subq    $136, %%rsp\n");
  fprintf(f,"        movq    %%rsi, -152(%%rbp)\n");
  fprintf(f,"        movq    $0, -56(%%rbp)\n");
  fprintf(f,"        movq    %%rdi, -64(%%rbp)\n");
  fprintf(f,"0:\n");
  fprintf(f,"        movq    -64(%%rbp), %%rdx\n");
  fprintf(f,"        movb    (%%rdx), %%al\n");
  fprintf(f,"        cmpb    $13, %%al\n");
  fprintf(f,"        jg      1f\n");
  fprintf(f,"        cmpb    $9, %%al\n");
  fprintf(f,"        jge     5f\n");
  fprintf(f,"        testb   %%al, %%al\n");
  fprintf(f,"        je      50f\n");
  fprintf(f,"        jmp     6f\n");
  fprintf(f,"1:\n");
  fprintf(f,"        cmpb    $43, %%al\n");
  fprintf(f,"        je      3f\n");
  fprintf(f,"        cmpb    $45, %%al\n");
  fprintf(f,"        je      2f\n");
  fprintf(f,"        cmpb    $32, %%al\n");
  fprintf(f,"        jne     6f\n");
  fprintf(f,"        jmp     5f\n");
  fprintf(f,"2:\n");
  fprintf(f,"        movl    $1, -84(%%rbp)\n");
  fprintf(f,"        jmp     4f\n");
  fprintf(f,"3:\n");
  fprintf(f,"        movl    $0, -84(%%rbp)\n");
  fprintf(f,"4:\n");
  fprintf(f,"        leaq    1(%%rdx), %%rax\n");
  fprintf(f,"        movq    %%rax, -64(%%rbp)\n");
  fprintf(f,"        cmpb    $0, 1(%%rdx)\n");
  fprintf(f,"        jne     7f\n");
  fprintf(f,"        jmp     50f\n");
  fprintf(f,"5:\n");
  fprintf(f,"        incq    %%rdx\n");
  fprintf(f,"        movq    %%rdx, -64(%%rbp)\n");
  fprintf(f,"        jmp     0b\n");
  fprintf(f,"6:\n");
  fprintf(f,"        movl    $0, -84(%%rbp)\n");
  fprintf(f,"7:\n");
  fprintf(f,"        movq    -64(%%rbp), %%rax\n");
  fprintf(f,"        movl    $0, -120(%%rbp)\n");
  fprintf(f,"        cmpb    $48, (%%rax)\n");
  fprintf(f,"        jne     10f\n");
  fprintf(f,"        movb    1(%%rax), %%al\n");
  fprintf(f,"        cmpb    $88, %%al\n");
  fprintf(f,"        je      8f\n");
  fprintf(f,"        cmpb    $120, %%al\n");
  fprintf(f,"        jne     9f\n");
  fprintf(f,"8:\n");
  fprintf(f,"        movl    -84(%%rbp), %%ecx\n");
  fprintf(f,"        leaq    -56(%%rbp), %%rsi\n");
  fprintf(f,"        movl    $1, %%edx\n");
  fprintf(f,"        leaq    -64(%%rbp), %%rdi\n");
  fprintf(f,"        movabsq $gethex, %%rax\n");
  fprintf(f,"        callq   *%%rax\n");
  fprintf(f,"        jmp     113f\n");
  fprintf(f,"9:\n");
  fprintf(f,"        movq    -64(%%rbp), %%rax\n");
  fprintf(f,"        leaq    1(%%rax), %%rdx\n");
  fprintf(f,"        movq    %%rdx, -64(%%rbp)\n");
  fprintf(f,"        movb    1(%%rax), %%al\n");
  fprintf(f,"        cmpb    $48, %%al\n");
  fprintf(f,"        je      9b\n");
  fprintf(f,"        testb   %%al, %%al\n");
  fprintf(f,"        je      113f\n");
  fprintf(f,"        movl    $1, -120(%%rbp)\n");
  fprintf(f,"10:\n");
  fprintf(f,"        movq    -64(%%rbp), %%rax\n");
  fprintf(f,"        xorl    %%r13d, %%r13d\n");
  fprintf(f,"        xorl    %%ecx, %%ecx\n");
  fprintf(f,"        xorl    %%esi, %%esi\n");
  fprintf(f,"        movq    %%rax, -128(%%rbp)\n");
  fprintf(f,"11:\n");
  fprintf(f,"        movq    -64(%%rbp), %%rdx\n");
  fprintf(f,"        movsbl  (%%rdx), %%eax\n");
  fprintf(f,"        leal    -48(%%rax), %%edi\n");
  fprintf(f,"        cmpb    $9, %%dil\n");
  fprintf(f,"        ja      14f\n");
  fprintf(f,"        cmpl    $8, %%esi\n");
  fprintf(f,"        jg      12f\n");
  fprintf(f,"        imull   $10, %%ecx, %%ecx\n");
  fprintf(f,"        leal    -48(%%rcx,%%rax), %%ecx\n");
  fprintf(f,"        jmp     13f\n");
  fprintf(f,"12:\n");
  fprintf(f,"        cmpl    $16, %%esi\n");
  fprintf(f,"        jg      13f\n");
  fprintf(f,"        imull   $10, %%r13d, %%r13d\n");
  fprintf(f,"        leal    -48(%%r13,%%rax), %%r13d\n");
  fprintf(f,"13:\n");
  fprintf(f,"        incq    %%rdx\n");
  fprintf(f,"        incl    %%esi\n");
  fprintf(f,"        movq    %%rdx, -64(%%rbp)\n");
  fprintf(f,"        jmp     11b\n");
  fprintf(f,"14:\n");
  fprintf(f,"        movq    %%rdx, %%rdi\n");
  fprintf(f,"15:\n");
  fprintf(f,"        movl    %%edi, %%r11d\n");
  fprintf(f,"        subl    %%edx, %%r11d\n");
  fprintf(f,"        cmpq    -128(%%rbp), %%rdx\n");
  fprintf(f,"        ja      17f\n");
  fprintf(f,"16:\n");
  fprintf(f,"        cmpl    $46, %%eax\n");
  fprintf(f,"        je      18f\n");
  fprintf(f,"        movl    $0, -136(%%rbp)\n");
  fprintf(f,"        movl    %%esi, %%r14d\n");
  fprintf(f,"        xorl    %%edx, %%edx\n");
  fprintf(f,"        jmp     32f\n");
  fprintf(f,"17:\n");
  fprintf(f,"        decq    %%rdx\n");
  fprintf(f,"        cmpb    $48, (%%rdx)\n");
  fprintf(f,"        je      15b\n");
  fprintf(f,"        jmp     16b\n");
  fprintf(f,"18:\n");
  fprintf(f,"        leaq    1(%%rdi), %%rdx\n");
  fprintf(f,"        movl    %%edx, %%ebx\n");
  fprintf(f,"        movq    %%rdx, -64(%%rbp)\n");
  fprintf(f,"        xorl    %%edx, %%edx\n");
  fprintf(f,"        subl    %%edi, %%ebx\n");
  fprintf(f,"        testl   %%esi, %%esi\n");
  fprintf(f,"        movsbl  1(%%rdi), %%eax\n");
  fprintf(f,"        movl    %%ebx, -136(%%rbp)\n");
  fprintf(f,"        jne     29f\n");
  fprintf(f,"19:\n");
  fprintf(f,"        cmpl    $48, %%eax\n");
  fprintf(f,"        jne     20f\n");
  fprintf(f,"        movq    -64(%%rbp), %%rax\n");
  fprintf(f,"        incl    %%edx\n");
  fprintf(f,"        leaq    1(%%rax), %%rdi\n");
  fprintf(f,"        movq    %%rdi, -64(%%rbp)\n");
  fprintf(f,"        movsbl  1(%%rax), %%eax\n");
  fprintf(f,"        jmp     19b\n");
  fprintf(f,"20:\n");
  fprintf(f,"        leal    -49(%%rax), %%edi\n");
  fprintf(f,"        cmpl    $8, %%edi\n");
  fprintf(f,"        ja      31f\n");
  fprintf(f,"        movq    -64(%%rbp), %%rbx\n");
  fprintf(f,"        movl    %%edx, %%edi\n");
  fprintf(f,"        xorl    %%r14d, %%r14d\n");
  fprintf(f,"        xorl    %%edx, %%edx\n");
  fprintf(f,"        movq    %%rbx, -128(%%rbp)\n");
  fprintf(f,"21:\n");
  fprintf(f,"        leal    1(%%rdx), %%r8d\n");
  fprintf(f,"        subl    $48, %%eax\n");
  fprintf(f,"        je      28f\n");
  fprintf(f,"        leal    1(%%r14), %%r9d\n");
  fprintf(f,"        addl    %%r8d, %%edi\n");
  fprintf(f,"22:\n");
  fprintf(f,"        movl    %%r9d, %%r10d\n");
  fprintf(f,"        subl    %%r14d, %%r10d\n");
  fprintf(f,"        cmpl    %%r8d, %%r10d\n");
  fprintf(f,"        jge     25f\n");
  fprintf(f,"        leal    -1(%%r9), %%r10d\n");
  fprintf(f,"        cmpl    $8, %%r10d\n");
  fprintf(f,"        jg      23f\n");
  fprintf(f,"        imull   $10, %%ecx, %%ecx\n");
  fprintf(f,"        jmp     24f\n");
  fprintf(f,"23:\n");
  fprintf(f,"        imull   $10, %%r13d, %%r10d\n");
  fprintf(f,"        cmpl    $17, %%r9d\n");
  fprintf(f,"        cmovle  %%r10, %%r13\n");
  fprintf(f,"24:\n");
  fprintf(f,"        incl    %%r9d\n");
  fprintf(f,"        jmp     22b\n");
  fprintf(f,"25:\n");
  fprintf(f,"        xorl    %%r8d, %%r8d\n");
  fprintf(f,"        testl   %%edx, %%edx\n");
  fprintf(f,"        cmovns  %%edx, %%r8d\n");
  fprintf(f,"        addl    %%r14d, %%r8d\n");
  fprintf(f,"        leal    1(%%r8), %%r14d\n");
  fprintf(f,"        cmpl    $8, %%r8d\n");
  fprintf(f,"        jg      26f\n");
  fprintf(f,"        imull   $10, %%ecx, %%ecx\n");
  fprintf(f,"        addl    %%eax, %%ecx\n");
  fprintf(f,"        jmp     27f\n");
  fprintf(f,"26:\n");
  fprintf(f,"        cmpl    $17, %%r14d\n");
  fprintf(f,"        jg      27f\n");
  fprintf(f,"        imull   $10, %%r13d, %%r13d\n");
  fprintf(f,"        addl    %%eax, %%r13d\n");
  fprintf(f,"27:\n");
  fprintf(f,"        xorl    %%r11d, %%r11d\n");
  fprintf(f,"        xorl    %%r8d, %%r8d\n");
  fprintf(f,"28:\n");
  fprintf(f,"        movq    -64(%%rbp), %%rax\n");
  fprintf(f,"        leaq    1(%%rax), %%rdx\n");
  fprintf(f,"        movq    %%rdx, -64(%%rbp)\n");
  fprintf(f,"        movl    %%r8d, %%edx\n");
  fprintf(f,"        movsbl  1(%%rax), %%eax\n");
  fprintf(f,"        jmp     30f\n");
  fprintf(f,"29:\n");
  fprintf(f,"        movl    %%esi, %%r14d\n");
  fprintf(f,"        xorl    %%edx, %%edx\n");
  fprintf(f,"        xorl    %%edi, %%edi\n");
  fprintf(f,"30:\n");
  fprintf(f,"        leal    -48(%%rax), %%r8d\n");
  fprintf(f,"        cmpl    $9, %%r8d\n");
  fprintf(f,"        jbe     21b\n");
  fprintf(f,"        jmp     33f\n");
  fprintf(f,"31:\n");
  fprintf(f,"        xorl    %%r14d, %%r14d\n");
  fprintf(f,"32:\n");
  fprintf(f,"        xorl    %%edi, %%edi\n");
  fprintf(f,"33:\n");
  fprintf(f,"        movl    %%eax, %%r8d\n");
  fprintf(f,"        andl    $-33, %%r8d\n");
  fprintf(f,"        cmpl    $69, %%r8d\n");
  fprintf(f,"        jne     44f\n");
  fprintf(f,"        movl    %%r14d, %%eax\n");
  fprintf(f,"        orl     %%edx, %%eax\n");
  fprintf(f,"        orl     -120(%%rbp), %%eax\n");
  fprintf(f,"        je      50f\n");
  fprintf(f,"        movq    -64(%%rbp), %%r12\n");
  fprintf(f,"        leaq    1(%%r12), %%rax\n");
  fprintf(f,"        movq    %%rax, -64(%%rbp)\n");
  fprintf(f,"        movsbl  1(%%r12), %%eax\n");
  fprintf(f,"        cmpb    $43, %%al\n");
  fprintf(f,"        je      34f\n");
  fprintf(f,"        cmpb    $45, %%al\n");
  fprintf(f,"        je      35f\n");
  fprintf(f,"        xorl    %%r9d, %%r9d\n");
  fprintf(f,"        jmp     37f\n");
  fprintf(f,"34:\n");
  fprintf(f,"        xorl    %%r9d, %%r9d\n");
  fprintf(f,"        jmp     36f\n");
  fprintf(f,"35:\n");
  fprintf(f,"        movl    $1, %%r9d\n");
  fprintf(f,"36:\n");
  fprintf(f,"        leaq    2(%%r12), %%rax\n");
  fprintf(f,"        movq    %%rax, -64(%%rbp)\n");
  fprintf(f,"        movsbl  2(%%r12), %%eax\n");
  fprintf(f,"37:\n");
  fprintf(f,"        leal    -48(%%rax), %%r8d\n");
  fprintf(f,"        cmpl    $9, %%r8d\n");
  fprintf(f,"        ja      43f\n");
  fprintf(f,"38:\n");
  fprintf(f,"        cmpl    $48, %%eax\n");
  fprintf(f,"        jne     39f\n");
  fprintf(f,"        movq    -64(%%rbp), %%rax\n");
  fprintf(f,"        leaq    1(%%rax), %%r8\n");
  fprintf(f,"        movq    %%r8, -64(%%rbp)\n");
  fprintf(f,"        movsbl  1(%%rax), %%eax\n");
  fprintf(f,"        jmp     38b\n");
  fprintf(f,"39:\n");
  fprintf(f,"        leal    -49(%%rax), %%r8d\n");
  fprintf(f,"        xorl    %%ebx, %%ebx\n");
  fprintf(f,"        cmpl    $8, %%r8d\n");
  fprintf(f,"        ja      45f\n");
  fprintf(f,"        leal    -48(%%rax), %%r8d\n");
  fprintf(f,"        movq    -64(%%rbp), %%rax\n");
  fprintf(f,"        movq    %%rax, -144(%%rbp)\n");
  fprintf(f,"40:\n");
  fprintf(f,"        movq    -64(%%rbp), %%rax\n");
  fprintf(f,"        leaq    1(%%rax), %%r10\n");
  fprintf(f,"        movq    %%r10, -64(%%rbp)\n");
  fprintf(f,"        movb    1(%%rax), %%bl\n");
  fprintf(f,"        movsbl  %%bl, %%eax\n");
  fprintf(f,"        subl    $48, %%ebx\n");
  fprintf(f,"        cmpb    $9, %%bl\n");
  fprintf(f,"        ja      41f\n");
  fprintf(f,"        imull   $10, %%r8d, %%r8d\n");
  fprintf(f,"        leal    -48(%%rax,%%r8), %%r8d\n");
  fprintf(f,"        jmp     40b\n");
  fprintf(f,"41:\n");
  fprintf(f,"        subq    -144(%%rbp), %%r10\n");
  fprintf(f,"        movl    $19999, %%ebx\n");
  fprintf(f,"        cmpq    $8, %%r10\n");
  fprintf(f,"        jg      42f\n");
  fprintf(f,"        cmpl    $19999, %%r8d\n");
  fprintf(f,"        cmovle  %%r8d, %%ebx\n");
  fprintf(f,"42:\n");
  fprintf(f,"        movl    %%ebx, %%r8d\n");
  fprintf(f,"        negl    %%r8d\n");
  fprintf(f,"        testl   %%r9d, %%r9d\n");
  fprintf(f,"        cmovne  %%r8d, %%ebx\n");
  fprintf(f,"        jmp     45f\n");
  fprintf(f,"43:\n");
  fprintf(f,"        movq    %%r12, -64(%%rbp)\n");
  fprintf(f,"44:\n");
  fprintf(f,"        xorl    %%ebx, %%ebx\n");
  fprintf(f,"45:\n");
  fprintf(f,"        testl   %%r14d, %%r14d\n");
  fprintf(f,"        jne     51f\n");
  fprintf(f,"        movl    -120(%%rbp), %%ecx\n");
  fprintf(f,"        orl     %%edx, %%ecx\n");
  fprintf(f,"        jne     113f\n");
  fprintf(f,"        cmpl    $0, -136(%%rbp)\n");
  fprintf(f,"        jne     50f\n");
  fprintf(f,"        cmpl    $78, %%eax\n");
  fprintf(f,"        je      49f\n");
  fprintf(f,"        jg      46f\n");
  fprintf(f,"        cmpl    $73, %%eax\n");
  fprintf(f,"        je      47f\n");
  fprintf(f,"        jmp     50f\n");
  fprintf(f,"46:\n");
  fprintf(f,"        cmpl    $105, %%eax\n");
  fprintf(f,"        je      47f\n");
  fprintf(f,"        cmpl    $110, %%eax\n");
  fprintf(f,"        je      49f\n");
  fprintf(f,"        jmp     50f\n");
  fprintf(f,"47:\n");
  fprintf(f,"        movabsq $.LdtoaC4, %%rsi\n");
  fprintf(f,"        movabsq $match, %%rbx\n");
  fprintf(f,"        leaq    -64(%%rbp), %%rdi\n");
  fprintf(f,"        callq   *%%rbx\n");
  fprintf(f,"        testl   %%eax, %%eax\n");
  fprintf(f,"        je      50f\n");
  fprintf(f,"        decq    -64(%%rbp)\n");
  fprintf(f,"        leaq    -64(%%rbp), %%rdi\n");
  fprintf(f,"        movabsq $.LdtoaC5, %%rsi\n");
  fprintf(f,"        callq   *%%rbx\n");
  fprintf(f,"        testl   %%eax, %%eax\n");
  fprintf(f,"        jne     48f\n");
  fprintf(f,"        incq    -64(%%rbp)\n");
  fprintf(f,"48:\n");
  fprintf(f,"        movl    $2146435072, -52(%%rbp)\n");
  fprintf(f,"        movl    $0, -56(%%rbp)\n");
  fprintf(f,"        jmp     113f\n");
  fprintf(f,"49:\n");
  fprintf(f,"        movabsq $.LdtoaC6, %%rsi\n");
  fprintf(f,"        movabsq $match, %%rax\n");
  fprintf(f,"        leaq    -64(%%rbp), %%rdi\n");
  fprintf(f,"        callq   *%%rax\n");
  fprintf(f,"        testl   %%eax, %%eax\n");
  fprintf(f,"        je      50f\n");
  fprintf(f,"        movq    -64(%%rbp), %%rax\n");
  fprintf(f,"        movl    $2146959360, -52(%%rbp)\n");
  fprintf(f,"        movl    $0, -56(%%rbp)\n");
  fprintf(f,"        cmpb    $40, (%%rax)\n");
  fprintf(f,"        jne     113f\n");
  fprintf(f,"        leaq    -64(%%rbp), %%rsi\n");
  fprintf(f,"        movabsq $hexnan, %%rax\n");
  fprintf(f,"        leaq    -56(%%rbp), %%rdi\n");
  fprintf(f,"        callq   *%%rax\n");
  fprintf(f,"        jmp     113f\n");
  fprintf(f,"50:\n");
  fprintf(f,"        movq    %%r12, -64(%%rbp)\n");
  fprintf(f,"        jmp     55f\n");
  fprintf(f,"51:\n");
  fprintf(f,"        subl    %%edi, %%ebx\n");
  fprintf(f,"        movl    %%ecx, %%edx\n");
  fprintf(f,"        testl   %%esi, %%esi\n");
  fprintf(f,"        cmove   %%r14d, %%esi\n");
  fprintf(f,"        movl    $17, %%eax\n");
  fprintf(f,"        cmpl    $17, %%r14d\n");
  fprintf(f,"        cmovle  %%r14d, %%eax\n");
  fprintf(f,"        cvtsi2sdq       %%rdx, %%xmm0\n");
  fprintf(f,"        cmpl    $9, %%eax\n");
  fprintf(f,"        movsd   %%xmm0, -56(%%rbp)\n");
  fprintf(f,"        jle     52f\n");
  fprintf(f,"        leal    -9(%%rax), %%edi\n");
  fprintf(f,"        movabsq $tens, %%rdx\n");
  fprintf(f,"        cvtsi2sdq       %%r13, %%xmm1\n");
  fprintf(f,"        movslq  %%edi, %%rdi\n");
  fprintf(f,"        mulsd   (%%rdx,%%rdi,8), %%xmm0\n");
  fprintf(f,"        addsd   %%xmm1, %%xmm0\n");
  fprintf(f,"        movsd   %%xmm0, -56(%%rbp)\n");
  fprintf(f,"52:\n");
  fprintf(f,"        cmpl    $15, %%r14d\n");
  fprintf(f,"        jg      60f\n");
  fprintf(f,"        cmpl    $0, %%ebx\n");
  fprintf(f,"        je      113f\n");
  fprintf(f,"        jle     58f\n");
  fprintf(f,"        cmpl    $22, %%ebx\n");
  fprintf(f,"        jg      56f\n");
  fprintf(f,"        cmpl    $0, -84(%%rbp)\n");
  fprintf(f,"        je      53f\n");
  fprintf(f,"        movabsq $.LdtoaC7, %%rax\n");
  fprintf(f,"        movsd   -56(%%rbp), %%xmm0\n");
  fprintf(f,"        movaps  (%%rax), %%xmm1\n");
  fprintf(f,"        xorps   %%xmm1, %%xmm0\n");
  fprintf(f,"        movsd   %%xmm0, -56(%%rbp)\n");
  fprintf(f,"53:\n");
  fprintf(f,"        movsd   -56(%%rbp), %%xmm0\n");
  fprintf(f,"        movslq  %%ebx, %%rbx\n");
  fprintf(f,"        movabsq $tens, %%rax\n");
  fprintf(f,"        mulsd   (%%rax,%%rbx,8), %%xmm0\n");
  fprintf(f,"54:\n");
  fprintf(f,"        movsd   %%xmm0, -56(%%rbp)\n");
  fprintf(f,"55:\n");
  fprintf(f,"        movl    $0, -84(%%rbp)\n");
  fprintf(f,"        jmp     113f\n");
  fprintf(f,"56:\n");
  fprintf(f,"        movl    $37, %%edx\n");
  fprintf(f,"        subl    %%r14d, %%edx\n");
  fprintf(f,"        cmpl    %%edx, %%ebx\n");
  fprintf(f,"        jg      60f\n");
  fprintf(f,"        cmpl    $0, -84(%%rbp)\n");
  fprintf(f,"        je      57f\n");
  fprintf(f,"        movabsq $.LdtoaC7, %%rax\n");
  fprintf(f,"        movsd   -56(%%rbp), %%xmm0\n");
  fprintf(f,"        movaps  (%%rax), %%xmm1\n");
  fprintf(f,"        xorps   %%xmm1, %%xmm0\n");
  fprintf(f,"        movsd   %%xmm0, -56(%%rbp)\n");
  fprintf(f,"57:\n");
  fprintf(f,"        movl    $15, %%eax\n");
  fprintf(f,"        movsd   -56(%%rbp), %%xmm0\n");
  fprintf(f,"        movabsq $tens, %%rdx\n");
  fprintf(f,"        subl    %%r14d, %%eax\n");
  fprintf(f,"        movslq  %%eax, %%rcx\n");
  fprintf(f,"        subl    %%eax, %%ebx\n");
  fprintf(f,"        mulsd   (%%rdx,%%rcx,8), %%xmm0\n");
  fprintf(f,"        movslq  %%ebx, %%rbx\n");
  fprintf(f,"        mulsd   (%%rdx,%%rbx,8), %%xmm0\n");
  fprintf(f,"        jmp     54b\n");
  fprintf(f,"58:\n");
  fprintf(f,"        cmpl    $-22, %%ebx\n");
  fprintf(f,"        jl      60f\n");
  fprintf(f,"        cmpl    $0, -84(%%rbp)\n");
  fprintf(f,"        je      59f\n");
  fprintf(f,"        movabsq $.LdtoaC7, %%rax\n");
  fprintf(f,"        movsd   -56(%%rbp), %%xmm0\n");
  fprintf(f,"        movaps  (%%rax), %%xmm1\n");
  fprintf(f,"        xorps   %%xmm1, %%xmm0\n");
  fprintf(f,"        movsd   %%xmm0, -56(%%rbp)\n");
  fprintf(f,"59:\n");
  fprintf(f,"        negl    %%ebx\n");
  fprintf(f,"        movsd   -56(%%rbp), %%xmm0\n");
  fprintf(f,"        movabsq $tens, %%rax\n");
  fprintf(f,"        movslq  %%ebx, %%rbx\n");
  fprintf(f,"        divsd   (%%rax,%%rbx,8), %%xmm0\n");
  fprintf(f,"        jmp     54b\n");
  fprintf(f,"60:\n");
  fprintf(f,"        movl    %%r14d, %%edi\n");
  fprintf(f,"        subl    %%eax, %%edi\n");
  fprintf(f,"        movl    %%edi, %%eax\n");
  fprintf(f,"        addl    %%ebx, %%eax\n");
  fprintf(f,"        cmpl    $0, %%eax\n");
  fprintf(f,"        jle     71f\n");
  fprintf(f,"        movl    %%eax, %%edi\n");
  fprintf(f,"        andl    $15, %%edi\n");
  fprintf(f,"        je      61f\n");
  fprintf(f,"        movsd   -56(%%rbp), %%xmm0\n");
  fprintf(f,"        movslq  %%edi, %%rdi\n");
  fprintf(f,"        movabsq $tens, %%rdx\n");
  fprintf(f,"        mulsd   (%%rdx,%%rdi,8), %%xmm0\n");
  fprintf(f,"        movsd   %%xmm0, -56(%%rbp)\n");
  fprintf(f,"61:\n");
  fprintf(f,"        andl    $-16, %%eax\n");
  fprintf(f,"        je      81f\n");
  fprintf(f,"        cmpl    $308, %%eax\n");
  fprintf(f,"        jle     66f\n");
  fprintf(f,"62:\n");
  fprintf(f,"        xorl    %%r12d, %%r12d\n");
  fprintf(f,"63:\n");
  fprintf(f,"        movl    $2146435072, -52(%%rbp)\n");
  fprintf(f,"        movl    $0, -56(%%rbp)\n");
  fprintf(f,"64:\n");
  fprintf(f,"        testq   %%r12, %%r12\n");
  fprintf(f,"        je      65f\n");
  fprintf(f,"        movabsq $Bfree, %%rbx\n");
  fprintf(f,"        movq    -104(%%rbp), %%rdi\n");
  fprintf(f,"        callq   *%%rbx\n");
  fprintf(f,"        movq    -112(%%rbp), %%rdi\n");
  fprintf(f,"        callq   *%%rbx\n");
  fprintf(f,"        movq    -96(%%rbp), %%rdi\n");
  fprintf(f,"        callq   *%%rbx\n");
  fprintf(f,"        movq    %%r12, %%rdi\n");
  fprintf(f,"        callq   *%%rbx\n");
  fprintf(f,"        movq    %%r15, %%rdi\n");
  fprintf(f,"        callq   *%%rbx\n");
  fprintf(f,"65:\n");
  fprintf(f,"        movabsq $errno, %%rax\n");
  fprintf(f,"        movl    $34, (%%rax)\n");
  fprintf(f,"        jmp     113f\n");
  fprintf(f,"66:\n");
  fprintf(f,"        movsd   -56(%%rbp), %%xmm0\n");
  fprintf(f,"        sarl    $4, %%eax\n");
  fprintf(f,"        xorl    %%edx, %%edx\n");
  fprintf(f,"        movabsq $bigtens, %%r8\n");
  fprintf(f,"67:\n");
  fprintf(f,"        cmpl    $1, %%eax\n");
  fprintf(f,"        jle     69f\n");
  fprintf(f,"        testb   $1, %%al\n");
  fprintf(f,"        je      68f\n");
  fprintf(f,"        mulsd   (%%r8,%%rdx,8), %%xmm0\n");
  fprintf(f,"68:\n");
  fprintf(f,"        sarl    %%eax\n");
  fprintf(f,"        incq    %%rdx\n");
  fprintf(f,"        jmp     67b\n");
  fprintf(f,"69:\n");
  fprintf(f,"        movsd   %%xmm0, -56(%%rbp)\n");
  fprintf(f,"        movslq  %%edx, %%rdx\n");
  fprintf(f,"        movabsq $bigtens, %%rax\n");
  fprintf(f,"        subl    $55574528, -52(%%rbp)\n");
  fprintf(f,"        movsd   -56(%%rbp), %%xmm0\n");
  fprintf(f,"        mulsd   (%%rax,%%rdx,8), %%xmm0\n");
  fprintf(f,"        movsd   %%xmm0, -56(%%rbp)\n");
  fprintf(f,"        movl    -52(%%rbp), %%eax\n");
  fprintf(f,"        movl    %%eax, %%edx\n");
  fprintf(f,"        andl    $2146435072, %%edx\n");
  fprintf(f,"        cmpl    $2090860544, %%edx\n");
  fprintf(f,"        ja      62b\n");
  fprintf(f,"        cmpl    $2089811968, %%edx\n");
  fprintf(f,"        jbe     70f\n");
  fprintf(f,"        movl    $2146435071, -52(%%rbp)\n");
  fprintf(f,"        movl    $-1, -56(%%rbp)\n");
  fprintf(f,"        jmp     81f\n");
  fprintf(f,"70:\n");
  fprintf(f,"        addl    $55574528, %%eax\n");
  fprintf(f,"        movl    %%eax, -52(%%rbp)\n");
  fprintf(f,"        jmp     81f\n");
  fprintf(f,"71:\n");
  fprintf(f,"        je      81f\n");
  fprintf(f,"        negl    %%eax\n");
  fprintf(f,"        movl    %%eax, %%edi\n");
  fprintf(f,"        andl    $15, %%edi\n");
  fprintf(f,"        je      72f\n");
  fprintf(f,"        movsd   -56(%%rbp), %%xmm0\n");
  fprintf(f,"        movslq  %%edi, %%rdi\n");
  fprintf(f,"        movabsq $tens, %%rdx\n");
  fprintf(f,"        divsd   (%%rdx,%%rdi,8), %%xmm0\n");
  fprintf(f,"        movsd   %%xmm0, -56(%%rbp)\n");
  fprintf(f,"72:\n");
  fprintf(f,"        sarl    $4, %%eax\n");
  fprintf(f,"        je      81f\n");
  fprintf(f,"        cmpl    $31, %%eax\n");
  fprintf(f,"        jg      76f\n");
  fprintf(f,"        movsd   -56(%%rbp), %%xmm0\n");
  fprintf(f,"        xorl    %%edx, %%edx\n");
  fprintf(f,"        movabsq $tinytens, %%r8\n");
  fprintf(f,"        jmp     75f\n");
  fprintf(f,"73:\n");
  fprintf(f,"        testb   $1, %%al\n");
  fprintf(f,"        je      74f\n");
  fprintf(f,"        mulsd   (%%r8,%%rdx,8), %%xmm0\n");
  fprintf(f,"74:\n");
  fprintf(f,"        sarl    %%eax\n");
  fprintf(f,"        incq    %%rdx\n");
  fprintf(f,"75:\n");
  fprintf(f,"        cmpl    $1, %%eax\n");
  fprintf(f,"        jg      73b\n");
  fprintf(f,"        movslq  %%edx, %%rdx\n");
  fprintf(f,"        movapd  %%xmm0, %%xmm3\n");
  fprintf(f,"        xorps   %%xmm1, %%xmm1\n");
  fprintf(f,"        movabsq $tinytens, %%rax\n");
  fprintf(f,"        movsd   (%%rax,%%rdx,8), %%xmm2\n");
  fprintf(f,"        mulsd   %%xmm2, %%xmm3\n");
  fprintf(f,"        ucomisd %%xmm1, %%xmm3\n");
  fprintf(f,"        movsd   %%xmm3, -56(%%rbp)\n");
  fprintf(f,"        jp      81f\n");
  fprintf(f,"        jne     81f\n");
  fprintf(f,"        addsd   %%xmm0, %%xmm0\n");
  fprintf(f,"        mulsd   %%xmm2, %%xmm0\n");
  fprintf(f,"        ucomisd %%xmm1, %%xmm0\n");
  fprintf(f,"        jp      80f\n");
  fprintf(f,"        jne     80f\n");
  fprintf(f,"76:\n");
  fprintf(f,"        xorl    %%r12d, %%r12d\n");
  fprintf(f,"        jmp     79f\n");
  fprintf(f,"77:\n");
  fprintf(f,"        callq   *%%rax\n");
  fprintf(f,"        movsd   -120(%%rbp), %%xmm4\n");
  fprintf(f,"        xorps   %%xmm1, %%xmm1\n");
  fprintf(f,"        subsd   %%xmm0, %%xmm4\n");
  fprintf(f,"        ucomisd %%xmm1, %%xmm4\n");
  fprintf(f,"        movsd   %%xmm4, -56(%%rbp)\n");
  fprintf(f,"        jp      112f\n");
  fprintf(f,"        jne     112f\n");
  fprintf(f,"78:\n");
  fprintf(f,"        cmpl    %%r14d, -144(%%rbp)\n");
  fprintf(f,"        jg      112f\n");
  fprintf(f,"79:\n");
  fprintf(f,"        movq    $0, -56(%%rbp)\n");
  fprintf(f,"        jmp     64b\n");
  fprintf(f,"80:\n");
  fprintf(f,"        movl    $0, -52(%%rbp)\n");
  fprintf(f,"        movl    $1, -56(%%rbp)\n");
  fprintf(f,"81:\n");
  fprintf(f,"        movl    -136(%%rbp), %%r8d\n");
  fprintf(f,"        movl    %%r14d, %%eax\n");
  fprintf(f,"        movl    %%r14d, %%edx\n");
  fprintf(f,"        movq    -128(%%rbp), %%rdi\n");
  fprintf(f,"        subl    %%r11d, %%eax\n");
  fprintf(f,"        movl    %%eax, -144(%%rbp)\n");
  fprintf(f,"        movabsq $s2b, %%rax\n");
  fprintf(f,"        callq   *%%rax\n");
  fprintf(f,"        xorl    %%ecx, %%ecx\n");
  fprintf(f,"        movq    %%rax, %%r12\n");
  fprintf(f,"        movl    %%ebx, %%eax\n");
  fprintf(f,"        negl    %%eax\n");
  fprintf(f,"        testl   %%ebx, %%ebx\n");
  fprintf(f,"        cmovns  %%ecx, %%eax\n");
  fprintf(f,"        movl    %%eax, -120(%%rbp)\n");
  fprintf(f,"82:\n");
  fprintf(f,"        movabsq $Balloc, %%rax\n");
  fprintf(f,"        movl    8(%%r12), %%edi\n");
  fprintf(f,"        callq   *%%rax\n");
  fprintf(f,"        leaq    16(%%r12), %%rsi\n");
  fprintf(f,"        movq    %%rax, %%rcx\n");
  fprintf(f,"        movq    %%rax, -112(%%rbp)\n");
  fprintf(f,"        movslq  20(%%r12), %%rax\n");
  fprintf(f,"        leaq    16(%%rcx), %%rdi\n");
  fprintf(f,"        leaq    8(,%%rax,4), %%rdx\n");
  fprintf(f,"        movabsq $memcpy, %%rax\n");
  fprintf(f,"        callq   *%%rax\n");
  fprintf(f,"        leaq    -68(%%rbp), %%rdx\n");
  fprintf(f,"        movabsq $d2b, %%rax\n");
  fprintf(f,"        leaq    -72(%%rbp), %%rsi\n");
  fprintf(f,"        leaq    -56(%%rbp), %%rdi\n");
  fprintf(f,"        callq   *%%rax\n");
  fprintf(f,"        movl    $1, %%edi\n");
  fprintf(f,"        movq    %%rax, -104(%%rbp)\n");
  fprintf(f,"        movabsq $i2b, %%rax\n");
  fprintf(f,"        callq   *%%rax\n");
  fprintf(f,"        movl    -72(%%rbp), %%r15d\n");
  fprintf(f,"        xorl    %%edx, %%edx\n");
  fprintf(f,"        testl   %%ebx, %%ebx\n");
  fprintf(f,"        movq    %%rax, -96(%%rbp)\n");
  fprintf(f,"        cmovg   %%ebx, %%edx\n");
  fprintf(f,"        testl   %%r15d, %%r15d\n");
  fprintf(f,"        js      83f\n");
  fprintf(f,"        movl    -120(%%rbp), %%eax\n");
  fprintf(f,"        leal    (%%rax,%%r15), %%r13d\n");
  fprintf(f,"        movl    %%edx, %%eax\n");
  fprintf(f,"        jmp     84f\n");
  fprintf(f,"83:\n");
  fprintf(f,"        movl    -120(%%rbp), %%r13d\n");
  fprintf(f,"        movl    %%edx, %%eax\n");
  fprintf(f,"        subl    %%r15d, %%eax\n");
  fprintf(f,"84:\n");
  fprintf(f,"        movl    -68(%%rbp), %%ecx\n");
  fprintf(f,"        leal    (%%r15,%%rcx), %%esi\n");
  fprintf(f,"        addl    $1075, %%r15d\n");
  fprintf(f,"        cmpl    $-1021, %%esi\n");
  fprintf(f,"        jl      85f\n");
  fprintf(f,"        movl    $54, %%r15d\n");
  fprintf(f,"        subl    %%ecx, %%r15d\n");
  fprintf(f,"85:\n");
  fprintf(f,"        leal    0(%%r13,%%r15), %%ecx\n");
  fprintf(f,"        addl    %%eax, %%r15d\n");
  fprintf(f,"        cmpl    %%r15d, %%ecx\n");
  fprintf(f,"        movl    %%r15d, %%eax\n");
  fprintf(f,"        cmovle  %%ecx, %%eax\n");
  fprintf(f,"        cmpl    %%r13d, %%eax\n");
  fprintf(f,"        cmovg   %%r13d, %%eax\n");
  fprintf(f,"        testl   %%eax, %%eax\n");
  fprintf(f,"        jle     86f\n");
  fprintf(f,"        subl    %%eax, %%ecx\n");
  fprintf(f,"        subl    %%eax, %%r15d\n");
  fprintf(f,"        subl    %%eax, %%r13d\n");
  fprintf(f,"86:\n");
  fprintf(f,"        movl    -120(%%rbp), %%esi\n");
  fprintf(f,"        testl   %%esi, %%esi\n");
  fprintf(f,"        je      87f\n");
  fprintf(f,"        movq    -96(%%rbp), %%rdi\n");
  fprintf(f,"        movl    %%edx, -160(%%rbp)\n");
  fprintf(f,"        movabsq $pow5mult, %%rax\n");
  fprintf(f,"        movl    %%ecx, -136(%%rbp)\n");
  fprintf(f,"        callq   *%%rax\n");
  fprintf(f,"        movq    -104(%%rbp), %%rsi\n");
  fprintf(f,"        movq    %%rax, %%rdi\n");
  fprintf(f,"        movq    %%rax, -96(%%rbp)\n");
  fprintf(f,"        movabsq $mult, %%rax\n");
  fprintf(f,"        callq   *%%rax\n");
  fprintf(f,"        movq    -104(%%rbp), %%rdi\n");
  fprintf(f,"        movabsq $Bfree, %%rsi\n");
  fprintf(f,"        movq    %%rax, -128(%%rbp)\n");
  fprintf(f,"        callq   *%%rsi\n");
  fprintf(f,"        movq    -128(%%rbp), %%rax\n");
  fprintf(f,"        movl    -160(%%rbp), %%edx\n");
  fprintf(f,"        movl    -136(%%rbp), %%ecx\n");
  fprintf(f,"        movq    %%rax, -104(%%rbp)\n");
  fprintf(f,"87:\n");
  fprintf(f,"        testl   %%ecx, %%ecx\n");
  fprintf(f,"        jle     88f\n");
  fprintf(f,"        movq    -104(%%rbp), %%rdi\n");
  fprintf(f,"        movl    %%ecx, %%esi\n");
  fprintf(f,"        movl    %%edx, -128(%%rbp)\n");
  fprintf(f,"        movabsq $lshift, %%rax\n");
  fprintf(f,"        callq   *%%rax\n");
  fprintf(f,"        movl    -128(%%rbp), %%edx\n");
  fprintf(f,"        movq    %%rax, -104(%%rbp)\n");
  fprintf(f,"88:\n");
  fprintf(f,"        testl   %%edx, %%edx\n");
  fprintf(f,"        je      89f\n");
  fprintf(f,"        movabsq $pow5mult, %%rax\n");
  fprintf(f,"        movq    -112(%%rbp), %%rdi\n");
  fprintf(f,"        movl    %%edx, %%esi\n");
  fprintf(f,"        callq   *%%rax\n");
  fprintf(f,"        movq    %%rax, -112(%%rbp)\n");
  fprintf(f,"89:\n");
  fprintf(f,"        testl   %%r15d, %%r15d\n");
  fprintf(f,"        jle     90f\n");
  fprintf(f,"        movq    -112(%%rbp), %%rdi\n");
  fprintf(f,"        movl    %%r15d, %%esi\n");
  fprintf(f,"        movabsq $lshift, %%rax\n");
  fprintf(f,"        callq   *%%rax\n");
  fprintf(f,"        movq    %%rax, -112(%%rbp)\n");
  fprintf(f,"90:\n");
  fprintf(f,"        testl   %%r13d, %%r13d\n");
  fprintf(f,"        jle     91f\n");
  fprintf(f,"        movq    -96(%%rbp), %%rdi\n");
  fprintf(f,"        movl    %%r13d, %%esi\n");
  fprintf(f,"        movabsq $lshift, %%rax\n");
  fprintf(f,"        callq   *%%rax\n");
  fprintf(f,"        movq    %%rax, -96(%%rbp)\n");
  fprintf(f,"91:\n");
  fprintf(f,"        movabsq $diff, %%rax\n");
  fprintf(f,"        movq    -112(%%rbp), %%rsi\n");
  fprintf(f,"        movq    -104(%%rbp), %%rdi\n");
  fprintf(f,"        callq   *%%rax\n");
  fprintf(f,"        movq    -96(%%rbp), %%rsi\n");
  fprintf(f,"        movabsq $cmp, %%rdx\n");
  fprintf(f,"        movl    16(%%rax), %%r13d\n");
  fprintf(f,"        movq    %%rax, %%rdi\n");
  fprintf(f,"        movl    $0, 16(%%rax)\n");
  fprintf(f,"        movq    %%rdx, -128(%%rbp)\n");
  fprintf(f,"        movq    %%rax, %%r15\n");
  fprintf(f,"        callq   *%%rdx\n");
  fprintf(f,"        movq    -128(%%rbp), %%rdx\n");
  fprintf(f,"        testl   %%eax, %%eax\n");
  fprintf(f,"        jns     93f\n");
  fprintf(f,"        testl   %%r13d, %%r13d\n");
  fprintf(f,"        jne     112f\n");
  fprintf(f,"        movabsq $4503599627370495, %%rax\n");
  fprintf(f,"        testq   %%rax, -56(%%rbp)\n");
  fprintf(f,"        jne     112f\n");
  fprintf(f,"        movl    -52(%%rbp), %%eax\n");
  fprintf(f,"        andl    $2146435072, %%eax\n");
  fprintf(f,"        cmpl    $1048576, %%eax\n");
  fprintf(f,"        jbe     112f\n");
  fprintf(f,"        cmpl    $0, 24(%%r15)\n");
  fprintf(f,"        jne     92f\n");
  fprintf(f,"        cmpl    $1, 20(%%r15)\n");
  fprintf(f,"        jle     112f\n");
  fprintf(f,"92:\n");
  fprintf(f,"        movq    %%r15, %%rdi\n");
  fprintf(f,"        movl    $1, %%esi\n");
  fprintf(f,"        movq    %%rdx, -120(%%rbp)\n");
  fprintf(f,"        movabsq $lshift, %%rax\n");
  fprintf(f,"        callq   *%%rax\n");
  fprintf(f,"        movq    -96(%%rbp), %%rsi\n");
  fprintf(f,"        movq    -120(%%rbp), %%rdx\n");
  fprintf(f,"        movq    %%rax, %%rdi\n");
  fprintf(f,"        movq    %%rax, %%r15\n");
  fprintf(f,"        callq   *%%rdx\n");
  fprintf(f,"        testl   %%eax, %%eax\n");
  fprintf(f,"        jg      95f\n");
  fprintf(f,"        jmp     112f\n");
  fprintf(f,"93:\n");
  fprintf(f,"        jne     97f\n");
  fprintf(f,"        movabsq $4503599627370495, %%rax\n");
  fprintf(f,"        testl   %%r13d, %%r13d\n");
  fprintf(f,"        je      94f\n");
  fprintf(f,"        movq    -56(%%rbp), %%rcx\n");
  fprintf(f,"        movq    %%rcx, %%rsi\n");
  fprintf(f,"        andq    %%rax, %%rsi\n");
  fprintf(f,"        cmpq    %%rax, %%rsi\n");
  fprintf(f,"        jne     96f\n");
  fprintf(f,"        movabsq $9218868437227405311, %%rax\n");
  fprintf(f,"        cmpq    %%rax, %%rcx\n");
  fprintf(f,"        je      63b\n");
  fprintf(f,"        movl    -52(%%rbp), %%eax\n");
  fprintf(f,"        movl    $0, -56(%%rbp)\n");
  fprintf(f,"        andl    $2146435072, %%eax\n");
  fprintf(f,"        addl    $1048576, %%eax\n");
  fprintf(f,"        movl    %%eax, -52(%%rbp)\n");
  fprintf(f,"        jmp     112f\n");
  fprintf(f,"94:\n");
  fprintf(f,"        testq   %%rax, -56(%%rbp)\n");
  fprintf(f,"        jne     96f\n");
  fprintf(f,"95:\n");
  fprintf(f,"        movl    -52(%%rbp), %%eax\n");
  fprintf(f,"        movl    $-1, -56(%%rbp)\n");
  fprintf(f,"        andl    $2146435072, %%eax\n");
  fprintf(f,"        subl    $1048576, %%eax\n");
  fprintf(f,"        orl     $1048575, %%eax\n");
  fprintf(f,"        movl    %%eax, -52(%%rbp)\n");
  fprintf(f,"        jmp     112f\n");
  fprintf(f,"96:\n");
  fprintf(f,"        testb   $1, -56(%%rbp)\n");
  fprintf(f,"        je      112f\n");
  fprintf(f,"        movsd   -56(%%rbp), %%xmm5\n");
  fprintf(f,"        testl   %%r13d, %%r13d\n");
  fprintf(f,"        movabsq $ulp.isra.0, %%rax\n");
  fprintf(f,"        movl    -52(%%rbp), %%edi\n");
  fprintf(f,"        movsd   %%xmm5, -120(%%rbp)\n");
  fprintf(f,"        je      77b\n");
  fprintf(f,"        callq   *%%rax\n");
  fprintf(f,"        addsd   -120(%%rbp), %%xmm0\n");
  fprintf(f,"        movsd   %%xmm0, -56(%%rbp)\n");
  fprintf(f,"        jmp     112f\n");
  fprintf(f,"97:\n");
  fprintf(f,"        movq    -96(%%rbp), %%rsi\n");
  fprintf(f,"        movq    %%r15, %%rdi\n");
  fprintf(f,"        movabsq $ratio, %%rax\n");
  fprintf(f,"        callq   *%%rax\n");
  fprintf(f,"        movabsq $.LdtoaC8, %%rax\n");
  fprintf(f,"        movsd   (%%rax), %%xmm1\n");
  fprintf(f,"        ucomisd %%xmm0, %%xmm1\n");
  fprintf(f,"        jb      99f\n");
  fprintf(f,"        testl   %%r13d, %%r13d\n");
  fprintf(f,"        jne     101f\n");
  fprintf(f,"        movabsq $4503599627370495, %%rcx\n");
  fprintf(f,"        movq    -56(%%rbp), %%rax\n");
  fprintf(f,"        testq   %%rcx, %%rax\n");
  fprintf(f,"        je      98f\n");
  fprintf(f,"        decq    %%rax\n");
  fprintf(f,"        jne     102f\n");
  fprintf(f,"        jmp     78b\n");
  fprintf(f,"98:\n");
  fprintf(f,"        movabsq $.LdtoaC3, %%rax\n");
  fprintf(f,"        movsd   (%%rax), %%xmm1\n");
  fprintf(f,"        movabsq $.LdtoaC1, %%rax\n");
  fprintf(f,"        ucomisd %%xmm0, %%xmm1\n");
  fprintf(f,"        movsd   (%%rax), %%xmm1\n");
  fprintf(f,"        ja      100f\n");
  fprintf(f,"        mulsd   %%xmm0, %%xmm1\n");
  fprintf(f,"        jmp     100f\n");
  fprintf(f,"99:\n");
  fprintf(f,"        movabsq $.LdtoaC1, %%rax\n");
  fprintf(f,"        testl   %%r13d, %%r13d\n");
  fprintf(f,"        movsd   (%%rax), %%xmm1\n");
  fprintf(f,"        mulsd   %%xmm0, %%xmm1\n");
  fprintf(f,"        movapd  %%xmm1, %%xmm2\n");
  fprintf(f,"        jne     103f\n");
  fprintf(f,"100:\n");
  fprintf(f,"        movabsq $.LdtoaC7, %%rax\n");
  fprintf(f,"        movaps  (%%rax), %%xmm2\n");
  fprintf(f,"        xorps   %%xmm1, %%xmm2\n");
  fprintf(f,"        jmp     103f\n");
  fprintf(f,"101:\n");
  fprintf(f,"        movabsq $.LdtoaC3, %%rax\n");
  fprintf(f,"        movsd   (%%rax), %%xmm2\n");
  fprintf(f,"        movapd  %%xmm2, %%xmm1\n");
  fprintf(f,"        jmp     103f\n");
  fprintf(f,"102:\n");
  fprintf(f,"        movabsq $.LdtoaC2, %%rax\n");
  fprintf(f,"        movsd   (%%rax), %%xmm2\n");
  fprintf(f,"        movabsq $.LdtoaC3, %%rax\n");
  fprintf(f,"        movsd   (%%rax), %%xmm1\n");
  fprintf(f,"103:\n");
  fprintf(f,"        movl    -52(%%rbp), %%edi\n");
  fprintf(f,"        movsd   -56(%%rbp), %%xmm6\n");
  fprintf(f,"        movsd   %%xmm6, -136(%%rbp)\n");
  fprintf(f,"        movl    %%edi, %%eax\n");
  fprintf(f,"        andl    $2146435072, %%eax\n");
  fprintf(f,"        cmpl    $2145386496, %%eax\n");
  fprintf(f,"        movl    %%eax, -128(%%rbp)\n");
  fprintf(f,"        jne     105f\n");
  fprintf(f,"        subl    $55574528, %%edi\n");
  fprintf(f,"        movsd   %%xmm2, -168(%%rbp)\n");
  fprintf(f,"        movabsq $ulp.isra.0, %%rax\n");
  fprintf(f,"        movsd   %%xmm1, -160(%%rbp)\n");
  fprintf(f,"        movl    %%edi, -52(%%rbp)\n");
  fprintf(f,"        callq   *%%rax\n");
  fprintf(f,"        movsd   -168(%%rbp), %%xmm2\n");
  fprintf(f,"        movsd   -160(%%rbp), %%xmm1\n");
  fprintf(f,"        mulsd   %%xmm2, %%xmm0\n");
  fprintf(f,"        addsd   -56(%%rbp), %%xmm0\n");
  fprintf(f,"        movsd   %%xmm0, -56(%%rbp)\n");
  fprintf(f,"        movl    -52(%%rbp), %%eax\n");
  fprintf(f,"        movl    %%eax, %%ecx\n");
  fprintf(f,"        andl    $2146435072, %%ecx\n");
  fprintf(f,"        cmpl    $2090860543, %%ecx\n");
  fprintf(f,"        jbe     104f\n");
  fprintf(f,"        movq    -136(%%rbp), %%rdx\n");
  fprintf(f,"        movabsq $9218868437227405311, %%rax\n");
  fprintf(f,"        cmpq    %%rax, %%rdx\n");
  fprintf(f,"        je      63b\n");
  fprintf(f,"        movl    $2146435071, -52(%%rbp)\n");
  fprintf(f,"        movl    $-1, -56(%%rbp)\n");
  fprintf(f,"        jmp     111f\n");
  fprintf(f,"104:\n");
  fprintf(f,"        addl    $55574528, %%eax\n");
  fprintf(f,"        movl    %%eax, -52(%%rbp)\n");
  fprintf(f,"        jmp     107f\n");
  fprintf(f,"105:\n");
  fprintf(f,"        cmpl    $54525952, -128(%%rbp)\n");
  fprintf(f,"        ja      106f\n");
  fprintf(f,"        movabsq $.LdtoaC3, %%rax\n");
  fprintf(f,"        movsd   (%%rax), %%xmm0\n");
  fprintf(f,"        ucomisd %%xmm0, %%xmm1\n");
  fprintf(f,"        jbe     106f\n");
  fprintf(f,"        movabsq $.LdtoaC1, %%rax\n");
  fprintf(f,"        testl   %%r13d, %%r13d\n");
  fprintf(f,"        movsd   (%%rax), %%xmm0\n");
  fprintf(f,"        addsd   %%xmm1, %%xmm0\n");
  fprintf(f,"        cvttsd2si       %%xmm0, %%eax\n");
  fprintf(f,"        cvtsi2sd        %%eax, %%xmm2\n");
  fprintf(f,"        jne     106f\n");
  fprintf(f,"        movabsq $.LdtoaC7, %%rax\n");
  fprintf(f,"        movaps  (%%rax), %%xmm0\n");
  fprintf(f,"        xorps   %%xmm0, %%xmm2\n");
  fprintf(f,"106:\n");
  fprintf(f,"        movabsq $ulp.isra.0, %%rax\n");
  fprintf(f,"        movsd   %%xmm2, -168(%%rbp)\n");
  fprintf(f,"        movsd   %%xmm1, -160(%%rbp)\n");
  fprintf(f,"        callq   *%%rax\n");
  fprintf(f,"        movsd   -168(%%rbp), %%xmm2\n");
  fprintf(f,"        movsd   -160(%%rbp), %%xmm1\n");
  fprintf(f,"        mulsd   %%xmm2, %%xmm0\n");
  fprintf(f,"        addsd   -136(%%rbp), %%xmm0\n");
  fprintf(f,"        movsd   %%xmm0, -56(%%rbp)\n");
  fprintf(f,"107:\n");
  fprintf(f,"        cmpl    %%r14d, -144(%%rbp)\n");
  fprintf(f,"        movl    -52(%%rbp), %%eax\n");
  fprintf(f,"        jne     111f\n");
  fprintf(f,"        andl    $2146435072, %%eax\n");
  fprintf(f,"        cmpl    %%eax, -128(%%rbp)\n");
  fprintf(f,"        jne     111f\n");
  fprintf(f,"        cvttsd2si       %%xmm1, %%eax\n");
  fprintf(f,"        testl   %%r13d, %%r13d\n");
  fprintf(f,"        cvtsi2sd        %%eax, %%xmm0\n");
  fprintf(f,"        subsd   %%xmm0, %%xmm1\n");
  fprintf(f,"        jne     108f\n");
  fprintf(f,"        movabsq $4503599627370495, %%rax\n");
  fprintf(f,"        testq   %%rax, -56(%%rbp)\n");
  fprintf(f,"        je      109f\n");
  fprintf(f,"108:\n");
  fprintf(f,"        movabsq $.LdtoaC9, %%rax\n");
  fprintf(f,"        movsd   (%%rax), %%xmm0\n");
  fprintf(f,"        ucomisd %%xmm1, %%xmm0\n");
  fprintf(f,"        ja      112f\n");
  fprintf(f,"        movabsq $.LdtoaC10, %%rax\n");
  fprintf(f,"        ucomisd (%%rax), %%xmm1\n");
  fprintf(f,"        jmp     110f\n");
  fprintf(f,"109:\n");
  fprintf(f,"        movabsq $.LdtoaC11, %%rax\n");
  fprintf(f,"        movsd   (%%rax), %%xmm0\n");
  fprintf(f,"        ucomisd %%xmm1, %%xmm0\n");
  fprintf(f,"110:\n");
  fprintf(f,"        ja      112f\n");
  fprintf(f,"111:\n");
  fprintf(f,"        movabsq $Bfree, %%r13\n");
  fprintf(f,"        movq    -104(%%rbp), %%rdi\n");
  fprintf(f,"        callq   *%%r13\n");
  fprintf(f,"        movq    -112(%%rbp), %%rdi\n");
  fprintf(f,"        callq   *%%r13\n");
  fprintf(f,"        movq    -96(%%rbp), %%rdi\n");
  fprintf(f,"        callq   *%%r13\n");
  fprintf(f,"        movq    %%r15, %%rdi\n");
  fprintf(f,"        callq   *%%r13\n");
  fprintf(f,"        jmp     82b\n");
  fprintf(f,"112:\n");
  fprintf(f,"        movabsq $Bfree, %%rbx\n");
  fprintf(f,"        movq    -104(%%rbp), %%rdi\n");
  fprintf(f,"        callq   *%%rbx\n");
  fprintf(f,"        movq    -112(%%rbp), %%rdi\n");
  fprintf(f,"        callq   *%%rbx\n");
  fprintf(f,"        movq    -96(%%rbp), %%rdi\n");
  fprintf(f,"        callq   *%%rbx\n");
  fprintf(f,"        movq    %%r12, %%rdi\n");
  fprintf(f,"        callq   *%%rbx\n");
  fprintf(f,"        movq    %%r15, %%rdi\n");
  fprintf(f,"        callq   *%%rbx\n");
  fprintf(f,"113:\n");
  fprintf(f,"        movq    -152(%%rbp), %%rcx\n");
  fprintf(f,"        testq   %%rcx, %%rcx\n");
  fprintf(f,"        je      114f\n");
  fprintf(f,"        movq    -64(%%rbp), %%rax\n");
  fprintf(f,"        movq    %%rax, (%%rcx)\n");
  fprintf(f,"114:\n");
  fprintf(f,"        cmpl    $0, -84(%%rbp)\n");
  fprintf(f,"        movsd   -56(%%rbp), %%xmm0\n");
  fprintf(f,"        je      115f\n");
  fprintf(f,"        movabsq $.LdtoaC7, %%rax\n");
  fprintf(f,"        movaps  (%%rax), %%xmm1\n");
  fprintf(f,"        xorps   %%xmm1, %%xmm0\n");
  fprintf(f,"115:\n");
  fprintf(f,"        addq    $136, %%rsp\n");
  fprintf(f,"        popq    %%rbx\n");
  fprintf(f,"        popq    %%r12\n");
  fprintf(f,"        popq    %%r13\n");
  fprintf(f,"        popq    %%r14\n");
  fprintf(f,"        popq    %%r15\n");
  fprintf(f,"        popq    %%rbp\n");
  fprintf(f,"        retq\n");
  fprintf(f,"        .size   strtod, .-strtod\n\n");

  fprintf(f,"        .type   freedtoa, @function\n");
  fprintf(f,"freedtoa:\n");
  fprintf(f,"        pushq   %%rbp\n");
  fprintf(f,"        movl    $1, %%eax\n");
  fprintf(f,"        movq    %%rsp, %%rbp\n");
  fprintf(f,"        pushq   %%rbx\n");
  fprintf(f,"        movq    %%rdi, %%rbx\n");
  fprintf(f,"        pushq   %%r10\n");
  fprintf(f,"        leaq    -4(%%rdi), %%rdi\n");
  fprintf(f,"        movl    (%%rdi), %%ecx\n");
  fprintf(f,"        sall    %%cl, %%eax\n");
  fprintf(f,"        movl    %%ecx, 8(%%rdi)\n");
  fprintf(f,"        movl    %%eax, 12(%%rdi)\n");
  fprintf(f,"        movabsq $Bfree, %%rax\n");
  fprintf(f,"        callq   *%%rax\n");
  fprintf(f,"        movabsq $dtoa_result, %%rax\n");
  fprintf(f,"        cmpq    (%%rax), %%rbx\n");
  fprintf(f,"        jne     0f\n");
  fprintf(f,"        movq    $0, (%%rax)\n");
  fprintf(f,"0:\n");
  fprintf(f,"        popq    %%r9\n");
  fprintf(f,"        popq    %%rbx\n");
  fprintf(f,"        popq    %%rbp\n");
  fprintf(f,"        retq\n");
  fprintf(f,"        .size   freedtoa, .-freedtoa\n");

  fprintf(f,"        .section .rodata.str1.1,\"aMS\",@progbits,1\n\n");

  fprintf(f,".LdtoaC12:\n");
  fprintf(f,"        .string \"Infinity\"\n");
  fprintf(f,".LdtoaC13:\n");
  fprintf(f,"        .string \"NaN\"\n");
  fprintf(f,".LdtoaC14:\n");
  fprintf(f,"        .string \"0\"\n");

  fprintf(f,"        .section .text,\"ax\",@progbits\n\n");

  fprintf(f,"        .type   dtoa, @function\n");
  fprintf(f,"dtoa:\n");
  fprintf(f,"        movabsq $dtoa_result, %%rax\n");
  fprintf(f,"        pushq   %%rbp\n");
  fprintf(f,"        movq    %%rsp, %%rbp\n");
  fprintf(f,"        pushq   %%r15\n");
  fprintf(f,"        pushq   %%r14\n");
  fprintf(f,"        pushq   %%r13\n");
  fprintf(f,"        pushq   %%r12\n");
  fprintf(f,"        movq    %%rcx, %%r12\n");
  fprintf(f,"        pushq   %%rbx\n");
  fprintf(f,"        subq    $120, %%rsp\n");
  fprintf(f,"        movl    %%edi, -96(%%rbp)\n");
  fprintf(f,"        movq    (%%rax), %%rdi\n");
  fprintf(f,"        movl    %%esi, -112(%%rbp)\n");
  fprintf(f,"        movq    %%rdx, -144(%%rbp)\n");
  fprintf(f,"        movq    %%r8, -136(%%rbp)\n");
  fprintf(f,"        testq   %%rdi, %%rdi\n");
  fprintf(f,"        je      0f\n");
  fprintf(f,"        movabsq $freedtoa, %%rax\n");
  fprintf(f,"        movsd   %%xmm0, -72(%%rbp)\n");
  fprintf(f,"        callq   *%%rax\n");
  fprintf(f,"        movsd   -72(%%rbp), %%xmm0\n");
  fprintf(f,"        movabsq $dtoa_result, %%rax\n");
  fprintf(f,"        movq    $0, (%%rax)\n");
  fprintf(f,"0:\n");
  fprintf(f,"        movsd   %%xmm0, -56(%%rbp)\n");
  fprintf(f,"        movl    -52(%%rbp), %%eax\n");
  fprintf(f,"        testl   %%eax, %%eax\n");
  fprintf(f,"        jns     1f\n");
  fprintf(f,"        andl    $2147483647, %%eax\n");
  fprintf(f,"        movl    $1, (%%r12)\n");
  fprintf(f,"        movl    %%eax, -52(%%rbp)\n");
  fprintf(f,"        jmp     2f\n");
  fprintf(f,"1:\n");
  fprintf(f,"        movl    $0, (%%r12)\n");
  fprintf(f,"2:\n");
  fprintf(f,"        movl    -52(%%rbp), %%eax\n");
  fprintf(f,"        andl    $2146435072, %%eax\n");
  fprintf(f,"        cmpl    $2146435072, %%eax\n");
  fprintf(f,"        jne     4f\n");
  fprintf(f,"        movq    -144(%%rbp), %%rax\n");
  fprintf(f,"        movl    $9999, (%%rax)\n");
  fprintf(f,"        movabsq $4503599627370495, %%rax\n");
  fprintf(f,"        testq   %%rax, -56(%%rbp)\n");
  fprintf(f,"        movabsq $nrv_alloc, %%rax\n");
  fprintf(f,"        jne     3f\n");
  fprintf(f,"        movl    $8, %%edx\n");
  fprintf(f,"        movq    -136(%%rbp), %%rsi\n");
  fprintf(f,"        movabsq $.LdtoaC12, %%rdi\n");
  fprintf(f,"        jmp     5f\n");
  fprintf(f,"3:\n");
  fprintf(f,"        movl    $3, %%edx\n");
  fprintf(f,"        movq    -136(%%rbp), %%rsi\n");
  fprintf(f,"        movabsq $.LdtoaC13, %%rdi\n");
  fprintf(f,"        jmp     5f\n");
  fprintf(f,"4:\n");
  fprintf(f,"        xorps   %%xmm2, %%xmm2\n");
  fprintf(f,"        ucomisd -56(%%rbp), %%xmm2\n");
  fprintf(f,"        jp      6f\n");
  fprintf(f,"        jne     6f\n");
  fprintf(f,"        movq    -144(%%rbp), %%rax\n");
  fprintf(f,"        movl    $1, %%edx\n");
  fprintf(f,"        movabsq $.LdtoaC14, %%rdi\n");
  fprintf(f,"        movq    -136(%%rbp), %%rsi\n");
  fprintf(f,"        movl    $1, (%%rax)\n");
  fprintf(f,"        movabsq $nrv_alloc, %%rax\n");
  fprintf(f,"5:\n");
  fprintf(f,"        callq   *%%rax\n");
  fprintf(f,"        jmp     114f\n");
  fprintf(f,"6:\n");
  fprintf(f,"        leaq    -64(%%rbp), %%rdx\n");
  fprintf(f,"        movsd   %%xmm2, -80(%%rbp)\n");
  fprintf(f,"        movabsq $d2b, %%rax\n");
  fprintf(f,"        leaq    -60(%%rbp), %%rsi\n");
  fprintf(f,"        leaq    -56(%%rbp), %%rdi\n");
  fprintf(f,"        callq   *%%rax\n");
  fprintf(f,"        movl    -52(%%rbp), %%edi\n");
  fprintf(f,"        movq    %%rax, -72(%%rbp)\n");
  fprintf(f,"        movl    -64(%%rbp), %%esi\n");
  fprintf(f,"        movsd   -80(%%rbp), %%xmm2\n");
  fprintf(f,"        movl    %%edi, %%eax\n");
  fprintf(f,"        shrl    $20, %%eax\n");
  fprintf(f,"        andl    $2047, %%eax\n");
  fprintf(f,"        movl    %%eax, %%edx\n");
  fprintf(f,"        je      7f\n");
  fprintf(f,"        movq    -56(%%rbp), %%rcx\n");
  fprintf(f,"        xorl    %%r10d, %%r10d\n");
  fprintf(f,"        movq    %%rcx, %%rax\n");
  fprintf(f,"        movl    %%ecx, %%ecx\n");
  fprintf(f,"        shrq    $32, %%rax\n");
  fprintf(f,"        andl    $1048575, %%eax\n");
  fprintf(f,"        orq     $1072693248, %%rax\n");
  fprintf(f,"        salq    $32, %%rax\n");
  fprintf(f,"        orq     %%rax, %%rcx\n");
  fprintf(f,"        leal    -1023(%%rdx), %%eax\n");
  fprintf(f,"        jmp     10f\n");
  fprintf(f,"7:\n");
  fprintf(f,"        movl    -60(%%rbp), %%eax\n");
  fprintf(f,"        movl    -56(%%rbp), %%edx\n");
  fprintf(f,"        addl    %%esi, %%eax\n");
  fprintf(f,"        cmpl    $-1041, %%eax\n");
  fprintf(f,"        jl      8f\n");
  fprintf(f,"        leal    1042(%%rax), %%ecx\n");
  fprintf(f,"        shrl    %%cl, %%edx\n");
  fprintf(f,"        movl    $-1010, %%ecx\n");
  fprintf(f,"        subl    %%eax, %%ecx\n");
  fprintf(f,"        sall    %%cl, %%edi\n");
  fprintf(f,"        orl     %%edi, %%edx\n");
  fprintf(f,"        jmp     9f\n");
  fprintf(f,"8:\n");
  fprintf(f,"        movl    $-1042, %%ecx\n");
  fprintf(f,"        subl    %%eax, %%ecx\n");
  fprintf(f,"        sall    %%cl, %%edx\n");
  fprintf(f,"9:\n");
  fprintf(f,"        movl    %%edx, %%ecx\n");
  fprintf(f,"        decl    %%eax\n");
  fprintf(f,"        movl    $1, %%r10d\n");
  fprintf(f,"        cvtsi2sdq       %%rcx, %%xmm6\n");
  fprintf(f,"        movd    %%xmm6, %%rdx\n");
  fprintf(f,"        movd    %%xmm6, %%rcx\n");
  fprintf(f,"        shrq    $32, %%rdx\n");
  fprintf(f,"        movl    %%ecx, %%ecx\n");
  fprintf(f,"        subl    $32505856, %%edx\n");
  fprintf(f,"        salq    $32, %%rdx\n");
  fprintf(f,"        orq     %%rdx, %%rcx\n");
  fprintf(f,"10:\n");
  fprintf(f,"        movabsq $.LdtoaC15, %%rdx\n");
  fprintf(f,"        movd    %%rcx, %%xmm0\n");
  fprintf(f,"        subsd   (%%rdx), %%xmm0\n");
  fprintf(f,"        movabsq $.LdtoaC16, %%rdx\n");
  fprintf(f,"        cvtsi2sd        %%eax, %%xmm1\n");
  fprintf(f,"        mulsd   (%%rdx), %%xmm0\n");
  fprintf(f,"        movabsq $.LdtoaC17, %%rdx\n");
  fprintf(f,"        addsd   (%%rdx), %%xmm0\n");
  fprintf(f,"        movabsq $.LdtoaC18, %%rdx\n");
  fprintf(f,"        mulsd   (%%rdx), %%xmm1\n");
  fprintf(f,"        addsd   %%xmm1, %%xmm0\n");
  fprintf(f,"        ucomisd %%xmm0, %%xmm2\n");
  fprintf(f,"        cvttsd2si       %%xmm0, %%r14d\n");
  fprintf(f,"        jbe     12f\n");
  fprintf(f,"        cvtsi2sd        %%r14d, %%xmm1\n");
  fprintf(f,"        ucomisd %%xmm1, %%xmm0\n");
  fprintf(f,"        jp      11f\n");
  fprintf(f,"        je      12f\n");
  fprintf(f,"11:\n");
  fprintf(f,"        decl    %%r14d\n");
  fprintf(f,"12:\n");
  fprintf(f,"        cmpl    $22, %%r14d\n");
  fprintf(f,"        movl    $1, -120(%%rbp)\n");
  fprintf(f,"        ja      14f\n");
  fprintf(f,"        movabsq $tens, %%rdx\n");
  fprintf(f,"        movslq  %%r14d, %%rcx\n");
  fprintf(f,"        movsd   (%%rdx,%%rcx,8), %%xmm0\n");
  fprintf(f,"        ucomisd -56(%%rbp), %%xmm0\n");
  fprintf(f,"        jbe     13f\n");
  fprintf(f,"        decl    %%r14d\n");
  fprintf(f,"13:\n");
  fprintf(f,"        movl    $0, -120(%%rbp)\n");
  fprintf(f,"14:\n");
  fprintf(f,"        subl    %%eax, %%esi\n");
  fprintf(f,"        movl    $0, -88(%%rbp)\n");
  fprintf(f,"        decl    %%esi\n");
  fprintf(f,"        movl    %%esi, -104(%%rbp)\n");
  fprintf(f,"        jns     15f\n");
  fprintf(f,"        movl    %%esi, %%eax\n");
  fprintf(f,"        movl    $0, -104(%%rbp)\n");
  fprintf(f,"        negl    %%eax\n");
  fprintf(f,"        movl    %%eax, -88(%%rbp)\n");
  fprintf(f,"15:\n");
  fprintf(f,"        testl   %%r14d, %%r14d\n");
  fprintf(f,"        js      16f\n");
  fprintf(f,"        addl    %%r14d, -104(%%rbp)\n");
  fprintf(f,"        movl    %%r14d, -148(%%rbp)\n");
  fprintf(f,"        xorl    %%r12d, %%r12d\n");
  fprintf(f,"        jmp     17f\n");
  fprintf(f,"16:\n");
  fprintf(f,"        subl    %%r14d, -88(%%rbp)\n");
  fprintf(f,"        movl    %%r14d, %%r12d\n");
  fprintf(f,"        movl    $0, -148(%%rbp)\n");
  fprintf(f,"        negl    %%r12d\n");
  fprintf(f,"17:\n");
  fprintf(f,"        cmpl    $9, -96(%%rbp)\n");
  fprintf(f,"        movl    $1, %%r15d\n");
  fprintf(f,"        ja      23f\n");
  fprintf(f,"        cmpl    $5, -96(%%rbp)\n");
  fprintf(f,"        jle     18f\n");
  fprintf(f,"        subl    $4, -96(%%rbp)\n");
  fprintf(f,"        xorb    %%r15b, %%r15b\n");
  fprintf(f,"18:\n");
  fprintf(f,"        cmpl    $3, -96(%%rbp)\n");
  fprintf(f,"        je      21f\n");
  fprintf(f,"        jg      19f\n");
  fprintf(f,"        xorl    %%r13d, %%r13d\n");
  fprintf(f,"        cmpl    $2, -96(%%rbp)\n");
  fprintf(f,"        je      20f\n");
  fprintf(f,"        jmp     24f\n");
  fprintf(f,"19:\n");
  fprintf(f,"        movl    -96(%%rbp), %%eax\n");
  fprintf(f,"        movl    $1, %%r13d\n");
  fprintf(f,"        cmpl    $4, %%eax\n");
  fprintf(f,"        je      20f\n");
  fprintf(f,"        cmpl    $5, %%eax\n");
  fprintf(f,"        je      22f\n");
  fprintf(f,"        jmp     24f\n");
  fprintf(f,"20:\n");
  fprintf(f,"        movl    -112(%%rbp), %%eax\n");
  fprintf(f,"        testl   %%eax, %%eax\n");
  fprintf(f,"        jle     25f\n");
  fprintf(f,"        movl    %%eax, -116(%%rbp)\n");
  fprintf(f,"        movl    %%eax, %%edi\n");
  fprintf(f,"        movl    %%eax, -84(%%rbp)\n");
  fprintf(f,"        jmp     26f\n");
  fprintf(f,"21:\n");
  fprintf(f,"        xorl    %%r13d, %%r13d\n");
  fprintf(f,"22:\n");
  fprintf(f,"        movl    -112(%%rbp), %%eax\n");
  fprintf(f,"        movl    $1, %%edi\n");
  fprintf(f,"        addl    %%r14d, %%eax\n");
  fprintf(f,"        movl    %%eax, -116(%%rbp)\n");
  fprintf(f,"        incl    %%eax\n");
  fprintf(f,"        testl   %%eax, %%eax\n");
  fprintf(f,"        movl    %%eax, -84(%%rbp)\n");
  fprintf(f,"        cmovg   %%eax, %%edi\n");
  fprintf(f,"        jmp     26f\n");
  fprintf(f,"23:\n");
  fprintf(f,"        movl    $0, -96(%%rbp)\n");
  fprintf(f,"24:\n");
  fprintf(f,"        orl     $-1, %%eax\n");
  fprintf(f,"        movl    $1, %%r13d\n");
  fprintf(f,"        movl    $18, %%edi\n");
  fprintf(f,"        movl    %%eax, -116(%%rbp)\n");
  fprintf(f,"        movl    %%eax, -84(%%rbp)\n");
  fprintf(f,"        movl    $0, -112(%%rbp)\n");
  fprintf(f,"        jmp     26f\n");
  fprintf(f,"25:\n");
  fprintf(f,"        movl    $1, -116(%%rbp)\n");
  fprintf(f,"        movl    $1, %%edi\n");
  fprintf(f,"        movl    $1, -84(%%rbp)\n");
  fprintf(f,"        movl    $1, -112(%%rbp)\n");
  fprintf(f,"26:\n");
  fprintf(f,"        movabsq $rv_alloc, %%rax\n");
  fprintf(f,"        movsd   %%xmm2, -160(%%rbp)\n");
  fprintf(f,"        movl    %%r10d, -128(%%rbp)\n");
  fprintf(f,"        callq   *%%rax\n");
  fprintf(f,"        cmpl    $14, -84(%%rbp)\n");
  fprintf(f,"        movq    %%rax, -80(%%rbp)\n");
  fprintf(f,"        movl    -128(%%rbp), %%r10d\n");
  fprintf(f,"        movsd   -160(%%rbp), %%xmm2\n");
  fprintf(f,"        ja      50f\n");
  fprintf(f,"        testl   %%r15d, %%r15d\n");
  fprintf(f,"        je      50f\n");
  fprintf(f,"        testl   %%r14d, %%r14d\n");
  fprintf(f,"        movsd   -56(%%rbp), %%xmm3\n");
  fprintf(f,"        jle     31f\n");
  fprintf(f,"        movabsq $tens, %%rax\n");
  fprintf(f,"        movl    %%r14d, %%edx\n");
  fprintf(f,"        andl    $15, %%edx\n");
  fprintf(f,"        movsd   (%%rax,%%rdx,8), %%xmm1\n");
  fprintf(f,"        movl    %%r14d, %%eax\n");
  fprintf(f,"        movl    $2, %%edx\n");
  fprintf(f,"        sarl    $4, %%eax\n");
  fprintf(f,"        testb   $16, %%al\n");
  fprintf(f,"        je      27f\n");
  fprintf(f,"        movabsq $.LdtoaC19, %%rdx\n");
  fprintf(f,"        movapd  %%xmm3, %%xmm0\n");
  fprintf(f,"        andl    $15, %%eax\n");
  fprintf(f,"        divsd   (%%rdx), %%xmm0\n");
  fprintf(f,"        movl    $3, %%edx\n");
  fprintf(f,"        movsd   %%xmm0, -56(%%rbp)\n");
  fprintf(f,"27:\n");
  fprintf(f,"        movabsq $bigtens, %%rcx\n");
  fprintf(f,"28:\n");
  fprintf(f,"        testl   %%eax, %%eax\n");
  fprintf(f,"        je      30f\n");
  fprintf(f,"        testb   $1, %%al\n");
  fprintf(f,"        je      29f\n");
  fprintf(f,"        mulsd   (%%rcx), %%xmm1\n");
  fprintf(f,"        incl    %%edx\n");
  fprintf(f,"29:\n");
  fprintf(f,"        sarl    %%eax\n");
  fprintf(f,"        addq    $8, %%rcx\n");
  fprintf(f,"        jmp     28b\n");
  fprintf(f,"30:\n");
  fprintf(f,"        movsd   -56(%%rbp), %%xmm0\n");
  fprintf(f,"        divsd   %%xmm1, %%xmm0\n");
  fprintf(f,"        jmp     34f\n");
  fprintf(f,"31:\n");
  fprintf(f,"        movl    %%r14d, %%ebx\n");
  fprintf(f,"        movl    $2, %%edx\n");
  fprintf(f,"        negl    %%ebx\n");
  fprintf(f,"        je      35f\n");
  fprintf(f,"        movabsq $tens, %%rsi\n");
  fprintf(f,"        movl    %%ebx, %%edi\n");
  fprintf(f,"        movl    %%ebx, %%eax\n");
  fprintf(f,"        movabsq $bigtens, %%rcx\n");
  fprintf(f,"        andl    $15, %%edi\n");
  fprintf(f,"        sarl    $4, %%eax\n");
  fprintf(f,"        movsd   (%%rsi,%%rdi,8), %%xmm0\n");
  fprintf(f,"        mulsd   %%xmm3, %%xmm0\n");
  fprintf(f,"32:\n");
  fprintf(f,"        testl   %%eax, %%eax\n");
  fprintf(f,"        je      34f\n");
  fprintf(f,"        testb   $1, %%al\n");
  fprintf(f,"        je      33f\n");
  fprintf(f,"        mulsd   (%%rcx), %%xmm0\n");
  fprintf(f,"        incl    %%edx\n");
  fprintf(f,"33:\n");
  fprintf(f,"        sarl    %%eax\n");
  fprintf(f,"        addq    $8, %%rcx\n");
  fprintf(f,"        jmp     32b\n");
  fprintf(f,"34:\n");
  fprintf(f,"        movsd   %%xmm0, -56(%%rbp)\n");
  fprintf(f,"35:\n");
  fprintf(f,"        cmpl    $0, -120(%%rbp)\n");
  fprintf(f,"        je      36f\n");
  fprintf(f,"        movabsq $.LdtoaC3, %%rax\n");
  fprintf(f,"        movsd   -56(%%rbp), %%xmm0\n");
  fprintf(f,"        movsd   (%%rax), %%xmm1\n");
  fprintf(f,"        ucomisd %%xmm0, %%xmm1\n");
  fprintf(f,"        jbe     36f\n");
  fprintf(f,"        cmpl    $0, -84(%%rbp)\n");
  fprintf(f,"        je      37f\n");
  fprintf(f,"        movl    -116(%%rbp), %%ecx\n");
  fprintf(f,"        testl   %%ecx, %%ecx\n");
  fprintf(f,"        jle     49f\n");
  fprintf(f,"        movabsq $.LdtoaC20, %%rax\n");
  fprintf(f,"        incl    %%edx\n");
  fprintf(f,"        mulsd   (%%rax), %%xmm0\n");
  fprintf(f,"        movl    %%ecx, %%eax\n");
  fprintf(f,"        leal    -1(%%r14), %%esi\n");
  fprintf(f,"        movsd   %%xmm0, -56(%%rbp)\n");
  fprintf(f,"        jmp     38f\n");
  fprintf(f,"36:\n");
  fprintf(f,"        movl    %%r14d, %%esi\n");
  fprintf(f,"        movl    -84(%%rbp), %%eax\n");
  fprintf(f,"        jmp     38f\n");
  fprintf(f,"37:\n");
  fprintf(f,"        movl    %%r14d, %%esi\n");
  fprintf(f,"        xorl    %%eax, %%eax\n");
  fprintf(f,"38:\n");
  fprintf(f,"        cvtsi2sd        %%edx, %%xmm1\n");
  fprintf(f,"        movsd   -56(%%rbp), %%xmm0\n");
  fprintf(f,"        movabsq $.LdtoaC21, %%rdx\n");
  fprintf(f,"        mulsd   %%xmm0, %%xmm1\n");
  fprintf(f,"        addsd   (%%rdx), %%xmm1\n");
  fprintf(f,"        movsd   %%xmm1, -128(%%rbp)\n");
  fprintf(f,"        movq    -128(%%rbp), %%rdx\n");
  fprintf(f,"        movl    -128(%%rbp), %%ecx\n");
  fprintf(f,"        shrq    $32, %%rdx\n");
  fprintf(f,"        subl    $54525952, %%edx\n");
  fprintf(f,"        movq    %%rcx, -128(%%rbp)\n");
  fprintf(f,"        salq    $32, %%rdx\n");
  fprintf(f,"        orq     %%rdx, -128(%%rbp)\n");
  fprintf(f,"        testl   %%eax, %%eax\n");
  fprintf(f,"        jne     39f\n");
  fprintf(f,"        movabsq $.LdtoaC22, %%rax\n");
  fprintf(f,"        movsd   -128(%%rbp), %%xmm1\n");
  fprintf(f,"        subsd   (%%rax), %%xmm0\n");
  fprintf(f,"        ucomisd %%xmm1, %%xmm0\n");
  fprintf(f,"        movsd   %%xmm0, -56(%%rbp)\n");
  fprintf(f,"        ja      76f\n");
  fprintf(f,"        movabsq $.LdtoaC7, %%rax\n");
  fprintf(f,"        movaps  (%%rax), %%xmm4\n");
  fprintf(f,"        xorps   %%xmm4, %%xmm1\n");
  fprintf(f,"        ucomisd %%xmm0, %%xmm1\n");
  fprintf(f,"        ja      74f\n");
  fprintf(f,"        jmp     49f\n");
  fprintf(f,"39:\n");
  fprintf(f,"        leal    -1(%%rax), %%ecx\n");
  fprintf(f,"        testl   %%r13d, %%r13d\n");
  fprintf(f,"        movabsq $tens, %%rdx\n");
  fprintf(f,"        movslq  %%ecx, %%rcx\n");
  fprintf(f,"        je      44f\n");
  fprintf(f,"        movabsq $.LdtoaC1, %%rdi\n");
  fprintf(f,"        cmpl    $306, %%ebx\n");
  fprintf(f,"        movsd   (%%rdi), %%xmm0\n");
  fprintf(f,"        divsd   (%%rdx,%%rcx,8), %%xmm0\n");
  fprintf(f,"        subsd   -128(%%rbp), %%xmm0\n");
  fprintf(f,"        movsd   %%xmm0, -128(%%rbp)\n");
  fprintf(f,"        jle     42f\n");
  fprintf(f,"        movl    %%r14d, %%ecx\n");
  fprintf(f,"        shrl    $31, %%ecx\n");
  fprintf(f,"        je      42f\n");
  fprintf(f,"        movabsq $.LdtoaC24, %%r11\n");
  fprintf(f,"        movl    %%ebx, %%edi\n");
  fprintf(f,"        subl    $256, %%ebx\n");
  fprintf(f,"        movabsq $bigtens, %%rcx\n");
  fprintf(f,"        andl    $15, %%edi\n");
  fprintf(f,"        sarl    $4, %%ebx\n");
  fprintf(f,"        movsd   (%%rdx,%%rdi,8), %%xmm1\n");
  fprintf(f,"        mulsd   (%%r11), %%xmm1\n");
  fprintf(f,"40:\n");
  fprintf(f,"        testb   $1, %%bl\n");
  fprintf(f,"        je      41f\n");
  fprintf(f,"        mulsd   (%%rcx), %%xmm1\n");
  fprintf(f,"41:\n");
  fprintf(f,"        addq    $8, %%rcx\n");
  fprintf(f,"        sarl    %%ebx\n");
  fprintf(f,"        jne     40b\n");
  fprintf(f,"        cmpnltsd        %%xmm1, %%xmm0\n");
  fprintf(f,"        movsd   -128(%%rbp), %%xmm4\n");
  fprintf(f,"        andps   %%xmm0, %%xmm4\n");
  fprintf(f,"        andnps  %%xmm1, %%xmm0\n");
  fprintf(f,"        orps    %%xmm4, %%xmm0\n");
  fprintf(f,"        movsd   %%xmm0, -128(%%rbp)\n");
  fprintf(f,"42:\n");
  fprintf(f,"        movabsq $.LdtoaC3, %%rbx\n");
  fprintf(f,"        movq    -80(%%rbp), %%r15\n");
  fprintf(f,"        movabsq $.LdtoaC20, %%rdi\n");
  fprintf(f,"        movsd   -128(%%rbp), %%xmm0\n");
  fprintf(f,"        movsd   (%%rbx), %%xmm5\n");
  fprintf(f,"43:\n");
  fprintf(f,"        movsd   -56(%%rbp), %%xmm1\n");
  fprintf(f,"        incq    %%r15\n");
  fprintf(f,"        cvttsd2si       %%xmm1, %%edx\n");
  fprintf(f,"        cvtsi2sd        %%edx, %%xmm4\n");
  fprintf(f,"        addl    $48, %%edx\n");
  fprintf(f,"        subsd   %%xmm4, %%xmm1\n");
  fprintf(f,"        movapd  %%xmm5, %%xmm4\n");
  fprintf(f,"        movsd   %%xmm1, -56(%%rbp)\n");
  fprintf(f,"        movb    %%dl, -1(%%r15)\n");
  fprintf(f,"        movsd   -56(%%rbp), %%xmm1\n");
  fprintf(f,"        subsd   %%xmm1, %%xmm4\n");
  fprintf(f,"        ucomisd %%xmm4, %%xmm0\n");
  fprintf(f,"        ja      55f\n");
  fprintf(f,"        ucomisd %%xmm1, %%xmm0\n");
  fprintf(f,"        ja      111f\n");
  fprintf(f,"        movl    %%r15d, %%edx\n");
  fprintf(f,"        subl    -80(%%rbp), %%edx\n");
  fprintf(f,"        cmpl    %%eax, %%edx\n");
  fprintf(f,"        jge     49f\n");
  fprintf(f,"        movsd   (%%rdi), %%xmm4\n");
  fprintf(f,"        mulsd   %%xmm4, %%xmm1\n");
  fprintf(f,"        mulsd   %%xmm4, %%xmm0\n");
  fprintf(f,"        movsd   %%xmm1, -56(%%rbp)\n");
  fprintf(f,"        jmp     43b\n");
  fprintf(f,"44:\n");
  fprintf(f,"        movq    -80(%%rbp), %%rbx\n");
  fprintf(f,"        xorps   %%xmm1, %%xmm1\n");
  fprintf(f,"        movabsq $.LdtoaC20, %%r9\n");
  fprintf(f,"        movsd   -128(%%rbp), %%xmm4\n");
  fprintf(f,"        mulsd   (%%rdx,%%rcx,8), %%xmm4\n");
  fprintf(f,"        movl    $1, %%edx\n");
  fprintf(f,"        movq    %%rbx, %%r15\n");
  fprintf(f,"        subl    %%ebx, %%edx\n");
  fprintf(f,"45:\n");
  fprintf(f,"        movsd   -56(%%rbp), %%xmm0\n");
  fprintf(f,"        leal    (%%rdx,%%r15), %%edi\n");
  fprintf(f,"        cvttsd2si       %%xmm0, %%r8d\n");
  fprintf(f,"        cvtsi2sd        %%r8d, %%xmm5\n");
  fprintf(f,"        subsd   %%xmm5, %%xmm0\n");
  fprintf(f,"        ucomisd %%xmm2, %%xmm0\n");
  fprintf(f,"        movsd   %%xmm0, -56(%%rbp)\n");
  fprintf(f,"        jp      46f\n");
  fprintf(f,"        ucomisd %%xmm1, %%xmm0\n");
  fprintf(f,"        cmove   %%edi, %%eax\n");
  fprintf(f,"46:\n");
  fprintf(f,"        incq    %%r15\n");
  fprintf(f,"        addl    $48, %%r8d\n");
  fprintf(f,"        cmpl    %%eax, %%edi\n");
  fprintf(f,"        movb    %%r8b, -1(%%r15)\n");
  fprintf(f,"        movsd   -56(%%rbp), %%xmm0\n");
  fprintf(f,"        jne     48f\n");
  fprintf(f,"        movabsq $.LdtoaC1, %%rax\n");
  fprintf(f,"        movapd  %%xmm4, %%xmm5\n");
  fprintf(f,"        movsd   (%%rax), %%xmm1\n");
  fprintf(f,"        addsd   %%xmm1, %%xmm5\n");
  fprintf(f,"        ucomisd %%xmm5, %%xmm0\n");
  fprintf(f,"        ja      55f\n");
  fprintf(f,"        subsd   %%xmm4, %%xmm1\n");
  fprintf(f,"        ucomisd %%xmm0, %%xmm1\n");
  fprintf(f,"        jbe     49f\n");
  fprintf(f,"47:\n");
  fprintf(f,"        cmpb    $48, -1(%%r15)\n");
  fprintf(f,"        leaq    -1(%%r15), %%rax\n");
  fprintf(f,"        jne     111f\n");
  fprintf(f,"        movq    %%rax, %%r15\n");
  fprintf(f,"        jmp     47b\n");
  fprintf(f,"48:\n");
  fprintf(f,"        mulsd   (%%r9), %%xmm0\n");
  fprintf(f,"        movsd   %%xmm0, -56(%%rbp)\n");
  fprintf(f,"        jmp     45b\n");
  fprintf(f,"49:\n");
  fprintf(f,"        movsd   %%xmm3, -56(%%rbp)\n");
  fprintf(f,"50:\n");
  fprintf(f,"        movl    -60(%%rbp), %%eax\n");
  fprintf(f,"        testl   %%eax, %%eax\n");
  fprintf(f,"        js      60f\n");
  fprintf(f,"        cmpl    $14, %%r14d\n");
  fprintf(f,"        jg      60f\n");
  fprintf(f,"        cmpl    $0, -84(%%rbp)\n");
  fprintf(f,"        movslq  %%r14d, %%rdx\n");
  fprintf(f,"        movabsq $tens, %%rax\n");
  fprintf(f,"        movsd   (%%rax,%%rdx,8), %%xmm1\n");
  fprintf(f,"        jg      51f\n");
  fprintf(f,"        movl    -112(%%rbp), %%eax\n");
  fprintf(f,"        shrl    $31, %%eax\n");
  fprintf(f,"        je      51f\n");
  fprintf(f,"        cmpl    $0, -84(%%rbp)\n");
  fprintf(f,"        jne     74f\n");
  fprintf(f,"        movabsq $.LdtoaC22, %%rax\n");
  fprintf(f,"        xorl    %%r12d, %%r12d\n");
  fprintf(f,"        xorl    %%ebx, %%ebx\n");
  fprintf(f,"        mulsd   (%%rax), %%xmm1\n");
  fprintf(f,"        ucomisd -56(%%rbp), %%xmm1\n");
  fprintf(f,"        jae     75f\n");
  fprintf(f,"        jmp     77f\n");
  fprintf(f,"51:\n");
  fprintf(f,"        movq    -80(%%rbp), %%r15\n");
  fprintf(f,"        xorps   %%xmm4, %%xmm4\n");
  fprintf(f,"        movabsq $.LdtoaC20, %%rdx\n");
  fprintf(f,"52:\n");
  fprintf(f,"        movsd   -56(%%rbp), %%xmm0\n");
  fprintf(f,"        movapd  %%xmm0, %%xmm3\n");
  fprintf(f,"        divsd   %%xmm1, %%xmm3\n");
  fprintf(f,"        cvttsd2si       %%xmm3, %%eax\n");
  fprintf(f,"        cvtsi2sd        %%eax, %%xmm3\n");
  fprintf(f,"        mulsd   %%xmm1, %%xmm3\n");
  fprintf(f,"        subsd   %%xmm3, %%xmm0\n");
  fprintf(f,"        ucomisd %%xmm0, %%xmm2\n");
  fprintf(f,"        jbe     53f\n");
  fprintf(f,"        addsd   %%xmm1, %%xmm0\n");
  fprintf(f,"        decl    %%eax\n");
  fprintf(f,"53:\n");
  fprintf(f,"        leal    48(%%rax), %%esi\n");
  fprintf(f,"        incq    %%r15\n");
  fprintf(f,"        movsd   %%xmm0, -56(%%rbp)\n");
  fprintf(f,"        movb    %%sil, -1(%%r15)\n");
  fprintf(f,"        movsd   -56(%%rbp), %%xmm0\n");
  fprintf(f,"        ucomisd %%xmm2, %%xmm0\n");
  fprintf(f,"        jp      54f\n");
  fprintf(f,"        ucomisd %%xmm4, %%xmm0\n");
  fprintf(f,"        je      112f\n");
  fprintf(f,"54:\n");
  fprintf(f,"        movl    %%r15d, %%esi\n");
  fprintf(f,"        subl    -80(%%rbp), %%esi\n");
  fprintf(f,"        cmpl    -84(%%rbp), %%esi\n");
  fprintf(f,"        jne     59f\n");
  fprintf(f,"        addsd   %%xmm0, %%xmm0\n");
  fprintf(f,"        ucomisd %%xmm1, %%xmm0\n");
  fprintf(f,"        movsd   %%xmm0, -56(%%rbp)\n");
  fprintf(f,"        ja      56f\n");
  fprintf(f,"        jp      112f\n");
  fprintf(f,"        jne     112f\n");
  fprintf(f,"        testb   $1, %%al\n");
  fprintf(f,"        jne     56f\n");
  fprintf(f,"        jmp     112f\n");
  fprintf(f,"55:\n");
  fprintf(f,"        movl    %%esi, %%r14d\n");
  fprintf(f,"56:\n");
  fprintf(f,"        cmpb    $57, -1(%%r15)\n");
  fprintf(f,"        leaq    -1(%%r15), %%rax\n");
  fprintf(f,"        jne     58f\n");
  fprintf(f,"        cmpq    -80(%%rbp), %%rax\n");
  fprintf(f,"        je      57f\n");
  fprintf(f,"        movq    %%rax, %%r15\n");
  fprintf(f,"        jmp     56b\n");
  fprintf(f,"57:\n");
  fprintf(f,"        movq    -80(%%rbp), %%rbx\n");
  fprintf(f,"        incl    %%r14d\n");
  fprintf(f,"        movb    $48, (%%rbx)\n");
  fprintf(f,"58:\n");
  fprintf(f,"        incb    (%%rax)\n");
  fprintf(f,"        jmp     112f\n");
  fprintf(f,"59:\n");
  fprintf(f,"        mulsd   (%%rdx), %%xmm0\n");
  fprintf(f,"        movsd   %%xmm0, -56(%%rbp)\n");
  fprintf(f,"        jmp     52b\n");
  fprintf(f,"60:\n");
  fprintf(f,"        testl   %%r13d, %%r13d\n");
  fprintf(f,"        je      62f\n");
  fprintf(f,"        addl    $1075, %%eax\n");
  fprintf(f,"        testl   %%r10d, %%r10d\n");
  fprintf(f,"        jne     61f\n");
  fprintf(f,"        movl    $54, %%eax\n");
  fprintf(f,"        subl    -64(%%rbp), %%eax\n");
  fprintf(f,"61:\n");
  fprintf(f,"        movl    -88(%%rbp), %%ebx\n");
  fprintf(f,"        movl    $1, %%edi\n");
  fprintf(f,"        addl    %%eax, -104(%%rbp)\n");
  fprintf(f,"        leal    (%%rbx,%%rax), %%r15d\n");
  fprintf(f,"        movabsq $i2b, %%rax\n");
  fprintf(f,"        callq   *%%rax\n");
  fprintf(f,"        movq    %%rax, %%rbx\n");
  fprintf(f,"        jmp     63f\n");
  fprintf(f,"62:\n");
  fprintf(f,"        movl    -88(%%rbp), %%r15d\n");
  fprintf(f,"        xorl    %%ebx, %%ebx\n");
  fprintf(f,"63:\n");
  fprintf(f,"        cmpl    $0, -104(%%rbp)\n");
  fprintf(f,"        jle     64f\n");
  fprintf(f,"        cmpl    $0, -88(%%rbp)\n");
  fprintf(f,"        jle     64f\n");
  fprintf(f,"        movl    -88(%%rbp), %%eax\n");
  fprintf(f,"        movl    -104(%%rbp), %%ecx\n");
  fprintf(f,"        cmpl    %%eax, %%ecx\n");
  fprintf(f,"        cmovle  %%ecx, %%eax\n");
  fprintf(f,"        subl    %%eax, -88(%%rbp)\n");
  fprintf(f,"        subl    %%eax, %%r15d\n");
  fprintf(f,"        subl    %%eax, -104(%%rbp)\n");
  fprintf(f,"64:\n");
  fprintf(f,"        testl   %%r12d, %%r12d\n");
  fprintf(f,"        je      66f\n");
  fprintf(f,"        movabsq $pow5mult, %%rax\n");
  fprintf(f,"        testl   %%r13d, %%r13d\n");
  fprintf(f,"        movl    %%r12d, %%esi\n");
  fprintf(f,"        je      65f\n");
  fprintf(f,"        movq    %%rbx, %%rdi\n");
  fprintf(f,"        callq   *%%rax\n");
  fprintf(f,"        movq    -72(%%rbp), %%rsi\n");
  fprintf(f,"        movq    %%rax, %%rdi\n");
  fprintf(f,"        movq    %%rax, %%rbx\n");
  fprintf(f,"        movabsq $mult, %%rax\n");
  fprintf(f,"        callq   *%%rax\n");
  fprintf(f,"        movq    -72(%%rbp), %%rdi\n");
  fprintf(f,"        movq    %%rax, %%r12\n");
  fprintf(f,"        movabsq $Bfree, %%rax\n");
  fprintf(f,"        callq   *%%rax\n");
  fprintf(f,"        movq    %%r12, -72(%%rbp)\n");
  fprintf(f,"        jmp     66f\n");
  fprintf(f,"65:\n");
  fprintf(f,"        movq    -72(%%rbp), %%rdi\n");
  fprintf(f,"        callq   *%%rax\n");
  fprintf(f,"        movq    %%rax, -72(%%rbp)\n");
  fprintf(f,"66:\n");
  fprintf(f,"        movabsq $i2b, %%rax\n");
  fprintf(f,"        movl    $1, %%edi\n");
  fprintf(f,"        callq   *%%rax\n");
  fprintf(f,"        movl    -148(%%rbp), %%esi\n");
  fprintf(f,"        movq    %%rax, %%r12\n");
  fprintf(f,"        testl   %%esi, %%esi\n");
  fprintf(f,"        je      67f\n");
  fprintf(f,"        movq    %%rax, %%rdi\n");
  fprintf(f,"        movabsq $pow5mult, %%rax\n");
  fprintf(f,"        callq   *%%rax\n");
  fprintf(f,"        movq    %%rax, %%r12\n");
  fprintf(f,"67:\n");
  fprintf(f,"        testl   %%r13d, %%r13d\n");
  fprintf(f,"        jne     68f\n");
  fprintf(f,"        cmpl    $1, -96(%%rbp)\n");
  fprintf(f,"        movl    $0, -128(%%rbp)\n");
  fprintf(f,"        jg      69f\n");
  fprintf(f,"68:\n");
  fprintf(f,"        movabsq $4503599627370495, %%rax\n");
  fprintf(f,"        movl    $0, -128(%%rbp)\n");
  fprintf(f,"        testq   %%rax, -56(%%rbp)\n");
  fprintf(f,"        jne     69f\n");
  fprintf(f,"        testl   $2145386496, -52(%%rbp)\n");
  fprintf(f,"        je      69f\n");
  fprintf(f,"        incl    -104(%%rbp)\n");
  fprintf(f,"        incl    %%r15d\n");
  fprintf(f,"        movl    $1, -128(%%rbp)\n");
  fprintf(f,"69:\n");
  fprintf(f,"        movabsq $dshift, %%rax\n");
  fprintf(f,"        movl    -104(%%rbp), %%esi\n");
  fprintf(f,"        movq    %%r12, %%rdi\n");
  fprintf(f,"        callq   *%%rax\n");
  fprintf(f,"        leal    (%%r15,%%rax), %%esi\n");
  fprintf(f,"        movl    -104(%%rbp), %%r15d\n");
  fprintf(f,"        movl    %%eax, -148(%%rbp)\n");
  fprintf(f,"        addl    %%eax, %%r15d\n");
  fprintf(f,"        testl   %%esi, %%esi\n");
  fprintf(f,"        jle     70f\n");
  fprintf(f,"        movabsq $lshift, %%rax\n");
  fprintf(f,"        movq    -72(%%rbp), %%rdi\n");
  fprintf(f,"        callq   *%%rax\n");
  fprintf(f,"        movq    %%rax, -72(%%rbp)\n");
  fprintf(f,"70:\n");
  fprintf(f,"        testl   %%r15d, %%r15d\n");
  fprintf(f,"        jle     71f\n");
  fprintf(f,"        movabsq $lshift, %%rax\n");
  fprintf(f,"        movq    %%r12, %%rdi\n");
  fprintf(f,"        movl    %%r15d, %%esi\n");
  fprintf(f,"        callq   *%%rax\n");
  fprintf(f,"        movq    %%rax, %%r12\n");
  fprintf(f,"71:\n");
  fprintf(f,"        cmpl    $0, -120(%%rbp)\n");
  fprintf(f,"        je      72f\n");
  fprintf(f,"        movq    -72(%%rbp), %%rdi\n");
  fprintf(f,"        movq    %%r12, %%rsi\n");
  fprintf(f,"        movabsq $cmp, %%rax\n");
  fprintf(f,"        callq   *%%rax\n");
  fprintf(f,"        testl   %%eax, %%eax\n");
  fprintf(f,"        jns     72f\n");
  fprintf(f,"        movq    -72(%%rbp), %%rdi\n");
  fprintf(f,"        xorl    %%edx, %%edx\n");
  fprintf(f,"        movl    $10, %%esi\n");
  fprintf(f,"        movabsq $multadd, %%r15\n");
  fprintf(f,"        decl    %%r14d\n");
  fprintf(f,"        callq   *%%r15\n");
  fprintf(f,"        testl   %%r13d, %%r13d\n");
  fprintf(f,"        movq    %%rax, -72(%%rbp)\n");
  fprintf(f,"        movl    -116(%%rbp), %%eax\n");
  fprintf(f,"        movl    %%eax, -84(%%rbp)\n");
  fprintf(f,"        je      72f\n");
  fprintf(f,"        movq    %%rbx, %%rdi\n");
  fprintf(f,"        xorl    %%edx, %%edx\n");
  fprintf(f,"        movl    $10, %%esi\n");
  fprintf(f,"        callq   *%%r15\n");
  fprintf(f,"        movq    %%rax, %%rbx\n");
  fprintf(f,"72:\n");
  fprintf(f,"        cmpl    $0, -84(%%rbp)\n");
  fprintf(f,"        jg      78f\n");
  fprintf(f,"        cmpl    $5, -96(%%rbp)\n");
  fprintf(f,"        je      73f\n");
  fprintf(f,"        cmpl    $3, -96(%%rbp)\n");
  fprintf(f,"        jne     78f\n");
  fprintf(f,"73:\n");
  fprintf(f,"        cmpl    $0, -84(%%rbp)\n");
  fprintf(f,"        jne     75f\n");
  fprintf(f,"        xorl    %%edx, %%edx\n");
  fprintf(f,"        movq    %%r12, %%rdi\n");
  fprintf(f,"        movl    $5, %%esi\n");
  fprintf(f,"        movabsq $multadd, %%rax\n");
  fprintf(f,"        callq   *%%rax\n");
  fprintf(f,"        movq    -72(%%rbp), %%rdi\n");
  fprintf(f,"        movq    %%rax, %%rsi\n");
  fprintf(f,"        movq    %%rax, %%r12\n");
  fprintf(f,"        movabsq $cmp, %%rax\n");
  fprintf(f,"        callq   *%%rax\n");
  fprintf(f,"        testl   %%eax, %%eax\n");
  fprintf(f,"        jg      77f\n");
  fprintf(f,"        jmp     75f\n");
  fprintf(f,"74:\n");
  fprintf(f,"        xorl    %%r12d, %%r12d\n");
  fprintf(f,"        xorl    %%ebx, %%ebx\n");
  fprintf(f,"75:\n");
  fprintf(f,"        movl    -112(%%rbp), %%r14d\n");
  fprintf(f,"        movq    -80(%%rbp), %%r15\n");
  fprintf(f,"        notl    %%r14d\n");
  fprintf(f,"        jmp     108f\n");
  fprintf(f,"76:\n");
  fprintf(f,"        movl    %%esi, %%r14d\n");
  fprintf(f,"        xorl    %%r12d, %%r12d\n");
  fprintf(f,"        xorl    %%ebx, %%ebx\n");
  fprintf(f,"77:\n");
  fprintf(f,"        movq    -80(%%rbp), %%rax\n");
  fprintf(f,"        incl    %%r14d\n");
  fprintf(f,"        leaq    1(%%rax), %%r15\n");
  fprintf(f,"        movb    $49, (%%rax)\n");
  fprintf(f,"        jmp     108f\n");
  fprintf(f,"78:\n");
  fprintf(f,"        testl   %%r13d, %%r13d\n");
  fprintf(f,"        je      97f\n");
  fprintf(f,"        movl    -88(%%rbp), %%esi\n");
  fprintf(f,"        addl    -148(%%rbp), %%esi\n");
  fprintf(f,"        testl   %%esi, %%esi\n");
  fprintf(f,"        jle     79f\n");
  fprintf(f,"        movabsq $lshift, %%rax\n");
  fprintf(f,"        movq    %%rbx, %%rdi\n");
  fprintf(f,"        callq   *%%rax\n");
  fprintf(f,"        movq    %%rax, %%rbx\n");
  fprintf(f,"79:\n");
  fprintf(f,"        cmpl    $0, -128(%%rbp)\n");
  fprintf(f,"        movq    %%rbx, %%rax\n");
  fprintf(f,"        je      80f\n");
  fprintf(f,"        movabsq $Balloc, %%rax\n");
  fprintf(f,"        movl    8(%%rbx), %%edi\n");
  fprintf(f,"        callq   *%%rax\n");
  fprintf(f,"        leaq    16(%%rbx), %%rsi\n");
  fprintf(f,"        movq    %%rax, %%r13\n");
  fprintf(f,"        movslq  20(%%rbx), %%rax\n");
  fprintf(f,"        leaq    16(%%r13), %%rdi\n");
  fprintf(f,"        leaq    8(,%%rax,4), %%rdx\n");
  fprintf(f,"        movabsq $memcpy, %%rax\n");
  fprintf(f,"        callq   *%%rax\n");
  fprintf(f,"        movl    $1, %%esi\n");
  fprintf(f,"        movq    %%r13, %%rdi\n");
  fprintf(f,"        movabsq $lshift, %%rax\n");
  fprintf(f,"        callq   *%%rax\n");
  fprintf(f,"80:\n");
  fprintf(f,"        movq    -80(%%rbp), %%r15\n");
  fprintf(f,"        movq    %%rbx, %%r13\n");
  fprintf(f,"81:\n");
  fprintf(f,"        movq    %%rax, %%rbx\n");
  fprintf(f,"82:\n");
  fprintf(f,"        movq    -72(%%rbp), %%rdi\n");
  fprintf(f,"        movq    %%r12, %%rsi\n");
  fprintf(f,"        movabsq $quorem, %%rax\n");
  fprintf(f,"        callq   *%%rax\n");
  fprintf(f,"        movq    -72(%%rbp), %%rdi\n");
  fprintf(f,"        movq    %%r13, %%rsi\n");
  fprintf(f,"        leal    48(%%rax), %%edx\n");
  fprintf(f,"        movl    %%eax, -88(%%rbp)\n");
  fprintf(f,"        movabsq $cmp, %%r8\n");
  fprintf(f,"        movl    %%edx, -112(%%rbp)\n");
  fprintf(f,"        movq    %%r8, -128(%%rbp)\n");
  fprintf(f,"        callq   *%%r8\n");
  fprintf(f,"        movq    %%rbx, %%rsi\n");
  fprintf(f,"        movq    %%r12, %%rdi\n");
  fprintf(f,"        movl    %%eax, -104(%%rbp)\n");
  fprintf(f,"        movabsq $diff, %%rax\n");
  fprintf(f,"        callq   *%%rax\n");
  fprintf(f,"        movl    -112(%%rbp), %%edx\n");
  fprintf(f,"        cmpl    $0, 16(%%rax)\n");
  fprintf(f,"        movq    %%rax, %%r9\n");
  fprintf(f,"        jne     83f\n");
  fprintf(f,"        movq    -128(%%rbp), %%r8\n");
  fprintf(f,"        movq    %%rax, %%rsi\n");
  fprintf(f,"        movl    %%edx, -116(%%rbp)\n");
  fprintf(f,"        movq    -72(%%rbp), %%rdi\n");
  fprintf(f,"        movq    %%rax, -112(%%rbp)\n");
  fprintf(f,"        callq   *%%r8\n");
  fprintf(f,"        movq    -112(%%rbp), %%r9\n");
  fprintf(f,"        movl    %%eax, %%r8d\n");
  fprintf(f,"        movl    -116(%%rbp), %%edx\n");
  fprintf(f,"        jmp     84f\n");
  fprintf(f,"83:\n");
  fprintf(f,"        movl    $1, %%r8d\n");
  fprintf(f,"84:\n");
  fprintf(f,"        movabsq $Bfree, %%rax\n");
  fprintf(f,"        movq    %%r9, %%rdi\n");
  fprintf(f,"        movl    %%edx, -112(%%rbp)\n");
  fprintf(f,"        movl    %%r8d, -116(%%rbp)\n");
  fprintf(f,"        callq   *%%rax\n");
  fprintf(f,"        cmpl    $1, -96(%%rbp)\n");
  fprintf(f,"        movl    -116(%%rbp), %%r8d\n");
  fprintf(f,"        movl    -112(%%rbp), %%edx\n");
  fprintf(f,"        setne   %%al\n");
  fprintf(f,"        testl   %%r8d, %%r8d\n");
  fprintf(f,"        jne     85f\n");
  fprintf(f,"        testb   %%al, %%al\n");
  fprintf(f,"        je      85f\n");
  fprintf(f,"        testb   $1, -56(%%rbp)\n");
  fprintf(f,"        jne     85f\n");
  fprintf(f,"        cmpl    $57, %%edx\n");
  fprintf(f,"        movq    %%r15, %%rax\n");
  fprintf(f,"        je      93f\n");
  fprintf(f,"        movl    -88(%%rbp), %%ecx\n");
  fprintf(f,"        addl    $49, %%ecx\n");
  fprintf(f,"        cmpl    $0, -104(%%rbp)\n");
  fprintf(f,"        cmovg   %%ecx, %%edx\n");
  fprintf(f,"        incq    %%r15\n");
  fprintf(f,"        jmp     106f\n");
  fprintf(f,"85:\n");
  fprintf(f,"        cmpl    $0, -104(%%rbp)\n");
  fprintf(f,"        jns     87f\n");
  fprintf(f,"86:\n");
  fprintf(f,"        movq    -72(%%rbp), %%rax\n");
  fprintf(f,"        movq    %%r15, %%rcx\n");
  fprintf(f,"        cmpl    $0, 24(%%rax)\n");
  fprintf(f,"        je      88f\n");
  fprintf(f,"        jmp     89f\n");
  fprintf(f,"87:\n");
  fprintf(f,"        jne     92f\n");
  fprintf(f,"        testb   %%al, %%al\n");
  fprintf(f,"        je      92f\n");
  fprintf(f,"        testb   $1, -56(%%rbp)\n");
  fprintf(f,"        jne     92f\n");
  fprintf(f,"        jmp     86b\n");
  fprintf(f,"88:\n");
  fprintf(f,"        movq    -72(%%rbp), %%rax\n");
  fprintf(f,"        cmpl    $1, 20(%%rax)\n");
  fprintf(f,"        jle     91f\n");
  fprintf(f,"89:\n");
  fprintf(f,"        testl   %%r8d, %%r8d\n");
  fprintf(f,"        jle     91f\n");
  fprintf(f,"        movq    -72(%%rbp), %%rdi\n");
  fprintf(f,"        movl    $1, %%esi\n");
  fprintf(f,"        movq    %%rcx, -96(%%rbp)\n");
  fprintf(f,"        movabsq $lshift, %%rax\n");
  fprintf(f,"        movl    %%edx, -84(%%rbp)\n");
  fprintf(f,"        callq   *%%rax\n");
  fprintf(f,"        movq    %%r12, %%rsi\n");
  fprintf(f,"        movq    %%rax, -72(%%rbp)\n");
  fprintf(f,"        movq    %%rax, %%rdi\n");
  fprintf(f,"        movabsq $cmp, %%rax\n");
  fprintf(f,"        callq   *%%rax\n");
  fprintf(f,"        movl    -84(%%rbp), %%edx\n");
  fprintf(f,"        cmpl    $0, %%eax\n");
  fprintf(f,"        movq    -96(%%rbp), %%rcx\n");
  fprintf(f,"        jg      90f\n");
  fprintf(f,"        jne     91f\n");
  fprintf(f,"        testb   $1, %%dl\n");
  fprintf(f,"        je      91f\n");
  fprintf(f,"90:\n");
  fprintf(f,"        cmpl    $57, %%edx\n");
  fprintf(f,"        je      93f\n");
  fprintf(f,"        movl    -88(%%rbp), %%edx\n");
  fprintf(f,"        addl    $49, %%edx\n");
  fprintf(f,"91:\n");
  fprintf(f,"        incq    %%r15\n");
  fprintf(f,"        movb    %%dl, (%%rcx)\n");
  fprintf(f,"        jmp     109f\n");
  fprintf(f,"92:\n");
  fprintf(f,"        leaq    1(%%r15), %%rax\n");
  fprintf(f,"        testl   %%r8d, %%r8d\n");
  fprintf(f,"        jle     95f\n");
  fprintf(f,"        cmpl    $57, %%edx\n");
  fprintf(f,"        movq    %%r15, %%rsi\n");
  fprintf(f,"        jne     94f\n");
  fprintf(f,"93:\n");
  fprintf(f,"        movb    $57, (%%r15)\n");
  fprintf(f,"        incq    %%r15\n");
  fprintf(f,"        jmp     103f\n");
  fprintf(f,"94:\n");
  fprintf(f,"        incl    %%edx\n");
  fprintf(f,"        movq    %%rax, %%r15\n");
  fprintf(f,"        movb    %%dl, (%%rsi)\n");
  fprintf(f,"        jmp     109f\n");
  fprintf(f,"95:\n");
  fprintf(f,"        movq    %%rax, %%r15\n");
  fprintf(f,"        movb    %%dl, -1(%%rax)\n");
  fprintf(f,"        subl    -80(%%rbp), %%eax\n");
  fprintf(f,"        cmpl    -84(%%rbp), %%eax\n");
  fprintf(f,"        je      101f\n");
  fprintf(f,"        movq    -72(%%rbp), %%rdi\n");
  fprintf(f,"        xorl    %%edx, %%edx\n");
  fprintf(f,"        movl    $10, %%esi\n");
  fprintf(f,"        movabsq $multadd, %%rcx\n");
  fprintf(f,"        movq    %%rcx, -104(%%rbp)\n");
  fprintf(f,"        callq   *%%rcx\n");
  fprintf(f,"        xorl    %%edx, %%edx\n");
  fprintf(f,"        cmpq    %%rbx, %%r13\n");
  fprintf(f,"        movq    %%rax, -72(%%rbp)\n");
  fprintf(f,"        movl    $10, %%esi\n");
  fprintf(f,"        movq    %%r13, %%rdi\n");
  fprintf(f,"        movq    -104(%%rbp), %%rcx\n");
  fprintf(f,"        jne     96f\n");
  fprintf(f,"        callq   *%%rcx\n");
  fprintf(f,"        movq    %%rax, %%r13\n");
  fprintf(f,"        movq    %%rax, %%rbx\n");
  fprintf(f,"        jmp     82b\n");
  fprintf(f,"96:\n");
  fprintf(f,"        movq    %%rcx, -104(%%rbp)\n");
  fprintf(f,"        callq   *%%rcx\n");
  fprintf(f,"        movq    -104(%%rbp), %%rcx\n");
  fprintf(f,"        xorl    %%edx, %%edx\n");
  fprintf(f,"        movl    $10, %%esi\n");
  fprintf(f,"        movq    %%rbx, %%rdi\n");
  fprintf(f,"        movq    %%rax, %%r13\n");
  fprintf(f,"        callq   *%%rcx\n");
  fprintf(f,"        jmp     81b\n");
  fprintf(f,"97:\n");
  fprintf(f,"        movabsq $quorem, %%r13\n");
  fprintf(f,"        movq    -80(%%rbp), %%r15\n");
  fprintf(f,"98:\n");
  fprintf(f,"        movq    -72(%%rbp), %%rdi\n");
  fprintf(f,"        movq    %%r12, %%rsi\n");
  fprintf(f,"        incq    %%r15\n");
  fprintf(f,"        callq   *%%r13\n");
  fprintf(f,"        leal    48(%%rax), %%edx\n");
  fprintf(f,"        movq    -72(%%rbp), %%rax\n");
  fprintf(f,"        movb    %%dl, -1(%%r15)\n");
  fprintf(f,"        cmpl    $0, 24(%%rax)\n");
  fprintf(f,"        jne     99f\n");
  fprintf(f,"        cmpl    $1, 20(%%rax)\n");
  fprintf(f,"        jle     108f\n");
  fprintf(f,"99:\n");
  fprintf(f,"        movl    %%r15d, %%eax\n");
  fprintf(f,"        subl    -80(%%rbp), %%eax\n");
  fprintf(f,"        cmpl    -84(%%rbp), %%eax\n");
  fprintf(f,"        jge     100f\n");
  fprintf(f,"        movq    -72(%%rbp), %%rdi\n");
  fprintf(f,"        xorl    %%edx, %%edx\n");
  fprintf(f,"        movl    $10, %%esi\n");
  fprintf(f,"        movabsq $multadd, %%rax\n");
  fprintf(f,"        callq   *%%rax\n");
  fprintf(f,"        movq    %%rax, -72(%%rbp)\n");
  fprintf(f,"        jmp     98b\n");
  fprintf(f,"100:\n");
  fprintf(f,"        xorl    %%r13d, %%r13d\n");
  fprintf(f,"101:\n");
  fprintf(f,"        movq    -72(%%rbp), %%rdi\n");
  fprintf(f,"        movl    $1, %%esi\n");
  fprintf(f,"        movl    %%edx, -84(%%rbp)\n");
  fprintf(f,"        movabsq $lshift, %%rax\n");
  fprintf(f,"        callq   *%%rax\n");
  fprintf(f,"        movq    %%r12, %%rsi\n");
  fprintf(f,"        movq    %%rax, -72(%%rbp)\n");
  fprintf(f,"        movq    %%rax, %%rdi\n");
  fprintf(f,"        movabsq $cmp, %%rax\n");
  fprintf(f,"        callq   *%%rax\n");
  fprintf(f,"        movl    -84(%%rbp), %%edx\n");
  fprintf(f,"        cmpl    $0, %%eax\n");
  fprintf(f,"        jg      103f\n");
  fprintf(f,"        jne     107f\n");
  fprintf(f,"        andb    $1, %%dl\n");
  fprintf(f,"        jne     103f\n");
  fprintf(f,"        jmp     107f\n");
  fprintf(f,"102:\n");
  fprintf(f,"        cmpq    -80(%%rbp), %%rax\n");
  fprintf(f,"        je      104f\n");
  fprintf(f,"        movq    %%rax, %%r15\n");
  fprintf(f,"103:\n");
  fprintf(f,"        movb    -1(%%r15), %%dl\n");
  fprintf(f,"        leaq    -1(%%r15), %%rax\n");
  fprintf(f,"        cmpb    $57, %%dl\n");
  fprintf(f,"        je      102b\n");
  fprintf(f,"        jmp     105f\n");
  fprintf(f,"104:\n");
  fprintf(f,"        movq    -80(%%rbp), %%rax\n");
  fprintf(f,"        incl    %%r14d\n");
  fprintf(f,"        movb    $49, (%%rax)\n");
  fprintf(f,"        jmp     109f\n");
  fprintf(f,"105:\n");
  fprintf(f,"        incl    %%edx\n");
  fprintf(f,"106:\n");
  fprintf(f,"        movb    %%dl, (%%rax)\n");
  fprintf(f,"        jmp     109f\n");
  fprintf(f,"107:\n");
  fprintf(f,"        cmpb    $48, -1(%%r15)\n");
  fprintf(f,"        leaq    -1(%%r15), %%rax\n");
  fprintf(f,"        jne     109f\n");
  fprintf(f,"        movq    %%rax, %%r15\n");
  fprintf(f,"        jmp     107b\n");
  fprintf(f,"108:\n");
  fprintf(f,"        xorl    %%r13d, %%r13d\n");
  fprintf(f,"109:\n");
  fprintf(f,"        movq    %%r12, %%rdi\n");
  fprintf(f,"        movabsq $Bfree, %%r12\n");
  fprintf(f,"        callq   *%%r12\n");
  fprintf(f,"        testq   %%rbx, %%rbx\n");
  fprintf(f,"        je      112f\n");
  fprintf(f,"        cmpq    %%rbx, %%r13\n");
  fprintf(f,"        je      110f\n");
  fprintf(f,"        testq   %%r13, %%r13\n");
  fprintf(f,"        je      110f\n");
  fprintf(f,"        movq    %%r13, %%rdi\n");
  fprintf(f,"        callq   *%%r12\n");
  fprintf(f,"110:\n");
  fprintf(f,"        movq    %%rbx, %%rdi\n");
  fprintf(f,"        callq   *%%r12\n");
  fprintf(f,"        jmp     112f\n");
  fprintf(f,"111:\n");
  fprintf(f,"        movl    %%esi, %%r14d\n");
  fprintf(f,"112:\n");
  fprintf(f,"        movabsq $Bfree, %%rax\n");
  fprintf(f,"        movq    -72(%%rbp), %%rdi\n");
  fprintf(f,"        callq   *%%rax\n");
  fprintf(f,"        movq    -144(%%rbp), %%rbx\n");
  fprintf(f,"        leal    1(%%r14), %%eax\n");
  fprintf(f,"        movb    $0, (%%r15)\n");
  fprintf(f,"        movl    %%eax, (%%rbx)\n");
  fprintf(f,"        movq    -136(%%rbp), %%rax\n");
  fprintf(f,"        testq   %%rax, %%rax\n");
  fprintf(f,"        je      113f\n");
  fprintf(f,"        movq    %%r15, (%%rax)\n");
  fprintf(f,"113:\n");
  fprintf(f,"        movq    -80(%%rbp), %%rax\n");
  fprintf(f,"114:\n");
  fprintf(f,"        addq    $120, %%rsp\n");
  fprintf(f,"        popq    %%rbx\n");
  fprintf(f,"        popq    %%r12\n");
  fprintf(f,"        popq    %%r13\n");
  fprintf(f,"        popq    %%r14\n");
  fprintf(f,"        popq    %%r15\n");
  fprintf(f,"        popq    %%rbp\n");
  fprintf(f,"        retq\n");
  fprintf(f,"        .size   dtoa, .-dtoa\n\n");

  fprintf(f,"        .section .rodata,\"a\",@progbits\n\n");

  fprintf(f,"        .align 8\n");
  fprintf(f,"        .type   p05.3208, @object\n");
  fprintf(f,"        .size   p05.3208, 12\n");
  fprintf(f,"p05.3208:\n");
  fprintf(f,"        .long   5\n");
  fprintf(f,"        .long   25\n");
  fprintf(f,"        .long   125\n");
  fprintf(f,"        .local  dtoa_result\n");
  fprintf(f,"        .comm   dtoa_result,8,8\n");
  fprintf(f,"        .align 16\n");
  fprintf(f,"        .type   hexdig, @object\n");
  fprintf(f,"        .size   hexdig, 256\n");
  fprintf(f,"hexdig:\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   16\n");
  fprintf(f,"        .byte   17\n");
  fprintf(f,"        .byte   18\n");
  fprintf(f,"        .byte   19\n");
  fprintf(f,"        .byte   20\n");
  fprintf(f,"        .byte   21\n");
  fprintf(f,"        .byte   22\n");
  fprintf(f,"        .byte   23\n");
  fprintf(f,"        .byte   24\n");
  fprintf(f,"        .byte   25\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   26\n");
  fprintf(f,"        .byte   27\n");
  fprintf(f,"        .byte   28\n");
  fprintf(f,"        .byte   29\n");
  fprintf(f,"        .byte   30\n");
  fprintf(f,"        .byte   31\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   26\n");
  fprintf(f,"        .byte   27\n");
  fprintf(f,"        .byte   28\n");
  fprintf(f,"        .byte   29\n");
  fprintf(f,"        .byte   30\n");
  fprintf(f,"        .byte   31\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .byte   0\n");
  fprintf(f,"        .align 16\n");
  fprintf(f,"        .type   tinytens, @object\n");
  fprintf(f,"        .size   tinytens, 40\n");
  fprintf(f,"tinytens:\n");
  fprintf(f,"        .long   2547550652\n");
  fprintf(f,"        .long   1016910514\n");
  fprintf(f,"        .long   3584599859\n");
  fprintf(f,"        .long   961148451\n");
  fprintf(f,"        .long   1156884285\n");
  fprintf(f,"        .long   849678333\n");
  fprintf(f,"        .long   3482097565\n");
  fprintf(f,"        .long   626768392\n");
  fprintf(f,"        .long   1689022275\n");
  fprintf(f,"        .long   180880936\n");
  fprintf(f,"        .align 16\n");
  fprintf(f,"        .type   bigtens, @object\n");
  fprintf(f,"        .size   bigtens, 40\n");
  fprintf(f,"bigtens:\n");
  fprintf(f,"        .long   937459712\n");
  fprintf(f,"        .long   1128383353\n");
  fprintf(f,"        .long   3037031959\n");
  fprintf(f,"        .long   1184086197\n");
  fprintf(f,"        .long   3913284085\n");
  fprintf(f,"        .long   1295535875\n");
  fprintf(f,"        .long   4180679986\n");
  fprintf(f,"        .long   1518499656\n");
  fprintf(f,"        .long   2138292028\n");
  fprintf(f,"        .long   1964330973\n");
  fprintf(f,"        .align 16\n");
  fprintf(f,"        .type   tens, @object\n");
  fprintf(f,"        .size   tens, 184\n");
  fprintf(f,"tens:\n");
  fprintf(f,"        .long   0\n");
  fprintf(f,"        .long   1072693248\n");
  fprintf(f,"        .long   0\n");
  fprintf(f,"        .long   1076101120\n");
  fprintf(f,"        .long   0\n");
  fprintf(f,"        .long   1079574528\n");
  fprintf(f,"        .long   0\n");
  fprintf(f,"        .long   1083129856\n");
  fprintf(f,"        .long   0\n");
  fprintf(f,"        .long   1086556160\n");
  fprintf(f,"        .long   0\n");
  fprintf(f,"        .long   1090021888\n");
  fprintf(f,"        .long   0\n");
  fprintf(f,"        .long   1093567616\n");
  fprintf(f,"        .long   0\n");
  fprintf(f,"        .long   1097011920\n");
  fprintf(f,"        .long   0\n");
  fprintf(f,"        .long   1100470148\n");
  fprintf(f,"        .long   0\n");
  fprintf(f,"        .long   1104006501\n");
  fprintf(f,"        .long   536870912\n");
  fprintf(f,"        .long   1107468383\n");
  fprintf(f,"        .long   3892314112\n");
  fprintf(f,"        .long   1110919286\n");
  fprintf(f,"        .long   2717908992\n");
  fprintf(f,"        .long   1114446484\n");
  fprintf(f,"        .long   3846176768\n");
  fprintf(f,"        .long   1117925532\n");
  fprintf(f,"        .long   512753664\n");
  fprintf(f,"        .long   1121369284\n");
  fprintf(f,"        .long   640942080\n");
  fprintf(f,"        .long   1124887541\n");
  fprintf(f,"        .long   937459712\n");
  fprintf(f,"        .long   1128383353\n");
  fprintf(f,"        .long   2245566464\n");
  fprintf(f,"        .long   1131820119\n");
  fprintf(f,"        .long   1733216256\n");
  fprintf(f,"        .long   1135329645\n");
  fprintf(f,"        .long   1620131072\n");
  fprintf(f,"        .long   1138841828\n");
  fprintf(f,"        .long   2025163840\n");
  fprintf(f,"        .long   1142271773\n");
  fprintf(f,"        .long   3605196624\n");
  fprintf(f,"        .long   1145772772\n");
  fprintf(f,"        .long   105764242\n");
  fprintf(f,"        .long   1149300943\n");
  fprintf(f,"        .local  p5s\n");
  fprintf(f,"        .comm   p5s,8,8\n");
  fprintf(f,"        .local  freelist\n");
  fprintf(f,"        .comm   freelist,64,16\n");

  fprintf(f,"        .section .bss,\"aw\",@nobits\n\n");

  fprintf(f,"        .align 4\n");
  fprintf(f,"        .type   errno, @object\n");
  fprintf(f,"        .size   errno, 4\n");
  fprintf(f,"errno:\n");
  fprintf(f,"        .zero   4\n");
  fprintf(f,"        .data\n");
  fprintf(f,"        .align 8\n");
  fprintf(f,"        .type   pmem_next, @object\n");
  fprintf(f,"        .size   pmem_next, 8\n");
  fprintf(f,"pmem_next:\n");
  fprintf(f,"        .quad   private_mem\n");
  fprintf(f,"        .local  private_mem\n");
  fprintf(f,"        .comm   private_mem,16384,16\n\n");

  fprintf(f,"        .section .rodata.cst8,\"aM\",@progbits,8\n\n");

  fprintf(f,"        .align 8\n");
  fprintf(f,".LdtoaC1:\n");
  fprintf(f,"        .long   0\n");
  fprintf(f,"        .long   1071644672\n");
  fprintf(f,"        .align 8\n");
  fprintf(f,".LdtoaC2:\n");
  fprintf(f,"        .long   0\n");
  fprintf(f,"        .long   -1074790400\n");
  fprintf(f,"        .align 8\n");
  fprintf(f,".LdtoaC3:\n");
  fprintf(f,"        .long   0\n");
  fprintf(f,"        .long   1072693248\n");

  fprintf(f,"        .section .rodata.cst16,\"aM\",@progbits,16\n\n");

  fprintf(f,"        .align 16\n");
  fprintf(f,".LdtoaC7:\n");
  fprintf(f,"        .long   0\n");
  fprintf(f,"        .long   -2147483648\n");
  fprintf(f,"        .long   0\n");
  fprintf(f,"        .long   0\n");

  fprintf(f,"        .section .rodata.cst8,\"aM\",@progbits,8\n\n");

  fprintf(f,"        .align 8\n");
  fprintf(f,".LdtoaC8:\n");
  fprintf(f,"        .long   0\n");
  fprintf(f,"        .long   1073741824\n");
  fprintf(f,"        .align 8\n");
  fprintf(f,".LdtoaC9:\n");
  fprintf(f,"        .long   2493527445\n");
  fprintf(f,"        .long   1071644671\n");
  fprintf(f,"        .align 8\n");
  fprintf(f,".LdtoaC10:\n");
  fprintf(f,"        .long   900719925\n");
  fprintf(f,"        .long   1071644672\n");
  fprintf(f,"        .align 8\n");
  fprintf(f,".LdtoaC11:\n");
  fprintf(f,"        .long   2493527445\n");
  fprintf(f,"        .long   1070596095\n");
  fprintf(f,"        .align 8\n");
  fprintf(f,".LdtoaC15:\n");
  fprintf(f,"        .long   0\n");
  fprintf(f,"        .long   1073217536\n");
  fprintf(f,"        .align 8\n");
  fprintf(f,".LdtoaC16:\n");
  fprintf(f,"        .long   1668236129\n");
  fprintf(f,"        .long   1070761895\n");
  fprintf(f,"        .align 8\n");
  fprintf(f,".LdtoaC17:\n");
  fprintf(f,"        .long   2338375859\n");
  fprintf(f,"        .long   1069976104\n");
  fprintf(f,"        .align 8\n");
  fprintf(f,".LdtoaC18:\n");
  fprintf(f,"        .long   1352628731\n");
  fprintf(f,"        .long   1070810131\n");
  fprintf(f,"        .align 8\n");
  fprintf(f,".LdtoaC19:\n");
  fprintf(f,"        .long   2138292028\n");
  fprintf(f,"        .long   1964330973\n");
  fprintf(f,"        .align 8\n");
  fprintf(f,".LdtoaC20:\n");
  fprintf(f,"        .long   0\n");
  fprintf(f,"        .long   1076101120\n");
  fprintf(f,"        .align 8\n");
  fprintf(f,".LdtoaC21:\n");
  fprintf(f,"        .long   0\n");
  fprintf(f,"        .long   1075576832\n");
  fprintf(f,"        .align 8\n");
  fprintf(f,".LdtoaC22:\n");
  fprintf(f,"        .long   0\n");
  fprintf(f,"        .long   1075052544\n");
  fprintf(f,"        .align 8\n");
  fprintf(f,".LdtoaC24:\n");
  fprintf(f,"        .quad   3595427678142423661\n");

  fprintf(f,"        .section .text,\"ax\",@progbits\n\n");

  fprintf(f,"#\n");
  fprintf(f,"# char *g_fmt(register char *b, double x, signed long int digits_after_decimal, char digits_in_exponent)\n");
  fprintf(f,"# reg     arg\n");
  fprintf(f,"# ----    --------------------\n");
  fprintf(f,"# rdi     b\n");
  fprintf(f,"# xmm0    x\n");
  fprintf(f,"# esi     digits_after_decimal\n");
  fprintf(f,"# edx     digits_in_exponent\n");
  fprintf(f,"#\n");
  fprintf(f,"        .type   g_fmt, @function\n");
  fprintf(f,"g_fmt:\n");
  fprintf(f,"        movabsq $dtoa, %%rax\n");
  fprintf(f,"        pushq   %%rbp\n");
  fprintf(f,"        movq    %%rsp, %%rbp\n");
  fprintf(f,"        pushq   %%r14\n");
  fprintf(f,"        movq    %%rsi, %%r14\n");
  fprintf(f,"        pushq   %%r13\n");
  fprintf(f,"        leaq    -40(%%rbp), %%r8\n");
  fprintf(f,"        movl    %%edx, %%r13d\n");
  fprintf(f,"        pushq   %%r12\n");
  fprintf(f,"        leaq    -44(%%rbp), %%rcx\n");
  fprintf(f,"        movq    %%rdi, %%r12\n");
  fprintf(f,"        pushq   %%rbx\n");
  fprintf(f,"        movl    $4, %%edi\n");
  fprintf(f,"        movsbl  %%dl, %%ebx\n");
  fprintf(f,"        leaq    -48(%%rbp), %%rdx\n");
  fprintf(f,"        subq    $16, %%rsp\n");
  fprintf(f,"        callq   *%%rax\n");
  fprintf(f,"        cmpl    $0, -44(%%rbp)\n");
  fprintf(f,"        leaq    1(%%r12), %%r8\n");
  fprintf(f,"        movq    %%rax, %%rdi\n");
  fprintf(f,"        movq    %%r8, %%rcx\n");
  fprintf(f,"        je      0f\n");
  fprintf(f,"        movb    $45, (%%r12)\n");
  fprintf(f,"        jmp     1f\n");
  fprintf(f,"0:\n");
  fprintf(f,"        movb    $32, (%%r12)\n");
  fprintf(f,"1:\n");
  fprintf(f,"        movl    -48(%%rbp), %%eax\n");
  fprintf(f,"        cmpl    $9999, %%eax\n");
  fprintf(f,"        jne     3f\n");
  fprintf(f,"        movq    %%rdi, %%rax\n");
  fprintf(f,"2:\n");
  fprintf(f,"        incq    %%rax\n");
  fprintf(f,"        movb    -1(%%rax), %%dl\n");
  fprintf(f,"        testb   %%dl, %%dl\n");
  fprintf(f,"        movb    %%dl, (%%rcx)\n");
  fprintf(f,"        je      29f\n");
  fprintf(f,"        incq    %%rcx\n");
  fprintf(f,"        jmp     2b\n");
  fprintf(f,"3:\n");
  fprintf(f,"        movslq  %%eax, %%rsi\n");
  fprintf(f,"        cmpq    %%r14, %%rsi\n");
  fprintf(f,"        jg      4f\n");
  fprintf(f,"        movq    -40(%%rbp), %%rdx\n");
  fprintf(f,"        subq    %%rdi, %%rdx\n");
  fprintf(f,"        subq    %%r14, %%rdx\n");
  fprintf(f,"        cmpq    %%rdx, %%rsi\n");
  fprintf(f,"        jge     16f\n");
  fprintf(f,"4:\n");
  fprintf(f,"        movb    (%%rdi), %%dl\n");
  fprintf(f,"        leaq    2(%%r8), %%rcx\n");
  fprintf(f,"        movb    %%dl, (%%r8)\n");
  fprintf(f,"        cmpb    $0, 1(%%rdi)\n");
  fprintf(f,"        je      6f\n");
  fprintf(f,"        leaq    1(%%rdi), %%rdx\n");
  fprintf(f,"        movb    $46, 1(%%r8)\n");
  fprintf(f,"5:\n");
  fprintf(f,"        incq    %%rdx\n");
  fprintf(f,"        movb    -1(%%rdx), %%sil\n");
  fprintf(f,"        testb   %%sil, %%sil\n");
  fprintf(f,"        movb    %%sil, (%%rcx)\n");
  fprintf(f,"        je      7f\n");
  fprintf(f,"        incq    %%rcx\n");
  fprintf(f,"        jmp     5b\n");
  fprintf(f,"6:\n");
  fprintf(f,"        movb    $46, 1(%%r8)\n");
  fprintf(f,"7:\n");
  fprintf(f,"        movb    $69, (%%rcx)\n");
  fprintf(f,"        leaq    2(%%rcx), %%rsi\n");
  fprintf(f,"        decl    %%eax\n");
  fprintf(f,"        testl   %%eax, %%eax\n");
  fprintf(f,"        movl    %%eax, -48(%%rbp)\n");
  fprintf(f,"        jns     8f\n");
  fprintf(f,"        negl    %%eax\n");
  fprintf(f,"        movb    $45, 1(%%rcx)\n");
  fprintf(f,"        movl    %%eax, -48(%%rbp)\n");
  fprintf(f,"        jmp     9f\n");
  fprintf(f,"8:\n");
  fprintf(f,"        movb    $43, 1(%%rcx)\n");
  fprintf(f,"9:\n");
  fprintf(f,"        cmpb    $3, %%r13b\n");
  fprintf(f,"        je      10f\n");
  fprintf(f,"        movl    $10, %%ecx\n");
  fprintf(f,"        cmpb    $4, %%r13b\n");
  fprintf(f,"        movl    $1000, %%eax\n");
  fprintf(f,"        cmove   %%eax, %%ecx\n");
  fprintf(f,"        jmp     11f\n");
  fprintf(f,"10:\n");
  fprintf(f,"        movl    $100, %%ecx\n");
  fprintf(f,"11:\n");
  fprintf(f,"        movl    -48(%%rbp), %%r8d\n");
  fprintf(f,"        xorl    %%eax, %%eax\n");
  fprintf(f,"12:\n");
  fprintf(f,"        imull   $10, %%ecx, %%edx\n");
  fprintf(f,"        leal    (%%rax,%%rbx), %%r9d\n");
  fprintf(f,"        incl    %%eax\n");
  fprintf(f,"        cmpl    %%r8d, %%edx\n");
  fprintf(f,"        jg      13f\n");
  fprintf(f,"        movl    %%edx, %%ecx\n");
  fprintf(f,"        jmp     12b\n");
  fprintf(f,"13:\n");
  fprintf(f,"        addl    %%esi, %%r9d\n");
  fprintf(f,"14:\n");
  fprintf(f,"        movl    %%r8d, %%eax\n");
  fprintf(f,"        cltd\n");
  fprintf(f,"        idivl   %%ecx\n");
  fprintf(f,"        leaq    1(%%rsi), %%rdx\n");
  fprintf(f,"        leal    48(%%rax), %%r10d\n");
  fprintf(f,"        movb    %%r10b, (%%rsi)\n");
  fprintf(f,"        movl    %%r9d, %%r10d\n");
  fprintf(f,"        subl    %%edx, %%r10d\n");
  fprintf(f,"        testl   %%r10d, %%r10d\n");
  fprintf(f,"        jle     15f\n");
  fprintf(f,"        imull   %%ecx, %%eax\n");
  fprintf(f,"        movq    %%rdx, %%rsi\n");
  fprintf(f,"        subl    %%eax, %%r8d\n");
  fprintf(f,"        imull   $10, %%r8d, %%r8d\n");
  fprintf(f,"        jmp     14b\n");
  fprintf(f,"15:\n");
  fprintf(f,"        movl    %%r8d, -48(%%rbp)\n");
  fprintf(f,"        movb    $32, 1(%%rsi)\n");
  fprintf(f,"        movb    $0, 2(%%rsi)\n");
  fprintf(f,"        jmp     30f\n");
  fprintf(f,"16:\n");
  fprintf(f,"        testl   %%eax, %%eax\n");
  fprintf(f,"        jle     17f\n");
  fprintf(f,"        movq    %%rdi, %%rdx\n");
  fprintf(f,"        jmp     24f\n");
  fprintf(f,"17:\n");
  fprintf(f,"        leaq    1(%%r8), %%rdx\n");
  fprintf(f,"        movb    $46, (%%r8)\n");
  fprintf(f,"18:\n");
  fprintf(f,"        testl   %%eax, %%eax\n");
  fprintf(f,"        jns     19f\n");
  fprintf(f,"        incq    %%rdx\n");
  fprintf(f,"        movb    $48, -1(%%rdx)\n");
  fprintf(f,"        incl    %%eax\n");
  fprintf(f,"        jmp     18b\n");
  fprintf(f,"19:\n");
  fprintf(f,"        movl    %%eax, -48(%%rbp)\n");
  fprintf(f,"        movq    %%rdi, %%rax\n");
  fprintf(f,"20:\n");
  fprintf(f,"        incq    %%rax\n");
  fprintf(f,"        movb    -1(%%rax), %%cl\n");
  fprintf(f,"        testb   %%cl, %%cl\n");
  fprintf(f,"        movb    %%cl, (%%rdx)\n");
  fprintf(f,"        je      21f\n");
  fprintf(f,"        incq    %%rdx\n");
  fprintf(f,"        jmp     20b\n");
  fprintf(f,"21:\n");
  fprintf(f,"        movb    $32, (%%rdx)\n");
  fprintf(f,"        movb    $0, 1(%%rdx)\n");
  fprintf(f,"        jmp     30f\n");
  fprintf(f,"22:\n");
  fprintf(f,"        leaq    1(%%rcx), %%rsi\n");
  fprintf(f,"        decl    %%eax\n");
  fprintf(f,"        testl   %%eax, %%eax\n");
  fprintf(f,"        movl    %%eax, -48(%%rbp)\n");
  fprintf(f,"        je      25f\n");
  fprintf(f,"23:\n");
  fprintf(f,"        movq    %%rsi, %%rcx\n");
  fprintf(f,"24:\n");
  fprintf(f,"        incq    %%rdx\n");
  fprintf(f,"        movb    -1(%%rdx), %%al\n");
  fprintf(f,"        testb   %%al, %%al\n");
  fprintf(f,"        movb    %%al, (%%rcx)\n");
  fprintf(f,"        movl    -48(%%rbp), %%eax\n");
  fprintf(f,"        jne     22b\n");
  fprintf(f,"        jmp     26f\n");
  fprintf(f,"25:\n");
  fprintf(f,"        cmpb    $0, (%%rdx)\n");
  fprintf(f,"        je      23b\n");
  fprintf(f,"        leaq    2(%%rcx), %%rsi\n");
  fprintf(f,"        movb    $46, 1(%%rcx)\n");
  fprintf(f,"        jmp     23b\n");
  fprintf(f,"26:\n");
  fprintf(f,"        xorl    %%edx, %%edx\n");
  fprintf(f,"27:\n");
  fprintf(f,"        movl    %%eax, %%esi\n");
  fprintf(f,"        subl    %%edx, %%esi\n");
  fprintf(f,"        testl   %%esi, %%esi\n");
  fprintf(f,"        jle     28f\n");
  fprintf(f,"        movb    $48, (%%rcx,%%rdx)\n");
  fprintf(f,"        incq    %%rdx\n");
  fprintf(f,"        jmp     27b\n");
  fprintf(f,"28:\n");
  fprintf(f,"        xorl    %%edx, %%edx\n");
  fprintf(f,"        testl   %%eax, %%eax\n");
  fprintf(f,"        cmovns  %%eax, %%edx\n");
  fprintf(f,"        movslq  %%edx, %%rsi\n");
  fprintf(f,"        subl    %%edx, %%eax\n");
  fprintf(f,"        addq    %%rsi, %%rcx\n");
  fprintf(f,"        movl    %%eax, -48(%%rbp)\n");
  fprintf(f,"29:\n");
  fprintf(f,"        movb    $32, (%%rcx)\n");
  fprintf(f,"        movb    $0, 1(%%rcx)\n");
  fprintf(f,"30:\n");
  fprintf(f,"        movabsq $freedtoa, %%rax\n");
  fprintf(f,"        callq   *%%rax\n");
  fprintf(f,"        popq    %%rdx\n");
  fprintf(f,"        movq    %%r12, %%rax\n");
  fprintf(f,"        popq    %%rcx\n");
  fprintf(f,"        popq    %%rbx\n");
  fprintf(f,"        popq    %%r12\n");
  fprintf(f,"        popq    %%r13\n");
  fprintf(f,"        popq    %%r14\n");
  fprintf(f,"        popq    %%rbp\n");
  fprintf(f,"        retq\n");
  fprintf(f,"        .size   g_fmt, .-g_fmt\n");
  fprintf(f,"################################################################\n");
  fprintf(f,"#\n");
  fprintf(f,"# End of David M. Gay code copyrighted by Lucent.\n");
  fprintf(f,"#\n");
  fprintf(f,"################################################################\n");
  fflush(f);
  return;
}

