//
// shibata.c - This file contains the code generation routines
//             from Naoki Shibata's SLEEF 2.80 that this compiler
//             uses to support the following Minimal BASIC functions:
//             SIN(), COS(), TAN(), ATAN(), LOG(), EXP(), and POW()
//             This code is in the public domain.
//             Upstream URL:
//             http://shibatch.sourceforge.net/
//
#include <stdlib.h>
#include "shibata.h"

//
//  This procedure will emit code for the runtime library support
//  of the Minimal BASIC SIN() function.  All math is done with
//  64bit float values.
//  The argument for xsin_u1 is passed in xmm0, and the result
//  is returned in xmm0.
//  This code was created by running a C compiler on Shibata's 
//  SLEEF version 2.80 files and having it emit assembly.  This 
//  requires SSE2 instruction set support.
//
void write_SIN(FILE *f) {
  if (NULL==f) {
    fprintf(stderr,"write_SIN(): file parameter is NULL\n");
    abort();
  }
  fprintf(f,"# http://shibatch.sourceforge.net/\n");
  fprintf(f,"        .section .rodata.cst16,\"aM\",@progbits,16\n");
  fprintf(f,"        .align  16\n");
  fprintf(f,".LCPI3_0:\n");
  fprintf(f,"        .quad   4599405781057128579     # double 3.183099e-01\n");
  fprintf(f,"        .quad   4599405781057128579     # double 3.183099e-01\n");
  fprintf(f,".LCPI3_1:\n");
  fprintf(f,"        .quad   -4746500136710963200    # double -1.984187e-09\n");
  fprintf(f,"        .quad   -4746500136710963200    # double -1.984187e-09\n");
  fprintf(f,".LCPI3_2:\n");
  fprintf(f,"        .quad   -4609115380307197952    # double -3.141593e+00\n");
  fprintf(f,"        .quad   -4609115380307197952    # double -3.141593e+00\n");
  fprintf(f,".LCPI3_3:\n");
  fprintf(f,"        .long   0                       # 0x0\n");
  fprintf(f,"        .long   2147483648              # 0x80000000\n");
  fprintf(f,"        .long   0                       # 0x0\n");
  fprintf(f,"        .long   2147483648              # 0x80000000\n");
  fprintf(f,".LCPI3_4:\n");
  fprintf(f,"        .quad   -134217728              # 0xfffffffff8000000\n");
  fprintf(f,"        .quad   -134217728              # 0xfffffffff8000000\n");
  fprintf(f,".LCPI3_5:\n");
  fprintf(f,"        .quad   -4876056140208668672    # double -4.503484e-18\n");
  fprintf(f,"        .quad   -4876056140208668672    # double -4.503484e-18\n");
  fprintf(f,".LCPI3_6:\n");
  fprintf(f,"        .quad   -5007598047184935648    # double -7.043120e-27\n");
  fprintf(f,"        .quad   -5007598047184935648    # double -7.043120e-27\n");
  fprintf(f,".LCPI3_7:\n");
  fprintf(f,"        .quad   4388899785603723549     # double 2.720524e-15\n");
  fprintf(f,"        .quad   4388899785603723549     # double 2.720524e-15\n");
  fprintf(f,".LCPI3_8:\n");
  fprintf(f,"        .quad   -4628199217061079723    # double -1.666667e-01\n");
  fprintf(f,"        .quad   -4628199217061079723    # double -1.666667e-01\n");
  fprintf(f,".LCPI3_9:\n");
  fprintf(f,"        .quad   4575957461383581881     # double 8.333333e-03\n");
  fprintf(f,"        .quad   4575957461383581881     # double 8.333333e-03\n");
  fprintf(f,".LCPI3_10:\n");
  fprintf(f,"        .quad   -4671919876300783067    # double -1.984127e-04\n");
  fprintf(f,"        .quad   -4671919876300783067    # double -1.984127e-04\n");
  fprintf(f,".LCPI3_11:\n");
  fprintf(f,"        .quad   4523617214282464237     # double 2.755732e-06\n");
  fprintf(f,"        .quad   4523617214282464237     # double 2.755732e-06\n");
  fprintf(f,".LCPI3_12:\n");
  fprintf(f,"        .quad   -4730215273302709927    # double -2.505211e-08\n");
  fprintf(f,"        .quad   -4730215273302709927    # double -2.505211e-08\n");
  fprintf(f,".LCPI3_13:\n");
  fprintf(f,"        .quad   4460272531817914215     # double 1.605894e-10\n");
  fprintf(f,"        .quad   4460272531817914215     # double 1.605894e-10\n");
  fprintf(f,".LCPI3_14:\n");
  fprintf(f,"        .quad   -4797771615172455600    # double -7.642926e-13\n");
  fprintf(f,"        .quad   -4797771615172455600    # double -7.642926e-13\n");
  fprintf(f,".LCPI3_15:\n");
  fprintf(f,"        .quad   4607182418800017408     # double 1.000000e+00\n");
  fprintf(f,"        .quad   4607182418800017408     # double 1.000000e+00\n");
  fprintf(f,".LCPI3_16:\n");
  fprintf(f,"        .long   1                       # 0x1\n");
  fprintf(f,"        .long   1                       # 0x1\n");
  fprintf(f,"        .long   0                       # 0x0\n");
  fprintf(f,"        .long   0                       # 0x0\n\n");

  fprintf(f,"        .section .text,\"ax\",@progbits\n\n");

  fprintf(f,"        .type   xsin_u1,@function\n");
  fprintf(f,"xsin_u1:\n");
  fprintf(f,"        pushq   %%r15\n");
  fprintf(f,"        pushq   %%r14\n");
  fprintf(f,"        pushq   %%rbx\n");
  fprintf(f,"        movabsq $.LCPI3_0, %%rax\n");
  fprintf(f,"        movapd  (%%rax), %%xmm1\n");
  fprintf(f,"        mulpd   %%xmm0, %%xmm1\n");
  fprintf(f,"        cvtpd2dq        %%xmm1, %%xmm11\n");
  fprintf(f,"        cvtdq2pd        %%xmm11, %%xmm8\n");
  fprintf(f,"        movabsq $.LCPI3_1, %%rax\n");
  fprintf(f,"        movapd  (%%rax), %%xmm2\n");
  fprintf(f,"        mulpd   %%xmm8, %%xmm2\n");
  fprintf(f,"        movabsq $.LCPI3_2, %%rax\n");
  fprintf(f,"        movapd  (%%rax), %%xmm4\n");
  fprintf(f,"        mulpd   %%xmm8, %%xmm4\n");
  fprintf(f,"        movapd  %%xmm4, %%xmm3\n");
  fprintf(f,"        addpd   %%xmm0, %%xmm3\n");
  fprintf(f,"        movapd  %%xmm2, %%xmm7\n");
  fprintf(f,"        addpd   %%xmm3, %%xmm7\n");
  fprintf(f,"        movapd  %%xmm3, %%xmm1\n");
  fprintf(f,"        subpd   %%xmm0, %%xmm1\n");
  fprintf(f,"        movabsq $0x100000001, %%rsi\n");
  fprintf(f,"        movabsq $.LCPI3_3, %%r10\n");
  fprintf(f,"        movabsq $.LCPI3_4, %%rcx\n");
  fprintf(f,"        movabsq $.LCPI3_5, %%rdi\n");
  fprintf(f,"        movabsq $.LCPI3_6, %%rbx\n");
  fprintf(f,"        movabsq $.LCPI3_7, %%rdx\n");
  fprintf(f,"        movabsq $.LCPI3_8, %%r8\n");
  fprintf(f,"        movabsq $.LCPI3_9, %%r9\n");
  fprintf(f,"        movabsq $.LCPI3_10, %%r11\n");
  fprintf(f,"        movabsq $.LCPI3_11, %%r14\n");
  fprintf(f,"        movabsq $.LCPI3_12, %%r15\n");
  fprintf(f,"        movapd  %%xmm7, %%xmm5\n");
  fprintf(f,"        subpd   %%xmm3, %%xmm5\n");
  fprintf(f,"        subpd   %%xmm1, %%xmm4\n");
  fprintf(f,"        movapd  %%xmm3, %%xmm6\n");
  fprintf(f,"        subpd   %%xmm1, %%xmm6\n");
  fprintf(f,"        movabsq $.LCPI3_13, %%rax\n");
  fprintf(f,"        movd    %%rsi, %%xmm9\n");
  fprintf(f,"        subpd   %%xmm6, %%xmm0\n");
  fprintf(f,"        movabsq $.LCPI3_14, %%rsi\n");
  fprintf(f,"        movapd  (%%rcx), %%xmm10\n");
  fprintf(f,"        addpd   %%xmm4, %%xmm0\n");
  fprintf(f,"        movapd  %%xmm7, %%xmm1\n");
  fprintf(f,"        subpd   %%xmm5, %%xmm1\n");
  fprintf(f,"        subpd   %%xmm1, %%xmm3\n");
  fprintf(f,"        movapd  (%%r10), %%xmm4\n");
  fprintf(f,"        movapd  %%xmm4, -32(%%rsp)\n");
  fprintf(f,"        subpd   %%xmm5, %%xmm2\n");
  fprintf(f,"        andpd   %%xmm11, %%xmm9\n");
  fprintf(f,"        movapd  %%xmm9, -16(%%rsp)\n");
  fprintf(f,"        addpd   %%xmm3, %%xmm2\n");
  fprintf(f,"        movapd  (%%rdi), %%xmm1\n");
  fprintf(f,"        mulpd   %%xmm8, %%xmm1\n");
  fprintf(f,"        movapd  %%xmm1, %%xmm14\n");
  fprintf(f,"        addpd   %%xmm7, %%xmm14\n");
  fprintf(f,"        movapd  %%xmm14, %%xmm3\n");
  fprintf(f,"        subpd   %%xmm7, %%xmm3\n");
  fprintf(f,"        addpd   %%xmm0, %%xmm2\n");
  fprintf(f,"        subpd   %%xmm3, %%xmm1\n");
  fprintf(f,"        movapd  %%xmm14, %%xmm0\n");
  fprintf(f,"        subpd   %%xmm3, %%xmm0\n");
  fprintf(f,"        subpd   %%xmm0, %%xmm7\n");
  fprintf(f,"        mulpd   (%%rbx), %%xmm8\n");
  fprintf(f,"        movapd  %%xmm8, %%xmm12\n");
  fprintf(f,"        addpd   %%xmm14, %%xmm12\n");
  fprintf(f,"        movapd  %%xmm12, %%xmm0\n");
  fprintf(f,"        subpd   %%xmm14, %%xmm0\n");
  fprintf(f,"        addpd   %%xmm1, %%xmm7\n");
  fprintf(f,"        addpd   %%xmm2, %%xmm7\n");
  fprintf(f,"        subpd   %%xmm0, %%xmm8\n");
  fprintf(f,"        movapd  %%xmm12, %%xmm1\n");
  fprintf(f,"        subpd   %%xmm0, %%xmm1\n");
  fprintf(f,"        subpd   %%xmm1, %%xmm14\n");
  fprintf(f,"        addpd   %%xmm8, %%xmm14\n");
  fprintf(f,"        movapd  %%xmm12, %%xmm2\n");
  fprintf(f,"        mulpd   %%xmm2, %%xmm2\n");
  fprintf(f,"        movapd  %%xmm2, %%xmm1\n");
  fprintf(f,"        movapd  %%xmm4, %%xmm5\n");
  fprintf(f,"        xorpd   %%xmm5, %%xmm1\n");
  fprintf(f,"        movapd  %%xmm12, %%xmm13\n");
  fprintf(f,"        andpd   %%xmm10, %%xmm13\n");
  fprintf(f,"        movapd  %%xmm13, %%xmm0\n");
  fprintf(f,"        mulpd   %%xmm0, %%xmm0\n");
  fprintf(f,"        addpd   %%xmm7, %%xmm14\n");
  fprintf(f,"        addpd   %%xmm1, %%xmm0\n");
  fprintf(f,"        movapd  %%xmm12, %%xmm11\n");
  fprintf(f,"        subpd   %%xmm13, %%xmm11\n");
  fprintf(f,"        movapd  %%xmm13, %%xmm1\n");
  fprintf(f,"        addpd   %%xmm1, %%xmm1\n");
  fprintf(f,"        mulpd   %%xmm11, %%xmm1\n");
  fprintf(f,"        addpd   %%xmm0, %%xmm1\n");
  fprintf(f,"        movapd  %%xmm11, %%xmm0\n");
  fprintf(f,"        mulpd   %%xmm0, %%xmm0\n");
  fprintf(f,"        addpd   %%xmm1, %%xmm0\n");
  fprintf(f,"        movapd  %%xmm14, %%xmm7\n");
  fprintf(f,"        addpd   %%xmm7, %%xmm7\n");
  fprintf(f,"        mulpd   %%xmm12, %%xmm7\n");
  fprintf(f,"        addpd   %%xmm0, %%xmm7\n");
  fprintf(f,"        movapd  (%%rdx), %%xmm4\n");
  fprintf(f,"        mulpd   %%xmm2, %%xmm4\n");
  fprintf(f,"        addpd   (%%rsi), %%xmm4\n");
  fprintf(f,"        mulpd   %%xmm2, %%xmm4\n");
  fprintf(f,"        addpd   (%%rax), %%xmm4\n");
  fprintf(f,"        mulpd   %%xmm2, %%xmm4\n");
  fprintf(f,"        addpd   (%%r15), %%xmm4\n");
  fprintf(f,"        mulpd   %%xmm2, %%xmm4\n");
  fprintf(f,"        addpd   (%%r14), %%xmm4\n");
  fprintf(f,"        mulpd   %%xmm2, %%xmm4\n");
  fprintf(f,"        addpd   (%%r11), %%xmm4\n");
  fprintf(f,"        mulpd   %%xmm2, %%xmm4\n");
  fprintf(f,"        addpd   (%%r9), %%xmm4\n");
  fprintf(f,"        mulpd   %%xmm2, %%xmm4\n");
  fprintf(f,"        movapd  (%%r8), %%xmm1\n");
  fprintf(f,"        movapd  %%xmm2, %%xmm0\n");
  fprintf(f,"        andpd   %%xmm10, %%xmm0\n");
  fprintf(f,"        movapd  %%xmm4, %%xmm3\n");
  fprintf(f,"        addpd   %%xmm1, %%xmm3\n");
  fprintf(f,"        movapd  %%xmm2, %%xmm15\n");
  fprintf(f,"        mulpd   %%xmm3, %%xmm15\n");
  fprintf(f,"        movapd  %%xmm15, %%xmm6\n");
  fprintf(f,"        xorpd   %%xmm5, %%xmm6\n");
  fprintf(f,"        movapd  %%xmm3, %%xmm5\n");
  fprintf(f,"        andpd   %%xmm10, %%xmm5\n");
  fprintf(f,"        movapd  %%xmm0, %%xmm9\n");
  fprintf(f,"        mulpd   %%xmm5, %%xmm9\n");
  fprintf(f,"        addpd   %%xmm6, %%xmm9\n");
  fprintf(f,"        movapd  %%xmm3, %%xmm8\n");
  fprintf(f,"        subpd   %%xmm5, %%xmm8\n");
  fprintf(f,"        movapd  %%xmm2, %%xmm6\n");
  fprintf(f,"        subpd   %%xmm0, %%xmm6\n");
  fprintf(f,"        mulpd   %%xmm8, %%xmm0\n");
  fprintf(f,"        addpd   %%xmm9, %%xmm0\n");
  fprintf(f,"        mulpd   %%xmm6, %%xmm5\n");
  fprintf(f,"        addpd   %%xmm0, %%xmm5\n");
  fprintf(f,"        mulpd   %%xmm8, %%xmm6\n");
  fprintf(f,"        addpd   %%xmm5, %%xmm6\n");
  fprintf(f,"        mulpd   %%xmm3, %%xmm7\n");
  fprintf(f,"        addpd   %%xmm6, %%xmm7\n");
  fprintf(f,"        subpd   %%xmm3, %%xmm1\n");
  fprintf(f,"        addpd   %%xmm4, %%xmm1\n");
  fprintf(f,"        mulpd   %%xmm2, %%xmm1\n");
  fprintf(f,"        movabsq $.LCPI3_15, %%rax\n");
  fprintf(f,"        addpd   %%xmm7, %%xmm1\n");
  fprintf(f,"        movapd  (%%rax), %%xmm2\n");
  fprintf(f,"        movapd  %%xmm15, %%xmm0\n");
  fprintf(f,"        addpd   %%xmm2, %%xmm0\n");
  fprintf(f,"        subpd   %%xmm0, %%xmm2\n");
  fprintf(f,"        addpd   %%xmm15, %%xmm2\n");
  fprintf(f,"        movabsq $.LCPI3_16, %%rax\n");
  fprintf(f,"        movdqa  -16(%%rsp), %%xmm3\n");
  fprintf(f,"        pcmpeqd (%%rax), %%xmm3\n");
  fprintf(f,"        pshufd  $80, %%xmm3, %%xmm4       # xmm4 = xmm3[0,0,1,1]\n");
  fprintf(f,"        pand    (%%r10), %%xmm4\n");
  fprintf(f,"        addpd   %%xmm1, %%xmm2\n");
  fprintf(f,"        movapd  %%xmm12, %%xmm1\n");
  fprintf(f,"        mulpd   %%xmm0, %%xmm1\n");
  fprintf(f,"        movapd  -32(%%rsp), %%xmm3\n");
  fprintf(f,"        xorpd   %%xmm1, %%xmm3\n");
  fprintf(f,"        andpd   %%xmm0, %%xmm10\n");
  fprintf(f,"        mulpd   %%xmm0, %%xmm14\n");
  fprintf(f,"        mulpd   %%xmm12, %%xmm2\n");
  fprintf(f,"        movapd  %%xmm13, %%xmm5\n");
  fprintf(f,"        mulpd   %%xmm10, %%xmm5\n");
  fprintf(f,"        addpd   %%xmm3, %%xmm5\n");
  fprintf(f,"        movapd  %%xmm11, %%xmm3\n");
  fprintf(f,"        mulpd   %%xmm10, %%xmm3\n");
  fprintf(f,"        addpd   %%xmm5, %%xmm3\n");
  fprintf(f,"        subpd   %%xmm10, %%xmm0\n");
  fprintf(f,"        mulpd   %%xmm0, %%xmm13\n");
  fprintf(f,"        addpd   %%xmm3, %%xmm13\n");
  fprintf(f,"        mulpd   %%xmm11, %%xmm0\n");
  fprintf(f,"        addpd   %%xmm13, %%xmm0\n");
  fprintf(f,"        addpd   %%xmm2, %%xmm0\n");
  fprintf(f,"        addpd   %%xmm14, %%xmm0\n");
  fprintf(f,"        addpd   %%xmm1, %%xmm0\n");
  fprintf(f,"        xorpd   %%xmm4, %%xmm0\n");
  fprintf(f,"        popq    %%rbx\n");
  fprintf(f,"        popq    %%r14\n");
  fprintf(f,"        popq    %%r15\n");
  fprintf(f,"        retq\n");
  fprintf(f,"        .size   xsin_u1, .-xsin_u1\n\n");
  fflush(f);
  return;
}

//
//  This procedure will emit code for the runtime library support
//  of the Minimal BASIC COS() function.  All math is done with
//  64bit float values.
//  The argument for xcos_u1 is passed in xmm0, and the result
//  is returned in xmm0.
//  This code was created by running a C compiler on Shibata's 
//  SLEEF version 2.80 files and having it emit assembly.  This 
//  requires SSE2 instruction set support.
//
void write_COS(FILE *f) {
  if (NULL==f) {
    fprintf(stderr,"write_COS(): file parameter is NULL\n");
    abort();
  }
  fprintf(f,"# http://shibatch.sourceforge.net/\n");
  fprintf(f,"        .section .rodata.cst16,\"aM\",@progbits,16\n");
  fprintf(f,"        .align  16\n");
  fprintf(f,".LCPI5_0:\n");
  fprintf(f,"        .quad   4599405781057128579     # double 3.183099e-01\n");
  fprintf(f,"        .quad   4599405781057128579     # double 3.183099e-01\n");
  fprintf(f,".LCPI5_1:\n");
  fprintf(f,"        .quad   -4620693217682128896    # double -5.000000e-01\n");
  fprintf(f,"        .quad   -4620693217682128896    # double -5.000000e-01\n");
  fprintf(f,".LCPI5_2:\n");
  fprintf(f,"        .long   1                       # 0x1\n");
  fprintf(f,"        .long   1                       # 0x1\n");
  fprintf(f,"        .long   0                       # 0x0\n");
  fprintf(f,"        .long   0                       # 0x0\n");
  fprintf(f,".LCPI5_3:\n");
  fprintf(f,"        .quad   -4613618979934568448    # double -1.570796e+00\n");
  fprintf(f,"        .quad   -4613618979934568448    # double -1.570796e+00\n");
  fprintf(f,".LCPI5_4:\n");
  fprintf(f,"        .quad   -4751003736338333696    # double -9.920936e-10\n");
  fprintf(f,"        .quad   -4751003736338333696    # double -9.920936e-10\n");
  fprintf(f,".LCPI5_5:\n");
  fprintf(f,"        .quad   -5012101646812306144    # double -3.521560e-27\n");
  fprintf(f,"        .quad   -5012101646812306144    # double -3.521560e-27\n");
  fprintf(f,".LCPI5_6:\n");
  fprintf(f,"        .quad   -4880559739836039168    # double -2.251742e-18\n");
  fprintf(f,"        .quad   -4880559739836039168    # double -2.251742e-18\n");
  fprintf(f,".LCPI5_7:\n");
  fprintf(f,"        .long   0                       # 0x0\n");
  fprintf(f,"        .long   2147483648              # 0x80000000\n");
  fprintf(f,"        .long   0                       # 0x0\n");
  fprintf(f,"        .long   2147483648              # 0x80000000\n");
  fprintf(f,".LCPI5_8:\n");
  fprintf(f,"        .quad   -134217728              # 0xfffffffff8000000\n");
  fprintf(f,"        .quad   -134217728              # 0xfffffffff8000000\n");
  fprintf(f,".LCPI5_9:\n");
  fprintf(f,"        .quad   4388899785603723549     # double 2.720524e-15\n");
  fprintf(f,"        .quad   4388899785603723549     # double 2.720524e-15\n");
  fprintf(f,".LCPI5_10:\n");
  fprintf(f,"        .quad   -4628199217061079723    # double -1.666667e-01\n");
  fprintf(f,"        .quad   -4628199217061079723    # double -1.666667e-01\n");
  fprintf(f,".LCPI5_11:\n");
  fprintf(f,"        .quad   4575957461383581881     # double 8.333333e-03\n");
  fprintf(f,"        .quad   4575957461383581881     # double 8.333333e-03\n");
  fprintf(f,".LCPI5_12:\n");
  fprintf(f,"        .quad   -4671919876300783067    # double -1.984127e-04\n");
  fprintf(f,"        .quad   -4671919876300783067    # double -1.984127e-04\n");
  fprintf(f,".LCPI5_13:\n");
  fprintf(f,"        .quad   4523617214282464237     # double 2.755732e-06\n");
  fprintf(f,"        .quad   4523617214282464237     # double 2.755732e-06\n");
  fprintf(f,".LCPI5_14:\n");
  fprintf(f,"        .quad   -4730215273302709927    # double -2.505211e-08\n");
  fprintf(f,"        .quad   -4730215273302709927    # double -2.505211e-08\n");
  fprintf(f,".LCPI5_15:\n");
  fprintf(f,"        .quad   4460272531817914215     # double 1.605894e-10\n");
  fprintf(f,"        .quad   4460272531817914215     # double 1.605894e-10\n");
  fprintf(f,".LCPI5_16:\n");
  fprintf(f,"        .quad   -4797771615172455600    # double -7.642926e-13\n");
  fprintf(f,"        .quad   -4797771615172455600    # double -7.642926e-13\n");
  fprintf(f,".LCPI5_17:\n");
  fprintf(f,"        .quad   4607182418800017408     # double 1.000000e+00\n");
  fprintf(f,"        .quad   4607182418800017408     # double 1.000000e+00\n\n");

  fprintf(f,"        .section .text,\"ax\",@progbits\n\n");

  fprintf(f,"        .type   xcos_u1,@function\n");
  fprintf(f,"xcos_u1:\n");
  fprintf(f,"        pushq   %%r15\n");
  fprintf(f,"        pushq   %%r14\n");
  fprintf(f,"        pushq   %%rbx\n");
  fprintf(f,"        movabsq $.LCPI5_0, %%rax\n");
  fprintf(f,"        movapd  (%%rax), %%xmm1\n");
  fprintf(f,"        mulpd   %%xmm0, %%xmm1\n");
  fprintf(f,"        movabsq $.LCPI5_1, %%rax\n");
  fprintf(f,"        addpd   (%%rax), %%xmm1\n");
  fprintf(f,"        cvtpd2dq        %%xmm1, %%xmm1\n");
  fprintf(f,"        paddd   %%xmm1, %%xmm1\n");
  fprintf(f,"        movabsq $.LCPI5_2, %%rax\n");
  fprintf(f,"        paddd   (%%rax), %%xmm1\n");
  fprintf(f,"        movabsq $0x200000002, %%rax\n");
  fprintf(f,"        cvtdq2pd        %%xmm1, %%xmm6\n");
  fprintf(f,"        movabsq $.LCPI5_3, %%rsi\n");
  fprintf(f,"        movabsq $.LCPI5_4, %%rdi\n");
  fprintf(f,"        movd    %%rax, %%xmm9\n");
  fprintf(f,"        pxor    %%xmm8, %%xmm8\n");
  fprintf(f,"        movabsq $.LCPI5_5, %%rbx\n");
  fprintf(f,"        movabsq $.LCPI5_6, %%rax\n");
  fprintf(f,"        movabsq $.LCPI5_7, %%r14\n");
  fprintf(f,"        movabsq $.LCPI5_8, %%rcx\n");
  fprintf(f,"        movabsq $.LCPI5_9, %%rdx\n");
  fprintf(f,"        movabsq $.LCPI5_10, %%r8\n");
  fprintf(f,"        movabsq $.LCPI5_11, %%r9\n");
  fprintf(f,"        movabsq $.LCPI5_12, %%r10\n");
  fprintf(f,"        movabsq $.LCPI5_13, %%r11\n");
  fprintf(f,"        movabsq $.LCPI5_14, %%r15\n");
  fprintf(f,"        movapd  (%%rsi), %%xmm7\n");
  fprintf(f,"        mulpd   %%xmm6, %%xmm7\n");
  fprintf(f,"        movapd  %%xmm7, %%xmm5\n");
  fprintf(f,"        addpd   %%xmm0, %%xmm5\n");
  fprintf(f,"        movapd  %%xmm5, %%xmm2\n");
  fprintf(f,"        subpd   %%xmm0, %%xmm2\n");
  fprintf(f,"        subpd   %%xmm2, %%xmm7\n");
  fprintf(f,"        movapd  %%xmm5, %%xmm3\n");
  fprintf(f,"        subpd   %%xmm2, %%xmm3\n");
  fprintf(f,"        subpd   %%xmm3, %%xmm0\n");
  fprintf(f,"        movabsq $.LCPI5_15, %%rsi\n");
  fprintf(f,"        movapd  (%%rdi), %%xmm2\n");
  fprintf(f,"        mulpd   %%xmm6, %%xmm2\n");
  fprintf(f,"        movapd  %%xmm2, %%xmm3\n");
  fprintf(f,"        addpd   %%xmm5, %%xmm3\n");
  fprintf(f,"        movapd  %%xmm3, %%xmm4\n");
  fprintf(f,"        subpd   %%xmm5, %%xmm4\n");
  fprintf(f,"        addpd   %%xmm7, %%xmm0\n");
  fprintf(f,"        movapd  %%xmm3, %%xmm7\n");
  fprintf(f,"        subpd   %%xmm4, %%xmm7\n");
  fprintf(f,"        subpd   %%xmm7, %%xmm5\n");
  fprintf(f,"        movabsq $.LCPI5_16, %%rdi\n");
  fprintf(f,"        movapd  (%%rcx), %%xmm10\n");
  fprintf(f,"        subpd   %%xmm4, %%xmm2\n");
  fprintf(f,"        movapd  (%%r14), %%xmm11\n");
  fprintf(f,"        movapd  %%xmm11, -32(%%rsp)\n");
  fprintf(f,"        andpd   %%xmm1, %%xmm9\n");
  fprintf(f,"        addpd   %%xmm5, %%xmm2\n");
  fprintf(f,"        movapd  (%%rax), %%xmm1\n");
  fprintf(f,"        mulpd   %%xmm6, %%xmm1\n");
  fprintf(f,"        mulpd   (%%rbx), %%xmm6\n");
  fprintf(f,"        movapd  %%xmm1, %%xmm13\n");
  fprintf(f,"        addpd   %%xmm3, %%xmm13\n");
  fprintf(f,"        pcmpeqd %%xmm8, %%xmm9\n");
  fprintf(f,"        movdqa  %%xmm9, -16(%%rsp)\n");
  fprintf(f,"        movapd  %%xmm13, %%xmm4\n");
  fprintf(f,"        subpd   %%xmm3, %%xmm4\n");
  fprintf(f,"        addpd   %%xmm0, %%xmm2\n");
  fprintf(f,"        subpd   %%xmm4, %%xmm1\n");
  fprintf(f,"        movapd  %%xmm13, %%xmm0\n");
  fprintf(f,"        subpd   %%xmm4, %%xmm0\n");
  fprintf(f,"        subpd   %%xmm0, %%xmm3\n");
  fprintf(f,"        addpd   %%xmm1, %%xmm3\n");
  fprintf(f,"        movapd  %%xmm6, %%xmm12\n");
  fprintf(f,"        addpd   %%xmm13, %%xmm12\n");
  fprintf(f,"        movapd  %%xmm12, %%xmm0\n");
  fprintf(f,"        subpd   %%xmm13, %%xmm0\n");
  fprintf(f,"        movapd  %%xmm12, %%xmm1\n");
  fprintf(f,"        subpd   %%xmm0, %%xmm1\n");
  fprintf(f,"        addpd   %%xmm2, %%xmm3\n");
  fprintf(f,"        subpd   %%xmm0, %%xmm6\n");
  fprintf(f,"        movapd  %%xmm12, %%xmm7\n");
  fprintf(f,"        mulpd   %%xmm7, %%xmm7\n");
  fprintf(f,"        movapd  %%xmm7, %%xmm0\n");
  fprintf(f,"        xorpd   %%xmm11, %%xmm0\n");
  fprintf(f,"        movapd  %%xmm11, %%xmm5\n");
  fprintf(f,"        subpd   %%xmm1, %%xmm13\n");
  fprintf(f,"        addpd   %%xmm6, %%xmm13\n");
  fprintf(f,"        movapd  %%xmm12, %%xmm14\n");
  fprintf(f,"        andpd   %%xmm10, %%xmm14\n");
  fprintf(f,"        movapd  %%xmm14, %%xmm1\n");
  fprintf(f,"        mulpd   %%xmm1, %%xmm1\n");
  fprintf(f,"        addpd   %%xmm3, %%xmm13\n");
  fprintf(f,"        addpd   %%xmm0, %%xmm1\n");
  fprintf(f,"        movapd  %%xmm12, %%xmm11\n");
  fprintf(f,"        subpd   %%xmm14, %%xmm11\n");
  fprintf(f,"        movapd  %%xmm14, %%xmm0\n");
  fprintf(f,"        addpd   %%xmm0, %%xmm0\n");
  fprintf(f,"        mulpd   %%xmm11, %%xmm0\n");
  fprintf(f,"        addpd   %%xmm1, %%xmm0\n");
  fprintf(f,"        movapd  %%xmm11, %%xmm1\n");
  fprintf(f,"        mulpd   %%xmm1, %%xmm1\n");
  fprintf(f,"        addpd   %%xmm0, %%xmm1\n");
  fprintf(f,"        movapd  %%xmm13, %%xmm3\n");
  fprintf(f,"        addpd   %%xmm3, %%xmm3\n");
  fprintf(f,"        mulpd   %%xmm12, %%xmm3\n");
  fprintf(f,"        addpd   %%xmm1, %%xmm3\n");
  fprintf(f,"        movapd  (%%rdx), %%xmm4\n");
  fprintf(f,"        mulpd   %%xmm7, %%xmm4\n");
  fprintf(f,"        addpd   (%%rdi), %%xmm4\n");
  fprintf(f,"        mulpd   %%xmm7, %%xmm4\n");
  fprintf(f,"        addpd   (%%rsi), %%xmm4\n");
  fprintf(f,"        mulpd   %%xmm7, %%xmm4\n");
  fprintf(f,"        addpd   (%%r15), %%xmm4\n");
  fprintf(f,"        mulpd   %%xmm7, %%xmm4\n");
  fprintf(f,"        addpd   (%%r11), %%xmm4\n");
  fprintf(f,"        mulpd   %%xmm7, %%xmm4\n");
  fprintf(f,"        addpd   (%%r10), %%xmm4\n");
  fprintf(f,"        mulpd   %%xmm7, %%xmm4\n");
  fprintf(f,"        addpd   (%%r9), %%xmm4\n");
  fprintf(f,"        mulpd   %%xmm7, %%xmm4\n");
  fprintf(f,"        movapd  (%%r8), %%xmm2\n");
  fprintf(f,"        movapd  %%xmm7, %%xmm0\n");
  fprintf(f,"        andpd   %%xmm10, %%xmm0\n");
  fprintf(f,"        movapd  %%xmm4, %%xmm6\n");
  fprintf(f,"        addpd   %%xmm2, %%xmm6\n");
  fprintf(f,"        movapd  %%xmm7, %%xmm15\n");
  fprintf(f,"        mulpd   %%xmm6, %%xmm15\n");
  fprintf(f,"        movapd  %%xmm15, %%xmm1\n");
  fprintf(f,"        xorpd   %%xmm5, %%xmm1\n");
  fprintf(f,"        movapd  %%xmm6, %%xmm5\n");
  fprintf(f,"        andpd   %%xmm10, %%xmm5\n");
  fprintf(f,"        movapd  %%xmm0, %%xmm9\n");
  fprintf(f,"        mulpd   %%xmm5, %%xmm9\n");
  fprintf(f,"        addpd   %%xmm1, %%xmm9\n");
  fprintf(f,"        movapd  %%xmm6, %%xmm8\n");
  fprintf(f,"        subpd   %%xmm5, %%xmm8\n");
  fprintf(f,"        movapd  %%xmm7, %%xmm1\n");
  fprintf(f,"        subpd   %%xmm0, %%xmm1\n");
  fprintf(f,"        mulpd   %%xmm8, %%xmm0\n");
  fprintf(f,"        addpd   %%xmm9, %%xmm0\n");
  fprintf(f,"        mulpd   %%xmm1, %%xmm5\n");
  fprintf(f,"        addpd   %%xmm0, %%xmm5\n");
  fprintf(f,"        mulpd   %%xmm8, %%xmm1\n");
  fprintf(f,"        addpd   %%xmm5, %%xmm1\n");
  fprintf(f,"        mulpd   %%xmm6, %%xmm3\n");
  fprintf(f,"        addpd   %%xmm1, %%xmm3\n");
  fprintf(f,"        subpd   %%xmm6, %%xmm2\n");
  fprintf(f,"        addpd   %%xmm4, %%xmm2\n");
  fprintf(f,"        mulpd   %%xmm7, %%xmm2\n");
  fprintf(f,"        movabsq $.LCPI5_17, %%rax\n");
  fprintf(f,"        addpd   %%xmm3, %%xmm2\n");
  fprintf(f,"        movapd  (%%rax), %%xmm3\n");
  fprintf(f,"        movapd  %%xmm15, %%xmm0\n");
  fprintf(f,"        addpd   %%xmm3, %%xmm0\n");
  fprintf(f,"        subpd   %%xmm0, %%xmm3\n");
  fprintf(f,"        addpd   %%xmm15, %%xmm3\n");
  fprintf(f,"        pshufd  $80, -16(%%rsp), %%xmm4 # xmm4 = mem[0,0,1,1]\n");
  fprintf(f,"        pand    (%%r14), %%xmm4\n");
  fprintf(f,"        addpd   %%xmm2, %%xmm3\n");
  fprintf(f,"        movapd  %%xmm12, %%xmm2\n");
  fprintf(f,"        mulpd   %%xmm0, %%xmm2\n");
  fprintf(f,"        movapd  -32(%%rsp), %%xmm1\n");
  fprintf(f,"        xorpd   %%xmm2, %%xmm1\n");
  fprintf(f,"        andpd   %%xmm0, %%xmm10\n");
  fprintf(f,"        mulpd   %%xmm0, %%xmm13\n");
  fprintf(f,"        mulpd   %%xmm12, %%xmm3\n");
  fprintf(f,"        movapd  %%xmm14, %%xmm5\n");
  fprintf(f,"        mulpd   %%xmm10, %%xmm5\n");
  fprintf(f,"        addpd   %%xmm1, %%xmm5\n");
  fprintf(f,"        movapd  %%xmm11, %%xmm1\n");
  fprintf(f,"        mulpd   %%xmm10, %%xmm1\n");
  fprintf(f,"        addpd   %%xmm5, %%xmm1\n");
  fprintf(f,"        subpd   %%xmm10, %%xmm0\n");
  fprintf(f,"        mulpd   %%xmm0, %%xmm14\n");
  fprintf(f,"        addpd   %%xmm1, %%xmm14\n");
  fprintf(f,"        mulpd   %%xmm11, %%xmm0\n");
  fprintf(f,"        addpd   %%xmm14, %%xmm0\n");
  fprintf(f,"        addpd   %%xmm3, %%xmm0\n");
  fprintf(f,"        addpd   %%xmm13, %%xmm0\n");
  fprintf(f,"        addpd   %%xmm2, %%xmm0\n");
  fprintf(f,"        xorpd   %%xmm4, %%xmm0\n");
  fprintf(f,"        popq    %%rbx\n");
  fprintf(f,"        popq    %%r14\n");
  fprintf(f,"        popq    %%r15\n");
  fprintf(f,"        retq\n");
  fprintf(f,"        .size   xcos_u1, .-xcos_u1\n\n");
  fflush(f);
  return;
}

//
//  This procedure will emit code for the runtime library support
//  of the Minimal BASIC TAN() function.  All math is done with
//  64bit float values.
//  The argument for xtan_u1 is passed in xmm0, and the result
//  is returned in xmm0.
//  This code was created by running a C compiler on Shibata's 
//  SLEEF version 2.80 files and having it emit assembly.  This 
//  requires SSE2 instruction set support.
//
void write_TAN(FILE *f) {
  if (NULL==f) {
    fprintf(stderr,"write_TAN(): file parameter is NULL\n");
    abort();
  }
  fprintf(f,"# http://shibatch.sourceforge.net/\n");
  fprintf(f,"        .section .rodata.cst16,\"aM\",@progbits,16\n");
  fprintf(f,"        .align  16\n");
  fprintf(f,".LCPI9_0:\n");
  fprintf(f,"        .quad   4603909380684499075     # double 6.366198e-01\n");
  fprintf(f,"        .quad   4603909380684499075     # double 6.366198e-01\n");
  fprintf(f,".LCPI9_1:\n");
  fprintf(f,"        .quad   -4613618979934568448    # double -1.570796e+00\n");
  fprintf(f,"        .quad   -4613618979934568448    # double -1.570796e+00\n");
  fprintf(f,".LCPI9_2:\n");
  fprintf(f,"        .quad   -4751003736338333696    # double -9.920936e-10\n");
  fprintf(f,"        .quad   -4751003736338333696    # double -9.920936e-10\n");
  fprintf(f,".LCPI9_3:\n");
  fprintf(f,"        .quad   -134217728              # 0xfffffffff8000000\n");
  fprintf(f,"        .quad   -134217728              # 0xfffffffff8000000\n");
  fprintf(f,".LCPI9_4:\n");
  fprintf(f,"        .quad   -9223372036854775808    # 0x8000000000000000\n");
  fprintf(f,"        .quad   -9223372036854775808    # 0x8000000000000000\n");
  fprintf(f,".LCPI9_5:\n");
  fprintf(f,"        .long   1                       # 0x1\n");
  fprintf(f,"        .long   1                       # 0x1\n");
  fprintf(f,"        .long   0                       # 0x0\n");
  fprintf(f,"        .long   0                       # 0x0\n");
  fprintf(f,".LCPI9_6:\n");
  fprintf(f,"        .quad   -4880559739836039168    # double -2.251742e-18\n");
  fprintf(f,"        .quad   -4880559739836039168    # double -2.251742e-18\n");
  fprintf(f,".LCPI9_7:\n");
  fprintf(f,"        .quad   -5012101646812306144    # double -3.521560e-27\n");
  fprintf(f,"        .quad   -5012101646812306144    # double -3.521560e-27\n");
  fprintf(f,".LCPI9_8:\n");
  fprintf(f,"        .quad   -4685091139436140041    # double -2.595198e-05\n");
  fprintf(f,"        .quad   -4685091139436140041    # double -2.595198e-05\n");
  fprintf(f,".LCPI9_9:\n");
  fprintf(f,"        .quad   4532104388999833658     # double 1.014197e-05\n");
  fprintf(f,"        .quad   4532104388999833658     # double 1.014197e-05\n");
  fprintf(f,".LCPI9_10:\n");
  fprintf(f,"        .quad   4599676419421066611     # double 3.333333e-01\n");
  fprintf(f,"        .quad   4599676419421066611     # double 3.333333e-01\n");
  fprintf(f,".LCPI9_11:\n");
  fprintf(f,"        .quad   4593971859893056481     # double 1.333333e-01\n");
  fprintf(f,"        .quad   4593971859893056481     # double 1.333333e-01\n");
  fprintf(f,".LCPI9_12:\n");
  fprintf(f,"        .quad   4587938466107978171     # double 5.396825e-02\n");
  fprintf(f,"        .quad   4587938466107978171     # double 5.396825e-02\n");
  fprintf(f,".LCPI9_13:\n");
  fprintf(f,"        .quad   4581960672243165804     # double 2.186949e-02\n");
  fprintf(f,"        .quad   4581960672243165804     # double 2.186949e-02\n");
  fprintf(f,".LCPI9_14:\n");
  fprintf(f,"        .quad   4576262931458570317     # double 8.863239e-03\n");
  fprintf(f,"        .quad   4576262931458570317     # double 8.863239e-03\n");
  fprintf(f,".LCPI9_15:\n");
  fprintf(f,"        .quad   4570429212234560085     # double 3.592087e-03\n");
  fprintf(f,"        .quad   4570429212234560085     # double 3.592087e-03\n");
  fprintf(f,".LCPI9_16:\n");
  fprintf(f,"        .quad   4564358027526737477     # double 1.456128e-03\n");
  fprintf(f,"        .quad   4564358027526737477     # double 1.456128e-03\n");
  fprintf(f,".LCPI9_17:\n");
  fprintf(f,"        .quad   4558567225388505559     # double 5.885052e-04\n");
  fprintf(f,"        .quad   4558567225388505559     # double 5.885052e-04\n");
  fprintf(f,".LCPI9_18:\n");
  fprintf(f,"        .quad   4553152953310086792     # double 2.448849e-04\n");
  fprintf(f,"        .quad   4553152953310086792     # double 2.448849e-04\n");
  fprintf(f,".LCPI9_19:\n");
  fprintf(f,"        .quad   4545602767838147233     # double 8.096745e-05\n");
  fprintf(f,"        .quad   4545602767838147233     # double 8.096745e-05\n");
  fprintf(f,".LCPI9_20:\n");
  fprintf(f,"        .quad   4544902034955620668     # double 7.147075e-05\n");
  fprintf(f,"        .quad   4544902034955620668     # double 7.147075e-05\n");
  fprintf(f,".LCPI9_21:\n");
  fprintf(f,"        .quad   4542848669560755340     # double 5.233881e-05\n");
  fprintf(f,"        .quad   4542848669560755340     # double 5.233881e-05\n");
  fprintf(f,".LCPI9_22:\n");
  fprintf(f,"        .quad   -4683747826197984572    # double -3.050330e-05\n");
  fprintf(f,"        .quad   -4683747826197984572    # double -3.050330e-05\n");
  fprintf(f,".LCPI9_23:\n");
  fprintf(f,"        .quad   4607182418800017408     # double 1.000000e+00\n");
  fprintf(f,"        .quad   4607182418800017408     # double 1.000000e+00\n\n");

  fprintf(f,"        .section .text,\"ax\",@progbits\n\n");

  fprintf(f,"        .type   xtan_u1,@function\n");
  fprintf(f,"xtan_u1:\n");
  fprintf(f,"        pushq   %%rbp\n");
  fprintf(f,"        pushq   %%r15\n");
  fprintf(f,"        pushq   %%r14\n");
  fprintf(f,"        pushq   %%r13\n");
  fprintf(f,"        pushq   %%r12\n");
  fprintf(f,"        pushq   %%rbx\n");
  fprintf(f,"        movapd  %%xmm0, %%xmm1\n");
  fprintf(f,"        movabsq $.LCPI9_0, %%rax\n");
  fprintf(f,"        movapd  (%%rax), %%xmm0\n");
  fprintf(f,"        mulpd   %%xmm1, %%xmm0\n");
  fprintf(f,"        cvtpd2dq        %%xmm0, %%xmm8\n");
  fprintf(f,"        cvtdq2pd        %%xmm8, %%xmm11\n");
  fprintf(f,"        movabsq $.LCPI9_1, %%rax\n");
  fprintf(f,"        movapd  (%%rax), %%xmm7\n");
  fprintf(f,"        mulpd   %%xmm11, %%xmm7\n");
  fprintf(f,"        movapd  %%xmm7, %%xmm6\n");
  fprintf(f,"        addpd   %%xmm1, %%xmm6\n");
  fprintf(f,"        movapd  %%xmm6, %%xmm3\n");
  fprintf(f,"        subpd   %%xmm1, %%xmm3\n");
  fprintf(f,"        movabsq $0x100000001, %%rdx\n");
  fprintf(f,"        movabsq $.LCPI9_2, %%rdi\n");
  fprintf(f,"        movabsq $.LCPI9_3, %%rsi\n");
  fprintf(f,"        movabsq $.LCPI9_4, %%rax\n");
  fprintf(f,"        movabsq $.LCPI9_5, %%rcx\n");
  fprintf(f,"        movabsq $.LCPI9_6, %%rbx\n");
  fprintf(f,"        movabsq $.LCPI9_7, %%rbp\n");
  fprintf(f,"        movabsq $.LCPI9_8, %%r12\n");
  fprintf(f,"        movabsq $.LCPI9_9, %%r13\n");
  fprintf(f,"        movabsq $.LCPI9_10, %%r8\n");
  fprintf(f,"        movapd  %%xmm6, %%xmm0\n");
  fprintf(f,"        subpd   %%xmm3, %%xmm0\n");
  fprintf(f,"        movabsq $.LCPI9_11, %%r9\n");
  fprintf(f,"        movapd  (%%rdi), %%xmm4\n");
  fprintf(f,"        mulpd   %%xmm11, %%xmm4\n");
  fprintf(f,"        movapd  %%xmm4, %%xmm5\n");
  fprintf(f,"        addpd   %%xmm6, %%xmm5\n");
  fprintf(f,"        movabsq $.LCPI9_12, %%r10\n");
  fprintf(f,"        movd    %%rdx, %%xmm2\n");
  fprintf(f,"        subpd   %%xmm3, %%xmm7\n");
  fprintf(f,"        movapd  %%xmm5, %%xmm3\n");
  fprintf(f,"        subpd   %%xmm6, %%xmm3\n");
  fprintf(f,"        subpd   %%xmm0, %%xmm1\n");
  fprintf(f,"        andpd   %%xmm8, %%xmm2\n");
  fprintf(f,"        movabsq $.LCPI9_13, %%r11\n");
  fprintf(f,"        pcmpeqd (%%rcx), %%xmm2\n");
  fprintf(f,"        pshufd  $80, %%xmm2, %%xmm0       # xmm0 = xmm2[0,0,1,1]\n");
  fprintf(f,"        movabsq $.LCPI9_14, %%r14\n");
  fprintf(f,"        movdqa  (%%rax), %%xmm9\n");
  fprintf(f,"        pand    %%xmm0, %%xmm9\n");
  fprintf(f,"        movabsq $.LCPI9_15, %%r15\n");
  fprintf(f,"        movapd  (%%rsi), %%xmm10\n");
  fprintf(f,"        addpd   %%xmm7, %%xmm1\n");
  fprintf(f,"        movapd  %%xmm5, %%xmm2\n");
  fprintf(f,"        subpd   %%xmm3, %%xmm2\n");
  fprintf(f,"        subpd   %%xmm2, %%xmm6\n");
  fprintf(f,"        subpd   %%xmm3, %%xmm4\n");
  fprintf(f,"        addpd   %%xmm6, %%xmm4\n");
  fprintf(f,"        movabsq $.LCPI9_16, %%rsi\n");
  fprintf(f,"        movapd  (%%rbx), %%xmm3\n");
  fprintf(f,"        mulpd   %%xmm11, %%xmm3\n");
  fprintf(f,"        movapd  %%xmm3, %%xmm13\n");
  fprintf(f,"        addpd   %%xmm5, %%xmm13\n");
  fprintf(f,"        movabsq $.LCPI9_17, %%rbx\n");
  fprintf(f,"        mulpd   (%%rbp), %%xmm11\n");
  fprintf(f,"        addpd   %%xmm1, %%xmm4\n");
  fprintf(f,"        movapd  %%xmm13, %%xmm2\n");
  fprintf(f,"        subpd   %%xmm5, %%xmm2\n");
  fprintf(f,"        movapd  %%xmm13, %%xmm1\n");
  fprintf(f,"        subpd   %%xmm2, %%xmm1\n");
  fprintf(f,"        subpd   %%xmm2, %%xmm3\n");
  fprintf(f,"        movabsq $.LCPI9_18, %%rdi\n");
  fprintf(f,"        movapd  (%%rax), %%xmm6\n");
  fprintf(f,"        movapd  %%xmm6, -40(%%rsp)\n");
  fprintf(f,"        subpd   %%xmm1, %%xmm5\n");
  fprintf(f,"        addpd   %%xmm3, %%xmm5\n");
  fprintf(f,"        addpd   %%xmm4, %%xmm5\n");
  fprintf(f,"        movapd  %%xmm11, %%xmm12\n");
  fprintf(f,"        addpd   %%xmm13, %%xmm12\n");
  fprintf(f,"        movapd  %%xmm12, %%xmm1\n");
  fprintf(f,"        subpd   %%xmm13, %%xmm1\n");
  fprintf(f,"        subpd   %%xmm1, %%xmm11\n");
  fprintf(f,"        movapd  %%xmm12, %%xmm2\n");
  fprintf(f,"        subpd   %%xmm1, %%xmm2\n");
  fprintf(f,"        subpd   %%xmm2, %%xmm13\n");
  fprintf(f,"        addpd   %%xmm11, %%xmm13\n");
  fprintf(f,"        addpd   %%xmm5, %%xmm13\n");
  fprintf(f,"        xorpd   %%xmm9, %%xmm12\n");
  fprintf(f,"        movapd  %%xmm12, %%xmm2\n");
  fprintf(f,"        mulpd   %%xmm2, %%xmm2\n");
  fprintf(f,"        movapd  %%xmm2, %%xmm3\n");
  fprintf(f,"        xorpd   %%xmm6, %%xmm3\n");
  fprintf(f,"        movapd  %%xmm12, %%xmm11\n");
  fprintf(f,"        andpd   %%xmm10, %%xmm11\n");
  fprintf(f,"        movapd  %%xmm11, %%xmm1\n");
  fprintf(f,"        mulpd   %%xmm1, %%xmm1\n");
  fprintf(f,"        xorpd   %%xmm9, %%xmm13\n");
  fprintf(f,"        addpd   %%xmm3, %%xmm1\n");
  fprintf(f,"        movapd  %%xmm12, %%xmm4\n");
  fprintf(f,"        subpd   %%xmm11, %%xmm4\n");
  fprintf(f,"        movapd  %%xmm4, -24(%%rsp)\n");
  fprintf(f,"        movapd  %%xmm11, %%xmm3\n");
  fprintf(f,"        addpd   %%xmm3, %%xmm3\n");
  fprintf(f,"        mulpd   %%xmm4, %%xmm3\n");
  fprintf(f,"        addpd   %%xmm1, %%xmm3\n");
  fprintf(f,"        movapd  %%xmm4, %%xmm1\n");
  fprintf(f,"        mulpd   %%xmm1, %%xmm1\n");
  fprintf(f,"        addpd   %%xmm3, %%xmm1\n");
  fprintf(f,"        movapd  %%xmm13, %%xmm15\n");
  fprintf(f,"        addpd   %%xmm15, %%xmm15\n");
  fprintf(f,"        mulpd   %%xmm12, %%xmm15\n");
  fprintf(f,"        addpd   %%xmm1, %%xmm15\n");
  fprintf(f,"        movabsq $.LCPI9_19, %%rax\n");
  fprintf(f,"        movapd  (%%r13), %%xmm1\n");
  fprintf(f,"        mulpd   %%xmm2, %%xmm1\n");
  fprintf(f,"        movabsq $.LCPI9_20, %%rcx\n");
  fprintf(f,"        addpd   (%%r12), %%xmm1\n");
  fprintf(f,"        mulpd   %%xmm2, %%xmm1\n");
  fprintf(f,"        movabsq $.LCPI9_21, %%rdx\n");
  fprintf(f,"        addpd   (%%rdx), %%xmm1\n");
  fprintf(f,"        mulpd   %%xmm2, %%xmm1\n");
  fprintf(f,"        movabsq $.LCPI9_22, %%rdx\n");
  fprintf(f,"        addpd   (%%rdx), %%xmm1\n");
  fprintf(f,"        mulpd   %%xmm2, %%xmm1\n");
  fprintf(f,"        addpd   (%%rcx), %%xmm1\n");
  fprintf(f,"        mulpd   %%xmm2, %%xmm1\n");
  fprintf(f,"        addpd   (%%rax), %%xmm1\n");
  fprintf(f,"        mulpd   %%xmm2, %%xmm1\n");
  fprintf(f,"        addpd   (%%rdi), %%xmm1\n");
  fprintf(f,"        mulpd   %%xmm2, %%xmm1\n");
  fprintf(f,"        addpd   (%%rbx), %%xmm1\n");
  fprintf(f,"        mulpd   %%xmm2, %%xmm1\n");
  fprintf(f,"        addpd   (%%rsi), %%xmm1\n");
  fprintf(f,"        mulpd   %%xmm2, %%xmm1\n");
  fprintf(f,"        addpd   (%%r15), %%xmm1\n");
  fprintf(f,"        mulpd   %%xmm2, %%xmm1\n");
  fprintf(f,"        addpd   (%%r14), %%xmm1\n");
  fprintf(f,"        mulpd   %%xmm2, %%xmm1\n");
  fprintf(f,"        addpd   (%%r11), %%xmm1\n");
  fprintf(f,"        mulpd   %%xmm2, %%xmm1\n");
  fprintf(f,"        addpd   (%%r10), %%xmm1\n");
  fprintf(f,"        mulpd   %%xmm2, %%xmm1\n");
  fprintf(f,"        addpd   (%%r9), %%xmm1\n");
  fprintf(f,"        mulpd   %%xmm2, %%xmm1\n");
  fprintf(f,"        movapd  (%%r8), %%xmm5\n");
  fprintf(f,"        movapd  %%xmm2, %%xmm4\n");
  fprintf(f,"        andpd   %%xmm10, %%xmm4\n");
  fprintf(f,"        movapd  %%xmm1, %%xmm3\n");
  fprintf(f,"        addpd   %%xmm5, %%xmm3\n");
  fprintf(f,"        movapd  %%xmm2, %%xmm14\n");
  fprintf(f,"        mulpd   %%xmm3, %%xmm14\n");
  fprintf(f,"        movapd  %%xmm14, %%xmm7\n");
  fprintf(f,"        xorpd   %%xmm6, %%xmm7\n");
  fprintf(f,"        movapd  %%xmm3, %%xmm6\n");
  fprintf(f,"        andpd   %%xmm10, %%xmm6\n");
  fprintf(f,"        movapd  %%xmm4, %%xmm9\n");
  fprintf(f,"        mulpd   %%xmm6, %%xmm9\n");
  fprintf(f,"        addpd   %%xmm7, %%xmm9\n");
  fprintf(f,"        movapd  %%xmm3, %%xmm8\n");
  fprintf(f,"        subpd   %%xmm6, %%xmm8\n");
  fprintf(f,"        movapd  %%xmm2, %%xmm7\n");
  fprintf(f,"        subpd   %%xmm4, %%xmm7\n");
  fprintf(f,"        mulpd   %%xmm8, %%xmm4\n");
  fprintf(f,"        addpd   %%xmm9, %%xmm4\n");
  fprintf(f,"        mulpd   %%xmm7, %%xmm6\n");
  fprintf(f,"        addpd   %%xmm4, %%xmm6\n");
  fprintf(f,"        mulpd   %%xmm8, %%xmm7\n");
  fprintf(f,"        addpd   %%xmm6, %%xmm7\n");
  fprintf(f,"        mulpd   %%xmm3, %%xmm15\n");
  fprintf(f,"        addpd   %%xmm7, %%xmm15\n");
  fprintf(f,"        subpd   %%xmm3, %%xmm5\n");
  fprintf(f,"        addpd   %%xmm1, %%xmm5\n");
  fprintf(f,"        mulpd   %%xmm2, %%xmm5\n");
  fprintf(f,"        movabsq $.LCPI9_23, %%rax\n");
  fprintf(f,"        addpd   %%xmm15, %%xmm5\n");
  fprintf(f,"        movapd  (%%rax), %%xmm2\n");
  fprintf(f,"        movapd  %%xmm14, %%xmm1\n");
  fprintf(f,"        addpd   %%xmm2, %%xmm1\n");
  fprintf(f,"        movapd  %%xmm12, %%xmm8\n");
  fprintf(f,"        mulpd   %%xmm1, %%xmm8\n");
  fprintf(f,"        movapd  %%xmm8, %%xmm4\n");
  fprintf(f,"        andpd   %%xmm10, %%xmm4\n");
  fprintf(f,"        movapd  %%xmm2, %%xmm15\n");
  fprintf(f,"        divpd   %%xmm8, %%xmm15\n");
  fprintf(f,"        movapd  %%xmm15, %%xmm3\n");
  fprintf(f,"        andpd   %%xmm10, %%xmm3\n");
  fprintf(f,"        movapd  %%xmm3, %%xmm6\n");
  fprintf(f,"        mulpd   %%xmm4, %%xmm6\n");
  fprintf(f,"        movapd  %%xmm2, %%xmm7\n");
  fprintf(f,"        subpd   %%xmm6, %%xmm7\n");
  fprintf(f,"        subpd   %%xmm1, %%xmm2\n");
  fprintf(f,"        addpd   %%xmm14, %%xmm2\n");
  fprintf(f,"        addpd   %%xmm5, %%xmm2\n");
  fprintf(f,"        movapd  %%xmm15, %%xmm5\n");
  fprintf(f,"        subpd   %%xmm3, %%xmm5\n");
  fprintf(f,"        movapd  %%xmm5, %%xmm6\n");
  fprintf(f,"        mulpd   %%xmm4, %%xmm6\n");
  fprintf(f,"        subpd   %%xmm6, %%xmm7\n");
  fprintf(f,"        movapd  %%xmm8, %%xmm6\n");
  fprintf(f,"        subpd   %%xmm4, %%xmm6\n");
  fprintf(f,"        mulpd   %%xmm6, %%xmm5\n");
  fprintf(f,"        mulpd   %%xmm3, %%xmm6\n");
  fprintf(f,"        subpd   %%xmm6, %%xmm7\n");
  fprintf(f,"        movapd  -40(%%rsp), %%xmm4\n");
  fprintf(f,"        xorpd   %%xmm8, %%xmm4\n");
  fprintf(f,"        andpd   %%xmm1, %%xmm10\n");
  fprintf(f,"        movapd  %%xmm11, %%xmm3\n");
  fprintf(f,"        mulpd   %%xmm10, %%xmm3\n");
  fprintf(f,"        addpd   %%xmm4, %%xmm3\n");
  fprintf(f,"        subpd   %%xmm5, %%xmm7\n");
  fprintf(f,"        mulpd   %%xmm12, %%xmm2\n");
  fprintf(f,"        movapd  -24(%%rsp), %%xmm5\n");
  fprintf(f,"        movapd  %%xmm5, %%xmm4\n");
  fprintf(f,"        mulpd   %%xmm10, %%xmm4\n");
  fprintf(f,"        addpd   %%xmm3, %%xmm4\n");
  fprintf(f,"        mulpd   %%xmm1, %%xmm13\n");
  fprintf(f,"        subpd   %%xmm10, %%xmm1\n");
  fprintf(f,"        mulpd   %%xmm1, %%xmm11\n");
  fprintf(f,"        addpd   %%xmm4, %%xmm11\n");
  fprintf(f,"        mulpd   %%xmm5, %%xmm1\n");
  fprintf(f,"        addpd   %%xmm11, %%xmm1\n");
  fprintf(f,"        movdqa  %%xmm0, %%xmm3\n");
  fprintf(f,"        pandn   %%xmm8, %%xmm3\n");
  fprintf(f,"        addpd   %%xmm2, %%xmm1\n");
  fprintf(f,"        movapd  %%xmm15, %%xmm2\n");
  fprintf(f,"        andpd   %%xmm0, %%xmm2\n");
  fprintf(f,"        addpd   %%xmm13, %%xmm1\n");
  fprintf(f,"        orpd    %%xmm3, %%xmm2\n");
  fprintf(f,"        movapd  %%xmm15, %%xmm3\n");
  fprintf(f,"        mulpd   %%xmm1, %%xmm3\n");
  fprintf(f,"        subpd   %%xmm3, %%xmm7\n");
  fprintf(f,"        mulpd   %%xmm15, %%xmm7\n");
  fprintf(f,"        andpd   %%xmm0, %%xmm7\n");
  fprintf(f,"        andnpd  %%xmm1, %%xmm0\n");
  fprintf(f,"        orpd    %%xmm7, %%xmm0\n");
  fprintf(f,"        addpd   %%xmm2, %%xmm0\n");
  fprintf(f,"        # This routine sometimes leaves flags set, so I manually clear them here\n");
  fprintf(f,"        andl     $0xFFC0,.Lnewmxcsr  # clear exception in memory copy of MXCSR\n");
  fprintf(f,"        ldmxcsr .Lnewmxcsr        # and reload that into real MXCSR\n");
  fprintf(f,"        popq    %%rbx\n");
  fprintf(f,"        popq    %%r12\n");
  fprintf(f,"        popq    %%r13\n");
  fprintf(f,"        popq    %%r14\n");
  fprintf(f,"        popq    %%r15\n");
  fprintf(f,"        popq    %%rbp\n");
  fprintf(f,"        retq\n");
  fprintf(f,"        .size   xtan_u1, .-xtan_u1\n\n");
  fflush(f);
  return;
}

//
//  This procedure will emit code for the runtime library support
//  of the Minimal BASIC ATN() function.  All math is done with
//  64bit float values.
//  The argument for xatan_u1 is passed in xmm0, and the result
//  is returned in xmm0.
//  This code was created by running a C compiler on Shibata's 
//  SLEEF version 2.80 files and having it emit assembly.  This 
//  requires SSE2 instruction set support.
//
void write_ATN(FILE *f) {
  if (NULL==f) {
    fprintf(stderr,"write_ATN(): file parameter is NULL\n");
    abort();
  }
  fprintf(f,"# http://shibatch.sourceforge.net/\n");
  fprintf(f,"        .section .rodata.cst16,\"aM\",@progbits,16\n");
  fprintf(f,"        .align  16\n");
  fprintf(f,".LCPI16_0:\n");
  fprintf(f,"        .quad   4607182418800017408     # double 1.000000e+00\n");
  fprintf(f,"        .quad   4607182418800017408     # double 1.000000e+00\n");
  fprintf(f,".LCPI16_1:\n");
  fprintf(f,"        .long   4294967295              # 0xffffffff\n");
  fprintf(f,"        .long   2147483647              # 0x7fffffff\n");
  fprintf(f,"        .long   4294967295              # 0xffffffff\n");
  fprintf(f,"        .long   2147483647              # 0x7fffffff\n");
  fprintf(f,".LCPI16_2:\n");
  fprintf(f,"        .quad   -9223372036854775808    # 0x8000000000000000\n");
  fprintf(f,"        .quad   -9223372036854775808    # 0x8000000000000000\n");
  fprintf(f,".LCPI16_3:\n");
  fprintf(f,"        .quad   -134217728              # 0xfffffffff8000000\n");
  fprintf(f,"        .quad   -134217728              # 0xfffffffff8000000\n");
  fprintf(f,".LCPI16_4:\n");
  fprintf(f,"        .long   1                       # 0x1\n");
  fprintf(f,"        .long   1                       # 0x1\n");
  fprintf(f,"        .long   0                       # 0x0\n");
  fprintf(f,"        .long   0                       # 0x0\n");
  fprintf(f,".LCPI16_5:\n");
  fprintf(f,"        .quad   4609753056924675352     # double 1.570796e+00\n");
  fprintf(f,"        .quad   4609753056924675352     # double 1.570796e+00\n");
  fprintf(f,".LCPI16_6:\n");
  fprintf(f,"        .quad   4609753056853098496     # double 1.570796e+00\n");
  fprintf(f,"        .quad   4609753056853098496     # double 1.570796e+00\n");
  fprintf(f,".LCPI16_7:\n");
  fprintf(f,"        .quad   4490388670355865600     # double 1.589325e-08\n");
  fprintf(f,"        .quad   4490388670355865600     # double 1.589325e-08\n");
  fprintf(f,".LCPI16_8:\n");
  fprintf(f,"        .quad   4364452196894661639     # double 6.123234e-17\n");
  fprintf(f,"        .quad   4364452196894661639     # double 6.123234e-17\n");
  fprintf(f,".LCPI16_9:\n");
  fprintf(f,"        .quad   -4644823499662713419    # double -1.282813e-02\n");
  fprintf(f,"        .quad   -4644823499662713419    # double -1.282813e-02\n");
  fprintf(f,".LCPI16_10:\n");
  fprintf(f,"        .quad   4574100926092976051     # double 6.462629e-03\n");
  fprintf(f,"        .quad   4574100926092976051     # double 6.462629e-03\n");
  fprintf(f,".LCPI16_11:\n");
  fprintf(f,"        .quad   -4655417988663931666    # double -2.518656e-03\n");
  fprintf(f,"        .quad   -4655417988663931666    # double -2.518656e-03\n");
  fprintf(f,".LCPI16_12:\n");
  fprintf(f,"        .quad   4559647019150142872     # double 7.055766e-04\n");
  fprintf(f,"        .quad   4559647019150142872     # double 7.055766e-04\n");
  fprintf(f,".LCPI16_13:\n");
  fprintf(f,"        .quad   -4674605428877306162    # double -1.256206e-04\n");
  fprintf(f,"        .quad   -4674605428877306162    # double -1.256206e-04\n");
  fprintf(f,".LCPI16_14:\n");
  fprintf(f,"        .quad   4532392380493917810     # double 1.062985e-05\n");
  fprintf(f,"        .quad   4532392380493917810     # double 1.062985e-05\n");
  fprintf(f,".LCPI16_15:\n");
  fprintf(f,"        .quad   4581653128041526048     # double 2.080248e-02\n");
  fprintf(f,"        .quad   4581653128041526048     # double 2.080248e-02\n");
  fprintf(f,".LCPI16_16:\n");
  fprintf(f,"        .quad   -4623695617433709510    # double -3.333333e-01\n");
  fprintf(f,"        .quad   -4623695617433709510    # double -3.333333e-01\n");
  fprintf(f,".LCPI16_17:\n");
  fprintf(f,"        .quad   4596373779694255344     # double 2.000000e-01\n");
  fprintf(f,"        .quad   4596373779694255344     # double 2.000000e-01\n");
  fprintf(f,".LCPI16_18:\n");
  fprintf(f,"        .quad   -4629057045565165625    # double -1.428571e-01\n");
  fprintf(f,"        .quad   -4629057045565165625    # double -1.428571e-01\n");
  fprintf(f,".LCPI16_19:\n");
  fprintf(f,"        .quad   4592670819803691531     # double 1.111111e-01\n");
  fprintf(f,"        .quad   4592670819803691531     # double 1.111111e-01\n");
  fprintf(f,".LCPI16_20:\n");
  fprintf(f,"        .quad   -4632156929184698388    # double -9.090904e-02\n");
  fprintf(f,"        .quad   -4632156929184698388    # double -9.090904e-02\n");
  fprintf(f,".LCPI16_21:\n");
  fprintf(f,"        .quad   4590207273320582416     # double 7.692253e-02\n");
  fprintf(f,"        .quad   4590207273320582416     # double 7.692253e-02\n");
  fprintf(f,".LCPI16_22:\n");
  fprintf(f,"        .quad   -4633904106482351402    # double -6.666209e-02\n");
  fprintf(f,"        .quad   -4633904106482351402    # double -6.666209e-02\n");
  fprintf(f,".LCPI16_23:\n");
  fprintf(f,"        .quad   4588634024389408379     # double 5.879466e-02\n");
  fprintf(f,"        .quad   4588634024389408379     # double 5.879466e-02\n");
  fprintf(f,".LCPI16_24:\n");
  fprintf(f,"        .quad   -4635646404800717010    # double -5.249142e-02\n");
  fprintf(f,"        .quad   -4635646404800717010    # double -5.249142e-02\n");
  fprintf(f,".LCPI16_25:\n");
  fprintf(f,"        .quad   4586946383953655974     # double 4.708430e-02\n");
  fprintf(f,"        .quad   4586946383953655974     # double 4.708430e-02\n");
  fprintf(f,".LCPI16_26:\n");
  fprintf(f,"        .quad   -4637180199884290697    # double -4.184858e-02\n");
  fprintf(f,"        .quad   -4637180199884290697    # double -4.184858e-02\n");
  fprintf(f,".LCPI16_27:\n");
  fprintf(f,"        .quad   -4639384889991748633    # double -2.890023e-02\n");
  fprintf(f,"        .quad   -4639384889991748633    # double -2.890023e-02\n");
  fprintf(f,".LCPI16_28:\n");
  fprintf(f,"        .quad   4585345869402545117     # double 3.597850e-02\n");
  fprintf(f,"        .quad   4585345869402545117     # double 3.597850e-02\n");
  fprintf(f,".LCPI16_29:\n");
  fprintf(f,"        .quad   9218868437227405312     # double inf\n");
  fprintf(f,"        .quad   9218868437227405312     # double inf\n\n");

  fprintf(f,"        .section .text,\"ax\",@progbits\n\n");

  fprintf(f,"        .type   xatan_u1,@function\n");
  fprintf(f,"xatan_u1:\n");
  fprintf(f,"        pushq   %%r15\n");
  fprintf(f,"        pushq   %%r14\n");
  fprintf(f,"        pushq   %%r13\n");
  fprintf(f,"        pushq   %%r12\n");
  fprintf(f,"        pushq   %%rbx\n");
  fprintf(f,"        movabsq $.LCPI16_0, %%rax\n");
  fprintf(f,"        movabsq $.LCPI16_1, %%rcx\n");
  fprintf(f,"        movapd  (%%rcx), %%xmm7\n");
  fprintf(f,"        andpd   %%xmm0, %%xmm7\n");
  fprintf(f,"        movapd  %%xmm7, -32(%%rsp)\n");
  fprintf(f,"        movapd  %%xmm0, %%xmm3\n");
  fprintf(f,"        movapd  (%%rax), %%xmm10\n");
  fprintf(f,"        xorpd   %%xmm0, %%xmm0\n");
  fprintf(f,"        movapd  %%xmm10, %%xmm9\n");
  fprintf(f,"        cmpltpd %%xmm0, %%xmm9\n");
  fprintf(f,"        xorps   %%xmm13, %%xmm13\n");
  fprintf(f,"        movabsq $.LCPI16_2, %%rcx\n");
  fprintf(f,"        movapd  (%%rcx), %%xmm4\n");
  fprintf(f,"        movapd  %%xmm4, -80(%%rsp)\n");
  fprintf(f,"        movapd  %%xmm9, %%xmm1\n");
  fprintf(f,"        andpd   %%xmm4, %%xmm1\n");
  fprintf(f,"        movapd  (%%rax), %%xmm0\n");
  fprintf(f,"        orpd    %%xmm1, %%xmm0\n");
  fprintf(f,"        movapd  %%xmm0, %%xmm12\n");
  fprintf(f,"        cmpltpd %%xmm7, %%xmm12\n");
  fprintf(f,"        movapd  %%xmm12, %%xmm2\n");
  fprintf(f,"        andnpd  %%xmm0, %%xmm2\n");
  fprintf(f,"        xorpd   %%xmm4, %%xmm0\n");
  fprintf(f,"        movapd  %%xmm4, %%xmm5\n");
  fprintf(f,"        andpd   %%xmm12, %%xmm0\n");
  fprintf(f,"        movapd  %%xmm12, %%xmm4\n");
  fprintf(f,"        andnpd  %%xmm7, %%xmm4\n");
  fprintf(f,"        orpd    %%xmm0, %%xmm4\n");
  fprintf(f,"        andpd   %%xmm12, %%xmm7\n");
  fprintf(f,"        orpd    %%xmm2, %%xmm7\n");
  fprintf(f,"        andpd   %%xmm5, %%xmm3\n");
  fprintf(f,"        movapd  %%xmm5, %%xmm15\n");
  fprintf(f,"        movapd  %%xmm3, -16(%%rsp)\n");
  fprintf(f,"        movabsq $.LCPI16_3, %%rax\n");
  fprintf(f,"        movapd  (%%rax), %%xmm2\n");
  fprintf(f,"        movapd  %%xmm10, %%xmm8\n");
  fprintf(f,"        divpd   %%xmm7, %%xmm8\n");
  fprintf(f,"        movapd  %%xmm8, %%xmm14\n");
  fprintf(f,"        mulpd   %%xmm4, %%xmm14\n");
  fprintf(f,"        movapd  %%xmm14, -96(%%rsp)\n");
  fprintf(f,"        movapd  %%xmm4, %%xmm0\n");
  fprintf(f,"        andpd   %%xmm2, %%xmm0\n");
  fprintf(f,"        movapd  %%xmm8, %%xmm6\n");
  fprintf(f,"        andpd   %%xmm2, %%xmm6\n");
  fprintf(f,"        movapd  %%xmm2, %%xmm3\n");
  fprintf(f,"        movapd  %%xmm6, %%xmm2\n");
  fprintf(f,"        mulpd   %%xmm0, %%xmm2\n");
  fprintf(f,"        subpd   %%xmm14, %%xmm2\n");
  fprintf(f,"        movapd  %%xmm8, %%xmm5\n");
  fprintf(f,"        subpd   %%xmm6, %%xmm5\n");
  fprintf(f,"        subpd   %%xmm0, %%xmm4\n");
  fprintf(f,"        mulpd   %%xmm5, %%xmm0\n");
  fprintf(f,"        addpd   %%xmm2, %%xmm0\n");
  fprintf(f,"        movapd  %%xmm6, %%xmm11\n");
  fprintf(f,"        mulpd   %%xmm4, %%xmm11\n");
  fprintf(f,"        addpd   %%xmm0, %%xmm11\n");
  fprintf(f,"        movapd  %%xmm12, %%xmm0\n");
  fprintf(f,"        andnpd  %%xmm1, %%xmm0\n");
  fprintf(f,"        mulpd   %%xmm14, %%xmm0\n");
  fprintf(f,"        xorpd   %%xmm15, %%xmm1\n");
  fprintf(f,"        andpd   %%xmm12, %%xmm1\n");
  fprintf(f,"        subpd   %%xmm0, %%xmm1\n");
  fprintf(f,"        cvtpd2ps        %%xmm9, %%xmm0\n");
  fprintf(f,"        cmpeqps %%xmm13, %%xmm0\n");
  fprintf(f,"        movabsq $0xFFFFFFFEFFFFFFFE, %%rax\n");
  fprintf(f,"        movd    %%rax, %%xmm2\n");
  fprintf(f,"        andnps  %%xmm2, %%xmm0\n");
  fprintf(f,"        movapd  %%xmm7, %%xmm2\n");
  fprintf(f,"        andpd   %%xmm3, %%xmm2\n");
  fprintf(f,"        movapd  %%xmm3, %%xmm13\n");
  fprintf(f,"        movapd  %%xmm2, %%xmm3\n");
  fprintf(f,"        mulpd   %%xmm6, %%xmm3\n");
  fprintf(f,"        movapd  %%xmm10, %%xmm9\n");
  fprintf(f,"        subpd   %%xmm3, %%xmm9\n");
  fprintf(f,"        subpd   %%xmm2, %%xmm7\n");
  fprintf(f,"        mulpd   %%xmm5, %%xmm2\n");
  fprintf(f,"        subpd   %%xmm2, %%xmm9\n");
  fprintf(f,"        mulpd   %%xmm7, %%xmm6\n");
  fprintf(f,"        subpd   %%xmm6, %%xmm9\n");
  fprintf(f,"        mulpd   %%xmm5, %%xmm7\n");
  fprintf(f,"        mulpd   %%xmm5, %%xmm4\n");
  fprintf(f,"        movapd  %%xmm14, %%xmm3\n");
  fprintf(f,"        movapd  %%xmm3, %%xmm15\n");
  fprintf(f,"        mulpd   %%xmm15, %%xmm15\n");
  fprintf(f,"        movapd  %%xmm15, %%xmm2\n");
  fprintf(f,"        movapd  -80(%%rsp), %%xmm14\n");
  fprintf(f,"        xorpd   %%xmm14, %%xmm2\n");
  fprintf(f,"        movapd  %%xmm3, %%xmm6\n");
  fprintf(f,"        andpd   %%xmm13, %%xmm6\n");
  fprintf(f,"        movapd  %%xmm6, -64(%%rsp)\n");
  fprintf(f,"        movapd  %%xmm6, %%xmm5\n");
  fprintf(f,"        mulpd   %%xmm5, %%xmm5\n");
  fprintf(f,"        addpd   %%xmm2, %%xmm5\n");
  fprintf(f,"        subpd   %%xmm6, %%xmm3\n");
  fprintf(f,"        movapd  %%xmm3, -48(%%rsp)\n");
  fprintf(f,"        movapd  %%xmm6, %%xmm2\n");
  fprintf(f,"        addpd   %%xmm2, %%xmm2\n");
  fprintf(f,"        mulpd   %%xmm3, %%xmm2\n");
  fprintf(f,"        movapd  %%xmm3, %%xmm6\n");
  fprintf(f,"        addpd   %%xmm5, %%xmm2\n");
  fprintf(f,"        addpd   %%xmm11, %%xmm4\n");
  fprintf(f,"        subpd   %%xmm7, %%xmm9\n");
  fprintf(f,"        mulpd   %%xmm8, %%xmm1\n");
  fprintf(f,"        cvtpd2ps        %%xmm12, %%xmm3\n");
  fprintf(f,"        xorpd   %%xmm5, %%xmm5\n");
  fprintf(f,"        cmpeqps %%xmm5, %%xmm3\n");
  fprintf(f,"        movabsq $.LCPI16_4, %%rax\n");
  fprintf(f,"        movaps  %%xmm3, %%xmm5\n");
  fprintf(f,"        andps   %%xmm0, %%xmm5\n");
  fprintf(f,"        paddd   (%%rax), %%xmm0\n");
  fprintf(f,"        andnps  %%xmm0, %%xmm3\n");
  fprintf(f,"        orps    %%xmm5, %%xmm3\n");
  fprintf(f,"        movapd  -96(%%rsp), %%xmm12\n");
  fprintf(f,"        mulpd   %%xmm12, %%xmm9\n");
  fprintf(f,"        addpd   %%xmm4, %%xmm9\n");
  fprintf(f,"        addpd   %%xmm1, %%xmm9\n");
  fprintf(f,"        movapd  %%xmm6, %%xmm1\n");
  fprintf(f,"        mulpd   %%xmm1, %%xmm1\n");
  fprintf(f,"        addpd   %%xmm2, %%xmm1\n");
  fprintf(f,"        movabsq $.LCPI16_5, %%r8\n");
  fprintf(f,"        movabsq $.LCPI16_6, %%rax\n");
  fprintf(f,"        cvtdq2pd        %%xmm3, %%xmm11\n");
  fprintf(f,"        movapd  (%%rax), %%xmm2\n");
  fprintf(f,"        movapd  (%%r8), %%xmm8\n");
  fprintf(f,"        mulpd   %%xmm11, %%xmm8\n");
  fprintf(f,"        movapd  %%xmm8, %%xmm5\n");
  fprintf(f,"        xorpd   %%xmm14, %%xmm5\n");
  fprintf(f,"        movapd  %%xmm11, %%xmm0\n");
  fprintf(f,"        andpd   %%xmm13, %%xmm0\n");
  fprintf(f,"        movapd  %%xmm0, %%xmm4\n");
  fprintf(f,"        mulpd   %%xmm2, %%xmm4\n");
  fprintf(f,"        addpd   %%xmm5, %%xmm4\n");
  fprintf(f,"        movabsq $.LCPI16_7, %%rax\n");
  fprintf(f,"        movapd  %%xmm11, %%xmm5\n");
  fprintf(f,"        subpd   %%xmm0, %%xmm5\n");
  fprintf(f,"        mulpd   %%xmm5, %%xmm2\n");
  fprintf(f,"        movapd  (%%rax), %%xmm6\n");
  fprintf(f,"        mulpd   %%xmm6, %%xmm5\n");
  fprintf(f,"        mulpd   %%xmm6, %%xmm0\n");
  fprintf(f,"        movapd  %%xmm9, %%xmm7\n");
  fprintf(f,"        addpd   %%xmm7, %%xmm7\n");
  fprintf(f,"        mulpd   %%xmm12, %%xmm7\n");
  fprintf(f,"        addpd   %%xmm1, %%xmm7\n");
  fprintf(f,"        addpd   %%xmm4, %%xmm0\n");
  fprintf(f,"        addpd   %%xmm2, %%xmm0\n");
  fprintf(f,"        addpd   %%xmm5, %%xmm0\n");
  fprintf(f,"        movapd  %%xmm15, %%xmm4\n");
  fprintf(f,"        addpd   %%xmm7, %%xmm4\n");
  fprintf(f,"        subpd   %%xmm4, %%xmm15\n");
  fprintf(f,"        addpd   %%xmm7, %%xmm15\n");
  fprintf(f,"        movabsq $.LCPI16_8, %%rax\n");
  fprintf(f,"        mulpd   (%%rax), %%xmm11\n");
  fprintf(f,"        addpd   %%xmm0, %%xmm11\n");
  fprintf(f,"        movabsq $.LCPI16_9, %%r13\n");
  fprintf(f,"        movabsq $.LCPI16_10, %%rcx\n");
  fprintf(f,"        movabsq $.LCPI16_11, %%rax\n");
  fprintf(f,"        movabsq $.LCPI16_12, %%rsi\n");
  fprintf(f,"        movabsq $.LCPI16_13, %%rdx\n");
  fprintf(f,"        movabsq $.LCPI16_14, %%rbx\n");
  fprintf(f,"        movabsq $.LCPI16_15, %%rdi\n");
  fprintf(f,"        movabsq $.LCPI16_16, %%r9\n");
  fprintf(f,"        movabsq $.LCPI16_17, %%r10\n");
  fprintf(f,"        movabsq $.LCPI16_18, %%r11\n");
  fprintf(f,"        movabsq $.LCPI16_19, %%r14\n");
  fprintf(f,"        movabsq $.LCPI16_20, %%r15\n");
  fprintf(f,"        movapd  (%%rbx), %%xmm1\n");
  fprintf(f,"        mulpd   %%xmm4, %%xmm1\n");
  fprintf(f,"        movabsq $.LCPI16_21, %%r12\n");
  fprintf(f,"        addpd   (%%rdx), %%xmm1\n");
  fprintf(f,"        mulpd   %%xmm4, %%xmm1\n");
  fprintf(f,"        movabsq $.LCPI16_22, %%rdx\n");
  fprintf(f,"        addpd   (%%rsi), %%xmm1\n");
  fprintf(f,"        mulpd   %%xmm4, %%xmm1\n");
  fprintf(f,"        movabsq $.LCPI16_23, %%rsi\n");
  fprintf(f,"        addpd   (%%rax), %%xmm1\n");
  fprintf(f,"        mulpd   %%xmm4, %%xmm1\n");
  fprintf(f,"        movabsq $.LCPI16_24, %%rax\n");
  fprintf(f,"        addpd   (%%rcx), %%xmm1\n");
  fprintf(f,"        mulpd   %%xmm4, %%xmm1\n");
  fprintf(f,"        movabsq $.LCPI16_25, %%rcx\n");
  fprintf(f,"        addpd   (%%r13), %%xmm1\n");
  fprintf(f,"        mulpd   %%xmm4, %%xmm1\n");
  fprintf(f,"        movabsq $.LCPI16_26, %%rbx\n");
  fprintf(f,"        addpd   (%%rdi), %%xmm1\n");
  fprintf(f,"        mulpd   %%xmm4, %%xmm1\n");
  fprintf(f,"        movabsq $.LCPI16_27, %%rdi\n");
  fprintf(f,"        addpd   (%%rdi), %%xmm1\n");
  fprintf(f,"        mulpd   %%xmm4, %%xmm1\n");
  fprintf(f,"        movabsq $.LCPI16_28, %%rdi\n");
  fprintf(f,"        addpd   (%%rdi), %%xmm1\n");
  fprintf(f,"        mulpd   %%xmm4, %%xmm1\n");
  fprintf(f,"        addpd   (%%rbx), %%xmm1\n");
  fprintf(f,"        mulpd   %%xmm4, %%xmm1\n");
  fprintf(f,"        addpd   (%%rcx), %%xmm1\n");
  fprintf(f,"        mulpd   %%xmm4, %%xmm1\n");
  fprintf(f,"        addpd   (%%rax), %%xmm1\n");
  fprintf(f,"        mulpd   %%xmm4, %%xmm1\n");
  fprintf(f,"        addpd   (%%rsi), %%xmm1\n");
  fprintf(f,"        mulpd   %%xmm4, %%xmm1\n");
  fprintf(f,"        addpd   (%%rdx), %%xmm1\n");
  fprintf(f,"        mulpd   %%xmm4, %%xmm1\n");
  fprintf(f,"        addpd   (%%r12), %%xmm1\n");
  fprintf(f,"        mulpd   %%xmm4, %%xmm1\n");
  fprintf(f,"        addpd   (%%r15), %%xmm1\n");
  fprintf(f,"        mulpd   %%xmm4, %%xmm1\n");
  fprintf(f,"        addpd   (%%r14), %%xmm1\n");
  fprintf(f,"        mulpd   %%xmm4, %%xmm1\n");
  fprintf(f,"        addpd   (%%r11), %%xmm1\n");
  fprintf(f,"        mulpd   %%xmm4, %%xmm1\n");
  fprintf(f,"        addpd   (%%r10), %%xmm1\n");
  fprintf(f,"        mulpd   %%xmm4, %%xmm1\n");
  fprintf(f,"        addpd   (%%r9), %%xmm1\n");
  fprintf(f,"        movapd  %%xmm4, %%xmm0\n");
  fprintf(f,"        movapd  %%xmm13, %%xmm3\n");
  fprintf(f,"        andpd   %%xmm3, %%xmm0\n");
  fprintf(f,"        movapd  %%xmm4, %%xmm2\n");
  fprintf(f,"        mulpd   %%xmm1, %%xmm2\n");
  fprintf(f,"        movapd  %%xmm2, %%xmm6\n");
  fprintf(f,"        xorpd   %%xmm14, %%xmm6\n");
  fprintf(f,"        movapd  %%xmm1, %%xmm5\n");
  fprintf(f,"        andpd   %%xmm3, %%xmm5\n");
  fprintf(f,"        movapd  %%xmm0, %%xmm7\n");
  fprintf(f,"        mulpd   %%xmm5, %%xmm7\n");
  fprintf(f,"        addpd   %%xmm6, %%xmm7\n");
  fprintf(f,"        subpd   %%xmm0, %%xmm4\n");
  fprintf(f,"        movapd  %%xmm4, %%xmm6\n");
  fprintf(f,"        mulpd   %%xmm5, %%xmm6\n");
  fprintf(f,"        addpd   %%xmm7, %%xmm6\n");
  fprintf(f,"        mulpd   %%xmm1, %%xmm15\n");
  fprintf(f,"        subpd   %%xmm5, %%xmm1\n");
  fprintf(f,"        mulpd   %%xmm1, %%xmm0\n");
  fprintf(f,"        addpd   %%xmm6, %%xmm0\n");
  fprintf(f,"        mulpd   %%xmm4, %%xmm1\n");
  fprintf(f,"        addpd   %%xmm0, %%xmm1\n");
  fprintf(f,"        movapd  %%xmm2, %%xmm4\n");
  fprintf(f,"        addpd   %%xmm10, %%xmm4\n");
  fprintf(f,"        movapd  %%xmm12, %%xmm0\n");
  fprintf(f,"        mulpd   %%xmm4, %%xmm0\n");
  fprintf(f,"        movapd  %%xmm14, %%xmm6\n");
  fprintf(f,"        xorpd   %%xmm0, %%xmm6\n");
  fprintf(f,"        andpd   %%xmm4, %%xmm3\n");
  fprintf(f,"        movapd  -64(%%rsp), %%xmm7\n");
  fprintf(f,"        movapd  %%xmm7, %%xmm5\n");
  fprintf(f,"        mulpd   %%xmm3, %%xmm5\n");
  fprintf(f,"        addpd   %%xmm6, %%xmm5\n");
  fprintf(f,"        addpd   %%xmm15, %%xmm1\n");
  fprintf(f,"        movapd  -48(%%rsp), %%xmm13\n");
  fprintf(f,"        movapd  %%xmm13, %%xmm6\n");
  fprintf(f,"        mulpd   %%xmm3, %%xmm6\n");
  fprintf(f,"        addpd   %%xmm5, %%xmm6\n");
  fprintf(f,"        movapd  %%xmm4, %%xmm5\n");
  fprintf(f,"        subpd   %%xmm3, %%xmm5\n");
  fprintf(f,"        movapd  %%xmm7, %%xmm3\n");
  fprintf(f,"        mulpd   %%xmm5, %%xmm3\n");
  fprintf(f,"        addpd   %%xmm6, %%xmm3\n");
  fprintf(f,"        subpd   %%xmm4, %%xmm10\n");
  fprintf(f,"        addpd   %%xmm2, %%xmm10\n");
  fprintf(f,"        mulpd   %%xmm13, %%xmm5\n");
  fprintf(f,"        addpd   %%xmm3, %%xmm5\n");
  fprintf(f,"        addpd   %%xmm1, %%xmm10\n");
  fprintf(f,"        movabsq $.LCPI16_29, %%rax\n");
  fprintf(f,"        movapd  -32(%%rsp), %%xmm2\n");
  fprintf(f,"        cmpeqpd (%%rax), %%xmm2\n");
  fprintf(f,"        mulpd   %%xmm12, %%xmm10\n");
  fprintf(f,"        movapd  (%%r8), %%xmm1\n");
  fprintf(f,"        andpd   %%xmm2, %%xmm1\n");
  fprintf(f,"        movapd  %%xmm2, %%xmm6\n");
  fprintf(f,"        mulpd   %%xmm9, %%xmm4\n");
  fprintf(f,"        movapd  %%xmm8, %%xmm2\n");
  fprintf(f,"        addpd   %%xmm0, %%xmm2\n");
  fprintf(f,"        addpd   %%xmm5, %%xmm10\n");
  fprintf(f,"        addpd   %%xmm4, %%xmm10\n");
  fprintf(f,"        movapd  %%xmm2, %%xmm4\n");
  fprintf(f,"        subpd   %%xmm8, %%xmm4\n");
  fprintf(f,"        addpd   %%xmm11, %%xmm10\n");
  fprintf(f,"        subpd   %%xmm4, %%xmm0\n");
  fprintf(f,"        movapd  %%xmm2, %%xmm5\n");
  fprintf(f,"        subpd   %%xmm4, %%xmm5\n");
  fprintf(f,"        subpd   %%xmm5, %%xmm8\n");
  fprintf(f,"        addpd   %%xmm0, %%xmm8\n");
  fprintf(f,"        addpd   %%xmm10, %%xmm8\n");
  fprintf(f,"        addpd   %%xmm2, %%xmm8\n");
  fprintf(f,"        movapd  %%xmm6, %%xmm0\n");
  fprintf(f,"        andnpd  %%xmm8, %%xmm0\n");
  fprintf(f,"        orpd    %%xmm1, %%xmm0\n");
  fprintf(f,"        xorpd   -16(%%rsp), %%xmm0\n");
  fprintf(f,"        # This routine sometimes leaves flags set, so I manually clear them here\n");
  fprintf(f,"        andl     $0xFFC0,.Lnewmxcsr  # clear exception in memory copy of MXCSR\n");
  fprintf(f,"        ldmxcsr .Lnewmxcsr        # and reload that into real MXCSR\n");
  fprintf(f,"        popq    %%rbx\n");
  fprintf(f,"        popq    %%r12\n");
  fprintf(f,"        popq    %%r13\n");
  fprintf(f,"        popq    %%r14\n");
  fprintf(f,"        popq    %%r15\n");
  fprintf(f,"        retq\n");
  fprintf(f,"        .size   xatan_u1, .-xatan_u1\n\n");
  fflush(f);
  return;
}

//
//  This procedure will emit code for the runtime library support
//  of the Minimal BASIC LOG() function.  All math is done with
//  64bit float values.
//  The argument for xlog_u1 is passed in xmm0, and the result
//  is returned in xmm0.
//  This code was created by running a C compiler on Shibata's 
//  SLEEF version 2.80 files and having it emit assembly.  This 
//  requires SSE2 instruction set support.
//
void write_LOG(FILE *f) {
  if (NULL==f) {
    fprintf(stderr,"write_LOG(): file parameter is NULL\n");
    abort();
  }
  fprintf(f,"# http://shibatch.sourceforge.net/\n");
  fprintf(f,"        .section .rodata.cst16,\"aM\",@progbits,16\n");
  fprintf(f,"        .align  16\n");
  fprintf(f,".LCPI20_0:\n");
  fprintf(f,"        .quad   4604544210138303771     # double 7.071000e-01\n");
  fprintf(f,"        .quad   4604544210138303771     # double 7.071000e-01\n");
  fprintf(f,".LCPI20_1:\n");
  fprintf(f,"        .quad   3256102530588868608     # double 4.909093e-91\n");
  fprintf(f,"        .quad   3256102530588868608     # double 4.909093e-91\n");
  fprintf(f,".LCPI20_2:\n");
  fprintf(f,"        .quad   5958262307011166208     # double 2.037036e+90\n");
  fprintf(f,"        .quad   5958262307011166208     # double 2.037036e+90\n");
  fprintf(f,".LCPI20_3:\n");
  fprintf(f,"        .quad   -4503599627370496       # 0xfff0000000000000\n");
  fprintf(f,"        .quad   -4503599627370496       # 0xfff0000000000000\n");
  fprintf(f,".LCPI20_4:\n");
  fprintf(f,"        .quad   -134217728              # 0xfffffffff8000000\n");
  fprintf(f,"        .quad   -134217728              # 0xfffffffff8000000\n");
  fprintf(f,".LCPI20_5:\n");
  fprintf(f,"        .quad   4604418534196379648     # double 6.931472e-01\n");
  fprintf(f,"        .quad   4604418534196379648     # double 6.931472e-01\n");
  fprintf(f,".LCPI20_6:\n");
  fprintf(f,"        .quad   4604418534313441775     # double 6.931472e-01\n");
  fprintf(f,"        .quad   4604418534313441775     # double 6.931472e-01\n");
  fprintf(f,".LCPI20_7:\n");
  fprintf(f,"        .long   0                       # 0x0\n");
  fprintf(f,"        .long   2147483648              # 0x80000000\n");
  fprintf(f,"        .long   0                       # 0x0\n");
  fprintf(f,"        .long   2147483648              # 0x80000000\n");
  fprintf(f,".LCPI20_8:\n");
  fprintf(f,"        .long   0                       # 0x0\n");
  fprintf(f,"        .long   4294965974              # 0xfffffad6\n");
  fprintf(f,"        .long   0                       # 0x0\n");
  fprintf(f,"        .long   4294965974              # 0xfffffad6\n");
  fprintf(f,".LCPI20_9:\n");
  fprintf(f,"        .long   0                       # 0x0\n");
  fprintf(f,"        .long   4294966274              # 0xfffffc02\n");
  fprintf(f,"        .long   0                       # 0x0\n");
  fprintf(f,"        .long   4294966274              # 0xfffffc02\n");
  fprintf(f,".LCPI20_10:\n");
  fprintf(f,"        .quad   4488937535594037248     # double 1.299651e-08\n");
  fprintf(f,"        .quad   4488937535594037248     # double 1.299651e-08\n");
  fprintf(f,".LCPI20_11:\n");
  fprintf(f,"        .long   1023                    # 0x3ff\n");
  fprintf(f,"        .long   1023                    # 0x3ff\n");
  fprintf(f,"        .long   0                       # 0x0\n");
  fprintf(f,"        .long   0                       # 0x0\n");
  fprintf(f,".LCPI20_12:\n");
  fprintf(f,"        .long   2047                    # 0x7ff\n");
  fprintf(f,"        .long   2047                    # 0x7ff\n");
  fprintf(f,"        .long   0                       # 0x0\n");
  fprintf(f,"        .long   0                       # 0x0\n");
  fprintf(f,".LCPI20_13:\n");
  fprintf(f,"        .long   1023                    # 0x3ff\n");
  fprintf(f,"        .long   1023                    # 0x3ff\n");
  fprintf(f,"        .zero   4\n");
  fprintf(f,"        .long   0                       # 0x0\n");
  fprintf(f,".LCPI20_14:\n");
  fprintf(f,"        .quad   4358002977218854975     # double 2.319047e-17\n");
  fprintf(f,"        .quad   4358002977218854975     # double 2.319047e-17\n");
  fprintf(f,".LCPI20_15:\n");
  fprintf(f,"        .quad   -4616189618054758400    # double -1.000000e+00\n");
  fprintf(f,"        .quad   -4616189618054758400    # double -1.000000e+00\n");
  fprintf(f,".LCPI20_16:\n");
  fprintf(f,"        .quad   4607182418800017408     # double 1.000000e+00\n");
  fprintf(f,"        .quad   4607182418800017408     # double 1.000000e+00\n");
  fprintf(f,".LCPI20_17:\n");
  fprintf(f,"        .quad   4600877379321702721     # double 4.000000e-01\n");
  fprintf(f,"        .quad   4600877379321702721     # double 4.000000e-01\n");
  fprintf(f,".LCPI20_18:\n");
  fprintf(f,"        .quad   4598818590919479713     # double 2.857143e-01\n");
  fprintf(f,"        .quad   4598818590919479713     # double 2.857143e-01\n");
  fprintf(f,".LCPI20_19:\n");
  fprintf(f,"        .quad   4597174419956087886     # double 2.222222e-01\n");
  fprintf(f,"        .quad   4597174419956087886     # double 2.222222e-01\n");
  fprintf(f,".LCPI20_20:\n");
  fprintf(f,"        .quad   4595718683483098638     # double 1.818174e-01\n");
  fprintf(f,"        .quad   4595718683483098638     # double 1.818174e-01\n");
  fprintf(f,".LCPI20_21:\n");
  fprintf(f,"        .quad   4594712256174854310     # double 1.538835e-01\n");
  fprintf(f,"        .quad   4594712256174854310     # double 1.538835e-01\n");
  fprintf(f,".LCPI20_22:\n");
  fprintf(f,"        .quad   4593932774978511152     # double 1.322485e-01\n");
  fprintf(f,"        .quad   4593932774978511152     # double 1.322485e-01\n");
  fprintf(f,".LCPI20_23:\n");
  fprintf(f,"        .quad   4594017567976567522     # double 1.346020e-01\n");
  fprintf(f,"        .quad   4594017567976567522     # double 1.346020e-01\n");
  fprintf(f,".LCPI20_24:\n");
  fprintf(f,"        .quad   4611686018427387904     # double 2.000000e+00\n");
  fprintf(f,"        .quad   4611686018427387904     # double 2.000000e+00\n");
  fprintf(f,".LCPI20_25:\n");
  fprintf(f,"        .quad   4604180019048437075     # double 6.666667e-01\n");
  fprintf(f,"        .quad   4604180019048437075     # double 6.666667e-01\n");
  fprintf(f,".LCPI20_26:\n");
  fprintf(f,"        .quad   9218868437227405312     # double inf\n");
  fprintf(f,"        .quad   9218868437227405312     # double inf\n\n");

  fprintf(f,"        .section .text,\"ax\",@progbits\n\n");

  fprintf(f,"        .type   xlog_u1,@function\n");
  fprintf(f,"xlog_u1:\n");
  fprintf(f,"        movapd  %%xmm0, -56(%%rsp)\n");
  fprintf(f,"        movabsq $.LCPI20_0, %%rax\n");
  fprintf(f,"        movapd  (%%rax), %%xmm1\n");
  fprintf(f,"        mulpd   %%xmm0, %%xmm1\n");
  fprintf(f,"        movapd  %%xmm0, %%xmm9\n");
  fprintf(f,"        movabsq $.LCPI20_1, %%rax\n");
  fprintf(f,"        movapd  %%xmm1, %%xmm3\n");
  fprintf(f,"        cmpltpd (%%rax), %%xmm3\n");
  fprintf(f,"        movapd  %%xmm3, %%xmm2\n");
  fprintf(f,"        andnpd  %%xmm1, %%xmm2\n");
  fprintf(f,"        movabsq $.LCPI20_2, %%rax\n");
  fprintf(f,"        mulpd   (%%rax), %%xmm1\n");
  fprintf(f,"        andpd   %%xmm3, %%xmm1\n");
  fprintf(f,"        orpd    %%xmm2, %%xmm1\n");
  fprintf(f,"        movabsq $.LCPI20_3, %%rax\n");
  fprintf(f,"        movapd  (%%rax), %%xmm0\n");
  fprintf(f,"        movapd  %%xmm0, -24(%%rsp)\n");
  fprintf(f,"        andpd   %%xmm0, %%xmm1\n");
  fprintf(f,"        psrld   $20, %%xmm1\n");
  fprintf(f,"        movabsq $.LCPI20_4, %%rax\n");
  fprintf(f,"        movabsq $.LCPI20_5, %%rcx\n");
  fprintf(f,"        movapd  (%%rcx), %%xmm8\n");
  fprintf(f,"        movapd  (%%rax), %%xmm11\n");
  fprintf(f,"        movabsq $.LCPI20_6, %%rax\n");
  fprintf(f,"        movabsq $.LCPI20_7, %%rcx\n");
  fprintf(f,"        movabsq $.LCPI20_8, %%rdx\n");
  fprintf(f,"        movabsq $.LCPI20_9, %%rsi\n");
  fprintf(f,"        movdqa  (%%rsi), %%xmm2\n");
  fprintf(f,"        paddd   %%xmm1, %%xmm2\n");
  fprintf(f,"        movapd  %%xmm3, %%xmm4\n");
  fprintf(f,"        andnpd  %%xmm2, %%xmm4\n");
  fprintf(f,"        paddd   (%%rdx), %%xmm1\n");
  fprintf(f,"        pand    %%xmm3, %%xmm1\n");
  fprintf(f,"        por     %%xmm4, %%xmm1\n");
  fprintf(f,"        pshufd  $13, %%xmm1, %%xmm4       # xmm4 = xmm1[1,3,0,0]\n");
  fprintf(f,"        pxor    %%xmm1, %%xmm1\n");
  fprintf(f,"        psubd   %%xmm4, %%xmm1\n");
  fprintf(f,"        movdqa  %%xmm1, %%xmm5\n");
  fprintf(f,"        psrad   $31, %%xmm5\n");
  fprintf(f,"        movdqa  %%xmm5, %%xmm3\n");
  fprintf(f,"        psubd   %%xmm4, %%xmm3\n");
  fprintf(f,"        psrad   $9, %%xmm3\n");
  fprintf(f,"        cvtdq2pd        %%xmm4, %%xmm2\n");
  fprintf(f,"        movapd  (%%rcx), %%xmm0\n");
  fprintf(f,"        movapd  %%xmm0, -72(%%rsp)\n");
  fprintf(f,"        psubd   %%xmm5, %%xmm3\n");
  fprintf(f,"        movapd  (%%rax), %%xmm13\n");
  fprintf(f,"        mulpd   %%xmm2, %%xmm13\n");
  fprintf(f,"        movapd  %%xmm13, %%xmm4\n");
  fprintf(f,"        xorpd   %%xmm0, %%xmm4\n");
  fprintf(f,"        movapd  %%xmm2, %%xmm6\n");
  fprintf(f,"        movapd  %%xmm2, %%xmm0\n");
  fprintf(f,"        andpd   %%xmm11, %%xmm6\n");
  fprintf(f,"        movapd  %%xmm6, %%xmm7\n");
  fprintf(f,"        mulpd   %%xmm8, %%xmm7\n");
  fprintf(f,"        addpd   %%xmm4, %%xmm7\n");
  fprintf(f,"        movabsq $.LCPI20_10, %%rax\n");
  fprintf(f,"        movapd  (%%rax), %%xmm5\n");
  fprintf(f,"        movapd  %%xmm0, %%xmm4\n");
  fprintf(f,"        subpd   %%xmm6, %%xmm4\n");
  fprintf(f,"        mulpd   %%xmm5, %%xmm6\n");
  fprintf(f,"        addpd   %%xmm7, %%xmm6\n");
  fprintf(f,"        mulpd   %%xmm4, %%xmm8\n");
  fprintf(f,"        addpd   %%xmm6, %%xmm8\n");
  fprintf(f,"        movabsq $.LCPI20_11, %%rax\n");
  fprintf(f,"        pslld   $7, %%xmm3\n");
  fprintf(f,"        movdqa  (%%rax), %%xmm7\n");
  fprintf(f,"        paddd   %%xmm3, %%xmm7\n");
  fprintf(f,"        movdqa  %%xmm7, %%xmm6\n");
  fprintf(f,"        psrad   $31, %%xmm6\n");
  fprintf(f,"        pandn   %%xmm7, %%xmm6\n");
  fprintf(f,"        mulpd   %%xmm5, %%xmm4\n");
  fprintf(f,"        movabsq $0x7FF000007FF, %%rax\n");
  fprintf(f,"        movabsq $.LCPI20_12, %%rcx\n");
  fprintf(f,"        movdqa  %%xmm6, %%xmm7\n");
  fprintf(f,"        pcmpgtd (%%rcx), %%xmm7\n");
  fprintf(f,"        movdqa  %%xmm7, %%xmm2\n");
  fprintf(f,"        pandn   %%xmm6, %%xmm2\n");
  fprintf(f,"        movd    %%rax, %%xmm5\n");
  fprintf(f,"        andpd   %%xmm7, %%xmm5\n");
  fprintf(f,"        orpd    %%xmm2, %%xmm5\n");
  fprintf(f,"        addpd   %%xmm8, %%xmm4\n");
  fprintf(f,"        pslld   $2, %%xmm3\n");
  fprintf(f,"        psubd   %%xmm3, %%xmm1\n");
  fprintf(f,"        movabsq $.LCPI20_13, %%rdx\n");
  fprintf(f,"        movabsq $.LCPI20_14, %%rsi\n");
  fprintf(f,"        movabsq $.LCPI20_15, %%rax\n");
  fprintf(f,"        movabsq $.LCPI20_16, %%rcx\n");
  fprintf(f,"        mulpd   (%%rsi), %%xmm0\n");
  fprintf(f,"        pshufd  $115, %%xmm5, %%xmm2      # xmm2 = xmm5[3,0,3,1]\n");
  fprintf(f,"        pslld   $20, %%xmm2\n");
  fprintf(f,"        paddd   (%%rdx), %%xmm1\n");
  fprintf(f,"        addpd   %%xmm4, %%xmm0\n");
  fprintf(f,"        movapd  %%xmm0, -40(%%rsp)\n");
  fprintf(f,"        movdqa  %%xmm2, %%xmm7\n");
  fprintf(f,"        mulpd   %%xmm9, %%xmm7\n");
  fprintf(f,"        mulpd   %%xmm2, %%xmm7\n");
  fprintf(f,"        pshufd  $115, %%xmm1, %%xmm1      # xmm1 = xmm1[3,0,3,1]\n");
  fprintf(f,"        mulpd   %%xmm2, %%xmm7\n");
  fprintf(f,"        pslld   $20, %%xmm1\n");
  fprintf(f,"        mulpd   %%xmm2, %%xmm7\n");
  fprintf(f,"        movapd  (%%rcx), %%xmm10\n");
  fprintf(f,"        mulpd   %%xmm1, %%xmm7\n");
  fprintf(f,"        movapd  (%%rax), %%xmm15\n");
  fprintf(f,"        movapd  %%xmm7, %%xmm9\n");
  fprintf(f,"        addpd   %%xmm15, %%xmm9\n");
  fprintf(f,"        movapd  %%xmm7, %%xmm12\n");
  fprintf(f,"        addpd   %%xmm10, %%xmm12\n");
  fprintf(f,"        movapd  %%xmm10, %%xmm5\n");
  fprintf(f,"        divpd   %%xmm12, %%xmm5\n");
  fprintf(f,"        movapd  %%xmm5, %%xmm8\n");
  fprintf(f,"        mulpd   %%xmm9, %%xmm8\n");
  fprintf(f,"        movapd  %%xmm9, %%xmm2\n");
  fprintf(f,"        andpd   %%xmm11, %%xmm2\n");
  fprintf(f,"        movapd  %%xmm5, %%xmm14\n");
  fprintf(f,"        andpd   %%xmm11, %%xmm14\n");
  fprintf(f,"        movapd  %%xmm11, %%xmm0\n");
  fprintf(f,"        movapd  %%xmm14, %%xmm1\n");
  fprintf(f,"        mulpd   %%xmm2, %%xmm1\n");
  fprintf(f,"        subpd   %%xmm8, %%xmm1\n");
  fprintf(f,"        movapd  %%xmm5, %%xmm3\n");
  fprintf(f,"        subpd   %%xmm14, %%xmm3\n");
  fprintf(f,"        movapd  %%xmm3, %%xmm6\n");
  fprintf(f,"        mulpd   %%xmm2, %%xmm6\n");
  fprintf(f,"        addpd   %%xmm1, %%xmm6\n");
  fprintf(f,"        movapd  %%xmm9, %%xmm1\n");
  fprintf(f,"        subpd   %%xmm2, %%xmm1\n");
  fprintf(f,"        movapd  %%xmm14, %%xmm2\n");
  fprintf(f,"        mulpd   %%xmm1, %%xmm2\n");
  fprintf(f,"        addpd   %%xmm6, %%xmm2\n");
  fprintf(f,"        movapd  %%xmm9, %%xmm6\n");
  fprintf(f,"        addpd   %%xmm10, %%xmm6\n");
  fprintf(f,"        subpd   %%xmm6, %%xmm9\n");
  fprintf(f,"        movapd  %%xmm7, %%xmm11\n");
  fprintf(f,"        subpd   %%xmm6, %%xmm11\n");
  fprintf(f,"        movapd  %%xmm12, %%xmm6\n");
  fprintf(f,"        addpd   %%xmm15, %%xmm6\n");
  fprintf(f,"        subpd   %%xmm9, %%xmm15\n");
  fprintf(f,"        addpd   %%xmm11, %%xmm15\n");
  fprintf(f,"        subpd   %%xmm6, %%xmm7\n");
  fprintf(f,"        movapd  %%xmm12, %%xmm4\n");
  fprintf(f,"        subpd   %%xmm6, %%xmm4\n");
  fprintf(f,"        movapd  %%xmm10, %%xmm6\n");
  fprintf(f,"        subpd   %%xmm4, %%xmm6\n");
  fprintf(f,"        addpd   %%xmm7, %%xmm6\n");
  fprintf(f,"        movapd  %%xmm12, %%xmm7\n");
  fprintf(f,"        andpd   %%xmm0, %%xmm7\n");
  fprintf(f,"        movapd  %%xmm14, %%xmm4\n");
  fprintf(f,"        mulpd   %%xmm7, %%xmm4\n");
  fprintf(f,"        subpd   %%xmm4, %%xmm10\n");
  fprintf(f,"        mulpd   %%xmm8, %%xmm6\n");
  fprintf(f,"        subpd   %%xmm6, %%xmm15\n");
  fprintf(f,"        mulpd   %%xmm3, %%xmm1\n");
  fprintf(f,"        addpd   %%xmm2, %%xmm1\n");
  fprintf(f,"        movapd  %%xmm3, %%xmm2\n");
  fprintf(f,"        mulpd   %%xmm7, %%xmm2\n");
  fprintf(f,"        subpd   %%xmm2, %%xmm10\n");
  fprintf(f,"        subpd   %%xmm7, %%xmm12\n");
  fprintf(f,"        mulpd   %%xmm12, %%xmm3\n");
  fprintf(f,"        mulpd   %%xmm14, %%xmm12\n");
  fprintf(f,"        subpd   %%xmm12, %%xmm10\n");
  fprintf(f,"        subpd   %%xmm3, %%xmm10\n");
  fprintf(f,"        mulpd   %%xmm8, %%xmm10\n");
  fprintf(f,"        addpd   %%xmm1, %%xmm10\n");
  fprintf(f,"        mulpd   %%xmm5, %%xmm15\n");
  fprintf(f,"        movapd  %%xmm8, %%xmm7\n");
  fprintf(f,"        mulpd   %%xmm7, %%xmm7\n");
  fprintf(f,"        movapd  %%xmm8, %%xmm2\n");
  fprintf(f,"        mulpd   %%xmm7, %%xmm2\n");
  fprintf(f,"        movapd  %%xmm2, %%xmm3\n");
  fprintf(f,"        movapd  -72(%%rsp), %%xmm9\n");
  fprintf(f,"        xorpd   %%xmm9, %%xmm3\n");
  fprintf(f,"        movapd  %%xmm8, %%xmm1\n");
  fprintf(f,"        andpd   %%xmm0, %%xmm1\n");
  fprintf(f,"        movapd  %%xmm7, %%xmm4\n");
  fprintf(f,"        andpd   %%xmm0, %%xmm4\n");
  fprintf(f,"        movapd  %%xmm1, %%xmm5\n");
  fprintf(f,"        mulpd   %%xmm4, %%xmm5\n");
  fprintf(f,"        addpd   %%xmm3, %%xmm5\n");
  fprintf(f,"        movapd  %%xmm7, %%xmm3\n");
  fprintf(f,"        subpd   %%xmm4, %%xmm3\n");
  fprintf(f,"        movapd  %%xmm1, %%xmm6\n");
  fprintf(f,"        mulpd   %%xmm3, %%xmm6\n");
  fprintf(f,"        addpd   %%xmm5, %%xmm6\n");
  fprintf(f,"        movapd  %%xmm8, %%xmm5\n");
  fprintf(f,"        subpd   %%xmm1, %%xmm5\n");
  fprintf(f,"        mulpd   %%xmm5, %%xmm4\n");
  fprintf(f,"        addpd   %%xmm6, %%xmm4\n");
  fprintf(f,"        mulpd   %%xmm5, %%xmm3\n");
  fprintf(f,"        addpd   %%xmm4, %%xmm3\n");
  fprintf(f,"        addpd   %%xmm10, %%xmm15\n");
  fprintf(f,"        movapd  %%xmm7, %%xmm6\n");
  fprintf(f,"        xorpd   %%xmm9, %%xmm6\n");
  fprintf(f,"        movapd  %%xmm1, %%xmm4\n");
  fprintf(f,"        mulpd   %%xmm4, %%xmm4\n");
  fprintf(f,"        addpd   %%xmm6, %%xmm4\n");
  fprintf(f,"        addpd   %%xmm1, %%xmm1\n");
  fprintf(f,"        mulpd   %%xmm5, %%xmm1\n");
  fprintf(f,"        addpd   %%xmm4, %%xmm1\n");
  fprintf(f,"        mulpd   %%xmm5, %%xmm5\n");
  fprintf(f,"        addpd   %%xmm1, %%xmm5\n");
  fprintf(f,"        movapd  %%xmm7, %%xmm1\n");
  fprintf(f,"        mulpd   %%xmm15, %%xmm1\n");
  fprintf(f,"        addpd   %%xmm3, %%xmm1\n");
  fprintf(f,"        movapd  %%xmm15, %%xmm4\n");
  fprintf(f,"        addpd   %%xmm4, %%xmm4\n");
  fprintf(f,"        mulpd   %%xmm8, %%xmm4\n");
  fprintf(f,"        addpd   %%xmm5, %%xmm4\n");
  fprintf(f,"        movabsq $.LCPI20_17, %%r8\n");
  fprintf(f,"        movabsq $.LCPI20_18, %%r9\n");
  fprintf(f,"        movabsq $.LCPI20_19, %%rdx\n");
  fprintf(f,"        movabsq $.LCPI20_20, %%rsi\n");
  fprintf(f,"        movabsq $.LCPI20_21, %%rdi\n");
  fprintf(f,"        movabsq $.LCPI20_22, %%rax\n");
  fprintf(f,"        movabsq $.LCPI20_23, %%rcx\n");
  fprintf(f,"        movapd  (%%rcx), %%xmm6\n");
  fprintf(f,"        mulpd   %%xmm7, %%xmm6\n");
  fprintf(f,"        addpd   (%%rax), %%xmm6\n");
  fprintf(f,"        mulpd   %%xmm7, %%xmm6\n");
  fprintf(f,"        addpd   (%%rdi), %%xmm6\n");
  fprintf(f,"        mulpd   %%xmm7, %%xmm6\n");
  fprintf(f,"        addpd   (%%rsi), %%xmm6\n");
  fprintf(f,"        mulpd   %%xmm7, %%xmm6\n");
  fprintf(f,"        addpd   (%%rdx), %%xmm6\n");
  fprintf(f,"        mulpd   %%xmm7, %%xmm6\n");
  fprintf(f,"        addpd   (%%r9), %%xmm6\n");
  fprintf(f,"        mulpd   %%xmm7, %%xmm6\n");
  fprintf(f,"        addpd   (%%r8), %%xmm6\n");
  fprintf(f,"        mulpd   %%xmm7, %%xmm6\n");
  fprintf(f,"        mulpd   %%xmm8, %%xmm4\n");
  fprintf(f,"        addpd   %%xmm1, %%xmm4\n");
  fprintf(f,"        movabsq $.LCPI20_24, %%rax\n");
  fprintf(f,"        movapd  (%%rax), %%xmm1\n");
  fprintf(f,"        mulpd   %%xmm1, %%xmm8\n");
  fprintf(f,"        mulpd   %%xmm1, %%xmm15\n");
  fprintf(f,"        movabsq $.LCPI20_25, %%rax\n");
  fprintf(f,"        addpd   (%%rax), %%xmm6\n");
  fprintf(f,"        movapd  %%xmm2, %%xmm3\n");
  fprintf(f,"        mulpd   %%xmm6, %%xmm3\n");
  fprintf(f,"        movapd  %%xmm9, %%xmm5\n");
  fprintf(f,"        xorpd   %%xmm3, %%xmm5\n");
  fprintf(f,"        movapd  %%xmm2, %%xmm1\n");
  fprintf(f,"        andpd   %%xmm0, %%xmm1\n");
  fprintf(f,"        andpd   %%xmm6, %%xmm0\n");
  fprintf(f,"        movapd  %%xmm1, %%xmm7\n");
  fprintf(f,"        mulpd   %%xmm0, %%xmm7\n");
  fprintf(f,"        addpd   %%xmm5, %%xmm7\n");
  fprintf(f,"        subpd   %%xmm1, %%xmm2\n");
  fprintf(f,"        movapd  %%xmm2, %%xmm5\n");
  fprintf(f,"        mulpd   %%xmm0, %%xmm5\n");
  fprintf(f,"        addpd   %%xmm7, %%xmm5\n");
  fprintf(f,"        mulpd   %%xmm6, %%xmm4\n");
  fprintf(f,"        subpd   %%xmm0, %%xmm6\n");
  fprintf(f,"        mulpd   %%xmm6, %%xmm2\n");
  fprintf(f,"        mulpd   %%xmm1, %%xmm6\n");
  fprintf(f,"        addpd   %%xmm5, %%xmm6\n");
  fprintf(f,"        addpd   %%xmm2, %%xmm6\n");
  fprintf(f,"        movabsq $.LCPI20_26, %%rax\n");
  fprintf(f,"        addpd   %%xmm4, %%xmm6\n");
  fprintf(f,"        movapd  %%xmm8, %%xmm2\n");
  fprintf(f,"        addpd   %%xmm3, %%xmm2\n");
  fprintf(f,"        addpd   %%xmm15, %%xmm6\n");
  fprintf(f,"        movapd  %%xmm2, %%xmm1\n");
  fprintf(f,"        subpd   %%xmm8, %%xmm1\n");
  fprintf(f,"        subpd   %%xmm1, %%xmm3\n");
  fprintf(f,"        movapd  %%xmm2, %%xmm0\n");
  fprintf(f,"        subpd   %%xmm1, %%xmm0\n");
  fprintf(f,"        subpd   %%xmm0, %%xmm8\n");
  fprintf(f,"        movapd  -56(%%rsp), %%xmm0\n");
  fprintf(f,"        movapd  %%xmm0, %%xmm1\n");
  fprintf(f,"        xorpd   %%xmm5, %%xmm5\n");
  fprintf(f,"        cmpeqpd %%xmm5, %%xmm1\n");
  fprintf(f,"        movapd  %%xmm0, %%xmm4\n");
  fprintf(f,"        cmpltpd %%xmm5, %%xmm4\n");
  fprintf(f,"        cmpeqpd (%%rax), %%xmm0\n");
  fprintf(f,"        movapd  -24(%%rsp), %%xmm7\n");
  fprintf(f,"        andpd   %%xmm1, %%xmm7\n");
  fprintf(f,"        addpd   %%xmm3, %%xmm8\n");
  fprintf(f,"        addpd   %%xmm6, %%xmm8\n");
  fprintf(f,"        movapd  %%xmm13, %%xmm3\n");
  fprintf(f,"        addpd   %%xmm2, %%xmm3\n");
  fprintf(f,"        addpd   -40(%%rsp), %%xmm8\n");
  fprintf(f,"        movapd  %%xmm3, %%xmm5\n");
  fprintf(f,"        subpd   %%xmm13, %%xmm5\n");
  fprintf(f,"        subpd   %%xmm5, %%xmm2\n");
  fprintf(f,"        movapd  %%xmm3, %%xmm6\n");
  fprintf(f,"        subpd   %%xmm5, %%xmm6\n");
  fprintf(f,"        movapd  (%%rax), %%xmm5\n");
  fprintf(f,"        andpd   %%xmm0, %%xmm5\n");
  fprintf(f,"        subpd   %%xmm6, %%xmm13\n");
  fprintf(f,"        addpd   %%xmm2, %%xmm13\n");
  fprintf(f,"        addpd   %%xmm8, %%xmm13\n");
  fprintf(f,"        addpd   %%xmm3, %%xmm13\n");
  fprintf(f,"        andnpd  %%xmm13, %%xmm0\n");
  fprintf(f,"        orpd    %%xmm5, %%xmm0\n");
  fprintf(f,"        orpd    %%xmm4, %%xmm0\n");
  fprintf(f,"        andnpd  %%xmm0, %%xmm1\n");
  fprintf(f,"        orpd    %%xmm7, %%xmm1\n");
  fprintf(f,"        movapd  %%xmm1, %%xmm0\n");
  fprintf(f,"        # This routine sometimes leaves flags set, so I manually clear them here\n");
  fprintf(f,"        andl     $0xFFC0,.Lnewmxcsr  # clear exception in memory copy of MXCSR\n");
  fprintf(f,"        ldmxcsr .Lnewmxcsr        # and reload that into real MXCSR\n");
  fprintf(f,"        retq\n");
  fprintf(f,"        .size   xlog_u1, .-xlog_u1\n\n");
  fflush(f);
  return;
}

//
//  This procedure will emit code for the runtime library support
//  of the Minimal BASIC EXP() function.  All math is done with
//  64bit float values.
//  The argument for xexp is passed in xmm0, and the result
//  is returned in xmm0.
//  This code was created by running a C compiler on Shibata's 
//  SLEEF version 2.80 files and having it emit assembly.  This 
//  requires SSE2 instruction set support.
//
void write_EXP(FILE *f) {
  if (NULL==f) {
    fprintf(stderr,"write_EXP(): file parameter is NULL\n");
    abort();
  }
  fprintf(f,"# http://shibatch.sourceforge.net/\n");
  fprintf(f,"        .section .rodata.cst16,\"aM\",@progbits,16\n");
  fprintf(f,"        .align  16\n");
  fprintf(f,".LCEXP_0:\n");
  fprintf(f,"        .quad   4609176140021203710     # double 1.442695e+00\n");
  fprintf(f,"        .quad   4609176140021203710     # double 1.442695e+00\n");
  fprintf(f,".LCEXP_1:\n");
  fprintf(f,"        .quad   -4804251826558276378    # double -2.823529e-13\n");
  fprintf(f,"        .quad   -4804251826558276378    # double -2.823529e-13\n");
  fprintf(f,".LCEXP_2:\n");
  fprintf(f,"        .quad   -4618953502541336576    # double -6.931472e-01\n");
  fprintf(f,"        .quad   -4618953502541336576    # double -6.931472e-01\n");
  fprintf(f,".LCEXP_3:\n");
  fprintf(f,"        .long   1023                    # 0x3ff\n");
  fprintf(f,"        .long   1023                    # 0x3ff\n");
  fprintf(f,"        .long   0                       # 0x0\n");
  fprintf(f,"        .long   0                       # 0x0\n");
  fprintf(f,".LCEXP_4:\n");
  fprintf(f,"        .quad   -4503599627370496       # double -inf\n");
  fprintf(f,"        .quad   -4503599627370496       # double -inf\n");
  fprintf(f,".LCEXP_5:\n");
  fprintf(f,"        .long   2047                    # 0x7ff\n");
  fprintf(f,"        .long   2047                    # 0x7ff\n");
  fprintf(f,"        .long   0                       # 0x0\n");
  fprintf(f,"        .long   0                       # 0x0\n");
  fprintf(f,".LCEXP_6:\n");
  fprintf(f,"        .long   1023                    # 0x3ff\n");
  fprintf(f,"        .long   1023                    # 0x3ff\n");
  fprintf(f,"        .zero   4\n");
  fprintf(f,"        .long   0                       # 0x0\n");
  fprintf(f,".LCEXP_7:\n");
  fprintf(f,"        .quad   4523617194703709902     # double 2.755724e-06\n");
  fprintf(f,"        .quad   4523617194703709902     # double 2.755724e-06\n");
  fprintf(f,".LCEXP_8:\n");
  fprintf(f,"        .quad   4508805071378371955     # double 2.755739e-07\n");
  fprintf(f,"        .quad   4508805071378371955     # double 2.755739e-07\n");
  fprintf(f,".LCEXP_9:\n");
  fprintf(f,"        .quad   4493174651505901578     # double 2.511129e-08\n");
  fprintf(f,"        .quad   4493174651505901578     # double 2.511129e-08\n");
  fprintf(f,".LCEXP_10:\n");
  fprintf(f,"        .quad   4477124369930049397     # double 2.088606e-09\n");
  fprintf(f,"        .quad   4477124369930049397     # double 2.088606e-09\n");
  fprintf(f,".LCEXP_11:\n");
  fprintf(f,"        .quad   4537941361629890462     # double 2.480159e-05\n");
  fprintf(f,"        .quad   4537941361629890462     # double 2.480159e-05\n");
  fprintf(f,".LCEXP_12:\n");
  fprintf(f,"        .quad   4607182418800017408     # double 1.000000e+00\n");
  fprintf(f,"        .quad   4607182418800017408     # double 1.000000e+00\n");
  fprintf(f,".LCEXP_13:\n");
  fprintf(f,"        .quad   4602678819172646912     # double 5.000000e-01\n");
  fprintf(f,"        .quad   4602678819172646912     # double 5.000000e-01\n");
  fprintf(f,".LCEXP_14:\n");
  fprintf(f,"        .quad   4595172819793696092     # double 1.666667e-01\n");
  fprintf(f,"        .quad   4595172819793696092     # double 1.666667e-01\n");
  fprintf(f,".LCEXP_15:\n");
  fprintf(f,"        .quad   4586165620538955070     # double 4.166667e-02\n");
  fprintf(f,"        .quad   4586165620538955070     # double 4.166667e-02\n");
  fprintf(f,".LCEXP_16:\n");
  fprintf(f,"        .quad   4575957461383572281     # double 8.333333e-03\n");
  fprintf(f,"        .quad   4575957461383572281     # double 8.333333e-03\n");
  fprintf(f,".LCEXP_17:\n");
  fprintf(f,"        .quad   4564047942369020832     # double 1.388889e-03\n");
  fprintf(f,"        .quad   4564047942369020832     # double 1.388889e-03\n");
  fprintf(f,".LCEXP_18:\n");
  fprintf(f,"        .quad   4551452160574227445     # double 1.984127e-04\n");
  fprintf(f,"        .quad   4551452160574227445     # double 1.984127e-04\n\n");

  fprintf(f,"        .section .text,\"ax\",@progbits\n\n");

  fprintf(f,"        .type   xexp,@function\n");
  fprintf(f,"xexp:\n");
  fprintf(f,"        pushq   %%r15\n");
  fprintf(f,"        # function fails with args < -100.0, so check for that (Big Hammer)\n");
  fprintf(f,"        movabsq $-100, %%r15\n");  fprintf(f,"        # and now for the original SLEEF code...\n");
  fprintf(f,"        cvtsi2sdq       %%r15, %%xmm1\n");
  fprintf(f,"        ucomisd %%xmm0, %%xmm1\n");
  fprintf(f,"        jbe     0f\n");
  fprintf(f,"        xorps   %%xmm0,%%xmm0\n");
  fprintf(f,"        popq    %%r15\n");
  fprintf(f,"        retq\n");
  fprintf(f,"0:\n");
  fprintf(f,"        # and now for the original SLEEF code...\n");
  fprintf(f,"        pushq   %%r14\n");
  fprintf(f,"        pushq   %%r13\n");
  fprintf(f,"        pushq   %%r12\n");
  fprintf(f,"        pushq   %%rbx\n");
  fprintf(f,"        movabsq $.LCEXP_0, %%rax\n");
  fprintf(f,"        movapd  (%%rax), %%xmm1\n");
  fprintf(f,"        mulpd   %%xmm0, %%xmm1\n");
  fprintf(f,"        cvtpd2dq        %%xmm1, %%xmm2\n");
  fprintf(f,"        movdqa  %%xmm2, %%xmm4\n");
  fprintf(f,"        psrad   $31, %%xmm4\n");
  fprintf(f,"        movdqa  %%xmm4, %%xmm3\n");
  fprintf(f,"        paddd   %%xmm2, %%xmm3\n");
  fprintf(f,"        psrad   $9, %%xmm3\n");
  fprintf(f,"        cvtdq2pd        %%xmm2, %%xmm1\n");
  fprintf(f,"        movabsq $.LCEXP_1, %%rax\n");
  fprintf(f,"        movapd  (%%rax), %%xmm5\n");
  fprintf(f,"        mulpd   %%xmm1, %%xmm5\n");
  fprintf(f,"        movabsq $.LCEXP_2, %%rax\n");
  fprintf(f,"        mulpd   (%%rax), %%xmm1\n");
  fprintf(f,"        addpd   %%xmm0, %%xmm1\n");
  fprintf(f,"        movabsq $0x7FF000007FF, %%rax\n");
  fprintf(f,"        psubd   %%xmm4, %%xmm3\n");
  fprintf(f,"        movabsq $.LCEXP_3, %%rdi\n");
  fprintf(f,"        movabsq $.LCEXP_4, %%rdx\n");
  fprintf(f,"        addpd   %%xmm5, %%xmm1\n");
  fprintf(f,"        movd    %%rax, %%xmm4\n");
  fprintf(f,"        movabsq $.LCEXP_5, %%rbx\n");
  fprintf(f,"        movabsq $.LCEXP_6, %%r12\n");
  fprintf(f,"        movabsq $.LCEXP_7, %%r13\n");
  fprintf(f,"        movabsq $.LCEXP_8, %%rsi\n");
  fprintf(f,"        movabsq $.LCEXP_9, %%rax\n");
  fprintf(f,"        movabsq $.LCEXP_10, %%rcx\n");
  fprintf(f,"        movabsq $.LCEXP_11, %%r14\n");
  fprintf(f,"        cmpeqpd (%%rdx), %%xmm0\n");
  fprintf(f,"        movabsq $.LCEXP_12, %%r8\n");
  fprintf(f,"        movabsq $.LCEXP_13, %%r9\n");
  fprintf(f,"        movabsq $.LCEXP_14, %%r10\n");
  fprintf(f,"        pslld   $7, %%xmm3\n");
  fprintf(f,"        movabsq $.LCEXP_15, %%r11\n");
  fprintf(f,"        movdqa  (%%rdi), %%xmm5\n");
  fprintf(f,"        paddd   %%xmm3, %%xmm5\n");
  fprintf(f,"        movdqa  %%xmm5, %%xmm6\n");
  fprintf(f,"        psrad   $31, %%xmm6\n");
  fprintf(f,"        pandn   %%xmm5, %%xmm6\n");
  fprintf(f,"        movabsq $.LCEXP_16, %%r15\n");
  fprintf(f,"        movdqa  %%xmm6, %%xmm5\n");
  fprintf(f,"        pcmpgtd (%%rbx), %%xmm5\n");
  fprintf(f,"        movdqa  %%xmm5, %%xmm7\n");
  fprintf(f,"        pandn   %%xmm6, %%xmm7\n");
  fprintf(f,"        pand    %%xmm4, %%xmm5\n");
  fprintf(f,"        por     %%xmm7, %%xmm5\n");
  fprintf(f,"        movabsq $.LCEXP_17, %%rbx\n");
  fprintf(f,"        movapd  (%%rcx), %%xmm4\n");
  fprintf(f,"        mulpd   %%xmm1, %%xmm4\n");
  fprintf(f,"        movabsq $.LCEXP_18, %%rcx\n");
  fprintf(f,"        addpd   (%%rax), %%xmm4\n");
  fprintf(f,"        mulpd   %%xmm1, %%xmm4\n");
  fprintf(f,"        addpd   (%%rsi), %%xmm4\n");
  fprintf(f,"        mulpd   %%xmm1, %%xmm4\n");
  fprintf(f,"        addpd   (%%r13), %%xmm4\n");
  fprintf(f,"        pshufd  $115, %%xmm5, %%xmm5      # xmm5 = xmm5[3,0,3,1]\n");
  fprintf(f,"        mulpd   %%xmm1, %%xmm4\n");
  fprintf(f,"        pslld   $2, %%xmm3\n");
  fprintf(f,"        psubd   %%xmm3, %%xmm2\n");
  fprintf(f,"        paddd   (%%r12), %%xmm2\n");
  fprintf(f,"        pshufd  $115, %%xmm2, %%xmm2      # xmm2 = xmm2[3,0,3,1]\n");
  fprintf(f,"        movapd  %%xmm1, %%xmm3\n");
  fprintf(f,"        mulpd   %%xmm3, %%xmm3\n");
  fprintf(f,"        addpd   (%%r14), %%xmm4\n");
  fprintf(f,"        mulpd   %%xmm1, %%xmm4\n");
  fprintf(f,"        addpd   (%%rcx), %%xmm4\n");
  fprintf(f,"        mulpd   %%xmm1, %%xmm4\n");
  fprintf(f,"        addpd   (%%rbx), %%xmm4\n");
  fprintf(f,"        pslld   $20, %%xmm5\n");
  fprintf(f,"        pslld   $20, %%xmm2\n");
  fprintf(f,"        mulpd   %%xmm1, %%xmm4\n");
  fprintf(f,"        addpd   (%%r15), %%xmm4\n");
  fprintf(f,"        mulpd   %%xmm1, %%xmm4\n");
  fprintf(f,"        addpd   (%%r11), %%xmm4\n");
  fprintf(f,"        mulpd   %%xmm1, %%xmm4\n");
  fprintf(f,"        addpd   (%%r10), %%xmm4\n");
  fprintf(f,"        mulpd   %%xmm1, %%xmm4\n");
  fprintf(f,"        addpd   (%%r9), %%xmm4\n");
  fprintf(f,"        mulpd   %%xmm3, %%xmm4\n");
  fprintf(f,"        addpd   %%xmm1, %%xmm4\n");
  fprintf(f,"        addpd   (%%r8), %%xmm4\n");
  fprintf(f,"        mulpd   %%xmm5, %%xmm4\n");
  fprintf(f,"        mulpd   %%xmm5, %%xmm4\n");
  fprintf(f,"        mulpd   %%xmm5, %%xmm4\n");
  fprintf(f,"        mulpd   %%xmm5, %%xmm4\n");
  fprintf(f,"        mulpd   %%xmm2, %%xmm4\n");
  fprintf(f,"        andnpd  %%xmm4, %%xmm0\n");
  fprintf(f,"        # This routine sometimes leaves flags set, so I manually clear them here\n");
  fprintf(f,"        andl     $0xFFC0,.Lnewmxcsr  # clear exception in memory copy of MXCSR\n");
  fprintf(f,"        ldmxcsr .Lnewmxcsr        # and reload that into real MXCSR\n");
  fprintf(f,"        popq    %%rbx\n");
  fprintf(f,"        popq    %%r12\n");
  fprintf(f,"        popq    %%r13\n");
  fprintf(f,"        popq    %%r14\n");
  fprintf(f,"        popq    %%r15\n");
  fprintf(f,"        retq\n");
  fprintf(f,"        .size   xexp, .-xexp\n\n");
  fflush(f);
  return;
}

//
//  This procedure will emit code for the runtime library support
//  of the Minimal BASIC power operator '^'.  All math is done with
//  64bit float values using a function similar to the C standard
//  library pow().  When called, the xpow function expects
//  the exponent in xmm1 and the base in xmm0, and the result is
//  returned in xmm0.
//  This code was created by running a C compiler on Shibata's 
//  SLEEF version 2.80 files and having it emit assembly.  This 
//  requires SSE2 instruction set support.  The assembly code was
//  updated to clear the IE and PE flags in MXCSR before returning.
//
void write_POW(FILE *f) {
  if (NULL==f) {
    fprintf(stderr,"write_POW(): file parameter is NULL\n");
    abort();
  }
  fprintf(f,"# http://shibatch.sourceforge.net/\n");
  fprintf(f,"        .section .rodata.cst16,\"aM\",@progbits,16\n");
  fprintf(f,"        .align  16\n");
  fprintf(f,".LCPOW_0:\n");
  fprintf(f,"        .quad   -9223372036854775808    # 0x8000000000000000\n");
  fprintf(f,"        .quad   -9223372036854775808    # 0x8000000000000000\n");
  fprintf(f,".LCPOW_1:\n");
  fprintf(f,"        .long   4294967295              # 0xffffffff\n");
  fprintf(f,"        .long   2147483647              # 0x7fffffff\n");
  fprintf(f,"        .long   4294967295              # 0xffffffff\n");
  fprintf(f,"        .long   2147483647              # 0x7fffffff\n");
  fprintf(f,".LCPOW_2:\n");
  fprintf(f,"        .quad   -4616189618054758400    # double -1.000000e+00\n");
  fprintf(f,"        .quad   -4616189618054758400    # double -1.000000e+00\n");
  fprintf(f,".LCPOW_3:\n");
  fprintf(f,"        .quad   5958262307011166208     # double 2.037036e+90\n");
  fprintf(f,"        .quad   5958262307011166208     # double 2.037036e+90\n");
  fprintf(f,".LCPOW_4:\n");
  fprintf(f,"        .quad   3256102530588868608     # double 4.909093e-91\n");
  fprintf(f,"        .quad   3256102530588868608     # double 4.909093e-91\n");
  fprintf(f,".LCPOW_5:\n");
  fprintf(f,"        .quad   4604544210138303771     # double 7.071000e-01\n");
  fprintf(f,"        .quad   4604544210138303771     # double 7.071000e-01\n");
  fprintf(f,".LCPOW_6:\n");
  fprintf(f,"        .quad   9218868437227405312     # 0x7ff0000000000000\n");
  fprintf(f,"        .quad   9218868437227405312     # 0x7ff0000000000000\n");
  fprintf(f,".LCPOW_7:\n");
  fprintf(f,"        .quad   4607182418800017408     # 0x3ff0000000000000\n");
  fprintf(f,"        .quad   4607182418800017408     # 0x3ff0000000000000\n");
  fprintf(f,".LCPOW_8:\n");
  fprintf(f,"        .long   1                       # 0x1\n");
  fprintf(f,"        .long   1                       # 0x1\n");
  fprintf(f,"        .long   0                       # 0x0\n");
  fprintf(f,"        .long   0                       # 0x0\n");
  fprintf(f,".LCPOW_9:\n");
  fprintf(f,"        .long   0                       # 0x0\n");
  fprintf(f,"        .long   4294965974              # 0xfffffad6\n");
  fprintf(f,"        .long   0                       # 0x0\n");
  fprintf(f,"        .long   4294965974              # 0xfffffad6\n");
  fprintf(f,".LCPOW_10:\n");
  fprintf(f,"        .long   0                       # 0x0\n");
  fprintf(f,"        .long   4294966274              # 0xfffffc02\n");
  fprintf(f,"        .long   0                       # 0x0\n");
  fprintf(f,"        .long   4294966274              # 0xfffffc02\n");
  fprintf(f,".LCPOW_11:\n");
  fprintf(f,"        .quad   -4503599627370496       # 0xfff0000000000000\n");
  fprintf(f,"        .quad   -4503599627370496       # 0xfff0000000000000\n");
  fprintf(f,".LCPOW_12:\n");
  fprintf(f,"        .long   1023                    # 0x3ff\n");
  fprintf(f,"        .long   1023                    # 0x3ff\n");
  fprintf(f,"        .long   0                       # 0x0\n");
  fprintf(f,"        .long   0                       # 0x0\n");
  fprintf(f,".LCPOW_13:\n");
  fprintf(f,"        .quad   4604418534313441775     # double 6.931472e-01\n");
  fprintf(f,"        .quad   4604418534313441775     # double 6.931472e-01\n");
  fprintf(f,".LCPOW_14:\n");
  fprintf(f,"        .quad   -134217728              # 0xfffffffff8000000\n");
  fprintf(f,"        .quad   -134217728              # 0xfffffffff8000000\n");
  fprintf(f,".LCPOW_15:\n");
  fprintf(f,"        .quad   4604418534196379648     # double 6.931472e-01\n");
  fprintf(f,"        .quad   4604418534196379648     # double 6.931472e-01\n");
  fprintf(f,".LCPOW_16:\n");
  fprintf(f,"        .quad   4488937535594037248     # double 1.299651e-08\n");
  fprintf(f,"        .quad   4488937535594037248     # double 1.299651e-08\n");
  fprintf(f,".LCPOW_17:\n");
  fprintf(f,"        .long   2047                    # 0x7ff\n");
  fprintf(f,"        .long   2047                    # 0x7ff\n");
  fprintf(f,"        .long   0                       # 0x0\n");
  fprintf(f,"        .long   0                       # 0x0\n");
  fprintf(f,".LCPOW_18:\n");
  fprintf(f,"        .quad   4358002977218854975     # double 2.319047e-17\n");
  fprintf(f,"        .quad   4358002977218854975     # double 2.319047e-17\n");
  fprintf(f,".LCPOW_19:\n");
  fprintf(f,"        .long   1023                    # 0x3ff\n");
  fprintf(f,"        .long   1023                    # 0x3ff\n");
  fprintf(f,"        .zero   4\n");
  fprintf(f,"        .long   0                       # 0x0\n");
  fprintf(f,".LCPOW_20:\n");
  fprintf(f,"        .quad   4600877379321702721     # double 4.000000e-01\n");
  fprintf(f,"        .quad   4600877379321702721     # double 4.000000e-01\n");
  fprintf(f,".LCPOW_21:\n");
  fprintf(f,"        .quad   4598818590919479713     # double 2.857143e-01\n");
  fprintf(f,"        .quad   4598818590919479713     # double 2.857143e-01\n");
  fprintf(f,".LCPOW_22:\n");
  fprintf(f,"        .quad   4597174419956087886     # double 2.222222e-01\n");
  fprintf(f,"        .quad   4597174419956087886     # double 2.222222e-01\n");
  fprintf(f,".LCPOW_23:\n");
  fprintf(f,"        .quad   4595718683483098638     # double 1.818174e-01\n");
  fprintf(f,"        .quad   4595718683483098638     # double 1.818174e-01\n");
  fprintf(f,".LCPOW_24:\n");
  fprintf(f,"        .quad   4594712256174854310     # double 1.538835e-01\n");
  fprintf(f,"        .quad   4594712256174854310     # double 1.538835e-01\n");
  fprintf(f,".LCPOW_25:\n");
  fprintf(f,"        .quad   4593932774978511152     # double 1.322485e-01\n");
  fprintf(f,"        .quad   4593932774978511152     # double 1.322485e-01\n");
  fprintf(f,".LCPOW_26:\n");
  fprintf(f,"        .quad   4594017567976567522     # double 1.346020e-01\n");
  fprintf(f,"        .quad   4594017567976567522     # double 1.346020e-01\n");
  fprintf(f,".LCPOW_27:\n");
  fprintf(f,"        .quad   4611686018427387904     # double 2.000000e+00\n");
  fprintf(f,"        .quad   4611686018427387904     # double 2.000000e+00\n");
  fprintf(f,".LCPOW_28:\n");
  fprintf(f,"        .quad   4604180019048437075     # double 6.666667e-01\n");
  fprintf(f,"        .quad   4604180019048437075     # double 6.666667e-01\n");
  fprintf(f,".LCPOW_29:\n");
  fprintf(f,"        .quad   -4618953502541336576    # double -6.931472e-01\n");
  fprintf(f,"        .quad   -4618953502541336576    # double -6.931472e-01\n");
  fprintf(f,".LCPOW_30:\n");
  fprintf(f,"        .quad   4609176140021203710     # double 1.442695e+00\n");
  fprintf(f,"        .quad   4609176140021203710     # double 1.442695e+00\n");
  fprintf(f,".LCPOW_31:\n");
  fprintf(f,"        .quad   -4804251826558276378    # double -2.823529e-13\n");
  fprintf(f,"        .quad   -4804251826558276378    # double -2.823529e-13\n");
  fprintf(f,".LCPOW_32:\n");
  fprintf(f,"        .quad   4602678819172646921     # double 5.000000e-01\n");
  fprintf(f,"        .quad   4602678819172646921     # double 5.000000e-01\n");
  fprintf(f,".LCPOW_33:\n");
  fprintf(f,"        .quad   4595172819793696088     # double 1.666667e-01\n");
  fprintf(f,"        .quad   4595172819793696088     # double 1.666667e-01\n");
  fprintf(f,".LCPOW_34:\n");
  fprintf(f,"        .quad   4586165620538936976     # double 4.166667e-02\n");
  fprintf(f,"        .quad   4586165620538936976     # double 4.166667e-02\n");
  fprintf(f,".LCPOW_35:\n");
  fprintf(f,"        .quad   4575957461383576424     # double 8.333333e-03\n");
  fprintf(f,"        .quad   4575957461383576424     # double 8.333333e-03\n");
  fprintf(f,".LCPOW_36:\n");
  fprintf(f,"        .quad   4564047942392540283     # double 1.388889e-03\n");
  fprintf(f,"        .quad   4564047942392540283     # double 1.388889e-03\n");
  fprintf(f,".LCPOW_37:\n");
  fprintf(f,"        .quad   4551452160568640317     # double 1.984127e-04\n");
  fprintf(f,"        .quad   4551452160568640317     # double 1.984127e-04\n");
  fprintf(f,".LCPOW_38:\n");
  fprintf(f,"        .quad   4537941335137339675     # double 2.480150e-05\n");
  fprintf(f,"        .quad   4537941335137339675     # double 2.480150e-05\n");
  fprintf(f,".LCPOW_39:\n");
  fprintf(f,"        .quad   4523617197863282270     # double 2.755725e-06\n");
  fprintf(f,"        .quad   4523617197863282270     # double 2.755725e-06\n");
  fprintf(f,".LCPOW_40:\n");
  fprintf(f,"        .quad   4508818525505041837     # double 2.762862e-07\n");
  fprintf(f,"        .quad   4508818525505041837     # double 2.762862e-07\n");
  fprintf(f,".LCPOW_41:\n");
  fprintf(f,"        .quad   4493173344431265465     # double 2.510697e-08\n");
  fprintf(f,"        .quad   4493173344431265465     # double 2.510697e-08\n\n");

  fprintf(f,"        .section .text,\"ax\",@progbits\n\n");

  fprintf(f,"        .type   xpow,@function\n");
  fprintf(f,"xpow:\n");
  fprintf(f,"        pushq   %%rbx\n");
  fprintf(f,"        subq    $144, %%rsp\n");
  fprintf(f,"        movapd  %%xmm1, -48(%%rsp)\n");
  fprintf(f,"        movabsq $.LCPOW_0, %%rax\n");
  fprintf(f,"        movapd  (%%rax), %%xmm3\n");
  fprintf(f,"        movapd  %%xmm3, 96(%%rsp)\n");
  fprintf(f,"        xorpd   %%xmm13, %%xmm13\n");
  fprintf(f,"        movapd  %%xmm0, %%xmm14\n");
  fprintf(f,"        cmpeqpd %%xmm13, %%xmm14\n");
  fprintf(f,"        movapd  %%xmm1, %%xmm2\n");
  fprintf(f,"        andpd   %%xmm3, %%xmm2\n");
  fprintf(f,"        movabsq $.LCPOW_1, %%rax\n");
  fprintf(f,"        movapd  (%%rax), %%xmm4\n");
  fprintf(f,"        movapd  %%xmm4, 128(%%rsp)\n");
  fprintf(f,"        movapd  %%xmm0, %%xmm11\n");
  fprintf(f,"        andpd   %%xmm4, %%xmm11\n");
  fprintf(f,"        movabsq $.LCPOW_2, %%rax\n");
  fprintf(f,"        movapd  (%%rax), %%xmm4\n");
  fprintf(f,"        movapd  %%xmm4, -112(%%rsp)\n");
  fprintf(f,"        movapd  %%xmm11, %%xmm5\n");
  fprintf(f,"        addpd   %%xmm4, %%xmm5\n");
  fprintf(f,"        xorpd   %%xmm2, %%xmm5\n");
  fprintf(f,"        movapd  %%xmm5, 112(%%rsp)\n");
  fprintf(f,"        movapd  %%xmm5, %%xmm4\n");
  fprintf(f,"        movapd  %%xmm1, %%xmm2\n");
  fprintf(f,"        xorpd   %%xmm3, %%xmm2\n");
  fprintf(f,"        movapd  %%xmm3, %%xmm9\n");
  fprintf(f,"        andpd   %%xmm14, %%xmm2\n");
  fprintf(f,"        movapd  %%xmm14, %%xmm5\n");
  fprintf(f,"        andnpd  %%xmm1, %%xmm5\n");
  fprintf(f,"        movapd  %%xmm1, %%xmm10\n");
  fprintf(f,"        orpd    %%xmm2, %%xmm5\n");
  fprintf(f,"        movabsq $.LCPOW_3, %%rcx\n");
  fprintf(f,"        movabsq $.LCPOW_4, %%rdx\n");
  fprintf(f,"        movabsq $.LCPOW_5, %%rsi\n");
  fprintf(f,"        movapd  (%%rsi), %%xmm7\n");
  fprintf(f,"        mulpd   %%xmm11, %%xmm7\n");
  fprintf(f,"        movapd  %%xmm7, %%xmm15\n");
  fprintf(f,"        cmpltpd (%%rdx), %%xmm15\n");
  fprintf(f,"        movapd  %%xmm15, %%xmm2\n");
  fprintf(f,"        andnpd  %%xmm7, %%xmm2\n");
  fprintf(f,"        mulpd   (%%rcx), %%xmm7\n");
  fprintf(f,"        andpd   %%xmm15, %%xmm7\n");
  fprintf(f,"        orpd    %%xmm2, %%xmm7\n");
  fprintf(f,"        cmpltpd %%xmm13, %%xmm5\n");
  fprintf(f,"        movabsq $.LCPOW_6, %%rcx\n");
  fprintf(f,"        movapd  (%%rcx), %%xmm3\n");
  fprintf(f,"        andnpd  %%xmm3, %%xmm5\n");
  fprintf(f,"        movapd  %%xmm4, %%xmm6\n");
  fprintf(f,"        cmpeqpd %%xmm13, %%xmm6\n");
  fprintf(f,"        movapd  %%xmm6, %%xmm2\n");
  fprintf(f,"        andnpd  %%xmm3, %%xmm2\n");
  fprintf(f,"        movabsq $.LCPOW_7, %%rdx\n");
  fprintf(f,"        movapd  (%%rdx), %%xmm12\n");
  fprintf(f,"        movapd  %%xmm12, (%%rsp)\n");
  fprintf(f,"        andpd   %%xmm12, %%xmm6\n");
  fprintf(f,"        orpd    %%xmm2, %%xmm6\n");
  fprintf(f,"        movabsq $0x100000001, %%rsi\n");
  fprintf(f,"        movd    %%rsi, %%xmm2\n");
  fprintf(f,"        cvtpd2dq        %%xmm10, %%xmm3\n");
  fprintf(f,"        andpd   %%xmm3, %%xmm2\n");
  fprintf(f,"        movabsq $.LCPOW_8, %%rsi\n");
  fprintf(f,"        pcmpeqd (%%rsi), %%xmm2\n");
  fprintf(f,"        pshufd  $80, %%xmm2, %%xmm4       # xmm4 = xmm2[0,0,1,1]\n");
  fprintf(f,"        pcmpeqd %%xmm2, %%xmm2\n");
  fprintf(f,"        pxor    %%xmm4, %%xmm2\n");
  fprintf(f,"        cvtdq2pd        %%xmm3, %%xmm8\n");
  fprintf(f,"        cmpneqpd        %%xmm10, %%xmm8\n");
  fprintf(f,"        movapd  %%xmm8, %%xmm3\n");
  fprintf(f,"        andnpd  %%xmm4, %%xmm3\n");
  fprintf(f,"        movapd  %%xmm0, %%xmm4\n");
  fprintf(f,"        andpd   %%xmm9, %%xmm4\n");
  fprintf(f,"        orpd    %%xmm12, %%xmm4\n");
  fprintf(f,"        andpd   %%xmm3, %%xmm4\n");
  fprintf(f,"        por     %%xmm8, %%xmm2\n");
  fprintf(f,"        pand    %%xmm12, %%xmm2\n");
  fprintf(f,"        orpd    %%xmm2, %%xmm4\n");
  fprintf(f,"        andpd   (%%rax), %%xmm3\n");
  fprintf(f,"        orpd    %%xmm2, %%xmm3\n");
  fprintf(f,"        movapd  %%xmm10, %%xmm2\n");
  fprintf(f,"        cmpneqpd        %%xmm2, %%xmm2\n");
  fprintf(f,"        movapd  %%xmm0, %%xmm1\n");
  fprintf(f,"        cmpneqpd        %%xmm1, %%xmm1\n");
  fprintf(f,"        orpd    %%xmm2, %%xmm1\n");
  fprintf(f,"        movapd  %%xmm1, 80(%%rsp)\n");
  fprintf(f,"        mulpd   %%xmm5, %%xmm4\n");
  fprintf(f,"        movapd  %%xmm4, 64(%%rsp)\n");
  fprintf(f,"        movapd  %%xmm10, %%xmm2\n");
  fprintf(f,"        cmpeqpd %%xmm13, %%xmm2\n");
  fprintf(f,"        movapd  (%%rdx), %%xmm4\n");
  fprintf(f,"        movapd  %%xmm4, -128(%%rsp)\n");
  fprintf(f,"        movapd  %%xmm0, %%xmm1\n");
  fprintf(f,"        cmpeqpd %%xmm4, %%xmm1\n");
  fprintf(f,"        orpd    %%xmm2, %%xmm1\n");
  fprintf(f,"        movapd  %%xmm1, 48(%%rsp)\n");
  fprintf(f,"        movapd  (%%rcx), %%xmm2\n");
  fprintf(f,"        movapd  %%xmm11, %%xmm1\n");
  fprintf(f,"        cmpeqpd %%xmm2, %%xmm1\n");
  fprintf(f,"        orpd    %%xmm14, %%xmm1\n");
  fprintf(f,"        movapd  %%xmm1, 32(%%rsp)\n");
  fprintf(f,"        movapd  128(%%rsp), %%xmm9\n");
  fprintf(f,"        andpd   %%xmm10, %%xmm9\n");
  fprintf(f,"        cmpeqpd %%xmm2, %%xmm9\n");
  fprintf(f,"        movapd  %%xmm9, 128(%%rsp)\n");
  fprintf(f,"        movabsq $.LCPOW_9, %%rax\n");
  fprintf(f,"        movabsq $.LCPOW_10, %%rcx\n");
  fprintf(f,"        movabsq $.LCPOW_11, %%rdx\n");
  fprintf(f,"        andpd   (%%rdx), %%xmm7\n");
  fprintf(f,"        psrld   $20, %%xmm7\n");
  fprintf(f,"        movdqa  (%%rcx), %%xmm4\n");
  fprintf(f,"        paddd   %%xmm7, %%xmm4\n");
  fprintf(f,"        movapd  %%xmm15, %%xmm2\n");
  fprintf(f,"        andnpd  %%xmm4, %%xmm2\n");
  fprintf(f,"        paddd   (%%rax), %%xmm7\n");
  fprintf(f,"        pand    %%xmm15, %%xmm7\n");
  fprintf(f,"        por     %%xmm2, %%xmm7\n");
  fprintf(f,"        orpd    %%xmm8, %%xmm3\n");
  fprintf(f,"        xorpd   %%xmm1, %%xmm1\n");
  fprintf(f,"        cmpltpd %%xmm0, %%xmm1\n");
  fprintf(f,"        movapd  %%xmm1, %%xmm0\n");
  fprintf(f,"        andnpd  %%xmm3, %%xmm0\n");
  fprintf(f,"        andpd   %%xmm12, %%xmm1\n");
  fprintf(f,"        orpd    %%xmm0, %%xmm1\n");
  fprintf(f,"        movapd  %%xmm1, 16(%%rsp)\n");
  fprintf(f,"        movapd  112(%%rsp), %%xmm10\n");
  fprintf(f,"        cmpltpd %%xmm13, %%xmm10\n");
  fprintf(f,"        andnpd  %%xmm6, %%xmm10\n");
  fprintf(f,"        movapd  %%xmm10, 112(%%rsp)\n");
  fprintf(f,"        pshufd  $13, %%xmm7, %%xmm3       # xmm3 = xmm7[1,3,0,0]\n");
  fprintf(f,"        psubd   %%xmm3, %%xmm13\n");
  fprintf(f,"        movdqa  %%xmm13, %%xmm2\n");
  fprintf(f,"        psrad   $31, %%xmm2\n");
  fprintf(f,"        movdqa  %%xmm2, %%xmm0\n");
  fprintf(f,"        psubd   %%xmm3, %%xmm0\n");
  fprintf(f,"        psrad   $9, %%xmm0\n");
  fprintf(f,"        psubd   %%xmm2, %%xmm0\n");
  fprintf(f,"        movabsq $.LCPOW_12, %%rax\n");
  fprintf(f,"        pslld   $7, %%xmm0\n");
  fprintf(f,"        movdqa  (%%rax), %%xmm1\n");
  fprintf(f,"        movdqa  %%xmm1, -80(%%rsp)\n");
  fprintf(f,"        movdqa  %%xmm0, %%xmm2\n");
  fprintf(f,"        paddd   %%xmm1, %%xmm2\n");
  fprintf(f,"        movdqa  %%xmm2, %%xmm4\n");
  fprintf(f,"        psrad   $31, %%xmm4\n");
  fprintf(f,"        pandn   %%xmm2, %%xmm4\n");
  fprintf(f,"        movabsq $.LCPOW_13, %%rax\n");
  fprintf(f,"        movabsq $.LCPOW_14, %%rcx\n");
  fprintf(f,"        movabsq $.LCPOW_15, %%rdx\n");
  fprintf(f,"        cvtdq2pd        %%xmm3, %%xmm8\n");
  fprintf(f,"        movapd  (%%rdx), %%xmm6\n");
  fprintf(f,"        movapd  (%%rcx), %%xmm9\n");
  fprintf(f,"        movapd  (%%rax), %%xmm14\n");
  fprintf(f,"        mulpd   %%xmm8, %%xmm14\n");
  fprintf(f,"        movapd  %%xmm14, %%xmm2\n");
  fprintf(f,"        xorpd   96(%%rsp), %%xmm2\n");
  fprintf(f,"        movapd  %%xmm8, %%xmm3\n");
  fprintf(f,"        andpd   %%xmm9, %%xmm3\n");
  fprintf(f,"        movapd  %%xmm3, %%xmm7\n");
  fprintf(f,"        mulpd   %%xmm6, %%xmm7\n");
  fprintf(f,"        addpd   %%xmm2, %%xmm7\n");
  fprintf(f,"        movabsq $.LCPOW_16, %%rax\n");
  fprintf(f,"        movapd  (%%rax), %%xmm5\n");
  fprintf(f,"        movapd  %%xmm8, %%xmm2\n");
  fprintf(f,"        subpd   %%xmm3, %%xmm2\n");
  fprintf(f,"        mulpd   %%xmm5, %%xmm3\n");
  fprintf(f,"        mulpd   %%xmm2, %%xmm5\n");
  fprintf(f,"        mulpd   %%xmm6, %%xmm2\n");
  fprintf(f,"        addpd   %%xmm7, %%xmm3\n");
  fprintf(f,"        movabsq $0x7FF000007FF, %%rax\n");
  fprintf(f,"        movabsq $.LCPOW_17, %%rcx\n");
  fprintf(f,"        movdqa  (%%rcx), %%xmm1\n");
  fprintf(f,"        movdqa  %%xmm1, -16(%%rsp)\n");
  fprintf(f,"        movdqa  %%xmm4, %%xmm6\n");
  fprintf(f,"        pcmpgtd %%xmm1, %%xmm6\n");
  fprintf(f,"        movdqa  %%xmm6, %%xmm7\n");
  fprintf(f,"        pandn   %%xmm4, %%xmm7\n");
  fprintf(f,"        movd    %%rax, %%xmm1\n");
  fprintf(f,"        movapd  %%xmm1, -32(%%rsp)\n");
  fprintf(f,"        pand    %%xmm1, %%xmm6\n");
  fprintf(f,"        por     %%xmm7, %%xmm6\n");
  fprintf(f,"        addpd   %%xmm2, %%xmm3\n");
  fprintf(f,"        addpd   %%xmm5, %%xmm3\n");
  fprintf(f,"        pslld   $2, %%xmm0\n");
  fprintf(f,"        psubd   %%xmm0, %%xmm13\n");
  fprintf(f,"        movabsq $.LCPOW_18, %%rax\n");
  fprintf(f,"        mulpd   (%%rax), %%xmm8\n");
  fprintf(f,"        addpd   %%xmm3, %%xmm8\n");
  fprintf(f,"        movapd  %%xmm8, -96(%%rsp)\n");
  fprintf(f,"        pshufd  $115, %%xmm6, %%xmm0      # xmm0 = xmm6[3,0,3,1]\n");
  fprintf(f,"        pslld   $20, %%xmm0\n");
  fprintf(f,"        mulpd   %%xmm0, %%xmm11\n");
  fprintf(f,"        mulpd   %%xmm0, %%xmm11\n");
  fprintf(f,"        mulpd   %%xmm0, %%xmm11\n");
  fprintf(f,"        mulpd   %%xmm0, %%xmm11\n");
  fprintf(f,"        movabsq $.LCPOW_19, %%rax\n");
  fprintf(f,"        movdqa  (%%rax), %%xmm0\n");
  fprintf(f,"        movdqa  %%xmm0, -64(%%rsp)\n");
  fprintf(f,"        paddd   %%xmm0, %%xmm13\n");
  fprintf(f,"        pshufd  $115, %%xmm13, %%xmm8     # xmm8 = xmm13[3,0,3,1]\n");
  fprintf(f,"        pslld   $20, %%xmm8\n");
  fprintf(f,"        mulpd   %%xmm11, %%xmm8\n");
  fprintf(f,"        movapd  %%xmm8, %%xmm7\n");
  fprintf(f,"        movapd  -112(%%rsp), %%xmm1\n");
  fprintf(f,"        addpd   %%xmm1, %%xmm7\n");
  fprintf(f,"        movapd  %%xmm8, %%xmm2\n");
  fprintf(f,"        movapd  -128(%%rsp), %%xmm0\n");
  fprintf(f,"        addpd   %%xmm0, %%xmm2\n");
  fprintf(f,"        movapd  %%xmm0, %%xmm11\n");
  fprintf(f,"        movapd  %%xmm0, %%xmm10\n");
  fprintf(f,"        divpd   %%xmm2, %%xmm11\n");
  fprintf(f,"        movapd  %%xmm11, %%xmm13\n");
  fprintf(f,"        mulpd   %%xmm7, %%xmm13\n");
  fprintf(f,"        movapd  %%xmm7, %%xmm5\n");
  fprintf(f,"        andpd   %%xmm9, %%xmm5\n");
  fprintf(f,"        movapd  %%xmm11, %%xmm3\n");
  fprintf(f,"        andpd   %%xmm9, %%xmm3\n");
  fprintf(f,"        movapd  %%xmm3, %%xmm6\n");
  fprintf(f,"        mulpd   %%xmm5, %%xmm6\n");
  fprintf(f,"        subpd   %%xmm13, %%xmm6\n");
  fprintf(f,"        movapd  %%xmm11, %%xmm0\n");
  fprintf(f,"        subpd   %%xmm3, %%xmm0\n");
  fprintf(f,"        movapd  %%xmm0, %%xmm4\n");
  fprintf(f,"        mulpd   %%xmm5, %%xmm4\n");
  fprintf(f,"        addpd   %%xmm6, %%xmm4\n");
  fprintf(f,"        movapd  %%xmm7, %%xmm6\n");
  fprintf(f,"        subpd   %%xmm5, %%xmm6\n");
  fprintf(f,"        movapd  %%xmm3, %%xmm5\n");
  fprintf(f,"        mulpd   %%xmm6, %%xmm5\n");
  fprintf(f,"        addpd   %%xmm4, %%xmm5\n");
  fprintf(f,"        movapd  %%xmm7, %%xmm4\n");
  fprintf(f,"        addpd   %%xmm10, %%xmm4\n");
  fprintf(f,"        subpd   %%xmm4, %%xmm7\n");
  fprintf(f,"        movapd  %%xmm8, %%xmm15\n");
  fprintf(f,"        subpd   %%xmm4, %%xmm15\n");
  fprintf(f,"        movapd  %%xmm2, %%xmm4\n");
  fprintf(f,"        addpd   %%xmm1, %%xmm4\n");
  fprintf(f,"        subpd   %%xmm7, %%xmm1\n");
  fprintf(f,"        addpd   %%xmm15, %%xmm1\n");
  fprintf(f,"        movapd  %%xmm1, %%xmm12\n");
  fprintf(f,"        subpd   %%xmm4, %%xmm8\n");
  fprintf(f,"        movapd  %%xmm2, %%xmm7\n");
  fprintf(f,"        subpd   %%xmm4, %%xmm7\n");
  fprintf(f,"        movapd  %%xmm10, %%xmm4\n");
  fprintf(f,"        subpd   %%xmm7, %%xmm4\n");
  fprintf(f,"        addpd   %%xmm8, %%xmm4\n");
  fprintf(f,"        mulpd   %%xmm13, %%xmm4\n");
  fprintf(f,"        subpd   %%xmm4, %%xmm12\n");
  fprintf(f,"        mulpd   %%xmm0, %%xmm6\n");
  fprintf(f,"        addpd   %%xmm5, %%xmm6\n");
  fprintf(f,"        movapd  %%xmm2, %%xmm5\n");
  fprintf(f,"        andpd   %%xmm9, %%xmm5\n");
  fprintf(f,"        movapd  %%xmm3, %%xmm4\n");
  fprintf(f,"        mulpd   %%xmm5, %%xmm4\n");
  fprintf(f,"        movapd  %%xmm10, %%xmm7\n");
  fprintf(f,"        subpd   %%xmm4, %%xmm7\n");
  fprintf(f,"        movapd  %%xmm0, %%xmm4\n");
  fprintf(f,"        mulpd   %%xmm5, %%xmm4\n");
  fprintf(f,"        subpd   %%xmm4, %%xmm7\n");
  fprintf(f,"        subpd   %%xmm5, %%xmm2\n");
  fprintf(f,"        mulpd   %%xmm2, %%xmm0\n");
  fprintf(f,"        mulpd   %%xmm3, %%xmm2\n");
  fprintf(f,"        subpd   %%xmm2, %%xmm7\n");
  fprintf(f,"        subpd   %%xmm0, %%xmm7\n");
  fprintf(f,"        mulpd   %%xmm13, %%xmm7\n");
  fprintf(f,"        addpd   %%xmm6, %%xmm7\n");
  fprintf(f,"        mulpd   %%xmm11, %%xmm12\n");
  fprintf(f,"        movapd  %%xmm13, %%xmm0\n");
  fprintf(f,"        mulpd   %%xmm0, %%xmm0\n");
  fprintf(f,"        movapd  %%xmm13, %%xmm8\n");
  fprintf(f,"        mulpd   %%xmm0, %%xmm8\n");
  fprintf(f,"        movapd  %%xmm8, %%xmm4\n");
  fprintf(f,"        movapd  96(%%rsp), %%xmm1\n");
  fprintf(f,"        xorpd   %%xmm1, %%xmm4\n");
  fprintf(f,"        movapd  %%xmm13, %%xmm3\n");
  fprintf(f,"        andpd   %%xmm9, %%xmm3\n");
  fprintf(f,"        movapd  %%xmm0, %%xmm6\n");
  fprintf(f,"        andpd   %%xmm9, %%xmm6\n");
  fprintf(f,"        movapd  %%xmm3, %%xmm5\n");
  fprintf(f,"        mulpd   %%xmm6, %%xmm5\n");
  fprintf(f,"        addpd   %%xmm4, %%xmm5\n");
  fprintf(f,"        movapd  %%xmm0, %%xmm4\n");
  fprintf(f,"        subpd   %%xmm6, %%xmm4\n");
  fprintf(f,"        movapd  %%xmm3, %%xmm2\n");
  fprintf(f,"        mulpd   %%xmm4, %%xmm2\n");
  fprintf(f,"        addpd   %%xmm5, %%xmm2\n");
  fprintf(f,"        movapd  %%xmm13, %%xmm5\n");
  fprintf(f,"        subpd   %%xmm3, %%xmm5\n");
  fprintf(f,"        mulpd   %%xmm5, %%xmm6\n");
  fprintf(f,"        addpd   %%xmm2, %%xmm6\n");
  fprintf(f,"        mulpd   %%xmm5, %%xmm4\n");
  fprintf(f,"        addpd   %%xmm6, %%xmm4\n");
  fprintf(f,"        addpd   %%xmm7, %%xmm12\n");
  fprintf(f,"        movapd  %%xmm0, %%xmm6\n");
  fprintf(f,"        xorpd   %%xmm1, %%xmm6\n");
  fprintf(f,"        movapd  %%xmm1, %%xmm15\n");
  fprintf(f,"        movapd  %%xmm3, %%xmm2\n");
  fprintf(f,"        mulpd   %%xmm2, %%xmm2\n");
  fprintf(f,"        addpd   %%xmm6, %%xmm2\n");
  fprintf(f,"        addpd   %%xmm3, %%xmm3\n");
  fprintf(f,"        mulpd   %%xmm5, %%xmm3\n");
  fprintf(f,"        addpd   %%xmm2, %%xmm3\n");
  fprintf(f,"        mulpd   %%xmm5, %%xmm5\n");
  fprintf(f,"        addpd   %%xmm3, %%xmm5\n");
  fprintf(f,"        movapd  %%xmm0, %%xmm6\n");
  fprintf(f,"        movapd  %%xmm12, %%xmm1\n");
  fprintf(f,"        mulpd   %%xmm1, %%xmm6\n");
  fprintf(f,"        addpd   %%xmm4, %%xmm6\n");
  fprintf(f,"        movapd  %%xmm1, %%xmm4\n");
  fprintf(f,"        addpd   %%xmm4, %%xmm4\n");
  fprintf(f,"        mulpd   %%xmm13, %%xmm4\n");
  fprintf(f,"        addpd   %%xmm5, %%xmm4\n");
  fprintf(f,"        movabsq $.LCPOW_20, %%r8\n");
  fprintf(f,"        movabsq $.LCPOW_21, %%r9\n");
  fprintf(f,"        movabsq $.LCPOW_22, %%rdx\n");
  fprintf(f,"        movabsq $.LCPOW_23, %%rsi\n");
  fprintf(f,"        movabsq $.LCPOW_24, %%rdi\n");
  fprintf(f,"        movabsq $.LCPOW_25, %%rax\n");
  fprintf(f,"        movabsq $.LCPOW_26, %%rcx\n");
  fprintf(f,"        movapd  (%%rcx), %%xmm3\n");
  fprintf(f,"        mulpd   %%xmm0, %%xmm3\n");
  fprintf(f,"        addpd   (%%rax), %%xmm3\n");
  fprintf(f,"        mulpd   %%xmm0, %%xmm3\n");
  fprintf(f,"        addpd   (%%rdi), %%xmm3\n");
  fprintf(f,"        mulpd   %%xmm0, %%xmm3\n");
  fprintf(f,"        addpd   (%%rsi), %%xmm3\n");
  fprintf(f,"        mulpd   %%xmm0, %%xmm3\n");
  fprintf(f,"        addpd   (%%rdx), %%xmm3\n");
  fprintf(f,"        mulpd   %%xmm0, %%xmm3\n");
  fprintf(f,"        addpd   (%%r9), %%xmm3\n");
  fprintf(f,"        mulpd   %%xmm0, %%xmm3\n");
  fprintf(f,"        addpd   (%%r8), %%xmm3\n");
  fprintf(f,"        mulpd   %%xmm0, %%xmm3\n");
  fprintf(f,"        mulpd   %%xmm13, %%xmm4\n");
  fprintf(f,"        addpd   %%xmm6, %%xmm4\n");
  fprintf(f,"        movabsq $.LCPOW_27, %%rax\n");
  fprintf(f,"        movapd  (%%rax), %%xmm0\n");
  fprintf(f,"        mulpd   %%xmm0, %%xmm13\n");
  fprintf(f,"        mulpd   %%xmm0, %%xmm1\n");
  fprintf(f,"        movabsq $.LCPOW_28, %%rax\n");
  fprintf(f,"        addpd   (%%rax), %%xmm3\n");
  fprintf(f,"        movapd  %%xmm8, %%xmm5\n");
  fprintf(f,"        andpd   %%xmm9, %%xmm5\n");
  fprintf(f,"        movapd  %%xmm8, %%xmm0\n");
  fprintf(f,"        mulpd   %%xmm3, %%xmm0\n");
  fprintf(f,"        movapd  %%xmm0, %%xmm7\n");
  fprintf(f,"        xorpd   %%xmm15, %%xmm7\n");
  fprintf(f,"        movapd  %%xmm3, %%xmm6\n");
  fprintf(f,"        andpd   %%xmm9, %%xmm6\n");
  fprintf(f,"        movapd  %%xmm5, %%xmm2\n");
  fprintf(f,"        mulpd   %%xmm6, %%xmm2\n");
  fprintf(f,"        addpd   %%xmm7, %%xmm2\n");
  fprintf(f,"        subpd   %%xmm5, %%xmm8\n");
  fprintf(f,"        movapd  %%xmm8, %%xmm7\n");
  fprintf(f,"        mulpd   %%xmm6, %%xmm7\n");
  fprintf(f,"        addpd   %%xmm2, %%xmm7\n");
  fprintf(f,"        mulpd   %%xmm3, %%xmm4\n");
  fprintf(f,"        subpd   %%xmm6, %%xmm3\n");
  fprintf(f,"        mulpd   %%xmm3, %%xmm8\n");
  fprintf(f,"        mulpd   %%xmm5, %%xmm3\n");
  fprintf(f,"        addpd   %%xmm7, %%xmm3\n");
  fprintf(f,"        addpd   %%xmm8, %%xmm3\n");
  fprintf(f,"        addpd   %%xmm4, %%xmm3\n");
  fprintf(f,"        addpd   %%xmm1, %%xmm3\n");
  fprintf(f,"        movapd  %%xmm13, %%xmm4\n");
  fprintf(f,"        addpd   %%xmm0, %%xmm4\n");
  fprintf(f,"        movapd  %%xmm4, %%xmm2\n");
  fprintf(f,"        subpd   %%xmm13, %%xmm2\n");
  fprintf(f,"        subpd   %%xmm2, %%xmm0\n");
  fprintf(f,"        movapd  %%xmm4, %%xmm5\n");
  fprintf(f,"        subpd   %%xmm2, %%xmm5\n");
  fprintf(f,"        subpd   %%xmm5, %%xmm13\n");
  fprintf(f,"        addpd   %%xmm0, %%xmm13\n");
  fprintf(f,"        addpd   %%xmm3, %%xmm13\n");
  fprintf(f,"        addpd   -96(%%rsp), %%xmm13\n");
  fprintf(f,"        movapd  %%xmm14, %%xmm2\n");
  fprintf(f,"        addpd   %%xmm4, %%xmm2\n");
  fprintf(f,"        movapd  %%xmm2, %%xmm0\n");
  fprintf(f,"        subpd   %%xmm14, %%xmm0\n");
  fprintf(f,"        subpd   %%xmm0, %%xmm4\n");
  fprintf(f,"        movapd  %%xmm2, %%xmm3\n");
  fprintf(f,"        subpd   %%xmm0, %%xmm3\n");
  fprintf(f,"        subpd   %%xmm3, %%xmm14\n");
  fprintf(f,"        addpd   %%xmm4, %%xmm14\n");
  fprintf(f,"        addpd   %%xmm13, %%xmm14\n");
  fprintf(f,"        movapd  -48(%%rsp), %%xmm1\n");
  fprintf(f,"        movapd  %%xmm1, %%xmm0\n");
  fprintf(f,"        andpd   %%xmm9, %%xmm0\n");
  fprintf(f,"        movapd  %%xmm2, %%xmm3\n");
  fprintf(f,"        mulpd   %%xmm1, %%xmm3\n");
  fprintf(f,"        movapd  %%xmm3, %%xmm6\n");
  fprintf(f,"        xorpd   %%xmm15, %%xmm6\n");
  fprintf(f,"        movapd  %%xmm2, %%xmm4\n");
  fprintf(f,"        andpd   %%xmm9, %%xmm4\n");
  fprintf(f,"        movapd  %%xmm0, %%xmm5\n");
  fprintf(f,"        mulpd   %%xmm4, %%xmm5\n");
  fprintf(f,"        addpd   %%xmm6, %%xmm5\n");
  fprintf(f,"        mulpd   %%xmm1, %%xmm14\n");
  fprintf(f,"        subpd   %%xmm0, %%xmm1\n");
  fprintf(f,"        subpd   %%xmm4, %%xmm2\n");
  fprintf(f,"        mulpd   %%xmm2, %%xmm0\n");
  fprintf(f,"        addpd   %%xmm5, %%xmm0\n");
  fprintf(f,"        mulpd   %%xmm1, %%xmm4\n");
  fprintf(f,"        addpd   %%xmm0, %%xmm4\n");
  fprintf(f,"        mulpd   %%xmm1, %%xmm2\n");
  fprintf(f,"        addpd   %%xmm4, %%xmm2\n");
  fprintf(f,"        addpd   %%xmm14, %%xmm2\n");
  fprintf(f,"        movabsq $.LCPOW_29, %%rax\n");
  fprintf(f,"        movabsq $.LCPOW_30, %%rcx\n");
  fprintf(f,"        movapd  %%xmm3, %%xmm0\n");
  fprintf(f,"        addpd   %%xmm2, %%xmm0\n");
  fprintf(f,"        mulpd   (%%rcx), %%xmm0\n");
  fprintf(f,"        cvtpd2dq        %%xmm0, %%xmm8\n");
  fprintf(f,"        cvtdq2pd        %%xmm8, %%xmm0\n");
  fprintf(f,"        movapd  (%%rax), %%xmm1\n");
  fprintf(f,"        mulpd   %%xmm0, %%xmm1\n");
  fprintf(f,"        movapd  %%xmm1, %%xmm4\n");
  fprintf(f,"        addpd   %%xmm3, %%xmm4\n");
  fprintf(f,"        movapd  %%xmm4, %%xmm5\n");
  fprintf(f,"        subpd   %%xmm3, %%xmm5\n");
  fprintf(f,"        subpd   %%xmm5, %%xmm1\n");
  fprintf(f,"        movapd  %%xmm4, %%xmm6\n");
  fprintf(f,"        subpd   %%xmm5, %%xmm6\n");
  fprintf(f,"        subpd   %%xmm6, %%xmm3\n");
  fprintf(f,"        addpd   %%xmm1, %%xmm3\n");
  fprintf(f,"        addpd   %%xmm2, %%xmm3\n");
  fprintf(f,"        movabsq $.LCPOW_31, %%rax\n");
  fprintf(f,"        mulpd   (%%rax), %%xmm0\n");
  fprintf(f,"        movapd  %%xmm0, %%xmm11\n");
  fprintf(f,"        addpd   %%xmm4, %%xmm11\n");
  fprintf(f,"        movapd  %%xmm11, %%xmm1\n");
  fprintf(f,"        subpd   %%xmm4, %%xmm1\n");
  fprintf(f,"        movapd  %%xmm11, %%xmm2\n");
  fprintf(f,"        subpd   %%xmm1, %%xmm2\n");
  fprintf(f,"        subpd   %%xmm2, %%xmm4\n");
  fprintf(f,"        subpd   %%xmm1, %%xmm0\n");
  fprintf(f,"        addpd   %%xmm4, %%xmm0\n");
  fprintf(f,"        addpd   %%xmm3, %%xmm0\n");
  fprintf(f,"        movdqa  %%xmm8, %%xmm1\n");
  fprintf(f,"        psrad   $31, %%xmm1\n");
  fprintf(f,"        movdqa  %%xmm1, %%xmm3\n");
  fprintf(f,"        paddd   %%xmm8, %%xmm3\n");
  fprintf(f,"        psrad   $9, %%xmm3\n");
  fprintf(f,"        psubd   %%xmm1, %%xmm3\n");
  fprintf(f,"        pslld   $7, %%xmm3\n");
  fprintf(f,"        movdqa  %%xmm3, %%xmm1\n");
  fprintf(f,"        pslld   $2, %%xmm1\n");
  fprintf(f,"        psubd   %%xmm1, %%xmm8\n");
  fprintf(f,"        paddd   -80(%%rsp), %%xmm3\n");
  fprintf(f,"        movapd  %%xmm11, %%xmm2\n");
  fprintf(f,"        addpd   %%xmm0, %%xmm2\n");
  fprintf(f,"        movapd  %%xmm2, %%xmm1\n");
  fprintf(f,"        mulpd   %%xmm1, %%xmm1\n");
  fprintf(f,"        movapd  %%xmm1, %%xmm5\n");
  fprintf(f,"        xorpd   %%xmm15, %%xmm5\n");
  fprintf(f,"        movapd  %%xmm2, %%xmm4\n");
  fprintf(f,"        andpd   %%xmm9, %%xmm4\n");
  fprintf(f,"        movapd  %%xmm4, %%xmm6\n");
  fprintf(f,"        mulpd   %%xmm6, %%xmm6\n");
  fprintf(f,"        addpd   %%xmm5, %%xmm6\n");
  fprintf(f,"        movapd  %%xmm2, %%xmm5\n");
  fprintf(f,"        subpd   %%xmm4, %%xmm5\n");
  fprintf(f,"        addpd   %%xmm4, %%xmm4\n");
  fprintf(f,"        mulpd   %%xmm5, %%xmm4\n");
  fprintf(f,"        addpd   %%xmm6, %%xmm4\n");
  fprintf(f,"        movdqa  %%xmm3, %%xmm6\n");
  fprintf(f,"        psrad   $31, %%xmm6\n");
  fprintf(f,"        pandn   %%xmm3, %%xmm6\n");
  fprintf(f,"        movdqa  %%xmm6, %%xmm12\n");
  fprintf(f,"        pcmpgtd -16(%%rsp), %%xmm12\n");
  fprintf(f,"        movdqa  %%xmm12, %%xmm3\n");
  fprintf(f,"        pandn   %%xmm6, %%xmm3\n");
  fprintf(f,"        pand    -32(%%rsp), %%xmm12\n");
  fprintf(f,"        por     %%xmm3, %%xmm12\n");
  fprintf(f,"        paddd   -64(%%rsp), %%xmm8\n");
  fprintf(f,"        mulpd   %%xmm5, %%xmm5\n");
  fprintf(f,"        addpd   %%xmm4, %%xmm5\n");
  fprintf(f,"        subpd   %%xmm2, %%xmm11\n");
  fprintf(f,"        addpd   %%xmm0, %%xmm11\n");
  fprintf(f,"        movapd  %%xmm11, %%xmm0\n");
  fprintf(f,"        addpd   %%xmm0, %%xmm0\n");
  fprintf(f,"        mulpd   %%xmm2, %%xmm0\n");
  fprintf(f,"        addpd   %%xmm5, %%xmm0\n");
  fprintf(f,"        movabsq $.LCPOW_32, %%r8\n");
  fprintf(f,"        movabsq $.LCPOW_33, %%r9\n");
  fprintf(f,"        movabsq $.LCPOW_34, %%r10\n");
  fprintf(f,"        movabsq $.LCPOW_35, %%r11\n");
  fprintf(f,"        movabsq $.LCPOW_36, %%rdi\n");
  fprintf(f,"        movabsq $.LCPOW_37, %%rax\n");
  fprintf(f,"        movabsq $.LCPOW_38, %%rcx\n");
  fprintf(f,"        movabsq $.LCPOW_39, %%rdx\n");
  fprintf(f,"        movabsq $.LCPOW_40, %%rsi\n");
  fprintf(f,"        movabsq $.LCPOW_41, %%rbx\n");
  fprintf(f,"        movapd  (%%rbx), %%xmm3\n");
  fprintf(f,"        mulpd   %%xmm2, %%xmm3\n");
  fprintf(f,"        addpd   (%%rsi), %%xmm3\n");
  fprintf(f,"        mulpd   %%xmm2, %%xmm3\n");
  fprintf(f,"        addpd   (%%rdx), %%xmm3\n");
  fprintf(f,"        mulpd   %%xmm2, %%xmm3\n");
  fprintf(f,"        addpd   (%%rcx), %%xmm3\n");
  fprintf(f,"        mulpd   %%xmm2, %%xmm3\n");
  fprintf(f,"        addpd   (%%rax), %%xmm3\n");
  fprintf(f,"        mulpd   %%xmm2, %%xmm3\n");
  fprintf(f,"        addpd   (%%rdi), %%xmm3\n");
  fprintf(f,"        mulpd   %%xmm2, %%xmm3\n");
  fprintf(f,"        addpd   (%%r11), %%xmm3\n");
  fprintf(f,"        mulpd   %%xmm2, %%xmm3\n");
  fprintf(f,"        addpd   (%%r10), %%xmm3\n");
  fprintf(f,"        mulpd   %%xmm2, %%xmm3\n");
  fprintf(f,"        addpd   (%%r9), %%xmm3\n");
  fprintf(f,"        mulpd   %%xmm2, %%xmm3\n");
  fprintf(f,"        addpd   (%%r8), %%xmm3\n");
  fprintf(f,"        movapd  %%xmm1, %%xmm5\n");
  fprintf(f,"        mulpd   %%xmm3, %%xmm5\n");
  fprintf(f,"        xorpd   %%xmm5, %%xmm15\n");
  fprintf(f,"        movapd  %%xmm1, %%xmm4\n");
  fprintf(f,"        andpd   %%xmm9, %%xmm4\n");
  fprintf(f,"        andpd   %%xmm3, %%xmm9\n");
  fprintf(f,"        movapd  %%xmm4, %%xmm6\n");
  fprintf(f,"        mulpd   %%xmm9, %%xmm6\n");
  fprintf(f,"        addpd   %%xmm15, %%xmm6\n");
  fprintf(f,"        subpd   %%xmm4, %%xmm1\n");
  fprintf(f,"        movapd  %%xmm1, %%xmm7\n");
  fprintf(f,"        mulpd   %%xmm9, %%xmm7\n");
  fprintf(f,"        addpd   %%xmm6, %%xmm7\n");
  fprintf(f,"        mulpd   %%xmm3, %%xmm0\n");
  fprintf(f,"        subpd   %%xmm9, %%xmm3\n");
  fprintf(f,"        mulpd   %%xmm3, %%xmm4\n");
  fprintf(f,"        addpd   %%xmm7, %%xmm4\n");
  fprintf(f,"        mulpd   %%xmm1, %%xmm3\n");
  fprintf(f,"        addpd   %%xmm4, %%xmm3\n");
  fprintf(f,"        addpd   %%xmm0, %%xmm3\n");
  fprintf(f,"        movapd  %%xmm2, %%xmm0\n");
  fprintf(f,"        addpd   %%xmm5, %%xmm0\n");
  fprintf(f,"        subpd   %%xmm0, %%xmm2\n");
  fprintf(f,"        addpd   %%xmm5, %%xmm2\n");
  fprintf(f,"        addpd   %%xmm11, %%xmm2\n");
  fprintf(f,"        addpd   %%xmm3, %%xmm2\n");
  fprintf(f,"        movapd  %%xmm0, %%xmm1\n");
  fprintf(f,"        movapd  %%xmm10, %%xmm3\n");
  fprintf(f,"        addpd   %%xmm3, %%xmm1\n");
  fprintf(f,"        subpd   %%xmm1, %%xmm3\n");
  fprintf(f,"        addpd   %%xmm0, %%xmm3\n");
  fprintf(f,"        addpd   %%xmm2, %%xmm3\n");
  fprintf(f,"        addpd   %%xmm1, %%xmm3\n");
  fprintf(f,"        pshufd  $115, %%xmm12, %%xmm0     # xmm0 = xmm12[3,0,3,1]\n");
  fprintf(f,"        pslld   $20, %%xmm0\n");
  fprintf(f,"        mulpd   %%xmm0, %%xmm3\n");
  fprintf(f,"        mulpd   %%xmm0, %%xmm3\n");
  fprintf(f,"        mulpd   %%xmm0, %%xmm3\n");
  fprintf(f,"        mulpd   %%xmm0, %%xmm3\n");
  fprintf(f,"        movapd  %%xmm3, %%xmm4\n");
  fprintf(f,"        movapd  64(%%rsp), %%xmm2\n");
  fprintf(f,"        movapd  32(%%rsp), %%xmm5\n");
  fprintf(f,"        andpd   %%xmm5, %%xmm2\n");
  fprintf(f,"        movapd  128(%%rsp), %%xmm6\n");
  fprintf(f,"        movapd  112(%%rsp), %%xmm7\n");
  fprintf(f,"        andpd   %%xmm6, %%xmm7\n");
  fprintf(f,"        movapd  (%%rsp), %%xmm1\n");
  fprintf(f,"        movapd  48(%%rsp), %%xmm3\n");
  fprintf(f,"        andpd   %%xmm3, %%xmm1\n");
  fprintf(f,"        pshufd  $115, %%xmm8, %%xmm0      # xmm0 = xmm8[3,0,3,1]\n");
  fprintf(f,"        pslld   $20, %%xmm0\n");
  fprintf(f,"        mulpd   %%xmm4, %%xmm0\n");
  fprintf(f,"        mulpd   16(%%rsp), %%xmm0\n");
  fprintf(f,"        andnpd  %%xmm0, %%xmm6\n");
  fprintf(f,"        orpd    %%xmm7, %%xmm6\n");
  fprintf(f,"        movapd  %%xmm5, %%xmm0\n");
  fprintf(f,"        andnpd  %%xmm6, %%xmm0\n");
  fprintf(f,"        orpd    %%xmm2, %%xmm0\n");
  fprintf(f,"        orpd    80(%%rsp), %%xmm0\n");
  fprintf(f,"        movapd  %%xmm0, %%xmm2\n");
  fprintf(f,"        movapd  %%xmm3, %%xmm0\n");
  fprintf(f,"        andnpd  %%xmm2, %%xmm0\n");
  fprintf(f,"        orpd    %%xmm1, %%xmm0\n");
  fprintf(f,"        # This routine sometimes leaves IE and PE flags set, so I manually clear them here\n");
  fprintf(f,"        andl     $0xFFC0,.Lnewmxcsr  # clear exception in memory copy of MXCSR\n");
  fprintf(f,"        ldmxcsr .Lnewmxcsr        # and reload that into real MXCSR\n");
  fprintf(f,"        addq    $144, %%rsp\n");
  fprintf(f,"        popq    %%rbx\n");
  fprintf(f,"        retq\n");
  fprintf(f,"        .size   xpow, .-xpow\n\n");
  fflush(f);
  return;
}

